/* This file is auto-generated by opth-gen.awk.  */

#ifndef OPTIONS_H
#define OPTIONS_H

#include "flag-types.h"

#include "config/i386/i386-opts.h"

#if !defined(IN_LIBGCC2) && !defined(IN_TARGET_LIBS) && !defined(IN_RTS)
#ifndef GENERATOR_FILE
#if !defined(IN_LIBGCC2) && !defined(IN_TARGET_LIBS)
struct GTY(()) gcc_options
#else
struct gcc_options
#endif
{
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT ix86_stack_protector_guard_offset;
#else
  HOST_WIDE_INT x_ix86_stack_protector_guard_offset;
#define ix86_stack_protector_guard_offset global_options.x_ix86_stack_protector_guard_offset
#endif
#ifdef GENERATOR_FILE
extern addr_space_t ix86_stack_protector_guard_reg;
#else
  addr_space_t x_ix86_stack_protector_guard_reg;
#define ix86_stack_protector_guard_reg global_options.x_ix86_stack_protector_guard_reg
#endif
#ifdef GENERATOR_FILE
extern bool ix86_unsafe_math_optimizations;
#else
  bool x_ix86_unsafe_math_optimizations;
#define ix86_unsafe_math_optimizations global_options.x_ix86_unsafe_math_optimizations
#endif
#ifdef GENERATOR_FILE
extern enum calling_abi ix86_abi;
#else
  enum calling_abi x_ix86_abi;
#define ix86_abi global_options.x_ix86_abi
#endif
#ifdef GENERATOR_FILE
extern enum cmodel ix86_cmodel;
#else
  enum cmodel x_ix86_cmodel;
#define ix86_cmodel global_options.x_ix86_cmodel
#endif
#ifdef GENERATOR_FILE
extern enum excess_precision ix86_excess_precision;
#else
  enum excess_precision x_ix86_excess_precision;
#define ix86_excess_precision global_options.x_ix86_excess_precision
#endif
#ifdef GENERATOR_FILE
extern enum ix86_veclibabi ix86_veclibabi_type;
#else
  enum ix86_veclibabi x_ix86_veclibabi_type;
#define ix86_veclibabi_type global_options.x_ix86_veclibabi_type
#endif
#ifdef GENERATOR_FILE
extern enum pmode ix86_pmode;
#else
  enum pmode x_ix86_pmode;
#define ix86_pmode global_options.x_ix86_pmode
#endif
#ifdef GENERATOR_FILE
extern int ix86_incoming_stack_boundary_arg;
#else
  int x_ix86_incoming_stack_boundary_arg;
#define ix86_incoming_stack_boundary_arg global_options.x_ix86_incoming_stack_boundary_arg
#endif
#ifdef GENERATOR_FILE
extern int ix86_preferred_stack_boundary_arg;
#else
  int x_ix86_preferred_stack_boundary_arg;
#define ix86_preferred_stack_boundary_arg global_options.x_ix86_preferred_stack_boundary_arg
#endif
#ifdef GENERATOR_FILE
extern int ix86_regparm;
#else
  int x_ix86_regparm;
#define ix86_regparm global_options.x_ix86_regparm
#endif
#ifdef GENERATOR_FILE
extern int recip_mask;
#else
  int x_recip_mask;
#define recip_mask global_options.x_recip_mask
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT ix86_isa_flags;
#else
  HOST_WIDE_INT x_ix86_isa_flags;
#define ix86_isa_flags global_options.x_ix86_isa_flags
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT ix86_isa_flags2;
#else
  HOST_WIDE_INT x_ix86_isa_flags2;
#define ix86_isa_flags2 global_options.x_ix86_isa_flags2
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT ix86_isa_flags2_explicit;
#else
  HOST_WIDE_INT x_ix86_isa_flags2_explicit;
#define ix86_isa_flags2_explicit global_options.x_ix86_isa_flags2_explicit
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT ix86_isa_flags_explicit;
#else
  HOST_WIDE_INT x_ix86_isa_flags_explicit;
#define ix86_isa_flags_explicit global_options.x_ix86_isa_flags_explicit
#endif
#ifdef GENERATOR_FILE
extern bool dump_base_name_prefixed;
#else
  bool x_dump_base_name_prefixed;
#define dump_base_name_prefixed global_options.x_dump_base_name_prefixed
#endif
#ifdef GENERATOR_FILE
extern bool exit_after_options;
#else
  bool x_exit_after_options;
#define exit_after_options global_options.x_exit_after_options
#endif
#ifdef GENERATOR_FILE
extern bool flag_dump_all_passed;
#else
  bool x_flag_dump_all_passed;
#define flag_dump_all_passed global_options.x_flag_dump_all_passed
#endif
#ifdef GENERATOR_FILE
extern bool flag_opts_finished;
#else
  bool x_flag_opts_finished;
#define flag_opts_finished global_options.x_flag_opts_finished
#endif
#ifdef GENERATOR_FILE
extern bool flag_stack_usage_info;
#else
  bool x_flag_stack_usage_info;
#define flag_stack_usage_info global_options.x_flag_stack_usage_info
#endif
#ifdef GENERATOR_FILE
extern bool flag_warn_unused_result;
#else
  bool x_flag_warn_unused_result;
#define flag_warn_unused_result global_options.x_flag_warn_unused_result
#endif
#ifdef GENERATOR_FILE
extern bool in_lto_p;
#else
  bool x_in_lto_p;
#define in_lto_p global_options.x_in_lto_p
#endif
#ifdef GENERATOR_FILE
extern char *help_enum_printed;
#else
  char * x_help_enum_printed;
#define help_enum_printed global_options.x_help_enum_printed
#endif
#ifdef GENERATOR_FILE
extern char *help_printed;
#else
  char * x_help_printed;
#define help_printed global_options.x_help_printed
#endif
#ifdef GENERATOR_FILE
extern const char *aux_base_name;
#else
  const char * x_aux_base_name;
#define aux_base_name global_options.x_aux_base_name
#endif
#ifdef GENERATOR_FILE
extern const char *main_input_basename;
#else
  const char * x_main_input_basename;
#define main_input_basename global_options.x_main_input_basename
#endif
#ifdef GENERATOR_FILE
extern const char *main_input_filename;
#else
  const char * x_main_input_filename;
#define main_input_filename global_options.x_main_input_filename
#endif
#ifdef GENERATOR_FILE
extern enum ctf_debug_info_levels ctf_debug_info_level;
#else
  enum ctf_debug_info_levels x_ctf_debug_info_level;
#define ctf_debug_info_level global_options.x_ctf_debug_info_level
#endif
#ifdef GENERATOR_FILE
extern enum debug_info_levels debug_info_level;
#else
  enum debug_info_levels x_debug_info_level;
#define debug_info_level global_options.x_debug_info_level
#endif
#ifdef GENERATOR_FILE
extern enum debug_struct_file debug_struct_generic[DINFO_USAGE_NUM_ENUMS];
#else
  enum debug_struct_file x_debug_struct_generic[DINFO_USAGE_NUM_ENUMS];
#define debug_struct_generic global_options.x_debug_struct_generic
#endif
#ifdef GENERATOR_FILE
extern enum debug_struct_file debug_struct_ordinary[DINFO_USAGE_NUM_ENUMS];
#else
  enum debug_struct_file x_debug_struct_ordinary[DINFO_USAGE_NUM_ENUMS];
#define debug_struct_ordinary global_options.x_debug_struct_ordinary
#endif
#ifdef GENERATOR_FILE
extern enum dwarf_gnat_encodings gnat_encodings;
#else
  enum dwarf_gnat_encodings x_gnat_encodings;
#define gnat_encodings global_options.x_gnat_encodings
#endif
#ifdef GENERATOR_FILE
extern enum incremental_link flag_incremental_link;
#else
  enum incremental_link x_flag_incremental_link;
#define flag_incremental_link global_options.x_flag_incremental_link
#endif
#ifdef GENERATOR_FILE
extern enum stack_check_type flag_stack_check;
#else
  enum stack_check_type x_flag_stack_check;
#define flag_stack_check global_options.x_flag_stack_check
#endif
#ifdef GENERATOR_FILE
extern int flag_complex_method;
#else
  int x_flag_complex_method;
#define flag_complex_method global_options.x_flag_complex_method
#endif
#ifdef GENERATOR_FILE
extern int flag_debug_asm;
#else
  int x_flag_debug_asm;
#define flag_debug_asm global_options.x_flag_debug_asm
#endif
#ifdef GENERATOR_FILE
extern int flag_default_complex_method;
#else
  int x_flag_default_complex_method;
#define flag_default_complex_method global_options.x_flag_default_complex_method
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_rtl_in_asm;
#else
  int x_flag_dump_rtl_in_asm;
#define flag_dump_rtl_in_asm global_options.x_flag_dump_rtl_in_asm
#endif
#ifdef GENERATOR_FILE
extern int flag_gen_aux_info;
#else
  int x_flag_gen_aux_info;
#define flag_gen_aux_info global_options.x_flag_gen_aux_info
#endif
#ifdef GENERATOR_FILE
extern int flag_generate_lto;
#else
  int x_flag_generate_lto;
#define flag_generate_lto global_options.x_flag_generate_lto
#endif
#ifdef GENERATOR_FILE
extern int flag_generate_offload;
#else
  int x_flag_generate_offload;
#define flag_generate_offload global_options.x_flag_generate_offload
#endif
#ifdef GENERATOR_FILE
extern int flag_print_asm_name;
#else
  int x_flag_print_asm_name;
#define flag_print_asm_name global_options.x_flag_print_asm_name
#endif
#ifdef GENERATOR_FILE
extern int flag_shlib;
#else
  int x_flag_shlib;
#define flag_shlib global_options.x_flag_shlib
#endif
#ifdef GENERATOR_FILE
extern int ix86_no_avx10_1_explicit;
#else
  int x_ix86_no_avx10_1_explicit;
#define ix86_no_avx10_1_explicit global_options.x_ix86_no_avx10_1_explicit
#endif
#ifdef GENERATOR_FILE
extern int ix86_no_avx512_explicit;
#else
  int x_ix86_no_avx512_explicit;
#define ix86_no_avx512_explicit global_options.x_ix86_no_avx512_explicit
#endif
#ifdef GENERATOR_FILE
extern int ix86_target_flags;
#else
  int x_ix86_target_flags;
#define ix86_target_flags global_options.x_ix86_target_flags
#endif
#ifdef GENERATOR_FILE
extern int main_input_baselength;
#else
  int x_main_input_baselength;
#define main_input_baselength global_options.x_main_input_baselength
#endif
#ifdef GENERATOR_FILE
extern int optimize;
#else
  int x_optimize;
#define optimize global_options.x_optimize
#endif
#ifdef GENERATOR_FILE
extern int optimize_debug;
#else
  int x_optimize_debug;
#define optimize_debug global_options.x_optimize_debug
#endif
#ifdef GENERATOR_FILE
extern int optimize_fast;
#else
  int x_optimize_fast;
#define optimize_fast global_options.x_optimize_fast
#endif
#ifdef GENERATOR_FILE
extern int optimize_size;
#else
  int x_optimize_size;
#define optimize_size global_options.x_optimize_size
#endif
#ifdef GENERATOR_FILE
extern int recip_mask_explicit;
#else
  int x_recip_mask_explicit;
#define recip_mask_explicit global_options.x_recip_mask_explicit
#endif
#ifdef GENERATOR_FILE
extern int rtl_dump_and_exit;
#else
  int x_rtl_dump_and_exit;
#define rtl_dump_and_exit global_options.x_rtl_dump_and_exit
#endif
#ifdef GENERATOR_FILE
extern int target_flags;
#else
  int x_target_flags;
#define target_flags global_options.x_target_flags
#endif
#ifdef GENERATOR_FILE
extern uint32_t write_symbols;
#else
  uint32_t x_write_symbols;
#define write_symbols global_options.x_write_symbols
#endif
#ifdef GENERATOR_FILE
extern unsigned int flag_sanitize;
#else
  unsigned int x_flag_sanitize;
#define flag_sanitize global_options.x_flag_sanitize
#endif
#ifdef GENERATOR_FILE
extern unsigned int flag_sanitize_recover;
#else
  unsigned int x_flag_sanitize_recover;
#define flag_sanitize_recover global_options.x_flag_sanitize_recover
#endif
#ifdef GENERATOR_FILE
extern unsigned int flag_sanitize_trap;
#else
  unsigned int x_flag_sanitize_trap;
#define flag_sanitize_trap global_options.x_flag_sanitize_trap
#endif
#ifdef GENERATOR_FILE
extern unsigned int flag_zero_call_used_regs;
#else
  unsigned int x_flag_zero_call_used_regs;
#define flag_zero_call_used_regs global_options.x_flag_zero_call_used_regs
#endif
#ifdef GENERATOR_FILE
extern unsigned int help_columns;
#else
  unsigned int x_help_columns;
#define help_columns global_options.x_help_columns
#endif
#ifdef GENERATOR_FILE
extern unsigned int initial_max_fld_align;
#else
  unsigned int x_initial_max_fld_align;
#define initial_max_fld_align global_options.x_initial_max_fld_align
#endif
#ifdef GENERATOR_FILE
extern void *flag_ignored_attributes;
#else
  void * x_flag_ignored_attributes;
#define flag_ignored_attributes global_options.x_flag_ignored_attributes
#endif
#ifdef GENERATOR_FILE
extern void *flag_instrument_functions_exclude_files;
#else
  void * x_flag_instrument_functions_exclude_files;
#define flag_instrument_functions_exclude_files global_options.x_flag_instrument_functions_exclude_files
#endif
#ifdef GENERATOR_FILE
extern void *flag_instrument_functions_exclude_functions;
#else
  void * x_flag_instrument_functions_exclude_functions;
#define flag_instrument_functions_exclude_functions global_options.x_flag_instrument_functions_exclude_functions
#endif
#ifdef GENERATOR_FILE
extern int help_flag;
#else
  int x_help_flag;
#define help_flag global_options.x_help_flag
#endif
#ifdef GENERATOR_FILE
extern int no_sysroot_suffix;
#else
  int x_no_sysroot_suffix;
#define no_sysroot_suffix global_options.x_no_sysroot_suffix
#endif
#ifdef GENERATOR_FILE
extern int param_align_loop_iterations;
#else
  int x_param_align_loop_iterations;
#define param_align_loop_iterations global_options.x_param_align_loop_iterations
#endif
#ifdef GENERATOR_FILE
extern int param_align_threshold;
#else
  int x_param_align_threshold;
#define param_align_threshold global_options.x_param_align_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_bb_explosion_factor;
#else
  int x_param_analyzer_bb_explosion_factor;
#define param_analyzer_bb_explosion_factor global_options.x_param_analyzer_bb_explosion_factor
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_max_constraints;
#else
  int x_param_analyzer_max_constraints;
#define param_analyzer_max_constraints global_options.x_param_analyzer_max_constraints
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_max_enodes_for_full_dump;
#else
  int x_param_analyzer_max_enodes_for_full_dump;
#define param_analyzer_max_enodes_for_full_dump global_options.x_param_analyzer_max_enodes_for_full_dump
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_max_enodes_per_program_point;
#else
  int x_param_analyzer_max_enodes_per_program_point;
#define param_analyzer_max_enodes_per_program_point global_options.x_param_analyzer_max_enodes_per_program_point
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_max_infeasible_edges;
#else
  int x_param_analyzer_max_infeasible_edges;
#define param_analyzer_max_infeasible_edges global_options.x_param_analyzer_max_infeasible_edges
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_max_recursion_depth;
#else
  int x_param_analyzer_max_recursion_depth;
#define param_analyzer_max_recursion_depth global_options.x_param_analyzer_max_recursion_depth
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_max_svalue_depth;
#else
  int x_param_analyzer_max_svalue_depth;
#define param_analyzer_max_svalue_depth global_options.x_param_analyzer_max_svalue_depth
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_min_snodes_for_call_summary;
#else
  int x_param_analyzer_min_snodes_for_call_summary;
#define param_analyzer_min_snodes_for_call_summary global_options.x_param_analyzer_min_snodes_for_call_summary
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_text_art_ideal_canvas_width;
#else
  int x_param_analyzer_text_art_ideal_canvas_width;
#define param_analyzer_text_art_ideal_canvas_width global_options.x_param_analyzer_text_art_ideal_canvas_width
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_text_art_string_ellipsis_head_len;
#else
  int x_param_analyzer_text_art_string_ellipsis_head_len;
#define param_analyzer_text_art_string_ellipsis_head_len global_options.x_param_analyzer_text_art_string_ellipsis_head_len
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_text_art_string_ellipsis_tail_len;
#else
  int x_param_analyzer_text_art_string_ellipsis_tail_len;
#define param_analyzer_text_art_string_ellipsis_tail_len global_options.x_param_analyzer_text_art_string_ellipsis_tail_len
#endif
#ifdef GENERATOR_FILE
extern int param_analyzer_text_art_string_ellipsis_threshold;
#else
  int x_param_analyzer_text_art_string_ellipsis_threshold;
#define param_analyzer_text_art_string_ellipsis_threshold global_options.x_param_analyzer_text_art_string_ellipsis_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_asan_globals;
#else
  int x_param_asan_globals;
#define param_asan_globals global_options.x_param_asan_globals
#endif
#ifdef GENERATOR_FILE
extern int param_asan_protect_allocas;
#else
  int x_param_asan_protect_allocas;
#define param_asan_protect_allocas global_options.x_param_asan_protect_allocas
#endif
#ifdef GENERATOR_FILE
extern int param_asan_instrument_reads;
#else
  int x_param_asan_instrument_reads;
#define param_asan_instrument_reads global_options.x_param_asan_instrument_reads
#endif
#ifdef GENERATOR_FILE
extern int param_asan_instrument_writes;
#else
  int x_param_asan_instrument_writes;
#define param_asan_instrument_writes global_options.x_param_asan_instrument_writes
#endif
#ifdef GENERATOR_FILE
extern int param_asan_instrumentation_with_call_threshold;
#else
  int x_param_asan_instrumentation_with_call_threshold;
#define param_asan_instrumentation_with_call_threshold global_options.x_param_asan_instrumentation_with_call_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_asan_kernel_mem_intrinsic_prefix;
#else
  int x_param_asan_kernel_mem_intrinsic_prefix;
#define param_asan_kernel_mem_intrinsic_prefix global_options.x_param_asan_kernel_mem_intrinsic_prefix
#endif
#ifdef GENERATOR_FILE
extern int param_asan_memintrin;
#else
  int x_param_asan_memintrin;
#define param_asan_memintrin global_options.x_param_asan_memintrin
#endif
#ifdef GENERATOR_FILE
extern int param_asan_stack;
#else
  int x_param_asan_stack;
#define param_asan_stack global_options.x_param_asan_stack
#endif
#ifdef GENERATOR_FILE
extern int param_asan_use_after_return;
#else
  int x_param_asan_use_after_return;
#define param_asan_use_after_return global_options.x_param_asan_use_after_return
#endif
#ifdef GENERATOR_FILE
extern int param_avg_loop_niter;
#else
  int x_param_avg_loop_niter;
#define param_avg_loop_niter global_options.x_param_avg_loop_niter
#endif
#ifdef GENERATOR_FILE
extern int param_avoid_fma_max_bits;
#else
  int x_param_avoid_fma_max_bits;
#define param_avoid_fma_max_bits global_options.x_param_avoid_fma_max_bits
#endif
#ifdef GENERATOR_FILE
extern int param_builtin_expect_probability;
#else
  int x_param_builtin_expect_probability;
#define param_builtin_expect_probability global_options.x_param_builtin_expect_probability
#endif
#ifdef GENERATOR_FILE
extern int param_builtin_string_cmp_inline_length;
#else
  int x_param_builtin_string_cmp_inline_length;
#define param_builtin_string_cmp_inline_length global_options.x_param_builtin_string_cmp_inline_length
#endif
#ifdef GENERATOR_FILE
extern int param_case_values_threshold;
#else
  int x_param_case_values_threshold;
#define param_case_values_threshold global_options.x_param_case_values_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_comdat_sharing_probability;
#else
  int x_param_comdat_sharing_probability;
#define param_comdat_sharing_probability global_options.x_param_comdat_sharing_probability
#endif
#ifdef GENERATOR_FILE
extern int param_construct_interfere_size;
#else
  int x_param_construct_interfere_size;
#define param_construct_interfere_size global_options.x_param_construct_interfere_size
#endif
#ifdef GENERATOR_FILE
extern int param_cxx_max_namespaces_for_diagnostic_help;
#else
  int x_param_cxx_max_namespaces_for_diagnostic_help;
#define param_cxx_max_namespaces_for_diagnostic_help global_options.x_param_cxx_max_namespaces_for_diagnostic_help
#endif
#ifdef GENERATOR_FILE
extern int param_destruct_interfere_size;
#else
  int x_param_destruct_interfere_size;
#define param_destruct_interfere_size global_options.x_param_destruct_interfere_size
#endif
#ifdef GENERATOR_FILE
extern int param_dse_max_alias_queries_per_store;
#else
  int x_param_dse_max_alias_queries_per_store;
#define param_dse_max_alias_queries_per_store global_options.x_param_dse_max_alias_queries_per_store
#endif
#ifdef GENERATOR_FILE
extern int param_dse_max_object_size;
#else
  int x_param_dse_max_object_size;
#define param_dse_max_object_size global_options.x_param_dse_max_object_size
#endif
#ifdef GENERATOR_FILE
extern int param_early_inlining_insns;
#else
  int x_param_early_inlining_insns;
#define param_early_inlining_insns global_options.x_param_early_inlining_insns
#endif
#ifdef GENERATOR_FILE
extern int param_fsm_scale_path_stmts;
#else
  int x_param_fsm_scale_path_stmts;
#define param_fsm_scale_path_stmts global_options.x_param_fsm_scale_path_stmts
#endif
#ifdef GENERATOR_FILE
extern int param_fully_pipelined_fma;
#else
  int x_param_fully_pipelined_fma;
#define param_fully_pipelined_fma global_options.x_param_fully_pipelined_fma
#endif
#ifdef GENERATOR_FILE
extern int param_gcse_after_reload_critical_fraction;
#else
  int x_param_gcse_after_reload_critical_fraction;
#define param_gcse_after_reload_critical_fraction global_options.x_param_gcse_after_reload_critical_fraction
#endif
#ifdef GENERATOR_FILE
extern int param_gcse_after_reload_partial_fraction;
#else
  int x_param_gcse_after_reload_partial_fraction;
#define param_gcse_after_reload_partial_fraction global_options.x_param_gcse_after_reload_partial_fraction
#endif
#ifdef GENERATOR_FILE
extern int param_gcse_cost_distance_ratio;
#else
  int x_param_gcse_cost_distance_ratio;
#define param_gcse_cost_distance_ratio global_options.x_param_gcse_cost_distance_ratio
#endif
#ifdef GENERATOR_FILE
extern int param_gcse_unrestricted_cost;
#else
  int x_param_gcse_unrestricted_cost;
#define param_gcse_unrestricted_cost global_options.x_param_gcse_unrestricted_cost
#endif
#ifdef GENERATOR_FILE
extern int param_ggc_min_expand;
#else
  int x_param_ggc_min_expand;
#define param_ggc_min_expand global_options.x_param_ggc_min_expand
#endif
#ifdef GENERATOR_FILE
extern int param_ggc_min_heapsize;
#else
  int x_param_ggc_min_heapsize;
#define param_ggc_min_heapsize global_options.x_param_ggc_min_heapsize
#endif
#ifdef GENERATOR_FILE
extern int param_gimple_fe_computed_hot_bb_threshold;
#else
  int x_param_gimple_fe_computed_hot_bb_threshold;
#define param_gimple_fe_computed_hot_bb_threshold global_options.x_param_gimple_fe_computed_hot_bb_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_graphite_allow_codegen_errors;
#else
  int x_param_graphite_allow_codegen_errors;
#define param_graphite_allow_codegen_errors global_options.x_param_graphite_allow_codegen_errors
#endif
#ifdef GENERATOR_FILE
extern int param_graphite_max_arrays_per_scop;
#else
  int x_param_graphite_max_arrays_per_scop;
#define param_graphite_max_arrays_per_scop global_options.x_param_graphite_max_arrays_per_scop
#endif
#ifdef GENERATOR_FILE
extern int param_graphite_max_nb_scop_params;
#else
  int x_param_graphite_max_nb_scop_params;
#define param_graphite_max_nb_scop_params global_options.x_param_graphite_max_nb_scop_params
#endif
#ifdef GENERATOR_FILE
extern int param_hardcfr_max_blocks;
#else
  int x_param_hardcfr_max_blocks;
#define param_hardcfr_max_blocks global_options.x_param_hardcfr_max_blocks
#endif
#ifdef GENERATOR_FILE
extern int param_hardcfr_max_inline_blocks;
#else
  int x_param_hardcfr_max_inline_blocks;
#define param_hardcfr_max_inline_blocks global_options.x_param_hardcfr_max_inline_blocks
#endif
#ifdef GENERATOR_FILE
extern int param_hash_table_verification_limit;
#else
  int x_param_hash_table_verification_limit;
#define param_hash_table_verification_limit global_options.x_param_hash_table_verification_limit
#endif
#ifdef GENERATOR_FILE
extern int param_hot_bb_count_fraction;
#else
  int x_param_hot_bb_count_fraction;
#define param_hot_bb_count_fraction global_options.x_param_hot_bb_count_fraction
#endif
#ifdef GENERATOR_FILE
extern int param_hot_bb_count_ws_permille;
#else
  int x_param_hot_bb_count_ws_permille;
#define param_hot_bb_count_ws_permille global_options.x_param_hot_bb_count_ws_permille
#endif
#ifdef GENERATOR_FILE
extern int param_hot_bb_frequency_fraction;
#else
  int x_param_hot_bb_frequency_fraction;
#define param_hot_bb_frequency_fraction global_options.x_param_hot_bb_frequency_fraction
#endif
#ifdef GENERATOR_FILE
extern int param_hwasan_instrument_allocas;
#else
  int x_param_hwasan_instrument_allocas;
#define param_hwasan_instrument_allocas global_options.x_param_hwasan_instrument_allocas
#endif
#ifdef GENERATOR_FILE
extern int param_hwasan_instrument_mem_intrinsics;
#else
  int x_param_hwasan_instrument_mem_intrinsics;
#define param_hwasan_instrument_mem_intrinsics global_options.x_param_hwasan_instrument_mem_intrinsics
#endif
#ifdef GENERATOR_FILE
extern int param_hwasan_instrument_reads;
#else
  int x_param_hwasan_instrument_reads;
#define param_hwasan_instrument_reads global_options.x_param_hwasan_instrument_reads
#endif
#ifdef GENERATOR_FILE
extern int param_hwasan_instrument_stack;
#else
  int x_param_hwasan_instrument_stack;
#define param_hwasan_instrument_stack global_options.x_param_hwasan_instrument_stack
#endif
#ifdef GENERATOR_FILE
extern int param_hwasan_instrument_writes;
#else
  int x_param_hwasan_instrument_writes;
#define param_hwasan_instrument_writes global_options.x_param_hwasan_instrument_writes
#endif
#ifdef GENERATOR_FILE
extern int param_hwasan_random_frame_tag;
#else
  int x_param_hwasan_random_frame_tag;
#define param_hwasan_random_frame_tag global_options.x_param_hwasan_random_frame_tag
#endif
#ifdef GENERATOR_FILE
extern int param_inline_heuristics_hint_percent;
#else
  int x_param_inline_heuristics_hint_percent;
#define param_inline_heuristics_hint_percent global_options.x_param_inline_heuristics_hint_percent
#endif
#ifdef GENERATOR_FILE
extern int param_inline_min_speedup;
#else
  int x_param_inline_min_speedup;
#define param_inline_min_speedup global_options.x_param_inline_min_speedup
#endif
#ifdef GENERATOR_FILE
extern int param_inline_unit_growth;
#else
  int x_param_inline_unit_growth;
#define param_inline_unit_growth global_options.x_param_inline_unit_growth
#endif
#ifdef GENERATOR_FILE
extern int param_integer_share_limit;
#else
  int x_param_integer_share_limit;
#define param_integer_share_limit global_options.x_param_integer_share_limit
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_eval_threshold;
#else
  int x_param_ipa_cp_eval_threshold;
#define param_ipa_cp_eval_threshold global_options.x_param_ipa_cp_eval_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_large_unit_insns;
#else
  int x_param_ipa_cp_large_unit_insns;
#define param_ipa_cp_large_unit_insns global_options.x_param_ipa_cp_large_unit_insns
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_loop_hint_bonus;
#else
  int x_param_ipa_cp_loop_hint_bonus;
#define param_ipa_cp_loop_hint_bonus global_options.x_param_ipa_cp_loop_hint_bonus
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_max_recursive_depth;
#else
  int x_param_ipa_cp_max_recursive_depth;
#define param_ipa_cp_max_recursive_depth global_options.x_param_ipa_cp_max_recursive_depth
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_min_recursive_probability;
#else
  int x_param_ipa_cp_min_recursive_probability;
#define param_ipa_cp_min_recursive_probability global_options.x_param_ipa_cp_min_recursive_probability
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_profile_count_base;
#else
  int x_param_ipa_cp_profile_count_base;
#define param_ipa_cp_profile_count_base global_options.x_param_ipa_cp_profile_count_base
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_recursion_penalty;
#else
  int x_param_ipa_cp_recursion_penalty;
#define param_ipa_cp_recursion_penalty global_options.x_param_ipa_cp_recursion_penalty
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_recursive_freq_factor;
#else
  int x_param_ipa_cp_recursive_freq_factor;
#define param_ipa_cp_recursive_freq_factor global_options.x_param_ipa_cp_recursive_freq_factor
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_single_call_penalty;
#else
  int x_param_ipa_cp_single_call_penalty;
#define param_ipa_cp_single_call_penalty global_options.x_param_ipa_cp_single_call_penalty
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_unit_growth;
#else
  int x_param_ipa_cp_unit_growth;
#define param_ipa_cp_unit_growth global_options.x_param_ipa_cp_unit_growth
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_cp_value_list_size;
#else
  int x_param_ipa_cp_value_list_size;
#define param_ipa_cp_value_list_size global_options.x_param_ipa_cp_value_list_size
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_jump_function_lookups;
#else
  int x_param_ipa_jump_function_lookups;
#define param_ipa_jump_function_lookups global_options.x_param_ipa_jump_function_lookups
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_max_aa_steps;
#else
  int x_param_ipa_max_aa_steps;
#define param_ipa_max_aa_steps global_options.x_param_ipa_max_aa_steps
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_max_agg_items;
#else
  int x_param_ipa_max_agg_items;
#define param_ipa_max_agg_items global_options.x_param_ipa_max_agg_items
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_max_loop_predicates;
#else
  int x_param_ipa_max_loop_predicates;
#define param_ipa_max_loop_predicates global_options.x_param_ipa_max_loop_predicates
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_max_param_expr_ops;
#else
  int x_param_ipa_max_param_expr_ops;
#define param_ipa_max_param_expr_ops global_options.x_param_ipa_max_param_expr_ops
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_max_switch_predicate_bounds;
#else
  int x_param_ipa_max_switch_predicate_bounds;
#define param_ipa_max_switch_predicate_bounds global_options.x_param_ipa_max_switch_predicate_bounds
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_sra_deref_prob_threshold;
#else
  int x_param_ipa_sra_deref_prob_threshold;
#define param_ipa_sra_deref_prob_threshold global_options.x_param_ipa_sra_deref_prob_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_sra_max_replacements;
#else
  int x_param_ipa_sra_max_replacements;
#define param_ipa_sra_max_replacements global_options.x_param_ipa_sra_max_replacements
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_sra_ptr_growth_factor;
#else
  int x_param_ipa_sra_ptr_growth_factor;
#define param_ipa_sra_ptr_growth_factor global_options.x_param_ipa_sra_ptr_growth_factor
#endif
#ifdef GENERATOR_FILE
extern int param_ipa_sra_ptrwrap_growth_factor;
#else
  int x_param_ipa_sra_ptrwrap_growth_factor;
#define param_ipa_sra_ptrwrap_growth_factor global_options.x_param_ipa_sra_ptrwrap_growth_factor
#endif
#ifdef GENERATOR_FILE
extern int param_ira_consider_dup_in_all_alts;
#else
  int x_param_ira_consider_dup_in_all_alts;
#define param_ira_consider_dup_in_all_alts global_options.x_param_ira_consider_dup_in_all_alts
#endif
#ifdef GENERATOR_FILE
extern int param_ira_loop_reserved_regs;
#else
  int x_param_ira_loop_reserved_regs;
#define param_ira_loop_reserved_regs global_options.x_param_ira_loop_reserved_regs
#endif
#ifdef GENERATOR_FILE
extern int param_ira_max_conflict_table_size;
#else
  int x_param_ira_max_conflict_table_size;
#define param_ira_max_conflict_table_size global_options.x_param_ira_max_conflict_table_size
#endif
#ifdef GENERATOR_FILE
extern int param_ira_max_loops_num;
#else
  int x_param_ira_max_loops_num;
#define param_ira_max_loops_num global_options.x_param_ira_max_loops_num
#endif
#ifdef GENERATOR_FILE
extern int param_ira_simple_lra_insn_threshold;
#else
  int x_param_ira_simple_lra_insn_threshold;
#define param_ira_simple_lra_insn_threshold global_options.x_param_ira_simple_lra_insn_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_iv_always_prune_cand_set_bound;
#else
  int x_param_iv_always_prune_cand_set_bound;
#define param_iv_always_prune_cand_set_bound global_options.x_param_iv_always_prune_cand_set_bound
#endif
#ifdef GENERATOR_FILE
extern int param_iv_consider_all_candidates_bound;
#else
  int x_param_iv_consider_all_candidates_bound;
#define param_iv_consider_all_candidates_bound global_options.x_param_iv_consider_all_candidates_bound
#endif
#ifdef GENERATOR_FILE
extern int param_iv_max_considered_uses;
#else
  int x_param_iv_max_considered_uses;
#define param_iv_max_considered_uses global_options.x_param_iv_max_considered_uses
#endif
#ifdef GENERATOR_FILE
extern int param_jump_table_max_growth_ratio_for_size;
#else
  int x_param_jump_table_max_growth_ratio_for_size;
#define param_jump_table_max_growth_ratio_for_size global_options.x_param_jump_table_max_growth_ratio_for_size
#endif
#ifdef GENERATOR_FILE
extern int param_jump_table_max_growth_ratio_for_speed;
#else
  int x_param_jump_table_max_growth_ratio_for_speed;
#define param_jump_table_max_growth_ratio_for_speed global_options.x_param_jump_table_max_growth_ratio_for_speed
#endif
#ifdef GENERATOR_FILE
extern int param_l1_cache_line_size;
#else
  int x_param_l1_cache_line_size;
#define param_l1_cache_line_size global_options.x_param_l1_cache_line_size
#endif
#ifdef GENERATOR_FILE
extern int param_l1_cache_size;
#else
  int x_param_l1_cache_size;
#define param_l1_cache_size global_options.x_param_l1_cache_size
#endif
#ifdef GENERATOR_FILE
extern int param_l2_cache_size;
#else
  int x_param_l2_cache_size;
#define param_l2_cache_size global_options.x_param_l2_cache_size
#endif
#ifdef GENERATOR_FILE
extern int param_large_function_growth;
#else
  int x_param_large_function_growth;
#define param_large_function_growth global_options.x_param_large_function_growth
#endif
#ifdef GENERATOR_FILE
extern int param_large_function_insns;
#else
  int x_param_large_function_insns;
#define param_large_function_insns global_options.x_param_large_function_insns
#endif
#ifdef GENERATOR_FILE
extern int param_stack_frame_growth;
#else
  int x_param_stack_frame_growth;
#define param_stack_frame_growth global_options.x_param_stack_frame_growth
#endif
#ifdef GENERATOR_FILE
extern int param_large_stack_frame;
#else
  int x_param_large_stack_frame;
#define param_large_stack_frame global_options.x_param_large_stack_frame
#endif
#ifdef GENERATOR_FILE
extern int param_large_unit_insns;
#else
  int x_param_large_unit_insns;
#define param_large_unit_insns global_options.x_param_large_unit_insns
#endif
#ifdef GENERATOR_FILE
extern int param_lazy_modules;
#else
  int x_param_lazy_modules;
#define param_lazy_modules global_options.x_param_lazy_modules
#endif
#ifdef GENERATOR_FILE
extern int param_lim_expensive;
#else
  int x_param_lim_expensive;
#define param_lim_expensive global_options.x_param_lim_expensive
#endif
#ifdef GENERATOR_FILE
extern int param_logical_op_non_short_circuit;
#else
  int x_param_logical_op_non_short_circuit;
#define param_logical_op_non_short_circuit global_options.x_param_logical_op_non_short_circuit
#endif
#ifdef GENERATOR_FILE
extern int param_loop_block_tile_size;
#else
  int x_param_loop_block_tile_size;
#define param_loop_block_tile_size global_options.x_param_loop_block_tile_size
#endif
#ifdef GENERATOR_FILE
extern int param_loop_interchange_max_num_stmts;
#else
  int x_param_loop_interchange_max_num_stmts;
#define param_loop_interchange_max_num_stmts global_options.x_param_loop_interchange_max_num_stmts
#endif
#ifdef GENERATOR_FILE
extern int param_loop_interchange_stride_ratio;
#else
  int x_param_loop_interchange_stride_ratio;
#define param_loop_interchange_stride_ratio global_options.x_param_loop_interchange_stride_ratio
#endif
#ifdef GENERATOR_FILE
extern int param_loop_invariant_max_bbs_in_loop;
#else
  int x_param_loop_invariant_max_bbs_in_loop;
#define param_loop_invariant_max_bbs_in_loop global_options.x_param_loop_invariant_max_bbs_in_loop
#endif
#ifdef GENERATOR_FILE
extern int param_loop_max_datarefs_for_datadeps;
#else
  int x_param_loop_max_datarefs_for_datadeps;
#define param_loop_max_datarefs_for_datadeps global_options.x_param_loop_max_datarefs_for_datadeps
#endif
#ifdef GENERATOR_FILE
extern int param_loop_versioning_max_inner_insns;
#else
  int x_param_loop_versioning_max_inner_insns;
#define param_loop_versioning_max_inner_insns global_options.x_param_loop_versioning_max_inner_insns
#endif
#ifdef GENERATOR_FILE
extern int param_loop_versioning_max_outer_insns;
#else
  int x_param_loop_versioning_max_outer_insns;
#define param_loop_versioning_max_outer_insns global_options.x_param_loop_versioning_max_outer_insns
#endif
#ifdef GENERATOR_FILE
extern int param_lra_inheritance_ebb_probability_cutoff;
#else
  int x_param_lra_inheritance_ebb_probability_cutoff;
#define param_lra_inheritance_ebb_probability_cutoff global_options.x_param_lra_inheritance_ebb_probability_cutoff
#endif
#ifdef GENERATOR_FILE
extern int param_lra_max_considered_reload_pseudos;
#else
  int x_param_lra_max_considered_reload_pseudos;
#define param_lra_max_considered_reload_pseudos global_options.x_param_lra_max_considered_reload_pseudos
#endif
#ifdef GENERATOR_FILE
extern int param_max_partition_size;
#else
  int x_param_max_partition_size;
#define param_max_partition_size global_options.x_param_max_partition_size
#endif
#ifdef GENERATOR_FILE
extern int param_max_lto_streaming_parallelism;
#else
  int x_param_max_lto_streaming_parallelism;
#define param_max_lto_streaming_parallelism global_options.x_param_max_lto_streaming_parallelism
#endif
#ifdef GENERATOR_FILE
extern int param_min_partition_size;
#else
  int x_param_min_partition_size;
#define param_min_partition_size global_options.x_param_min_partition_size
#endif
#ifdef GENERATOR_FILE
extern int param_lto_partitions;
#else
  int x_param_lto_partitions;
#define param_lto_partitions global_options.x_param_lto_partitions
#endif
#ifdef GENERATOR_FILE
extern int param_max_average_unrolled_insns;
#else
  int x_param_max_average_unrolled_insns;
#define param_max_average_unrolled_insns global_options.x_param_max_average_unrolled_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_combine_insns;
#else
  int x_param_max_combine_insns;
#define param_max_combine_insns global_options.x_param_max_combine_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_unroll_iterations;
#else
  int x_param_max_unroll_iterations;
#define param_max_unroll_iterations global_options.x_param_max_unroll_iterations
#endif
#ifdef GENERATOR_FILE
extern int param_max_completely_peel_times;
#else
  int x_param_max_completely_peel_times;
#define param_max_completely_peel_times global_options.x_param_max_completely_peel_times
#endif
#ifdef GENERATOR_FILE
extern int param_max_completely_peeled_insns;
#else
  int x_param_max_completely_peeled_insns;
#define param_max_completely_peeled_insns global_options.x_param_max_completely_peeled_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_crossjump_edges;
#else
  int x_param_max_crossjump_edges;
#define param_max_crossjump_edges global_options.x_param_max_crossjump_edges
#endif
#ifdef GENERATOR_FILE
extern int param_max_cse_insns;
#else
  int x_param_max_cse_insns;
#define param_max_cse_insns global_options.x_param_max_cse_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_cse_path_length;
#else
  int x_param_max_cse_path_length;
#define param_max_cse_path_length global_options.x_param_max_cse_path_length
#endif
#ifdef GENERATOR_FILE
extern int param_max_cselib_memory_locations;
#else
  int x_param_max_cselib_memory_locations;
#define param_max_cselib_memory_locations global_options.x_param_max_cselib_memory_locations
#endif
#ifdef GENERATOR_FILE
extern int param_max_debug_marker_count;
#else
  int x_param_max_debug_marker_count;
#define param_max_debug_marker_count global_options.x_param_max_debug_marker_count
#endif
#ifdef GENERATOR_FILE
extern int param_max_delay_slot_insn_search;
#else
  int x_param_max_delay_slot_insn_search;
#define param_max_delay_slot_insn_search global_options.x_param_max_delay_slot_insn_search
#endif
#ifdef GENERATOR_FILE
extern int param_max_delay_slot_live_search;
#else
  int x_param_max_delay_slot_live_search;
#define param_max_delay_slot_live_search global_options.x_param_max_delay_slot_live_search
#endif
#ifdef GENERATOR_FILE
extern int param_max_dse_active_local_stores;
#else
  int x_param_max_dse_active_local_stores;
#define param_max_dse_active_local_stores global_options.x_param_max_dse_active_local_stores
#endif
#ifdef GENERATOR_FILE
extern int param_early_inliner_max_iterations;
#else
  int x_param_early_inliner_max_iterations;
#define param_early_inliner_max_iterations global_options.x_param_early_inliner_max_iterations
#endif
#ifdef GENERATOR_FILE
extern int param_max_fields_for_field_sensitive;
#else
  int x_param_max_fields_for_field_sensitive;
#define param_max_fields_for_field_sensitive global_options.x_param_max_fields_for_field_sensitive
#endif
#ifdef GENERATOR_FILE
extern int param_max_find_base_term_values;
#else
  int x_param_max_find_base_term_values;
#define param_max_find_base_term_values global_options.x_param_max_find_base_term_values
#endif
#ifdef GENERATOR_FILE
extern int param_max_fsm_thread_path_insns;
#else
  int x_param_max_fsm_thread_path_insns;
#define param_max_fsm_thread_path_insns global_options.x_param_max_fsm_thread_path_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_gcse_insertion_ratio;
#else
  int x_param_max_gcse_insertion_ratio;
#define param_max_gcse_insertion_ratio global_options.x_param_max_gcse_insertion_ratio
#endif
#ifdef GENERATOR_FILE
extern int param_max_gcse_memory;
#else
  int x_param_max_gcse_memory;
#define param_max_gcse_memory global_options.x_param_max_gcse_memory
#endif
#ifdef GENERATOR_FILE
extern int param_max_goto_duplication_insns;
#else
  int x_param_max_goto_duplication_insns;
#define param_max_goto_duplication_insns global_options.x_param_max_goto_duplication_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_grow_copy_bb_insns;
#else
  int x_param_max_grow_copy_bb_insns;
#define param_max_grow_copy_bb_insns global_options.x_param_max_grow_copy_bb_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_hoist_depth;
#else
  int x_param_max_hoist_depth;
#define param_max_hoist_depth global_options.x_param_max_hoist_depth
#endif
#ifdef GENERATOR_FILE
extern int param_inline_functions_called_once_insns;
#else
  int x_param_inline_functions_called_once_insns;
#define param_inline_functions_called_once_insns global_options.x_param_inline_functions_called_once_insns
#endif
#ifdef GENERATOR_FILE
extern int param_inline_functions_called_once_loop_depth;
#else
  int x_param_inline_functions_called_once_loop_depth;
#define param_inline_functions_called_once_loop_depth global_options.x_param_inline_functions_called_once_loop_depth
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_insns_auto;
#else
  int x_param_max_inline_insns_auto;
#define param_max_inline_insns_auto global_options.x_param_max_inline_insns_auto
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_insns_recursive_auto;
#else
  int x_param_max_inline_insns_recursive_auto;
#define param_max_inline_insns_recursive_auto global_options.x_param_max_inline_insns_recursive_auto
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_insns_recursive;
#else
  int x_param_max_inline_insns_recursive;
#define param_max_inline_insns_recursive global_options.x_param_max_inline_insns_recursive
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_insns_single;
#else
  int x_param_max_inline_insns_single;
#define param_max_inline_insns_single global_options.x_param_max_inline_insns_single
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_insns_size;
#else
  int x_param_max_inline_insns_size;
#define param_max_inline_insns_size global_options.x_param_max_inline_insns_size
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_insns_small;
#else
  int x_param_max_inline_insns_small;
#define param_max_inline_insns_small global_options.x_param_max_inline_insns_small
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_recursive_depth_auto;
#else
  int x_param_max_inline_recursive_depth_auto;
#define param_max_inline_recursive_depth_auto global_options.x_param_max_inline_recursive_depth_auto
#endif
#ifdef GENERATOR_FILE
extern int param_max_inline_recursive_depth;
#else
  int x_param_max_inline_recursive_depth;
#define param_max_inline_recursive_depth global_options.x_param_max_inline_recursive_depth
#endif
#ifdef GENERATOR_FILE
extern int param_max_isl_operations;
#else
  int x_param_max_isl_operations;
#define param_max_isl_operations global_options.x_param_max_isl_operations
#endif
#ifdef GENERATOR_FILE
extern int param_max_iterations_computation_cost;
#else
  int x_param_max_iterations_computation_cost;
#define param_max_iterations_computation_cost global_options.x_param_max_iterations_computation_cost
#endif
#ifdef GENERATOR_FILE
extern int param_max_iterations_to_track;
#else
  int x_param_max_iterations_to_track;
#define param_max_iterations_to_track global_options.x_param_max_iterations_to_track
#endif
#ifdef GENERATOR_FILE
extern int param_max_jump_thread_duplication_stmts;
#else
  int x_param_max_jump_thread_duplication_stmts;
#define param_max_jump_thread_duplication_stmts global_options.x_param_max_jump_thread_duplication_stmts
#endif
#ifdef GENERATOR_FILE
extern int param_max_jump_thread_paths;
#else
  int x_param_max_jump_thread_paths;
#define param_max_jump_thread_paths global_options.x_param_max_jump_thread_paths
#endif
#ifdef GENERATOR_FILE
extern int param_max_last_value_rtl;
#else
  int x_param_max_last_value_rtl;
#define param_max_last_value_rtl global_options.x_param_max_last_value_rtl
#endif
#ifdef GENERATOR_FILE
extern int param_max_loop_header_insns;
#else
  int x_param_max_loop_header_insns;
#define param_max_loop_header_insns global_options.x_param_max_loop_header_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_modulo_backtrack_attempts;
#else
  int x_param_max_modulo_backtrack_attempts;
#define param_max_modulo_backtrack_attempts global_options.x_param_max_modulo_backtrack_attempts
#endif
#ifdef GENERATOR_FILE
extern int param_max_partial_antic_length;
#else
  int x_param_max_partial_antic_length;
#define param_max_partial_antic_length global_options.x_param_max_partial_antic_length
#endif
#ifdef GENERATOR_FILE
extern int param_max_peel_branches;
#else
  int x_param_max_peel_branches;
#define param_max_peel_branches global_options.x_param_max_peel_branches
#endif
#ifdef GENERATOR_FILE
extern int param_max_peel_times;
#else
  int x_param_max_peel_times;
#define param_max_peel_times global_options.x_param_max_peel_times
#endif
#ifdef GENERATOR_FILE
extern int param_max_peeled_insns;
#else
  int x_param_max_peeled_insns;
#define param_max_peeled_insns global_options.x_param_max_peeled_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_pending_list_length;
#else
  int x_param_max_pending_list_length;
#define param_max_pending_list_length global_options.x_param_max_pending_list_length
#endif
#ifdef GENERATOR_FILE
extern int param_max_pipeline_region_blocks;
#else
  int x_param_max_pipeline_region_blocks;
#define param_max_pipeline_region_blocks global_options.x_param_max_pipeline_region_blocks
#endif
#ifdef GENERATOR_FILE
extern int param_max_pipeline_region_insns;
#else
  int x_param_max_pipeline_region_insns;
#define param_max_pipeline_region_insns global_options.x_param_max_pipeline_region_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_pow_sqrt_depth;
#else
  int x_param_max_pow_sqrt_depth;
#define param_max_pow_sqrt_depth global_options.x_param_max_pow_sqrt_depth
#endif
#ifdef GENERATOR_FILE
extern int param_max_predicted_iterations;
#else
  int x_param_max_predicted_iterations;
#define param_max_predicted_iterations global_options.x_param_max_predicted_iterations
#endif
#ifdef GENERATOR_FILE
extern int param_max_reload_search_insns;
#else
  int x_param_max_reload_search_insns;
#define param_max_reload_search_insns global_options.x_param_max_reload_search_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_rtl_if_conversion_insns;
#else
  int x_param_max_rtl_if_conversion_insns;
#define param_max_rtl_if_conversion_insns global_options.x_param_max_rtl_if_conversion_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_rtl_if_conversion_predictable_cost;
#else
  int x_param_max_rtl_if_conversion_predictable_cost;
#define param_max_rtl_if_conversion_predictable_cost global_options.x_param_max_rtl_if_conversion_predictable_cost
#endif
#ifdef GENERATOR_FILE
extern int param_max_rtl_if_conversion_unpredictable_cost;
#else
  int x_param_max_rtl_if_conversion_unpredictable_cost;
#define param_max_rtl_if_conversion_unpredictable_cost global_options.x_param_max_rtl_if_conversion_unpredictable_cost
#endif
#ifdef GENERATOR_FILE
extern int param_max_sched_extend_regions_iters;
#else
  int x_param_max_sched_extend_regions_iters;
#define param_max_sched_extend_regions_iters global_options.x_param_max_sched_extend_regions_iters
#endif
#ifdef GENERATOR_FILE
extern int param_max_sched_insn_conflict_delay;
#else
  int x_param_max_sched_insn_conflict_delay;
#define param_max_sched_insn_conflict_delay global_options.x_param_max_sched_insn_conflict_delay
#endif
#ifdef GENERATOR_FILE
extern int param_max_sched_ready_insns;
#else
  int x_param_max_sched_ready_insns;
#define param_max_sched_ready_insns global_options.x_param_max_sched_ready_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_sched_region_blocks;
#else
  int x_param_max_sched_region_blocks;
#define param_max_sched_region_blocks global_options.x_param_max_sched_region_blocks
#endif
#ifdef GENERATOR_FILE
extern int param_max_sched_region_insns;
#else
  int x_param_max_sched_region_insns;
#define param_max_sched_region_insns global_options.x_param_max_sched_region_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_slsr_candidate_scan;
#else
  int x_param_max_slsr_candidate_scan;
#define param_max_slsr_candidate_scan global_options.x_param_max_slsr_candidate_scan
#endif
#ifdef GENERATOR_FILE
extern int param_max_speculative_devirt_maydefs;
#else
  int x_param_max_speculative_devirt_maydefs;
#define param_max_speculative_devirt_maydefs global_options.x_param_max_speculative_devirt_maydefs
#endif
#ifdef GENERATOR_FILE
extern int param_max_ssa_name_query_depth;
#else
  int x_param_max_ssa_name_query_depth;
#define param_max_ssa_name_query_depth global_options.x_param_max_ssa_name_query_depth
#endif
#ifdef GENERATOR_FILE
extern int param_max_store_chains_to_track;
#else
  int x_param_max_store_chains_to_track;
#define param_max_store_chains_to_track global_options.x_param_max_store_chains_to_track
#endif
#ifdef GENERATOR_FILE
extern int param_max_stores_to_merge;
#else
  int x_param_max_stores_to_merge;
#define param_max_stores_to_merge global_options.x_param_max_stores_to_merge
#endif
#ifdef GENERATOR_FILE
extern int param_max_stores_to_sink;
#else
  int x_param_max_stores_to_sink;
#define param_max_stores_to_sink global_options.x_param_max_stores_to_sink
#endif
#ifdef GENERATOR_FILE
extern int param_max_stores_to_track;
#else
  int x_param_max_stores_to_track;
#define param_max_stores_to_track global_options.x_param_max_stores_to_track
#endif
#ifdef GENERATOR_FILE
extern int param_max_tail_merge_comparisons;
#else
  int x_param_max_tail_merge_comparisons;
#define param_max_tail_merge_comparisons global_options.x_param_max_tail_merge_comparisons
#endif
#ifdef GENERATOR_FILE
extern int param_max_tail_merge_iterations;
#else
  int x_param_max_tail_merge_iterations;
#define param_max_tail_merge_iterations global_options.x_param_max_tail_merge_iterations
#endif
#ifdef GENERATOR_FILE
extern int param_max_tracked_strlens;
#else
  int x_param_max_tracked_strlens;
#define param_max_tracked_strlens global_options.x_param_max_tracked_strlens
#endif
#ifdef GENERATOR_FILE
extern int param_max_tree_if_conversion_phi_args;
#else
  int x_param_max_tree_if_conversion_phi_args;
#define param_max_tree_if_conversion_phi_args global_options.x_param_max_tree_if_conversion_phi_args
#endif
#ifdef GENERATOR_FILE
extern int param_max_unroll_times;
#else
  int x_param_max_unroll_times;
#define param_max_unroll_times global_options.x_param_max_unroll_times
#endif
#ifdef GENERATOR_FILE
extern int param_max_unrolled_insns;
#else
  int x_param_max_unrolled_insns;
#define param_max_unrolled_insns global_options.x_param_max_unrolled_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_unswitch_depth;
#else
  int x_param_max_unswitch_depth;
#define param_max_unswitch_depth global_options.x_param_max_unswitch_depth
#endif
#ifdef GENERATOR_FILE
extern int param_max_unswitch_insns;
#else
  int x_param_max_unswitch_insns;
#define param_max_unswitch_insns global_options.x_param_max_unswitch_insns
#endif
#ifdef GENERATOR_FILE
extern int param_max_variable_expansions;
#else
  int x_param_max_variable_expansions;
#define param_max_variable_expansions global_options.x_param_max_variable_expansions
#endif
#ifdef GENERATOR_FILE
extern int param_max_vartrack_expr_depth;
#else
  int x_param_max_vartrack_expr_depth;
#define param_max_vartrack_expr_depth global_options.x_param_max_vartrack_expr_depth
#endif
#ifdef GENERATOR_FILE
extern int param_max_vartrack_reverse_op_size;
#else
  int x_param_max_vartrack_reverse_op_size;
#define param_max_vartrack_reverse_op_size global_options.x_param_max_vartrack_reverse_op_size
#endif
#ifdef GENERATOR_FILE
extern int param_max_vartrack_size;
#else
  int x_param_max_vartrack_size;
#define param_max_vartrack_size global_options.x_param_max_vartrack_size
#endif
#ifdef GENERATOR_FILE
extern int param_min_crossjump_insns;
#else
  int x_param_min_crossjump_insns;
#define param_min_crossjump_insns global_options.x_param_min_crossjump_insns
#endif
#ifdef GENERATOR_FILE
extern int param_min_inline_recursive_probability;
#else
  int x_param_min_inline_recursive_probability;
#define param_min_inline_recursive_probability global_options.x_param_min_inline_recursive_probability
#endif
#ifdef GENERATOR_FILE
extern int param_min_insn_to_prefetch_ratio;
#else
  int x_param_min_insn_to_prefetch_ratio;
#define param_min_insn_to_prefetch_ratio global_options.x_param_min_insn_to_prefetch_ratio
#endif
#ifdef GENERATOR_FILE
extern int param_min_loop_cond_split_prob;
#else
  int x_param_min_loop_cond_split_prob;
#define param_min_loop_cond_split_prob global_options.x_param_min_loop_cond_split_prob
#endif
#ifdef GENERATOR_FILE
extern int param_min_nondebug_insn_uid;
#else
  int x_param_min_nondebug_insn_uid;
#define param_min_nondebug_insn_uid global_options.x_param_min_nondebug_insn_uid
#endif
#ifdef GENERATOR_FILE
extern int param_min_pagesize;
#else
  int x_param_min_pagesize;
#define param_min_pagesize global_options.x_param_min_pagesize
#endif
#ifdef GENERATOR_FILE
extern int param_min_size_for_stack_sharing;
#else
  int x_param_min_size_for_stack_sharing;
#define param_min_size_for_stack_sharing global_options.x_param_min_size_for_stack_sharing
#endif
#ifdef GENERATOR_FILE
extern int param_min_spec_prob;
#else
  int x_param_min_spec_prob;
#define param_min_spec_prob global_options.x_param_min_spec_prob
#endif
#ifdef GENERATOR_FILE
extern int param_min_vect_loop_bound;
#else
  int x_param_min_vect_loop_bound;
#define param_min_vect_loop_bound global_options.x_param_min_vect_loop_bound
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_accesses;
#else
  int x_param_modref_max_accesses;
#define param_modref_max_accesses global_options.x_param_modref_max_accesses
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_adjustments;
#else
  int x_param_modref_max_adjustments;
#define param_modref_max_adjustments global_options.x_param_modref_max_adjustments
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_bases;
#else
  int x_param_modref_max_bases;
#define param_modref_max_bases global_options.x_param_modref_max_bases
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_depth;
#else
  int x_param_modref_max_depth;
#define param_modref_max_depth global_options.x_param_modref_max_depth
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_escape_points;
#else
  int x_param_modref_max_escape_points;
#define param_modref_max_escape_points global_options.x_param_modref_max_escape_points
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_refs;
#else
  int x_param_modref_max_refs;
#define param_modref_max_refs global_options.x_param_modref_max_refs
#endif
#ifdef GENERATOR_FILE
extern int param_modref_max_tests;
#else
  int x_param_modref_max_tests;
#define param_modref_max_tests global_options.x_param_modref_max_tests
#endif
#ifdef GENERATOR_FILE
extern enum openacc_kernels param_openacc_kernels;
#else
  enum openacc_kernels x_param_openacc_kernels;
#define param_openacc_kernels global_options.x_param_openacc_kernels
#endif
#ifdef GENERATOR_FILE
extern enum openacc_privatization param_openacc_privatization;
#else
  enum openacc_privatization x_param_openacc_privatization;
#define param_openacc_privatization global_options.x_param_openacc_privatization
#endif
#ifdef GENERATOR_FILE
extern int param_parloops_chunk_size;
#else
  int x_param_parloops_chunk_size;
#define param_parloops_chunk_size global_options.x_param_parloops_chunk_size
#endif
#ifdef GENERATOR_FILE
extern int param_parloops_min_per_thread;
#else
  int x_param_parloops_min_per_thread;
#define param_parloops_min_per_thread global_options.x_param_parloops_min_per_thread
#endif
#ifdef GENERATOR_FILE
extern int param_parloops_schedule;
#else
  int x_param_parloops_schedule;
#define param_parloops_schedule global_options.x_param_parloops_schedule
#endif
#ifdef GENERATOR_FILE
extern int param_partial_inlining_entry_probability;
#else
  int x_param_partial_inlining_entry_probability;
#define param_partial_inlining_entry_probability global_options.x_param_partial_inlining_entry_probability
#endif
#ifdef GENERATOR_FILE
extern int param_predictable_branch_outcome;
#else
  int x_param_predictable_branch_outcome;
#define param_predictable_branch_outcome global_options.x_param_predictable_branch_outcome
#endif
#ifdef GENERATOR_FILE
extern int param_prefetch_dynamic_strides;
#else
  int x_param_prefetch_dynamic_strides;
#define param_prefetch_dynamic_strides global_options.x_param_prefetch_dynamic_strides
#endif
#ifdef GENERATOR_FILE
extern int param_prefetch_latency;
#else
  int x_param_prefetch_latency;
#define param_prefetch_latency global_options.x_param_prefetch_latency
#endif
#ifdef GENERATOR_FILE
extern int param_prefetch_min_insn_to_mem_ratio;
#else
  int x_param_prefetch_min_insn_to_mem_ratio;
#define param_prefetch_min_insn_to_mem_ratio global_options.x_param_prefetch_min_insn_to_mem_ratio
#endif
#ifdef GENERATOR_FILE
extern int param_prefetch_minimum_stride;
#else
  int x_param_prefetch_minimum_stride;
#define param_prefetch_minimum_stride global_options.x_param_prefetch_minimum_stride
#endif
#ifdef GENERATOR_FILE
extern int param_profile_func_internal_id;
#else
  int x_param_profile_func_internal_id;
#define param_profile_func_internal_id global_options.x_param_profile_func_internal_id
#endif
#ifdef GENERATOR_FILE
extern enum ranger_debug param_ranger_debug;
#else
  enum ranger_debug x_param_ranger_debug;
#define param_ranger_debug global_options.x_param_ranger_debug
#endif
#ifdef GENERATOR_FILE
extern int param_ranger_logical_depth;
#else
  int x_param_ranger_logical_depth;
#define param_ranger_logical_depth global_options.x_param_ranger_logical_depth
#endif
#ifdef GENERATOR_FILE
extern int param_ranger_recompute_depth;
#else
  int x_param_ranger_recompute_depth;
#define param_ranger_recompute_depth global_options.x_param_ranger_recompute_depth
#endif
#ifdef GENERATOR_FILE
extern int param_relation_block_limit;
#else
  int x_param_relation_block_limit;
#define param_relation_block_limit global_options.x_param_relation_block_limit
#endif
#ifdef GENERATOR_FILE
extern int param_rpo_vn_max_loop_depth;
#else
  int x_param_rpo_vn_max_loop_depth;
#define param_rpo_vn_max_loop_depth global_options.x_param_rpo_vn_max_loop_depth
#endif
#ifdef GENERATOR_FILE
extern int param_sccvn_max_alias_queries_per_access;
#else
  int x_param_sccvn_max_alias_queries_per_access;
#define param_sccvn_max_alias_queries_per_access global_options.x_param_sccvn_max_alias_queries_per_access
#endif
#ifdef GENERATOR_FILE
extern int param_scev_max_expr_complexity;
#else
  int x_param_scev_max_expr_complexity;
#define param_scev_max_expr_complexity global_options.x_param_scev_max_expr_complexity
#endif
#ifdef GENERATOR_FILE
extern int param_scev_max_expr_size;
#else
  int x_param_scev_max_expr_size;
#define param_scev_max_expr_size global_options.x_param_scev_max_expr_size
#endif
#ifdef GENERATOR_FILE
extern int param_sched_autopref_queue_depth;
#else
  int x_param_sched_autopref_queue_depth;
#define param_sched_autopref_queue_depth global_options.x_param_sched_autopref_queue_depth
#endif
#ifdef GENERATOR_FILE
extern int param_sched_mem_true_dep_cost;
#else
  int x_param_sched_mem_true_dep_cost;
#define param_sched_mem_true_dep_cost global_options.x_param_sched_mem_true_dep_cost
#endif
#ifdef GENERATOR_FILE
extern int param_sched_pressure_algorithm;
#else
  int x_param_sched_pressure_algorithm;
#define param_sched_pressure_algorithm global_options.x_param_sched_pressure_algorithm
#endif
#ifdef GENERATOR_FILE
extern int param_sched_spec_prob_cutoff;
#else
  int x_param_sched_spec_prob_cutoff;
#define param_sched_spec_prob_cutoff global_options.x_param_sched_spec_prob_cutoff
#endif
#ifdef GENERATOR_FILE
extern int param_sched_state_edge_prob_cutoff;
#else
  int x_param_sched_state_edge_prob_cutoff;
#define param_sched_state_edge_prob_cutoff global_options.x_param_sched_state_edge_prob_cutoff
#endif
#ifdef GENERATOR_FILE
extern int param_selsched_insns_to_rename;
#else
  int x_param_selsched_insns_to_rename;
#define param_selsched_insns_to_rename global_options.x_param_selsched_insns_to_rename
#endif
#ifdef GENERATOR_FILE
extern int param_selsched_max_lookahead;
#else
  int x_param_selsched_max_lookahead;
#define param_selsched_max_lookahead global_options.x_param_selsched_max_lookahead
#endif
#ifdef GENERATOR_FILE
extern int param_selsched_max_sched_times;
#else
  int x_param_selsched_max_sched_times;
#define param_selsched_max_sched_times global_options.x_param_selsched_max_sched_times
#endif
#ifdef GENERATOR_FILE
extern int param_simultaneous_prefetches;
#else
  int x_param_simultaneous_prefetches;
#define param_simultaneous_prefetches global_options.x_param_simultaneous_prefetches
#endif
#ifdef GENERATOR_FILE
extern int param_sink_frequency_threshold;
#else
  int x_param_sink_frequency_threshold;
#define param_sink_frequency_threshold global_options.x_param_sink_frequency_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_sms_dfa_history;
#else
  int x_param_sms_dfa_history;
#define param_sms_dfa_history global_options.x_param_sms_dfa_history
#endif
#ifdef GENERATOR_FILE
extern int param_sms_loop_average_count_threshold;
#else
  int x_param_sms_loop_average_count_threshold;
#define param_sms_loop_average_count_threshold global_options.x_param_sms_loop_average_count_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_sms_max_ii_factor;
#else
  int x_param_sms_max_ii_factor;
#define param_sms_max_ii_factor global_options.x_param_sms_max_ii_factor
#endif
#ifdef GENERATOR_FILE
extern int param_sms_min_sc;
#else
  int x_param_sms_min_sc;
#define param_sms_min_sc global_options.x_param_sms_min_sc
#endif
#ifdef GENERATOR_FILE
extern int param_sra_max_propagations;
#else
  int x_param_sra_max_propagations;
#define param_sra_max_propagations global_options.x_param_sra_max_propagations
#endif
#ifdef GENERATOR_FILE
extern int param_sra_max_scalarization_size_size;
#else
  int x_param_sra_max_scalarization_size_size;
#define param_sra_max_scalarization_size_size global_options.x_param_sra_max_scalarization_size_size
#endif
#ifdef GENERATOR_FILE
extern int param_sra_max_scalarization_size_speed;
#else
  int x_param_sra_max_scalarization_size_speed;
#define param_sra_max_scalarization_size_speed global_options.x_param_sra_max_scalarization_size_speed
#endif
#ifdef GENERATOR_FILE
extern int param_ssa_name_def_chain_limit;
#else
  int x_param_ssa_name_def_chain_limit;
#define param_ssa_name_def_chain_limit global_options.x_param_ssa_name_def_chain_limit
#endif
#ifdef GENERATOR_FILE
extern int param_ssp_buffer_size;
#else
  int x_param_ssp_buffer_size;
#define param_ssp_buffer_size global_options.x_param_ssp_buffer_size
#endif
#ifdef GENERATOR_FILE
extern int param_stack_clash_protection_guard_size;
#else
  int x_param_stack_clash_protection_guard_size;
#define param_stack_clash_protection_guard_size global_options.x_param_stack_clash_protection_guard_size
#endif
#ifdef GENERATOR_FILE
extern int param_stack_clash_protection_probe_interval;
#else
  int x_param_stack_clash_protection_probe_interval;
#define param_stack_clash_protection_probe_interval global_options.x_param_stack_clash_protection_probe_interval
#endif
#ifdef GENERATOR_FILE
extern int param_store_merging_allow_unaligned;
#else
  int x_param_store_merging_allow_unaligned;
#define param_store_merging_allow_unaligned global_options.x_param_store_merging_allow_unaligned
#endif
#ifdef GENERATOR_FILE
extern int param_store_merging_max_size;
#else
  int x_param_store_merging_max_size;
#define param_store_merging_max_size global_options.x_param_store_merging_max_size
#endif
#ifdef GENERATOR_FILE
extern int param_switch_conversion_branch_ratio;
#else
  int x_param_switch_conversion_branch_ratio;
#define param_switch_conversion_branch_ratio global_options.x_param_switch_conversion_branch_ratio
#endif
#ifdef GENERATOR_FILE
extern enum threader_debug param_threader_debug;
#else
  enum threader_debug x_param_threader_debug;
#define param_threader_debug global_options.x_param_threader_debug
#endif
#ifdef GENERATOR_FILE
extern int param_tm_max_aggregate_size;
#else
  int x_param_tm_max_aggregate_size;
#define param_tm_max_aggregate_size global_options.x_param_tm_max_aggregate_size
#endif
#ifdef GENERATOR_FILE
extern int param_tracer_dynamic_coverage_feedback;
#else
  int x_param_tracer_dynamic_coverage_feedback;
#define param_tracer_dynamic_coverage_feedback global_options.x_param_tracer_dynamic_coverage_feedback
#endif
#ifdef GENERATOR_FILE
extern int param_tracer_dynamic_coverage;
#else
  int x_param_tracer_dynamic_coverage;
#define param_tracer_dynamic_coverage global_options.x_param_tracer_dynamic_coverage
#endif
#ifdef GENERATOR_FILE
extern int param_tracer_max_code_growth;
#else
  int x_param_tracer_max_code_growth;
#define param_tracer_max_code_growth global_options.x_param_tracer_max_code_growth
#endif
#ifdef GENERATOR_FILE
extern int param_tracer_min_branch_probability_feedback;
#else
  int x_param_tracer_min_branch_probability_feedback;
#define param_tracer_min_branch_probability_feedback global_options.x_param_tracer_min_branch_probability_feedback
#endif
#ifdef GENERATOR_FILE
extern int param_tracer_min_branch_probability;
#else
  int x_param_tracer_min_branch_probability;
#define param_tracer_min_branch_probability global_options.x_param_tracer_min_branch_probability
#endif
#ifdef GENERATOR_FILE
extern int param_tracer_min_branch_ratio;
#else
  int x_param_tracer_min_branch_ratio;
#define param_tracer_min_branch_ratio global_options.x_param_tracer_min_branch_ratio
#endif
#ifdef GENERATOR_FILE
extern int param_tree_reassoc_width;
#else
  int x_param_tree_reassoc_width;
#define param_tree_reassoc_width global_options.x_param_tree_reassoc_width
#endif
#ifdef GENERATOR_FILE
extern int param_tsan_distinguish_volatile;
#else
  int x_param_tsan_distinguish_volatile;
#define param_tsan_distinguish_volatile global_options.x_param_tsan_distinguish_volatile
#endif
#ifdef GENERATOR_FILE
extern int param_tsan_instrument_func_entry_exit;
#else
  int x_param_tsan_instrument_func_entry_exit;
#define param_tsan_instrument_func_entry_exit global_options.x_param_tsan_instrument_func_entry_exit
#endif
#ifdef GENERATOR_FILE
extern int param_uninit_control_dep_attempts;
#else
  int x_param_uninit_control_dep_attempts;
#define param_uninit_control_dep_attempts global_options.x_param_uninit_control_dep_attempts
#endif
#ifdef GENERATOR_FILE
extern int param_uninit_max_chain_len;
#else
  int x_param_uninit_max_chain_len;
#define param_uninit_max_chain_len global_options.x_param_uninit_max_chain_len
#endif
#ifdef GENERATOR_FILE
extern int param_uninit_max_num_chains;
#else
  int x_param_uninit_max_num_chains;
#define param_uninit_max_num_chains global_options.x_param_uninit_max_num_chains
#endif
#ifdef GENERATOR_FILE
extern int param_uninlined_function_insns;
#else
  int x_param_uninlined_function_insns;
#define param_uninlined_function_insns global_options.x_param_uninlined_function_insns
#endif
#ifdef GENERATOR_FILE
extern int param_uninlined_function_time;
#else
  int x_param_uninlined_function_time;
#define param_uninlined_function_time global_options.x_param_uninlined_function_time
#endif
#ifdef GENERATOR_FILE
extern int param_uninlined_function_thunk_insns;
#else
  int x_param_uninlined_function_thunk_insns;
#define param_uninlined_function_thunk_insns global_options.x_param_uninlined_function_thunk_insns
#endif
#ifdef GENERATOR_FILE
extern int param_uninlined_function_thunk_time;
#else
  int x_param_uninlined_function_thunk_time;
#define param_uninlined_function_thunk_time global_options.x_param_uninlined_function_thunk_time
#endif
#ifdef GENERATOR_FILE
extern int param_unlikely_bb_count_fraction;
#else
  int x_param_unlikely_bb_count_fraction;
#define param_unlikely_bb_count_fraction global_options.x_param_unlikely_bb_count_fraction
#endif
#ifdef GENERATOR_FILE
extern int param_unroll_jam_max_unroll;
#else
  int x_param_unroll_jam_max_unroll;
#define param_unroll_jam_max_unroll global_options.x_param_unroll_jam_max_unroll
#endif
#ifdef GENERATOR_FILE
extern int param_unroll_jam_min_percent;
#else
  int x_param_unroll_jam_min_percent;
#define param_unroll_jam_min_percent global_options.x_param_unroll_jam_min_percent
#endif
#ifdef GENERATOR_FILE
extern int param_use_after_scope_direct_emission_threshold;
#else
  int x_param_use_after_scope_direct_emission_threshold;
#define param_use_after_scope_direct_emission_threshold global_options.x_param_use_after_scope_direct_emission_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_use_canonical_types;
#else
  int x_param_use_canonical_types;
#define param_use_canonical_types global_options.x_param_use_canonical_types
#endif
#ifdef GENERATOR_FILE
extern int param_vect_epilogues_nomask;
#else
  int x_param_vect_epilogues_nomask;
#define param_vect_epilogues_nomask global_options.x_param_vect_epilogues_nomask
#endif
#ifdef GENERATOR_FILE
extern int param_vect_induction_float;
#else
  int x_param_vect_induction_float;
#define param_vect_induction_float global_options.x_param_vect_induction_float
#endif
#ifdef GENERATOR_FILE
extern int param_vect_inner_loop_cost_factor;
#else
  int x_param_vect_inner_loop_cost_factor;
#define param_vect_inner_loop_cost_factor global_options.x_param_vect_inner_loop_cost_factor
#endif
#ifdef GENERATOR_FILE
extern int param_vect_max_layout_candidates;
#else
  int x_param_vect_max_layout_candidates;
#define param_vect_max_layout_candidates global_options.x_param_vect_max_layout_candidates
#endif
#ifdef GENERATOR_FILE
extern int param_vect_max_peeling_for_alignment;
#else
  int x_param_vect_max_peeling_for_alignment;
#define param_vect_max_peeling_for_alignment global_options.x_param_vect_max_peeling_for_alignment
#endif
#ifdef GENERATOR_FILE
extern int param_vect_max_version_for_alias_checks;
#else
  int x_param_vect_max_version_for_alias_checks;
#define param_vect_max_version_for_alias_checks global_options.x_param_vect_max_version_for_alias_checks
#endif
#ifdef GENERATOR_FILE
extern int param_vect_max_version_for_alignment_checks;
#else
  int x_param_vect_max_version_for_alignment_checks;
#define param_vect_max_version_for_alignment_checks global_options.x_param_vect_max_version_for_alignment_checks
#endif
#ifdef GENERATOR_FILE
extern int param_vect_partial_vector_usage;
#else
  int x_param_vect_partial_vector_usage;
#define param_vect_partial_vector_usage global_options.x_param_vect_partial_vector_usage
#endif
#ifdef GENERATOR_FILE
extern int param_vrp_sparse_threshold;
#else
  int x_param_vrp_sparse_threshold;
#define param_vrp_sparse_threshold global_options.x_param_vrp_sparse_threshold
#endif
#ifdef GENERATOR_FILE
extern int param_vrp_switch_limit;
#else
  int x_param_vrp_switch_limit;
#define param_vrp_switch_limit global_options.x_param_vrp_switch_limit
#endif
#ifdef GENERATOR_FILE
extern int param_vrp_vector_threshold;
#else
  int x_param_vrp_vector_threshold;
#define param_vrp_vector_threshold global_options.x_param_vrp_vector_threshold
#endif
#ifdef GENERATOR_FILE
extern int x86_stlf_window_ninsns;
#else
  int x_x86_stlf_window_ninsns;
#define x86_stlf_window_ninsns global_options.x_x86_stlf_window_ninsns
#endif
#ifdef GENERATOR_FILE
extern int x86_stv_max_visits;
#else
  int x_x86_stv_max_visits;
#define x86_stv_max_visits global_options.x_x86_stv_max_visits
#endif
#ifdef GENERATOR_FILE
extern int flag_preprocess_only;
#else
  int x_flag_preprocess_only;
#define flag_preprocess_only global_options.x_flag_preprocess_only
#endif
#ifdef GENERATOR_FILE
extern int warn_nsobject_attribute;
#else
  int x_warn_nsobject_attribute;
#define warn_nsobject_attribute global_options.x_warn_nsobject_attribute
#endif
#ifdef GENERATOR_FILE
extern int warn_abi;
#else
  int x_warn_abi;
#define warn_abi global_options.x_warn_abi
#endif
#ifdef GENERATOR_FILE
extern int warn_abi_tag;
#else
  int x_warn_abi_tag;
#define warn_abi_tag global_options.x_warn_abi_tag
#endif
#ifdef GENERATOR_FILE
extern int warn_absolute_value;
#else
  int x_warn_absolute_value;
#define warn_absolute_value global_options.x_warn_absolute_value
#endif
#ifdef GENERATOR_FILE
extern int warn_address;
#else
  int x_warn_address;
#define warn_address global_options.x_warn_address
#endif
#ifdef GENERATOR_FILE
extern int warn_address_of_packed_member;
#else
  int x_warn_address_of_packed_member;
#define warn_address_of_packed_member global_options.x_warn_address_of_packed_member
#endif
#ifdef GENERATOR_FILE
extern int warn_aggregate_return;
#else
  int x_warn_aggregate_return;
#define warn_aggregate_return global_options.x_warn_aggregate_return
#endif
#ifdef GENERATOR_FILE
extern int warn_aggressive_loop_optimizations;
#else
  int x_warn_aggressive_loop_optimizations;
#define warn_aggressive_loop_optimizations global_options.x_warn_aggressive_loop_optimizations
#endif
#ifdef GENERATOR_FILE
extern int warn_aliasing;
#else
  int x_warn_aliasing;
#define warn_aliasing global_options.x_warn_aliasing
#endif
#ifdef GENERATOR_FILE
extern int warn_align_commons;
#else
  int x_warn_align_commons;
#define warn_align_commons global_options.x_warn_align_commons
#endif
#ifdef GENERATOR_FILE
extern int warn_aligned_new;
#else
  int x_warn_aligned_new;
#define warn_aligned_new global_options.x_warn_aligned_new
#endif
#ifdef GENERATOR_FILE
extern int warn_alloc_size;
#else
  int x_warn_alloc_size;
#define warn_alloc_size global_options.x_warn_alloc_size
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT warn_alloc_size_limit;
#else
  HOST_WIDE_INT x_warn_alloc_size_limit;
#define warn_alloc_size_limit global_options.x_warn_alloc_size_limit
#endif
#ifdef GENERATOR_FILE
extern int warn_alloc_zero;
#else
  int x_warn_alloc_zero;
#define warn_alloc_zero global_options.x_warn_alloc_zero
#endif
#ifdef GENERATOR_FILE
extern int warn_alloca;
#else
  int x_warn_alloca;
#define warn_alloca global_options.x_warn_alloca
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT warn_alloca_limit;
#else
  HOST_WIDE_INT x_warn_alloca_limit;
#define warn_alloca_limit global_options.x_warn_alloca_limit
#endif
#ifdef GENERATOR_FILE
extern int warn_ampersand;
#else
  int x_warn_ampersand;
#define warn_ampersand global_options.x_warn_ampersand
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_allocation_size;
#else
  int x_warn_analyzer_allocation_size;
#define warn_analyzer_allocation_size global_options.x_warn_analyzer_allocation_size
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_deref_before_check;
#else
  int x_warn_analyzer_deref_before_check;
#define warn_analyzer_deref_before_check global_options.x_warn_analyzer_deref_before_check
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_double_fclose;
#else
  int x_warn_analyzer_double_fclose;
#define warn_analyzer_double_fclose global_options.x_warn_analyzer_double_fclose
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_double_free;
#else
  int x_warn_analyzer_double_free;
#define warn_analyzer_double_free global_options.x_warn_analyzer_double_free
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_exposure_through_output_file;
#else
  int x_warn_analyzer_exposure_through_output_file;
#define warn_analyzer_exposure_through_output_file global_options.x_warn_analyzer_exposure_through_output_file
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_exposure_through_uninit_copy;
#else
  int x_warn_analyzer_exposure_through_uninit_copy;
#define warn_analyzer_exposure_through_uninit_copy global_options.x_warn_analyzer_exposure_through_uninit_copy
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_mode_mismatch;
#else
  int x_warn_analyzer_fd_mode_mismatch;
#define warn_analyzer_fd_mode_mismatch global_options.x_warn_analyzer_fd_mode_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_double_close;
#else
  int x_warn_analyzer_fd_double_close;
#define warn_analyzer_fd_double_close global_options.x_warn_analyzer_fd_double_close
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_leak;
#else
  int x_warn_analyzer_fd_leak;
#define warn_analyzer_fd_leak global_options.x_warn_analyzer_fd_leak
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_phase_mismatch;
#else
  int x_warn_analyzer_fd_phase_mismatch;
#define warn_analyzer_fd_phase_mismatch global_options.x_warn_analyzer_fd_phase_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_type_mismatch;
#else
  int x_warn_analyzer_fd_type_mismatch;
#define warn_analyzer_fd_type_mismatch global_options.x_warn_analyzer_fd_type_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_use_after_close;
#else
  int x_warn_analyzer_fd_use_after_close;
#define warn_analyzer_fd_use_after_close global_options.x_warn_analyzer_fd_use_after_close
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_fd_use_without_check;
#else
  int x_warn_analyzer_fd_use_without_check;
#define warn_analyzer_fd_use_without_check global_options.x_warn_analyzer_fd_use_without_check
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_file_leak;
#else
  int x_warn_analyzer_file_leak;
#define warn_analyzer_file_leak global_options.x_warn_analyzer_file_leak
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_free_of_non_heap;
#else
  int x_warn_analyzer_free_of_non_heap;
#define warn_analyzer_free_of_non_heap global_options.x_warn_analyzer_free_of_non_heap
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_imprecise_fp_arithmetic;
#else
  int x_warn_analyzer_imprecise_fp_arithmetic;
#define warn_analyzer_imprecise_fp_arithmetic global_options.x_warn_analyzer_imprecise_fp_arithmetic
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_infinite_loop;
#else
  int x_warn_analyzer_infinite_loop;
#define warn_analyzer_infinite_loop global_options.x_warn_analyzer_infinite_loop
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_infinite_recursion;
#else
  int x_warn_analyzer_infinite_recursion;
#define warn_analyzer_infinite_recursion global_options.x_warn_analyzer_infinite_recursion
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_jump_through_null;
#else
  int x_warn_analyzer_jump_through_null;
#define warn_analyzer_jump_through_null global_options.x_warn_analyzer_jump_through_null
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_malloc_leak;
#else
  int x_warn_analyzer_malloc_leak;
#define warn_analyzer_malloc_leak global_options.x_warn_analyzer_malloc_leak
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_mismatching_deallocation;
#else
  int x_warn_analyzer_mismatching_deallocation;
#define warn_analyzer_mismatching_deallocation global_options.x_warn_analyzer_mismatching_deallocation
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_null_argument;
#else
  int x_warn_analyzer_null_argument;
#define warn_analyzer_null_argument global_options.x_warn_analyzer_null_argument
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_null_dereference;
#else
  int x_warn_analyzer_null_dereference;
#define warn_analyzer_null_dereference global_options.x_warn_analyzer_null_dereference
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_out_of_bounds;
#else
  int x_warn_analyzer_out_of_bounds;
#define warn_analyzer_out_of_bounds global_options.x_warn_analyzer_out_of_bounds
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_overlapping_buffers;
#else
  int x_warn_analyzer_overlapping_buffers;
#define warn_analyzer_overlapping_buffers global_options.x_warn_analyzer_overlapping_buffers
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_possible_null_argument;
#else
  int x_warn_analyzer_possible_null_argument;
#define warn_analyzer_possible_null_argument global_options.x_warn_analyzer_possible_null_argument
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_possible_null_dereference;
#else
  int x_warn_analyzer_possible_null_dereference;
#define warn_analyzer_possible_null_dereference global_options.x_warn_analyzer_possible_null_dereference
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_putenv_of_auto_var;
#else
  int x_warn_analyzer_putenv_of_auto_var;
#define warn_analyzer_putenv_of_auto_var global_options.x_warn_analyzer_putenv_of_auto_var
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_shift_count_negative;
#else
  int x_warn_analyzer_shift_count_negative;
#define warn_analyzer_shift_count_negative global_options.x_warn_analyzer_shift_count_negative
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_shift_count_overflow;
#else
  int x_warn_analyzer_shift_count_overflow;
#define warn_analyzer_shift_count_overflow global_options.x_warn_analyzer_shift_count_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_stale_setjmp_buffer;
#else
  int x_warn_analyzer_stale_setjmp_buffer;
#define warn_analyzer_stale_setjmp_buffer global_options.x_warn_analyzer_stale_setjmp_buffer
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_symbol_too_complex;
#else
  int x_warn_analyzer_symbol_too_complex;
#define warn_analyzer_symbol_too_complex global_options.x_warn_analyzer_symbol_too_complex
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_tainted_allocation_size;
#else
  int x_warn_analyzer_tainted_allocation_size;
#define warn_analyzer_tainted_allocation_size global_options.x_warn_analyzer_tainted_allocation_size
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_tainted_array_index;
#else
  int x_warn_analyzer_tainted_array_index;
#define warn_analyzer_tainted_array_index global_options.x_warn_analyzer_tainted_array_index
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_tainted_assertion;
#else
  int x_warn_analyzer_tainted_assertion;
#define warn_analyzer_tainted_assertion global_options.x_warn_analyzer_tainted_assertion
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_tainted_divisor;
#else
  int x_warn_analyzer_tainted_divisor;
#define warn_analyzer_tainted_divisor global_options.x_warn_analyzer_tainted_divisor
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_tainted_offset;
#else
  int x_warn_analyzer_tainted_offset;
#define warn_analyzer_tainted_offset global_options.x_warn_analyzer_tainted_offset
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_tainted_size;
#else
  int x_warn_analyzer_tainted_size;
#define warn_analyzer_tainted_size global_options.x_warn_analyzer_tainted_size
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_too_complex;
#else
  int x_warn_analyzer_too_complex;
#define warn_analyzer_too_complex global_options.x_warn_analyzer_too_complex
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_undefined_behavior_strtok;
#else
  int x_warn_analyzer_undefined_behavior_strtok;
#define warn_analyzer_undefined_behavior_strtok global_options.x_warn_analyzer_undefined_behavior_strtok
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_unsafe_call_within_signal_handler;
#else
  int x_warn_analyzer_unsafe_call_within_signal_handler;
#define warn_analyzer_unsafe_call_within_signal_handler global_options.x_warn_analyzer_unsafe_call_within_signal_handler
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_use_after_free;
#else
  int x_warn_analyzer_use_after_free;
#define warn_analyzer_use_after_free global_options.x_warn_analyzer_use_after_free
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_use_of_pointer_in_stale_stack_frame;
#else
  int x_warn_analyzer_use_of_pointer_in_stale_stack_frame;
#define warn_analyzer_use_of_pointer_in_stale_stack_frame global_options.x_warn_analyzer_use_of_pointer_in_stale_stack_frame
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_use_of_uninitialized_value;
#else
  int x_warn_analyzer_use_of_uninitialized_value;
#define warn_analyzer_use_of_uninitialized_value global_options.x_warn_analyzer_use_of_uninitialized_value
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_va_arg_type_mismatch;
#else
  int x_warn_analyzer_va_arg_type_mismatch;
#define warn_analyzer_va_arg_type_mismatch global_options.x_warn_analyzer_va_arg_type_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_va_list_exhausted;
#else
  int x_warn_analyzer_va_list_exhausted;
#define warn_analyzer_va_list_exhausted global_options.x_warn_analyzer_va_list_exhausted
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_va_list_leak;
#else
  int x_warn_analyzer_va_list_leak;
#define warn_analyzer_va_list_leak global_options.x_warn_analyzer_va_list_leak
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_va_list_use_after_va_end;
#else
  int x_warn_analyzer_va_list_use_after_va_end;
#define warn_analyzer_va_list_use_after_va_end global_options.x_warn_analyzer_va_list_use_after_va_end
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_write_to_const;
#else
  int x_warn_analyzer_write_to_const;
#define warn_analyzer_write_to_const global_options.x_warn_analyzer_write_to_const
#endif
#ifdef GENERATOR_FILE
extern int warn_analyzer_write_to_string_literal;
#else
  int x_warn_analyzer_write_to_string_literal;
#define warn_analyzer_write_to_string_literal global_options.x_warn_analyzer_write_to_string_literal
#endif
#ifdef GENERATOR_FILE
extern int warn_arith_conv;
#else
  int x_warn_arith_conv;
#define warn_arith_conv global_options.x_warn_arith_conv
#endif
#ifdef GENERATOR_FILE
extern int warn_array_bounds;
#else
  int x_warn_array_bounds;
#define warn_array_bounds global_options.x_warn_array_bounds
#endif
#ifdef GENERATOR_FILE
extern int warn_array_compare;
#else
  int x_warn_array_compare;
#define warn_array_compare global_options.x_warn_array_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_array_parameter;
#else
  int x_warn_array_parameter;
#define warn_array_parameter global_options.x_warn_array_parameter
#endif
#ifdef GENERATOR_FILE
extern int warn_array_temporaries;
#else
  int x_warn_array_temporaries;
#define warn_array_temporaries global_options.x_warn_array_temporaries
#endif
#ifdef GENERATOR_FILE
extern int warn_assign_intercept;
#else
  int x_warn_assign_intercept;
#define warn_assign_intercept global_options.x_warn_assign_intercept
#endif
#ifdef GENERATOR_FILE
extern int warn_attribute_alias;
#else
  int x_warn_attribute_alias;
#define warn_attribute_alias global_options.x_warn_attribute_alias
#endif
#ifdef GENERATOR_FILE
extern int warn_attribute_warning;
#else
  int x_warn_attribute_warning;
#define warn_attribute_warning global_options.x_warn_attribute_warning
#endif
#ifdef GENERATOR_FILE
extern int warn_attributes;
#else
  int x_warn_attributes;
#define warn_attributes global_options.x_warn_attributes
#endif
#ifdef GENERATOR_FILE
extern int warn_bad_function_cast;
#else
  int x_warn_bad_function_cast;
#define warn_bad_function_cast global_options.x_warn_bad_function_cast
#endif
#ifdef GENERATOR_FILE
extern int warn_bidirectional;
#else
  int x_warn_bidirectional;
#define warn_bidirectional global_options.x_warn_bidirectional
#endif
#ifdef GENERATOR_FILE
extern int warn_bool_compare;
#else
  int x_warn_bool_compare;
#define warn_bool_compare global_options.x_warn_bool_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_bool_op;
#else
  int x_warn_bool_op;
#define warn_bool_op global_options.x_warn_bool_op
#endif
#ifdef GENERATOR_FILE
extern int warn_builtin_declaration_mismatch;
#else
  int x_warn_builtin_declaration_mismatch;
#define warn_builtin_declaration_mismatch global_options.x_warn_builtin_declaration_mismatch
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_builtin_macro_redefined;
#else
  int x_cpp_warn_builtin_macro_redefined;
#define cpp_warn_builtin_macro_redefined global_options.x_cpp_warn_builtin_macro_redefined
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx_compat;
#else
  int x_warn_cxx_compat;
#define warn_cxx_compat global_options.x_warn_cxx_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx11_compat;
#else
  int x_warn_cxx11_compat;
#define warn_cxx11_compat global_options.x_warn_cxx11_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx11_extensions;
#else
  int x_warn_cxx11_extensions;
#define warn_cxx11_extensions global_options.x_warn_cxx11_extensions
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx14_compat;
#else
  int x_warn_cxx14_compat;
#define warn_cxx14_compat global_options.x_warn_cxx14_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx14_extensions;
#else
  int x_warn_cxx14_extensions;
#define warn_cxx14_extensions global_options.x_warn_cxx14_extensions
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx17_compat;
#else
  int x_warn_cxx17_compat;
#define warn_cxx17_compat global_options.x_warn_cxx17_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx17_extensions;
#else
  int x_warn_cxx17_extensions;
#define warn_cxx17_extensions global_options.x_warn_cxx17_extensions
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx20_compat;
#else
  int x_warn_cxx20_compat;
#define warn_cxx20_compat global_options.x_warn_cxx20_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx20_extensions;
#else
  int x_warn_cxx20_extensions;
#define warn_cxx20_extensions global_options.x_warn_cxx20_extensions
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx23_extensions;
#else
  int x_warn_cxx23_extensions;
#define warn_cxx23_extensions global_options.x_warn_cxx23_extensions
#endif
#ifdef GENERATOR_FILE
extern int warn_cxx26_extensions;
#else
  int x_warn_cxx26_extensions;
#define warn_cxx26_extensions global_options.x_warn_cxx26_extensions
#endif
#ifdef GENERATOR_FILE
extern int warn_c_binding_type;
#else
  int x_warn_c_binding_type;
#define warn_c_binding_type global_options.x_warn_c_binding_type
#endif
#ifdef GENERATOR_FILE
extern int warn_c11_c23_compat;
#else
  int x_warn_c11_c23_compat;
#define warn_c11_c23_compat global_options.x_warn_c11_c23_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_c90_c99_compat;
#else
  int x_warn_c90_c99_compat;
#define warn_c90_c99_compat global_options.x_warn_c90_c99_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_c99_c11_compat;
#else
  int x_warn_c99_c11_compat;
#define warn_c99_c11_compat global_options.x_warn_c99_c11_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_calloc_transposed_args;
#else
  int x_warn_calloc_transposed_args;
#define warn_calloc_transposed_args global_options.x_warn_calloc_transposed_args
#endif
#ifdef GENERATOR_FILE
extern int warn_cannot_profile;
#else
  int x_warn_cannot_profile;
#define warn_cannot_profile global_options.x_warn_cannot_profile
#endif
#ifdef GENERATOR_FILE
extern int warn_cast_align;
#else
  int x_warn_cast_align;
#define warn_cast_align global_options.x_warn_cast_align
#endif
#ifdef GENERATOR_FILE
extern int warn_cast_function_type;
#else
  int x_warn_cast_function_type;
#define warn_cast_function_type global_options.x_warn_cast_function_type
#endif
#ifdef GENERATOR_FILE
extern int warn_cast_qual;
#else
  int x_warn_cast_qual;
#define warn_cast_qual global_options.x_warn_cast_qual
#endif
#ifdef GENERATOR_FILE
extern int warn_cast_result;
#else
  int x_warn_cast_result;
#define warn_cast_result global_options.x_warn_cast_result
#endif
#ifdef GENERATOR_FILE
extern int warn_cast_user_defined;
#else
  int x_warn_cast_user_defined;
#define warn_cast_user_defined global_options.x_warn_cast_user_defined
#endif
#ifdef GENERATOR_FILE
extern int warn_catch_value;
#else
  int x_warn_catch_value;
#define warn_catch_value global_options.x_warn_catch_value
#endif
#ifdef GENERATOR_FILE
extern int warn_changes_meaning;
#else
  int x_warn_changes_meaning;
#define warn_changes_meaning global_options.x_warn_changes_meaning
#endif
#ifdef GENERATOR_FILE
extern int warn_char_subscripts;
#else
  int x_warn_char_subscripts;
#define warn_char_subscripts global_options.x_warn_char_subscripts
#endif
#ifdef GENERATOR_FILE
extern int warn_character_truncation;
#else
  int x_warn_character_truncation;
#define warn_character_truncation global_options.x_warn_character_truncation
#endif
#ifdef GENERATOR_FILE
extern int warn_class_conversion;
#else
  int x_warn_class_conversion;
#define warn_class_conversion global_options.x_warn_class_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_class_memaccess;
#else
  int x_warn_class_memaccess;
#define warn_class_memaccess global_options.x_warn_class_memaccess
#endif
#ifdef GENERATOR_FILE
extern int warn_clobbered;
#else
  int x_warn_clobbered;
#define warn_clobbered global_options.x_warn_clobbered
#endif
#ifdef GENERATOR_FILE
extern int warn_comma_subscript;
#else
  int x_warn_comma_subscript;
#define warn_comma_subscript global_options.x_warn_comma_subscript
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_comment;
#else
  int x_cpp_warn_comment;
#define cpp_warn_comment global_options.x_cpp_warn_comment
#endif
#ifdef GENERATOR_FILE
extern int warn_compare_distinct_pointer_types;
#else
  int x_warn_compare_distinct_pointer_types;
#define warn_compare_distinct_pointer_types global_options.x_warn_compare_distinct_pointer_types
#endif
#ifdef GENERATOR_FILE
extern int warn_compare_reals;
#else
  int x_warn_compare_reals;
#define warn_compare_reals global_options.x_warn_compare_reals
#endif
#ifdef GENERATOR_FILE
extern int warn_complain_wrong_lang;
#else
  int x_warn_complain_wrong_lang;
#define warn_complain_wrong_lang global_options.x_warn_complain_wrong_lang
#endif
#ifdef GENERATOR_FILE
extern int warn_conditionally_supported;
#else
  int x_warn_conditionally_supported;
#define warn_conditionally_supported global_options.x_warn_conditionally_supported
#endif
#ifdef GENERATOR_FILE
extern int warn_conversion;
#else
  int x_warn_conversion;
#define warn_conversion global_options.x_warn_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_conversion_extra;
#else
  int x_warn_conversion_extra;
#define warn_conversion_extra global_options.x_warn_conversion_extra
#endif
#ifdef GENERATOR_FILE
extern int warn_conversion_null;
#else
  int x_warn_conversion_null;
#define warn_conversion_null global_options.x_warn_conversion_null
#endif
#ifdef GENERATOR_FILE
extern int warn_coverage_invalid_linenum;
#else
  int x_warn_coverage_invalid_linenum;
#define warn_coverage_invalid_linenum global_options.x_warn_coverage_invalid_linenum
#endif
#ifdef GENERATOR_FILE
extern int warn_coverage_mismatch;
#else
  int x_warn_coverage_mismatch;
#define warn_coverage_mismatch global_options.x_warn_coverage_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_too_many_conditions;
#else
  int x_warn_too_many_conditions;
#define warn_too_many_conditions global_options.x_warn_too_many_conditions
#endif
#ifdef GENERATOR_FILE
extern int warn_cpp;
#else
  int x_warn_cpp;
#define warn_cpp global_options.x_warn_cpp
#endif
#ifdef GENERATOR_FILE
extern int warn_ctad_maybe_unsupported;
#else
  int x_warn_ctad_maybe_unsupported;
#define warn_ctad_maybe_unsupported global_options.x_warn_ctad_maybe_unsupported
#endif
#ifdef GENERATOR_FILE
extern int warn_ctor_dtor_privacy;
#else
  int x_warn_ctor_dtor_privacy;
#define warn_ctor_dtor_privacy global_options.x_warn_ctor_dtor_privacy
#endif
#ifdef GENERATOR_FILE
extern int warn_dangling_else;
#else
  int x_warn_dangling_else;
#define warn_dangling_else global_options.x_warn_dangling_else
#endif
#ifdef GENERATOR_FILE
extern int warn_dangling_pointer;
#else
  int x_warn_dangling_pointer;
#define warn_dangling_pointer global_options.x_warn_dangling_pointer
#endif
#ifdef GENERATOR_FILE
extern int warn_dangling_reference;
#else
  int x_warn_dangling_reference;
#define warn_dangling_reference global_options.x_warn_dangling_reference
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_date_time;
#else
  int x_cpp_warn_date_time;
#define cpp_warn_date_time global_options.x_cpp_warn_date_time
#endif
#ifdef GENERATOR_FILE
extern int warn_declaration_after_statement;
#else
  int x_warn_declaration_after_statement;
#define warn_declaration_after_statement global_options.x_warn_declaration_after_statement
#endif
#ifdef GENERATOR_FILE
extern int warn_declaration_missing_parameter;
#else
  int x_warn_declaration_missing_parameter;
#define warn_declaration_missing_parameter global_options.x_warn_declaration_missing_parameter
#endif
#ifdef GENERATOR_FILE
extern int warn_delete_incomplete;
#else
  int x_warn_delete_incomplete;
#define warn_delete_incomplete global_options.x_warn_delete_incomplete
#endif
#ifdef GENERATOR_FILE
extern int warn_delnonvdtor;
#else
  int x_warn_delnonvdtor;
#define warn_delnonvdtor global_options.x_warn_delnonvdtor
#endif
#ifdef GENERATOR_FILE
extern int warn_deprecated;
#else
  int x_warn_deprecated;
#define warn_deprecated global_options.x_warn_deprecated
#endif
#ifdef GENERATOR_FILE
extern int warn_deprecated_copy;
#else
  int x_warn_deprecated_copy;
#define warn_deprecated_copy global_options.x_warn_deprecated_copy
#endif
#ifdef GENERATOR_FILE
extern int warn_deprecated_decl;
#else
  int x_warn_deprecated_decl;
#define warn_deprecated_decl global_options.x_warn_deprecated_decl
#endif
#ifdef GENERATOR_FILE
extern int warn_deprecated_enum_enum_conv;
#else
  int x_warn_deprecated_enum_enum_conv;
#define warn_deprecated_enum_enum_conv global_options.x_warn_deprecated_enum_enum_conv
#endif
#ifdef GENERATOR_FILE
extern int warn_deprecated_enum_float_conv;
#else
  int x_warn_deprecated_enum_float_conv;
#define warn_deprecated_enum_float_conv global_options.x_warn_deprecated_enum_float_conv
#endif
#ifdef GENERATOR_FILE
extern int warn_designated_init;
#else
  int x_warn_designated_init;
#define warn_designated_init global_options.x_warn_designated_init
#endif
#ifdef GENERATOR_FILE
extern int warn_disabled_optimization;
#else
  int x_warn_disabled_optimization;
#define warn_disabled_optimization global_options.x_warn_disabled_optimization
#endif
#ifdef GENERATOR_FILE
extern int warn_discarded_array_qualifiers;
#else
  int x_warn_discarded_array_qualifiers;
#define warn_discarded_array_qualifiers global_options.x_warn_discarded_array_qualifiers
#endif
#ifdef GENERATOR_FILE
extern int warn_discarded_qualifiers;
#else
  int x_warn_discarded_qualifiers;
#define warn_discarded_qualifiers global_options.x_warn_discarded_qualifiers
#endif
#ifdef GENERATOR_FILE
extern int warn_div_by_zero;
#else
  int x_warn_div_by_zero;
#define warn_div_by_zero global_options.x_warn_div_by_zero
#endif
#ifdef GENERATOR_FILE
extern int warn_do_subscript;
#else
  int x_warn_do_subscript;
#define warn_do_subscript global_options.x_warn_do_subscript
#endif
#ifdef GENERATOR_FILE
extern int warn_double_promotion;
#else
  int x_warn_double_promotion;
#define warn_double_promotion global_options.x_warn_double_promotion
#endif
#ifdef GENERATOR_FILE
extern int warn_duplicate_decl_specifier;
#else
  int x_warn_duplicate_decl_specifier;
#define warn_duplicate_decl_specifier global_options.x_warn_duplicate_decl_specifier
#endif
#ifdef GENERATOR_FILE
extern int warn_duplicated_branches;
#else
  int x_warn_duplicated_branches;
#define warn_duplicated_branches global_options.x_warn_duplicated_branches
#endif
#ifdef GENERATOR_FILE
extern int warn_duplicated_cond;
#else
  int x_warn_duplicated_cond;
#define warn_duplicated_cond global_options.x_warn_duplicated_cond
#endif
#ifdef GENERATOR_FILE
extern int warn_ecpp;
#else
  int x_warn_ecpp;
#define warn_ecpp global_options.x_warn_ecpp
#endif
#ifdef GENERATOR_FILE
extern int warn_elaborated_enum_base;
#else
  int x_warn_elaborated_enum_base;
#define warn_elaborated_enum_base global_options.x_warn_elaborated_enum_base
#endif
#ifdef GENERATOR_FILE
extern int warn_empty_body;
#else
  int x_warn_empty_body;
#define warn_empty_body global_options.x_warn_empty_body
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_endif_labels;
#else
  int x_cpp_warn_endif_labels;
#define cpp_warn_endif_labels global_options.x_cpp_warn_endif_labels
#endif
#ifdef GENERATOR_FILE
extern int warn_enum_compare;
#else
  int x_warn_enum_compare;
#define warn_enum_compare global_options.x_warn_enum_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_enum_conversion;
#else
  int x_warn_enum_conversion;
#define warn_enum_conversion global_options.x_warn_enum_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_enum_int_mismatch;
#else
  int x_warn_enum_int_mismatch;
#define warn_enum_int_mismatch global_options.x_warn_enum_int_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warnings_are_errors;
#else
  int x_warnings_are_errors;
#define warnings_are_errors global_options.x_warnings_are_errors
#endif
#ifdef GENERATOR_FILE
extern int warn_exceptions;
#else
  int x_warn_exceptions;
#define warn_exceptions global_options.x_warn_exceptions
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_expansion_to_defined;
#else
  int x_cpp_warn_expansion_to_defined;
#define cpp_warn_expansion_to_defined global_options.x_cpp_warn_expansion_to_defined
#endif
#ifdef GENERATOR_FILE
extern int extra_warnings;
#else
  int x_extra_warnings;
#define extra_warnings global_options.x_extra_warnings
#endif
#ifdef GENERATOR_FILE
extern int warn_extra_semi;
#else
  int x_warn_extra_semi;
#define warn_extra_semi global_options.x_warn_extra_semi
#endif
#ifdef GENERATOR_FILE
extern int flag_fatal_errors;
#else
  int x_flag_fatal_errors;
#define flag_fatal_errors global_options.x_flag_fatal_errors
#endif
#ifdef GENERATOR_FILE
extern int warn_flex_array_member_not_at_end;
#else
  int x_warn_flex_array_member_not_at_end;
#define warn_flex_array_member_not_at_end global_options.x_warn_flex_array_member_not_at_end
#endif
#ifdef GENERATOR_FILE
extern int warn_float_conversion;
#else
  int x_warn_float_conversion;
#define warn_float_conversion global_options.x_warn_float_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_float_equal;
#else
  int x_warn_float_equal;
#define warn_float_equal global_options.x_warn_float_equal
#endif
#ifdef GENERATOR_FILE
extern int warn_format_contains_nul;
#else
  int x_warn_format_contains_nul;
#define warn_format_contains_nul global_options.x_warn_format_contains_nul
#endif
#ifdef GENERATOR_FILE
extern int warn_format_diag;
#else
  int x_warn_format_diag;
#define warn_format_diag global_options.x_warn_format_diag
#endif
#ifdef GENERATOR_FILE
extern int warn_format_extra_args;
#else
  int x_warn_format_extra_args;
#define warn_format_extra_args global_options.x_warn_format_extra_args
#endif
#ifdef GENERATOR_FILE
extern int warn_format_nonliteral;
#else
  int x_warn_format_nonliteral;
#define warn_format_nonliteral global_options.x_warn_format_nonliteral
#endif
#ifdef GENERATOR_FILE
extern int warn_format_overflow;
#else
  int x_warn_format_overflow;
#define warn_format_overflow global_options.x_warn_format_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_format_security;
#else
  int x_warn_format_security;
#define warn_format_security global_options.x_warn_format_security
#endif
#ifdef GENERATOR_FILE
extern int warn_format_signedness;
#else
  int x_warn_format_signedness;
#define warn_format_signedness global_options.x_warn_format_signedness
#endif
#ifdef GENERATOR_FILE
extern int warn_format_trunc;
#else
  int x_warn_format_trunc;
#define warn_format_trunc global_options.x_warn_format_trunc
#endif
#ifdef GENERATOR_FILE
extern int warn_format_y2k;
#else
  int x_warn_format_y2k;
#define warn_format_y2k global_options.x_warn_format_y2k
#endif
#ifdef GENERATOR_FILE
extern int warn_format_zero_length;
#else
  int x_warn_format_zero_length;
#define warn_format_zero_length global_options.x_warn_format_zero_length
#endif
#ifdef GENERATOR_FILE
extern int warn_format;
#else
  int x_warn_format;
#define warn_format global_options.x_warn_format
#endif
#ifdef GENERATOR_FILE
extern int warn_frame_address;
#else
  int x_warn_frame_address;
#define warn_frame_address global_options.x_warn_frame_address
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT warn_frame_larger_than_size;
#else
  HOST_WIDE_INT x_warn_frame_larger_than_size;
#define warn_frame_larger_than_size global_options.x_warn_frame_larger_than_size
#endif
#ifdef GENERATOR_FILE
extern int warn_free_nonheap_object;
#else
  int x_warn_free_nonheap_object;
#define warn_free_nonheap_object global_options.x_warn_free_nonheap_object
#endif
#ifdef GENERATOR_FILE
extern int flag_warn_frontend_loop_interchange;
#else
  int x_flag_warn_frontend_loop_interchange;
#define flag_warn_frontend_loop_interchange global_options.x_flag_warn_frontend_loop_interchange
#endif
#ifdef GENERATOR_FILE
extern int warn_function_elimination;
#else
  int x_warn_function_elimination;
#define warn_function_elimination global_options.x_warn_function_elimination
#endif
#ifdef GENERATOR_FILE
extern int warn_global_module;
#else
  int x_warn_global_module;
#define warn_global_module global_options.x_warn_global_module
#endif
#ifdef GENERATOR_FILE
extern int warn_hardened;
#else
  int x_warn_hardened;
#define warn_hardened global_options.x_warn_hardened
#endif
#ifdef GENERATOR_FILE
extern int warn_if_not_aligned;
#else
  int x_warn_if_not_aligned;
#define warn_if_not_aligned global_options.x_warn_if_not_aligned
#endif
#ifdef GENERATOR_FILE
extern int warn_ignored_attributes;
#else
  int x_warn_ignored_attributes;
#define warn_ignored_attributes global_options.x_warn_ignored_attributes
#endif
#ifdef GENERATOR_FILE
extern int warn_ignored_qualifiers;
#else
  int x_warn_ignored_qualifiers;
#define warn_ignored_qualifiers global_options.x_warn_ignored_qualifiers
#endif
#ifdef GENERATOR_FILE
extern int warn_implicit;
#else
  int x_warn_implicit;
#define warn_implicit global_options.x_warn_implicit
#endif
#ifdef GENERATOR_FILE
extern int warn_implicit_fallthrough;
#else
  int x_warn_implicit_fallthrough;
#define warn_implicit_fallthrough global_options.x_warn_implicit_fallthrough
#endif
#ifdef GENERATOR_FILE
extern int warn_implicit_function_declaration;
#else
  int x_warn_implicit_function_declaration;
#define warn_implicit_function_declaration global_options.x_warn_implicit_function_declaration
#endif
#ifdef GENERATOR_FILE
extern int warn_implicit_int;
#else
  int x_warn_implicit_int;
#define warn_implicit_int global_options.x_warn_implicit_int
#endif
#ifdef GENERATOR_FILE
extern int warn_implicit_interface;
#else
  int x_warn_implicit_interface;
#define warn_implicit_interface global_options.x_warn_implicit_interface
#endif
#ifdef GENERATOR_FILE
extern int warn_implicit_procedure;
#else
  int x_warn_implicit_procedure;
#define warn_implicit_procedure global_options.x_warn_implicit_procedure
#endif
#ifdef GENERATOR_FILE
extern int warn_inaccessible_base;
#else
  int x_warn_inaccessible_base;
#define warn_inaccessible_base global_options.x_warn_inaccessible_base
#endif
#ifdef GENERATOR_FILE
extern int warn_incompatible_pointer_types;
#else
  int x_warn_incompatible_pointer_types;
#define warn_incompatible_pointer_types global_options.x_warn_incompatible_pointer_types
#endif
#ifdef GENERATOR_FILE
extern int warn_infinite_recursion;
#else
  int x_warn_infinite_recursion;
#define warn_infinite_recursion global_options.x_warn_infinite_recursion
#endif
#ifdef GENERATOR_FILE
extern int warn_inh_var_ctor;
#else
  int x_warn_inh_var_ctor;
#define warn_inh_var_ctor global_options.x_warn_inh_var_ctor
#endif
#ifdef GENERATOR_FILE
extern int warn_init_list;
#else
  int x_warn_init_list;
#define warn_init_list global_options.x_warn_init_list
#endif
#ifdef GENERATOR_FILE
extern int warn_init_self;
#else
  int x_warn_init_self;
#define warn_init_self global_options.x_warn_init_self
#endif
#ifdef GENERATOR_FILE
extern int warn_inline;
#else
  int x_warn_inline;
#define warn_inline global_options.x_warn_inline
#endif
#ifdef GENERATOR_FILE
extern int warn_int_conversion;
#else
  int x_warn_int_conversion;
#define warn_int_conversion global_options.x_warn_int_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_int_in_bool_context;
#else
  int x_warn_int_in_bool_context;
#define warn_int_in_bool_context global_options.x_warn_int_in_bool_context
#endif
#ifdef GENERATOR_FILE
extern int warn_int_to_pointer_cast;
#else
  int x_warn_int_to_pointer_cast;
#define warn_int_to_pointer_cast global_options.x_warn_int_to_pointer_cast
#endif
#ifdef GENERATOR_FILE
extern int warn_integer_division;
#else
  int x_warn_integer_division;
#define warn_integer_division global_options.x_warn_integer_division
#endif
#ifdef GENERATOR_FILE
extern int warn_interference_size;
#else
  int x_warn_interference_size;
#define warn_interference_size global_options.x_warn_interference_size
#endif
#ifdef GENERATOR_FILE
extern int warn_intrinsic_shadow;
#else
  int x_warn_intrinsic_shadow;
#define warn_intrinsic_shadow global_options.x_warn_intrinsic_shadow
#endif
#ifdef GENERATOR_FILE
extern int warn_intrinsics_std;
#else
  int x_warn_intrinsics_std;
#define warn_intrinsics_std global_options.x_warn_intrinsics_std
#endif
#ifdef GENERATOR_FILE
extern int warn_invalid_constexpr;
#else
  int x_warn_invalid_constexpr;
#define warn_invalid_constexpr global_options.x_warn_invalid_constexpr
#endif
#ifdef GENERATOR_FILE
extern int warn_imported_macros;
#else
  int x_warn_imported_macros;
#define warn_imported_macros global_options.x_warn_imported_macros
#endif
#ifdef GENERATOR_FILE
extern int warn_invalid_memory_model;
#else
  int x_warn_invalid_memory_model;
#define warn_invalid_memory_model global_options.x_warn_invalid_memory_model
#endif
#ifdef GENERATOR_FILE
extern int warn_invalid_offsetof;
#else
  int x_warn_invalid_offsetof;
#define warn_invalid_offsetof global_options.x_warn_invalid_offsetof
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_invalid_pch;
#else
  int x_cpp_warn_invalid_pch;
#define cpp_warn_invalid_pch global_options.x_cpp_warn_invalid_pch
#endif
#ifdef GENERATOR_FILE
extern int warn_invalid_utf8;
#else
  int x_warn_invalid_utf8;
#define warn_invalid_utf8 global_options.x_warn_invalid_utf8
#endif
#ifdef GENERATOR_FILE
extern int warn_jump_misses_init;
#else
  int x_warn_jump_misses_init;
#define warn_jump_misses_init global_options.x_warn_jump_misses_init
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT warn_larger_than_size;
#else
  HOST_WIDE_INT x_warn_larger_than_size;
#define warn_larger_than_size global_options.x_warn_larger_than_size
#endif
#ifdef GENERATOR_FILE
extern int warn_line_truncation;
#else
  int x_warn_line_truncation;
#define warn_line_truncation global_options.x_warn_line_truncation
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_literal_suffix;
#else
  int x_cpp_warn_literal_suffix;
#define cpp_warn_literal_suffix global_options.x_cpp_warn_literal_suffix
#endif
#ifdef GENERATOR_FILE
extern int warn_logical_not_paren;
#else
  int x_warn_logical_not_paren;
#define warn_logical_not_paren global_options.x_warn_logical_not_paren
#endif
#ifdef GENERATOR_FILE
extern int warn_logical_op;
#else
  int x_warn_logical_op;
#define warn_logical_op global_options.x_warn_logical_op
#endif
#ifdef GENERATOR_FILE
extern int warn_long_long;
#else
  int x_warn_long_long;
#define warn_long_long global_options.x_warn_long_long
#endif
#ifdef GENERATOR_FILE
extern int warn_lto_type_mismatch;
#else
  int x_warn_lto_type_mismatch;
#define warn_lto_type_mismatch global_options.x_warn_lto_type_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_main;
#else
  int x_warn_main;
#define warn_main global_options.x_warn_main
#endif
#ifdef GENERATOR_FILE
extern int warn_maybe_uninitialized;
#else
  int x_warn_maybe_uninitialized;
#define warn_maybe_uninitialized global_options.x_warn_maybe_uninitialized
#endif
#ifdef GENERATOR_FILE
extern int warn_memset_elt_size;
#else
  int x_warn_memset_elt_size;
#define warn_memset_elt_size global_options.x_warn_memset_elt_size
#endif
#ifdef GENERATOR_FILE
extern int warn_memset_transposed_args;
#else
  int x_warn_memset_transposed_args;
#define warn_memset_transposed_args global_options.x_warn_memset_transposed_args
#endif
#ifdef GENERATOR_FILE
extern int warn_misleading_indentation;
#else
  int x_warn_misleading_indentation;
#define warn_misleading_indentation global_options.x_warn_misleading_indentation
#endif
#ifdef GENERATOR_FILE
extern int warn_mismatched_alloc;
#else
  int x_warn_mismatched_alloc;
#define warn_mismatched_alloc global_options.x_warn_mismatched_alloc
#endif
#ifdef GENERATOR_FILE
extern int warn_mismatched_new_delete;
#else
  int x_warn_mismatched_new_delete;
#define warn_mismatched_new_delete global_options.x_warn_mismatched_new_delete
#endif
#ifdef GENERATOR_FILE
extern int warn_mismatched_special_enum;
#else
  int x_warn_mismatched_special_enum;
#define warn_mismatched_special_enum global_options.x_warn_mismatched_special_enum
#endif
#ifdef GENERATOR_FILE
extern int warn_mismatched_tags;
#else
  int x_warn_mismatched_tags;
#define warn_mismatched_tags global_options.x_warn_mismatched_tags
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_attributes;
#else
  int x_warn_missing_attributes;
#define warn_missing_attributes global_options.x_warn_missing_attributes
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_braces;
#else
  int x_warn_missing_braces;
#define warn_missing_braces global_options.x_warn_missing_braces
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_declarations;
#else
  int x_warn_missing_declarations;
#define warn_missing_declarations global_options.x_warn_missing_declarations
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_field_initializers;
#else
  int x_warn_missing_field_initializers;
#define warn_missing_field_initializers global_options.x_warn_missing_field_initializers
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_missing_include_dirs;
#else
  int x_cpp_warn_missing_include_dirs;
#define cpp_warn_missing_include_dirs global_options.x_cpp_warn_missing_include_dirs
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_parameter_type;
#else
  int x_warn_missing_parameter_type;
#define warn_missing_parameter_type global_options.x_warn_missing_parameter_type
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_profile;
#else
  int x_warn_missing_profile;
#define warn_missing_profile global_options.x_warn_missing_profile
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_prototypes;
#else
  int x_warn_missing_prototypes;
#define warn_missing_prototypes global_options.x_warn_missing_prototypes
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_requires;
#else
  int x_warn_missing_requires;
#define warn_missing_requires global_options.x_warn_missing_requires
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_template_keyword;
#else
  int x_warn_missing_template_keyword;
#define warn_missing_template_keyword global_options.x_warn_missing_template_keyword
#endif
#ifdef GENERATOR_FILE
extern int warn_missing_variable_declarations;
#else
  int x_warn_missing_variable_declarations;
#define warn_missing_variable_declarations global_options.x_warn_missing_variable_declarations
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_multichar;
#else
  int x_cpp_warn_multichar;
#define cpp_warn_multichar global_options.x_cpp_warn_multichar
#endif
#ifdef GENERATOR_FILE
extern int warn_multiple_inheritance;
#else
  int x_warn_multiple_inheritance;
#define warn_multiple_inheritance global_options.x_warn_multiple_inheritance
#endif
#ifdef GENERATOR_FILE
extern int warn_multistatement_macros;
#else
  int x_warn_multistatement_macros;
#define warn_multistatement_macros global_options.x_warn_multistatement_macros
#endif
#ifdef GENERATOR_FILE
extern int warn_namespaces;
#else
  int x_warn_namespaces;
#define warn_namespaces global_options.x_warn_namespaces
#endif
#ifdef GENERATOR_FILE
extern int warn_narrowing;
#else
  int x_warn_narrowing;
#define warn_narrowing global_options.x_warn_narrowing
#endif
#ifdef GENERATOR_FILE
extern int warn_nested_externs;
#else
  int x_warn_nested_externs;
#define warn_nested_externs global_options.x_warn_nested_externs
#endif
#ifdef GENERATOR_FILE
extern int warn_noexcept;
#else
  int x_warn_noexcept;
#define warn_noexcept global_options.x_warn_noexcept
#endif
#ifdef GENERATOR_FILE
extern int warn_noexcept_type;
#else
  int x_warn_noexcept_type;
#define warn_noexcept_type global_options.x_warn_noexcept_type
#endif
#ifdef GENERATOR_FILE
extern int warn_nontemplate_friend;
#else
  int x_warn_nontemplate_friend;
#define warn_nontemplate_friend global_options.x_warn_nontemplate_friend
#endif
#ifdef GENERATOR_FILE
extern int warn_nonvdtor;
#else
  int x_warn_nonvdtor;
#define warn_nonvdtor global_options.x_warn_nonvdtor
#endif
#ifdef GENERATOR_FILE
extern int warn_nonnull;
#else
  int x_warn_nonnull;
#define warn_nonnull global_options.x_warn_nonnull
#endif
#ifdef GENERATOR_FILE
extern int warn_nonnull_compare;
#else
  int x_warn_nonnull_compare;
#define warn_nonnull_compare global_options.x_warn_nonnull_compare
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_normalize;
#else
  int x_cpp_warn_normalize;
#define cpp_warn_normalize global_options.x_cpp_warn_normalize
#endif
#ifdef GENERATOR_FILE
extern int warn_nrvo;
#else
  int x_warn_nrvo;
#define warn_nrvo global_options.x_warn_nrvo
#endif
#ifdef GENERATOR_FILE
extern int warn_null_dereference;
#else
  int x_warn_null_dereference;
#define warn_null_dereference global_options.x_warn_null_dereference
#endif
#ifdef GENERATOR_FILE
extern int warn_objc_root_class;
#else
  int x_warn_objc_root_class;
#define warn_objc_root_class global_options.x_warn_objc_root_class
#endif
#ifdef GENERATOR_FILE
extern int warn_odr_violations;
#else
  int x_warn_odr_violations;
#define warn_odr_violations global_options.x_warn_odr_violations
#endif
#ifdef GENERATOR_FILE
extern int warn_old_style_cast;
#else
  int x_warn_old_style_cast;
#define warn_old_style_cast global_options.x_warn_old_style_cast
#endif
#ifdef GENERATOR_FILE
extern int warn_old_style_declaration;
#else
  int x_warn_old_style_declaration;
#define warn_old_style_declaration global_options.x_warn_old_style_declaration
#endif
#ifdef GENERATOR_FILE
extern int warn_old_style_definition;
#else
  int x_warn_old_style_definition;
#define warn_old_style_definition global_options.x_warn_old_style_definition
#endif
#ifdef GENERATOR_FILE
extern int warn_openacc_parallelism;
#else
  int x_warn_openacc_parallelism;
#define warn_openacc_parallelism global_options.x_warn_openacc_parallelism
#endif
#ifdef GENERATOR_FILE
extern int warn_openmp;
#else
  int x_warn_openmp;
#define warn_openmp global_options.x_warn_openmp
#endif
#ifdef GENERATOR_FILE
extern int warn_openmp_simd;
#else
  int x_warn_openmp_simd;
#define warn_openmp_simd global_options.x_warn_openmp_simd
#endif
#ifdef GENERATOR_FILE
extern int warn_overflow;
#else
  int x_warn_overflow;
#define warn_overflow global_options.x_warn_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_overlength_strings;
#else
  int x_warn_overlength_strings;
#define warn_overlength_strings global_options.x_warn_overlength_strings
#endif
#ifdef GENERATOR_FILE
extern int warn_overloaded_virtual;
#else
  int x_warn_overloaded_virtual;
#define warn_overloaded_virtual global_options.x_warn_overloaded_virtual
#endif
#ifdef GENERATOR_FILE
extern int warn_override_init;
#else
  int x_warn_override_init;
#define warn_override_init global_options.x_warn_override_init
#endif
#ifdef GENERATOR_FILE
extern int warn_override_init_side_effects;
#else
  int x_warn_override_init_side_effects;
#define warn_override_init_side_effects global_options.x_warn_override_init_side_effects
#endif
#ifdef GENERATOR_FILE
extern int warn_overwrite_recursive;
#else
  int x_warn_overwrite_recursive;
#define warn_overwrite_recursive global_options.x_warn_overwrite_recursive
#endif
#ifdef GENERATOR_FILE
extern int warn_packed;
#else
  int x_warn_packed;
#define warn_packed global_options.x_warn_packed
#endif
#ifdef GENERATOR_FILE
extern int warn_packed_bitfield_compat;
#else
  int x_warn_packed_bitfield_compat;
#define warn_packed_bitfield_compat global_options.x_warn_packed_bitfield_compat
#endif
#ifdef GENERATOR_FILE
extern int warn_packed_not_aligned;
#else
  int x_warn_packed_not_aligned;
#define warn_packed_not_aligned global_options.x_warn_packed_not_aligned
#endif
#ifdef GENERATOR_FILE
extern int warn_padded;
#else
  int x_warn_padded;
#define warn_padded global_options.x_warn_padded
#endif
#ifdef GENERATOR_FILE
extern int warn_parentheses;
#else
  int x_warn_parentheses;
#define warn_parentheses global_options.x_warn_parentheses
#endif
#ifdef GENERATOR_FILE
extern int pedantic;
#else
  int x_pedantic;
#define pedantic global_options.x_pedantic
#endif
#ifdef GENERATOR_FILE
extern int warn_pessimizing_move;
#else
  int x_warn_pessimizing_move;
#define warn_pessimizing_move global_options.x_warn_pessimizing_move
#endif
#ifdef GENERATOR_FILE
extern int warn_placement_new;
#else
  int x_warn_placement_new;
#define warn_placement_new global_options.x_warn_placement_new
#endif
#ifdef GENERATOR_FILE
extern int warn_pmf2ptr;
#else
  int x_warn_pmf2ptr;
#define warn_pmf2ptr global_options.x_warn_pmf2ptr
#endif
#ifdef GENERATOR_FILE
extern int warn_pointer_arith;
#else
  int x_warn_pointer_arith;
#define warn_pointer_arith global_options.x_warn_pointer_arith
#endif
#ifdef GENERATOR_FILE
extern int warn_pointer_compare;
#else
  int x_warn_pointer_compare;
#define warn_pointer_compare global_options.x_warn_pointer_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_pointer_sign;
#else
  int x_warn_pointer_sign;
#define warn_pointer_sign global_options.x_warn_pointer_sign
#endif
#ifdef GENERATOR_FILE
extern int warn_pointer_to_int_cast;
#else
  int x_warn_pointer_to_int_cast;
#define warn_pointer_to_int_cast global_options.x_warn_pointer_to_int_cast
#endif
#ifdef GENERATOR_FILE
extern int warn_pragmas;
#else
  int x_warn_pragmas;
#define warn_pragmas global_options.x_warn_pragmas
#endif
#ifdef GENERATOR_FILE
extern int warn_prio_ctor_dtor;
#else
  int x_warn_prio_ctor_dtor;
#define warn_prio_ctor_dtor global_options.x_warn_prio_ctor_dtor
#endif
#ifdef GENERATOR_FILE
extern int warn_property_assign_default;
#else
  int x_warn_property_assign_default;
#define warn_property_assign_default global_options.x_warn_property_assign_default
#endif
#ifdef GENERATOR_FILE
extern int warn_protocol;
#else
  int x_warn_protocol;
#define warn_protocol global_options.x_warn_protocol
#endif
#ifdef GENERATOR_FILE
extern int warn_psabi;
#else
  int x_warn_psabi;
#define warn_psabi global_options.x_warn_psabi
#endif
#ifdef GENERATOR_FILE
extern int warn_range_loop_construct;
#else
  int x_warn_range_loop_construct;
#define warn_range_loop_construct global_options.x_warn_range_loop_construct
#endif
#ifdef GENERATOR_FILE
extern int warn_real_q_constant;
#else
  int x_warn_real_q_constant;
#define warn_real_q_constant global_options.x_warn_real_q_constant
#endif
#ifdef GENERATOR_FILE
extern int warn_realloc_lhs;
#else
  int x_warn_realloc_lhs;
#define warn_realloc_lhs global_options.x_warn_realloc_lhs
#endif
#ifdef GENERATOR_FILE
extern int warn_realloc_lhs_all;
#else
  int x_warn_realloc_lhs_all;
#define warn_realloc_lhs_all global_options.x_warn_realloc_lhs_all
#endif
#ifdef GENERATOR_FILE
extern int warn_redundant_decls;
#else
  int x_warn_redundant_decls;
#define warn_redundant_decls global_options.x_warn_redundant_decls
#endif
#ifdef GENERATOR_FILE
extern int warn_redundant_move;
#else
  int x_warn_redundant_move;
#define warn_redundant_move global_options.x_warn_redundant_move
#endif
#ifdef GENERATOR_FILE
extern int warn_redundant_tags;
#else
  int x_warn_redundant_tags;
#define warn_redundant_tags global_options.x_warn_redundant_tags
#endif
#ifdef GENERATOR_FILE
extern int warn_register;
#else
  int x_warn_register;
#define warn_register global_options.x_warn_register
#endif
#ifdef GENERATOR_FILE
extern int warn_reorder;
#else
  int x_warn_reorder;
#define warn_reorder global_options.x_warn_reorder
#endif
#ifdef GENERATOR_FILE
extern int warn_restrict;
#else
  int x_warn_restrict;
#define warn_restrict global_options.x_warn_restrict
#endif
#ifdef GENERATOR_FILE
extern int warn_return_local_addr;
#else
  int x_warn_return_local_addr;
#define warn_return_local_addr global_options.x_warn_return_local_addr
#endif
#ifdef GENERATOR_FILE
extern int warn_return_mismatch;
#else
  int x_warn_return_mismatch;
#define warn_return_mismatch global_options.x_warn_return_mismatch
#endif
#ifdef GENERATOR_FILE
extern int warn_return_type;
#else
  int x_warn_return_type;
#define warn_return_type global_options.x_warn_return_type
#endif
#ifdef GENERATOR_FILE
extern int warn_scalar_storage_order;
#else
  int x_warn_scalar_storage_order;
#define warn_scalar_storage_order global_options.x_warn_scalar_storage_order
#endif
#ifdef GENERATOR_FILE
extern int warn_selector;
#else
  int x_warn_selector;
#define warn_selector global_options.x_warn_selector
#endif
#ifdef GENERATOR_FILE
extern int warn_self_move;
#else
  int x_warn_self_move;
#define warn_self_move global_options.x_warn_self_move
#endif
#ifdef GENERATOR_FILE
extern int warn_sequence_point;
#else
  int x_warn_sequence_point;
#define warn_sequence_point global_options.x_warn_sequence_point
#endif
#ifdef GENERATOR_FILE
extern int warn_shadow;
#else
  int x_warn_shadow;
#define warn_shadow global_options.x_warn_shadow
#endif
#ifdef GENERATOR_FILE
extern int warn_shadow_ivar;
#else
  int x_warn_shadow_ivar;
#define warn_shadow_ivar global_options.x_warn_shadow_ivar
#endif
#ifdef GENERATOR_FILE
extern int warn_shadow_compatible_local;
#else
  int x_warn_shadow_compatible_local;
#define warn_shadow_compatible_local global_options.x_warn_shadow_compatible_local
#endif
#ifdef GENERATOR_FILE
extern int warn_shadow_local;
#else
  int x_warn_shadow_local;
#define warn_shadow_local global_options.x_warn_shadow_local
#endif
#ifdef GENERATOR_FILE
extern int warn_shift_count_negative;
#else
  int x_warn_shift_count_negative;
#define warn_shift_count_negative global_options.x_warn_shift_count_negative
#endif
#ifdef GENERATOR_FILE
extern int warn_shift_count_overflow;
#else
  int x_warn_shift_count_overflow;
#define warn_shift_count_overflow global_options.x_warn_shift_count_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_shift_negative_value;
#else
  int x_warn_shift_negative_value;
#define warn_shift_negative_value global_options.x_warn_shift_negative_value
#endif
#ifdef GENERATOR_FILE
extern int warn_shift_overflow;
#else
  int x_warn_shift_overflow;
#define warn_shift_overflow global_options.x_warn_shift_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_sign_compare;
#else
  int x_warn_sign_compare;
#define warn_sign_compare global_options.x_warn_sign_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_sign_conversion;
#else
  int x_warn_sign_conversion;
#define warn_sign_conversion global_options.x_warn_sign_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_sign_promo;
#else
  int x_warn_sign_promo;
#define warn_sign_promo global_options.x_warn_sign_promo
#endif
#ifdef GENERATOR_FILE
extern int warn_sized_deallocation;
#else
  int x_warn_sized_deallocation;
#define warn_sized_deallocation global_options.x_warn_sized_deallocation
#endif
#ifdef GENERATOR_FILE
extern int warn_sizeof_array_argument;
#else
  int x_warn_sizeof_array_argument;
#define warn_sizeof_array_argument global_options.x_warn_sizeof_array_argument
#endif
#ifdef GENERATOR_FILE
extern int warn_sizeof_array_div;
#else
  int x_warn_sizeof_array_div;
#define warn_sizeof_array_div global_options.x_warn_sizeof_array_div
#endif
#ifdef GENERATOR_FILE
extern int warn_sizeof_pointer_div;
#else
  int x_warn_sizeof_pointer_div;
#define warn_sizeof_pointer_div global_options.x_warn_sizeof_pointer_div
#endif
#ifdef GENERATOR_FILE
extern int warn_sizeof_pointer_memaccess;
#else
  int x_warn_sizeof_pointer_memaccess;
#define warn_sizeof_pointer_memaccess global_options.x_warn_sizeof_pointer_memaccess
#endif
#ifdef GENERATOR_FILE
extern int warn_stack_protect;
#else
  int x_warn_stack_protect;
#define warn_stack_protect global_options.x_warn_stack_protect
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT warn_stack_usage;
#else
  HOST_WIDE_INT x_warn_stack_usage;
#define warn_stack_usage global_options.x_warn_stack_usage
#endif
#ifdef GENERATOR_FILE
extern int warn_strict_aliasing;
#else
  int x_warn_strict_aliasing;
#define warn_strict_aliasing global_options.x_warn_strict_aliasing
#endif
#ifdef GENERATOR_FILE
extern int warn_strict_flex_arrays;
#else
  int x_warn_strict_flex_arrays;
#define warn_strict_flex_arrays global_options.x_warn_strict_flex_arrays
#endif
#ifdef GENERATOR_FILE
extern int warn_strict_null_sentinel;
#else
  int x_warn_strict_null_sentinel;
#define warn_strict_null_sentinel global_options.x_warn_strict_null_sentinel
#endif
#ifdef GENERATOR_FILE
extern int warn_strict_overflow;
#else
  int x_warn_strict_overflow;
#define warn_strict_overflow global_options.x_warn_strict_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_strict_prototypes;
#else
  int x_warn_strict_prototypes;
#define warn_strict_prototypes global_options.x_warn_strict_prototypes
#endif
#ifdef GENERATOR_FILE
extern int warn_strict_selector_match;
#else
  int x_warn_strict_selector_match;
#define warn_strict_selector_match global_options.x_warn_strict_selector_match
#endif
#ifdef GENERATOR_FILE
extern int warn_string_compare;
#else
  int x_warn_string_compare;
#define warn_string_compare global_options.x_warn_string_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_stringop_overflow;
#else
  int x_warn_stringop_overflow;
#define warn_stringop_overflow global_options.x_warn_stringop_overflow
#endif
#ifdef GENERATOR_FILE
extern int warn_stringop_overread;
#else
  int x_warn_stringop_overread;
#define warn_stringop_overread global_options.x_warn_stringop_overread
#endif
#ifdef GENERATOR_FILE
extern int warn_stringop_truncation;
#else
  int x_warn_stringop_truncation;
#define warn_stringop_truncation global_options.x_warn_stringop_truncation
#endif
#ifdef GENERATOR_FILE
extern int warn_subobject_linkage;
#else
  int x_warn_subobject_linkage;
#define warn_subobject_linkage global_options.x_warn_subobject_linkage
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_cold;
#else
  int x_warn_suggest_attribute_cold;
#define warn_suggest_attribute_cold global_options.x_warn_suggest_attribute_cold
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_const;
#else
  int x_warn_suggest_attribute_const;
#define warn_suggest_attribute_const global_options.x_warn_suggest_attribute_const
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_format;
#else
  int x_warn_suggest_attribute_format;
#define warn_suggest_attribute_format global_options.x_warn_suggest_attribute_format
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_malloc;
#else
  int x_warn_suggest_attribute_malloc;
#define warn_suggest_attribute_malloc global_options.x_warn_suggest_attribute_malloc
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_noreturn;
#else
  int x_warn_suggest_attribute_noreturn;
#define warn_suggest_attribute_noreturn global_options.x_warn_suggest_attribute_noreturn
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_pure;
#else
  int x_warn_suggest_attribute_pure;
#define warn_suggest_attribute_pure global_options.x_warn_suggest_attribute_pure
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_attribute_returns_nonnull;
#else
  int x_warn_suggest_attribute_returns_nonnull;
#define warn_suggest_attribute_returns_nonnull global_options.x_warn_suggest_attribute_returns_nonnull
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_final_methods;
#else
  int x_warn_suggest_final_methods;
#define warn_suggest_final_methods global_options.x_warn_suggest_final_methods
#endif
#ifdef GENERATOR_FILE
extern int warn_suggest_final_types;
#else
  int x_warn_suggest_final_types;
#define warn_suggest_final_types global_options.x_warn_suggest_final_types
#endif
#ifdef GENERATOR_FILE
extern int warn_override;
#else
  int x_warn_override;
#define warn_override global_options.x_warn_override
#endif
#ifdef GENERATOR_FILE
extern int warn_surprising;
#else
  int x_warn_surprising;
#define warn_surprising global_options.x_warn_surprising
#endif
#ifdef GENERATOR_FILE
extern int warn_switch;
#else
  int x_warn_switch;
#define warn_switch global_options.x_warn_switch
#endif
#ifdef GENERATOR_FILE
extern int warn_switch_bool;
#else
  int x_warn_switch_bool;
#define warn_switch_bool global_options.x_warn_switch_bool
#endif
#ifdef GENERATOR_FILE
extern int warn_switch_default;
#else
  int x_warn_switch_default;
#define warn_switch_default global_options.x_warn_switch_default
#endif
#ifdef GENERATOR_FILE
extern int warn_switch_enum;
#else
  int x_warn_switch_enum;
#define warn_switch_enum global_options.x_warn_switch_enum
#endif
#ifdef GENERATOR_FILE
extern int warn_switch_outside_range;
#else
  int x_warn_switch_outside_range;
#define warn_switch_outside_range global_options.x_warn_switch_outside_range
#endif
#ifdef GENERATOR_FILE
extern int warn_switch_unreachable;
#else
  int x_warn_switch_unreachable;
#define warn_switch_unreachable global_options.x_warn_switch_unreachable
#endif
#ifdef GENERATOR_FILE
extern int warn_sync_nand;
#else
  int x_warn_sync_nand;
#define warn_sync_nand global_options.x_warn_sync_nand
#endif
#ifdef GENERATOR_FILE
extern int warn_synth;
#else
  int x_warn_synth;
#define warn_synth global_options.x_warn_synth
#endif
#ifdef GENERATOR_FILE
extern int warn_system_headers;
#else
  int x_warn_system_headers;
#define warn_system_headers global_options.x_warn_system_headers
#endif
#ifdef GENERATOR_FILE
extern int warn_tabs;
#else
  int x_warn_tabs;
#define warn_tabs global_options.x_warn_tabs
#endif
#ifdef GENERATOR_FILE
extern int warn_target_lifetime;
#else
  int x_warn_target_lifetime;
#define warn_target_lifetime global_options.x_warn_target_lifetime
#endif
#ifdef GENERATOR_FILE
extern int warn_tautological_compare;
#else
  int x_warn_tautological_compare;
#define warn_tautological_compare global_options.x_warn_tautological_compare
#endif
#ifdef GENERATOR_FILE
extern int warn_template_id_cdtor;
#else
  int x_warn_template_id_cdtor;
#define warn_template_id_cdtor global_options.x_warn_template_id_cdtor
#endif
#ifdef GENERATOR_FILE
extern int warn_templates;
#else
  int x_warn_templates;
#define warn_templates global_options.x_warn_templates
#endif
#ifdef GENERATOR_FILE
extern int warn_terminate;
#else
  int x_warn_terminate;
#define warn_terminate global_options.x_warn_terminate
#endif
#ifdef GENERATOR_FILE
extern int warn_traditional;
#else
  int x_warn_traditional;
#define warn_traditional global_options.x_warn_traditional
#endif
#ifdef GENERATOR_FILE
extern int warn_traditional_conversion;
#else
  int x_warn_traditional_conversion;
#define warn_traditional_conversion global_options.x_warn_traditional_conversion
#endif
#ifdef GENERATOR_FILE
extern int warn_trampolines;
#else
  int x_warn_trampolines;
#define warn_trampolines global_options.x_warn_trampolines
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_trigraphs;
#else
  int x_cpp_warn_trigraphs;
#define cpp_warn_trigraphs global_options.x_cpp_warn_trigraphs
#endif
#ifdef GENERATOR_FILE
extern int warn_trivial_auto_var_init;
#else
  int x_warn_trivial_auto_var_init;
#define warn_trivial_auto_var_init global_options.x_warn_trivial_auto_var_init
#endif
#ifdef GENERATOR_FILE
extern int warn_tsan;
#else
  int x_warn_tsan;
#define warn_tsan global_options.x_warn_tsan
#endif
#ifdef GENERATOR_FILE
extern int warn_type_limits;
#else
  int x_warn_type_limits;
#define warn_type_limits global_options.x_warn_type_limits
#endif
#ifdef GENERATOR_FILE
extern int warn_undeclared_selector;
#else
  int x_warn_undeclared_selector;
#define warn_undeclared_selector global_options.x_warn_undeclared_selector
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_undef;
#else
  int x_cpp_warn_undef;
#define cpp_warn_undef global_options.x_cpp_warn_undef
#endif
#ifdef GENERATOR_FILE
extern int warn_undefined_do_loop;
#else
  int x_warn_undefined_do_loop;
#define warn_undefined_do_loop global_options.x_warn_undefined_do_loop
#endif
#ifdef GENERATOR_FILE
extern int warn_underflow;
#else
  int x_warn_underflow;
#define warn_underflow global_options.x_warn_underflow
#endif
#ifdef GENERATOR_FILE
extern int warn_unicode;
#else
  int x_warn_unicode;
#define warn_unicode global_options.x_warn_unicode
#endif
#ifdef GENERATOR_FILE
extern int warn_uninitialized;
#else
  int x_warn_uninitialized;
#define warn_uninitialized global_options.x_warn_uninitialized
#endif
#ifdef GENERATOR_FILE
extern int warn_unknown_pragmas;
#else
  int x_warn_unknown_pragmas;
#define warn_unknown_pragmas global_options.x_warn_unknown_pragmas
#endif
#ifdef GENERATOR_FILE
extern int warn_unsuffixed_float_constants;
#else
  int x_warn_unsuffixed_float_constants;
#define warn_unsuffixed_float_constants global_options.x_warn_unsuffixed_float_constants
#endif
#ifdef GENERATOR_FILE
extern int warn_unused;
#else
  int x_warn_unused;
#define warn_unused global_options.x_warn_unused
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_but_set_parameter;
#else
  int x_warn_unused_but_set_parameter;
#define warn_unused_but_set_parameter global_options.x_warn_unused_but_set_parameter
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_but_set_variable;
#else
  int x_warn_unused_but_set_variable;
#define warn_unused_but_set_variable global_options.x_warn_unused_but_set_variable
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_const_variable;
#else
  int x_warn_unused_const_variable;
#define warn_unused_const_variable global_options.x_warn_unused_const_variable
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_dummy_argument;
#else
  int x_warn_unused_dummy_argument;
#define warn_unused_dummy_argument global_options.x_warn_unused_dummy_argument
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_function;
#else
  int x_warn_unused_function;
#define warn_unused_function global_options.x_warn_unused_function
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_label;
#else
  int x_warn_unused_label;
#define warn_unused_label global_options.x_warn_unused_label
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_local_typedefs;
#else
  int x_warn_unused_local_typedefs;
#define warn_unused_local_typedefs global_options.x_warn_unused_local_typedefs
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_unused_macros;
#else
  int x_cpp_warn_unused_macros;
#define cpp_warn_unused_macros global_options.x_cpp_warn_unused_macros
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_parameter;
#else
  int x_warn_unused_parameter;
#define warn_unused_parameter global_options.x_warn_unused_parameter
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_result;
#else
  int x_warn_unused_result;
#define warn_unused_result global_options.x_warn_unused_result
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_value;
#else
  int x_warn_unused_value;
#define warn_unused_value global_options.x_warn_unused_value
#endif
#ifdef GENERATOR_FILE
extern int warn_unused_variable;
#else
  int x_warn_unused_variable;
#define warn_unused_variable global_options.x_warn_unused_variable
#endif
#ifdef GENERATOR_FILE
extern int warn_use_after_free;
#else
  int x_warn_use_after_free;
#define warn_use_after_free global_options.x_warn_use_after_free
#endif
#ifdef GENERATOR_FILE
extern int warn_use_without_only;
#else
  int x_warn_use_without_only;
#define warn_use_without_only global_options.x_warn_use_without_only
#endif
#ifdef GENERATOR_FILE
extern int warn_useless_cast;
#else
  int x_warn_useless_cast;
#define warn_useless_cast global_options.x_warn_useless_cast
#endif
#ifdef GENERATOR_FILE
extern int warn_varargs;
#else
  int x_warn_varargs;
#define warn_varargs global_options.x_warn_varargs
#endif
#ifdef GENERATOR_FILE
extern int cpp_warn_variadic_macros;
#else
  int x_cpp_warn_variadic_macros;
#define cpp_warn_variadic_macros global_options.x_cpp_warn_variadic_macros
#endif
#ifdef GENERATOR_FILE
extern int warn_vector_operation_performance;
#else
  int x_warn_vector_operation_performance;
#define warn_vector_operation_performance global_options.x_warn_vector_operation_performance
#endif
#ifdef GENERATOR_FILE
extern int warn_vexing_parse;
#else
  int x_warn_vexing_parse;
#define warn_vexing_parse global_options.x_warn_vexing_parse
#endif
#ifdef GENERATOR_FILE
extern int warn_virtual_inheritance;
#else
  int x_warn_virtual_inheritance;
#define warn_virtual_inheritance global_options.x_warn_virtual_inheritance
#endif
#ifdef GENERATOR_FILE
extern int warn_virtual_move_assign;
#else
  int x_warn_virtual_move_assign;
#define warn_virtual_move_assign global_options.x_warn_virtual_move_assign
#endif
#ifdef GENERATOR_FILE
extern int warn_vla;
#else
  int x_warn_vla;
#define warn_vla global_options.x_warn_vla
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT warn_vla_limit;
#else
  HOST_WIDE_INT x_warn_vla_limit;
#define warn_vla_limit global_options.x_warn_vla_limit
#endif
#ifdef GENERATOR_FILE
extern int warn_vla_parameter;
#else
  int x_warn_vla_parameter;
#define warn_vla_parameter global_options.x_warn_vla_parameter
#endif
#ifdef GENERATOR_FILE
extern int warn_volatile;
#else
  int x_warn_volatile;
#define warn_volatile global_options.x_warn_volatile
#endif
#ifdef GENERATOR_FILE
extern int warn_volatile_register_var;
#else
  int x_warn_volatile_register_var;
#define warn_volatile_register_var global_options.x_warn_volatile_register_var
#endif
#ifdef GENERATOR_FILE
extern int warn_write_strings;
#else
  int x_warn_write_strings;
#define warn_write_strings global_options.x_warn_write_strings
#endif
#ifdef GENERATOR_FILE
extern int warn_xor_used_as_pow;
#else
  int x_warn_xor_used_as_pow;
#define warn_xor_used_as_pow global_options.x_warn_xor_used_as_pow
#endif
#ifdef GENERATOR_FILE
extern int warn_zero_as_null_pointer_constant;
#else
  int x_warn_zero_as_null_pointer_constant;
#define warn_zero_as_null_pointer_constant global_options.x_warn_zero_as_null_pointer_constant
#endif
#ifdef GENERATOR_FILE
extern int warn_zero_length_bounds;
#else
  int x_warn_zero_length_bounds;
#define warn_zero_length_bounds global_options.x_warn_zero_length_bounds
#endif
#ifdef GENERATOR_FILE
extern int warn_zerotrip;
#else
  int x_warn_zerotrip;
#define warn_zerotrip global_options.x_warn_zerotrip
#endif
#ifdef GENERATOR_FILE
extern const char *aux_info_file_name;
#else
  const char *x_aux_info_file_name;
#define aux_info_file_name global_options.x_aux_info_file_name
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_callgraph;
#else
  int x_flag_dump_callgraph;
#define flag_dump_callgraph global_options.x_flag_dump_callgraph
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_dump_defined;
#else
  int x_flag_lto_dump_defined;
#define flag_lto_dump_defined global_options.x_flag_lto_dump_defined
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_dump_demangle;
#else
  int x_flag_lto_dump_demangle;
#define flag_lto_dump_demangle global_options.x_flag_lto_dump_demangle
#endif
#ifdef GENERATOR_FILE
extern const char *flag_dump_body;
#else
  const char *x_flag_dump_body;
#define flag_dump_body global_options.x_flag_dump_body
#endif
#ifdef GENERATOR_FILE
extern const char *flag_dump_level;
#else
  const char *x_flag_dump_level;
#define flag_dump_level global_options.x_flag_dump_level
#endif
#ifdef GENERATOR_FILE
extern const char *dump_base_name;
#else
  const char *x_dump_base_name;
#define dump_base_name global_options.x_dump_base_name
#endif
#ifdef GENERATOR_FILE
extern const char *dump_base_ext;
#else
  const char *x_dump_base_ext;
#define dump_base_ext global_options.x_dump_base_ext
#endif
#ifdef GENERATOR_FILE
extern const char *dump_dir_name;
#else
  const char *x_dump_dir_name;
#define dump_dir_name global_options.x_dump_dir_name
#endif
#ifdef GENERATOR_FILE
extern int flag_pic;
#else
  int x_flag_pic;
#define flag_pic global_options.x_flag_pic
#endif
#ifdef GENERATOR_FILE
extern int flag_pie;
#else
  int x_flag_pie;
#define flag_pie global_options.x_flag_pie
#endif
#ifdef GENERATOR_FILE
extern int flag_abi_compat_version;
#else
  int x_flag_abi_compat_version;
#define flag_abi_compat_version global_options.x_flag_abi_compat_version
#endif
#ifdef GENERATOR_FILE
extern int flag_abi_version;
#else
  int x_flag_abi_version;
#define flag_abi_version global_options.x_flag_abi_version
#endif
#ifdef GENERATOR_FILE
extern int flag_access_control;
#else
  int x_flag_access_control;
#define flag_access_control global_options.x_flag_access_control
#endif
#ifdef GENERATOR_FILE
extern const char *ada_specs_parent;
#else
  const char *x_ada_specs_parent;
#define ada_specs_parent global_options.x_ada_specs_parent
#endif
#ifdef GENERATOR_FILE
extern int flag_aggressive_function_elimination;
#else
  int x_flag_aggressive_function_elimination;
#define flag_aggressive_function_elimination global_options.x_flag_aggressive_function_elimination
#endif
#ifdef GENERATOR_FILE
extern int flag_aggressive_loop_optimizations;
#else
  int x_flag_aggressive_loop_optimizations;
#define flag_aggressive_loop_optimizations global_options.x_flag_aggressive_loop_optimizations
#endif
#ifdef GENERATOR_FILE
extern int flag_align_commons;
#else
  int x_flag_align_commons;
#define flag_align_commons global_options.x_flag_align_commons
#endif
#ifdef GENERATOR_FILE
extern int flag_align_functions;
#else
  int x_flag_align_functions;
#define flag_align_functions global_options.x_flag_align_functions
#endif
#ifdef GENERATOR_FILE
extern const char *str_align_functions;
#else
  const char *x_str_align_functions;
#define str_align_functions global_options.x_str_align_functions
#endif
#ifdef GENERATOR_FILE
extern int flag_align_jumps;
#else
  int x_flag_align_jumps;
#define flag_align_jumps global_options.x_flag_align_jumps
#endif
#ifdef GENERATOR_FILE
extern const char *str_align_jumps;
#else
  const char *x_str_align_jumps;
#define str_align_jumps global_options.x_str_align_jumps
#endif
#ifdef GENERATOR_FILE
extern int flag_align_labels;
#else
  int x_flag_align_labels;
#define flag_align_labels global_options.x_flag_align_labels
#endif
#ifdef GENERATOR_FILE
extern const char *str_align_labels;
#else
  const char *x_str_align_labels;
#define str_align_labels global_options.x_str_align_labels
#endif
#ifdef GENERATOR_FILE
extern int flag_align_loops;
#else
  int x_flag_align_loops;
#define flag_align_loops global_options.x_flag_align_loops
#endif
#ifdef GENERATOR_FILE
extern const char *str_align_loops;
#else
  const char *x_str_align_loops;
#define str_align_loops global_options.x_str_align_loops
#endif
#ifdef GENERATOR_FILE
extern int aligned_new_threshold;
#else
  int x_aligned_new_threshold;
#define aligned_new_threshold global_options.x_aligned_new_threshold
#endif
#ifdef GENERATOR_FILE
extern int flag_all_intrinsics;
#else
  int x_flag_all_intrinsics;
#define flag_all_intrinsics global_options.x_flag_all_intrinsics
#endif
#ifdef GENERATOR_FILE
extern int flag_allocation_dce;
#else
  int x_flag_allocation_dce;
#define flag_allocation_dce global_options.x_flag_allocation_dce
#endif
#ifdef GENERATOR_FILE
extern int flag_allow_argument_mismatch;
#else
  int x_flag_allow_argument_mismatch;
#define flag_allow_argument_mismatch global_options.x_flag_allow_argument_mismatch
#endif
#ifdef GENERATOR_FILE
extern int flag_allow_invalid_boz;
#else
  int x_flag_allow_invalid_boz;
#define flag_allow_invalid_boz global_options.x_flag_allow_invalid_boz
#endif
#ifdef GENERATOR_FILE
extern int flag_allow_leading_underscore;
#else
  int x_flag_allow_leading_underscore;
#define flag_allow_leading_underscore global_options.x_flag_allow_leading_underscore
#endif
#ifdef GENERATOR_FILE
extern int flag_store_data_races;
#else
  int x_flag_store_data_races;
#define flag_store_data_races global_options.x_flag_store_data_races
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer;
#else
  int x_flag_analyzer;
#define flag_analyzer global_options.x_flag_analyzer
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_call_summaries;
#else
  int x_flag_analyzer_call_summaries;
#define flag_analyzer_call_summaries global_options.x_flag_analyzer_call_summaries
#endif
#ifdef GENERATOR_FILE
extern const char *flag_analyzer_checker;
#else
  const char *x_flag_analyzer_checker;
#define flag_analyzer_checker global_options.x_flag_analyzer_checker
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_debug_text_art;
#else
  int x_flag_analyzer_debug_text_art;
#define flag_analyzer_debug_text_art global_options.x_flag_analyzer_debug_text_art
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_feasibility;
#else
  int x_flag_analyzer_feasibility;
#define flag_analyzer_feasibility global_options.x_flag_analyzer_feasibility
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_fine_grained;
#else
  int x_flag_analyzer_fine_grained;
#define flag_analyzer_fine_grained global_options.x_flag_analyzer_fine_grained
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_show_duplicate_count;
#else
  int x_flag_analyzer_show_duplicate_count;
#define flag_analyzer_show_duplicate_count global_options.x_flag_analyzer_show_duplicate_count
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_show_events_in_system_headers;
#else
  int x_flag_analyzer_show_events_in_system_headers;
#define flag_analyzer_show_events_in_system_headers global_options.x_flag_analyzer_show_events_in_system_headers
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_state_merge;
#else
  int x_flag_analyzer_state_merge;
#define flag_analyzer_state_merge global_options.x_flag_analyzer_state_merge
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_state_purge;
#else
  int x_flag_analyzer_state_purge;
#define flag_analyzer_state_purge global_options.x_flag_analyzer_state_purge
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_suppress_followups;
#else
  int x_flag_analyzer_suppress_followups;
#define flag_analyzer_suppress_followups global_options.x_flag_analyzer_suppress_followups
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_transitivity;
#else
  int x_flag_analyzer_transitivity;
#define flag_analyzer_transitivity global_options.x_flag_analyzer_transitivity
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_undo_inlining;
#else
  int x_flag_analyzer_undo_inlining;
#define flag_analyzer_undo_inlining global_options.x_flag_analyzer_undo_inlining
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_verbose_edges;
#else
  int x_flag_analyzer_verbose_edges;
#define flag_analyzer_verbose_edges global_options.x_flag_analyzer_verbose_edges
#endif
#ifdef GENERATOR_FILE
extern int flag_analyzer_verbose_state_changes;
#else
  int x_flag_analyzer_verbose_state_changes;
#define flag_analyzer_verbose_state_changes global_options.x_flag_analyzer_verbose_state_changes
#endif
#ifdef GENERATOR_FILE
extern int analyzer_verbosity;
#else
  int x_analyzer_verbosity;
#define analyzer_verbosity global_options.x_analyzer_verbosity
#endif
#ifdef GENERATOR_FILE
extern void *common_deferred_options;
#else
  void *x_common_deferred_options;
#define common_deferred_options global_options.x_common_deferred_options
#endif
#ifdef GENERATOR_FILE
extern int flag_no_asm;
#else
  int x_flag_no_asm;
#define flag_no_asm global_options.x_flag_no_asm
#endif
#ifdef GENERATOR_FILE
extern int flag_assert;
#else
  int x_flag_assert;
#define flag_assert global_options.x_flag_assert
#endif
#ifdef GENERATOR_FILE
extern int flag_associative_math;
#else
  int x_flag_associative_math;
#define flag_associative_math global_options.x_flag_associative_math
#endif
#ifdef GENERATOR_FILE
extern int flag_asynchronous_unwind_tables;
#else
  int x_flag_asynchronous_unwind_tables;
#define flag_asynchronous_unwind_tables global_options.x_flag_asynchronous_unwind_tables
#endif
#ifdef GENERATOR_FILE
extern int flag_auto_inc_dec;
#else
  int x_flag_auto_inc_dec;
#define flag_auto_inc_dec global_options.x_flag_auto_inc_dec
#endif
#ifdef GENERATOR_FILE
extern int flag_auto_profile;
#else
  int x_flag_auto_profile;
#define flag_auto_profile global_options.x_flag_auto_profile
#endif
#ifdef GENERATOR_FILE
extern const char *auto_profile_file;
#else
  const char *x_auto_profile_file;
#define auto_profile_file global_options.x_auto_profile_file
#endif
#ifdef GENERATOR_FILE
extern int flag_automatic;
#else
  int x_flag_automatic;
#define flag_automatic global_options.x_flag_automatic
#endif
#ifdef GENERATOR_FILE
extern int flag_backslash;
#else
  int x_flag_backslash;
#define flag_backslash global_options.x_flag_backslash
#endif
#ifdef GENERATOR_FILE
extern int flag_backtrace;
#else
  int x_flag_backtrace;
#define flag_backtrace global_options.x_flag_backtrace
#endif
#ifdef GENERATOR_FILE
extern int flag_bit_tests;
#else
  int x_flag_bit_tests;
#define flag_bit_tests global_options.x_flag_bit_tests
#endif
#ifdef GENERATOR_FILE
extern int flag_blas_matmul_limit;
#else
  int x_flag_blas_matmul_limit;
#define flag_blas_matmul_limit global_options.x_flag_blas_matmul_limit
#endif
#ifdef GENERATOR_FILE
extern int flag_bounds_check;
#else
  int x_flag_bounds_check;
#define flag_bounds_check global_options.x_flag_bounds_check
#endif
#ifdef GENERATOR_FILE
extern int flag_branch_on_count_reg;
#else
  int x_flag_branch_on_count_reg;
#define flag_branch_on_count_reg global_options.x_flag_branch_on_count_reg
#endif
#ifdef GENERATOR_FILE
extern int flag_branch_probabilities;
#else
  int x_flag_branch_probabilities;
#define flag_branch_probabilities global_options.x_flag_branch_probabilities
#endif
#ifdef GENERATOR_FILE
extern int flag_building_libgcc;
#else
  int x_flag_building_libgcc;
#define flag_building_libgcc global_options.x_flag_building_libgcc
#endif
#ifdef GENERATOR_FILE
extern int flag_building_libgfortran;
#else
  int x_flag_building_libgfortran;
#define flag_building_libgfortran global_options.x_flag_building_libgfortran
#endif
#ifdef GENERATOR_FILE
extern int flag_building_libphobos_tests;
#else
  int x_flag_building_libphobos_tests;
#define flag_building_libphobos_tests global_options.x_flag_building_libphobos_tests
#endif
#ifdef GENERATOR_FILE
extern int flag_no_builtin;
#else
  int x_flag_no_builtin;
#define flag_no_builtin global_options.x_flag_no_builtin
#endif
#ifdef GENERATOR_FILE
extern int flag_c_prototypes;
#else
  int x_flag_c_prototypes;
#define flag_c_prototypes global_options.x_flag_c_prototypes
#endif
#ifdef GENERATOR_FILE
extern int flag_c_prototypes_external;
#else
  int x_flag_c_prototypes_external;
#define flag_c_prototypes_external global_options.x_flag_c_prototypes_external
#endif
#ifdef GENERATOR_FILE
extern int flag_caller_saves;
#else
  int x_flag_caller_saves;
#define flag_caller_saves global_options.x_flag_caller_saves
#endif
#ifdef GENERATOR_FILE
extern int flag_callgraph_info;
#else
  int x_flag_callgraph_info;
#define flag_callgraph_info global_options.x_flag_callgraph_info
#endif
#ifdef GENERATOR_FILE
extern enum cf_protection_level flag_cf_protection;
#else
  enum cf_protection_level x_flag_cf_protection;
#define flag_cf_protection global_options.x_flag_cf_protection
#endif
#ifdef GENERATOR_FILE
extern int flag_char8_t;
#else
  int x_flag_char8_t;
#define flag_char8_t global_options.x_flag_char8_t
#endif
#ifdef GENERATOR_FILE
extern int flag_check_new;
#else
  int x_flag_check_new;
#define flag_check_new global_options.x_flag_check_new
#endif
#ifdef GENERATOR_FILE
extern int flag_check_action;
#else
  int x_flag_check_action;
#define flag_check_action global_options.x_flag_check_action
#endif
#ifdef GENERATOR_FILE
extern int flag_checking;
#else
  int x_flag_checking;
#define flag_checking global_options.x_flag_checking
#endif
#ifdef GENERATOR_FILE
extern enum gfc_fcoarray flag_coarray;
#else
  enum gfc_fcoarray x_flag_coarray;
#define flag_coarray global_options.x_flag_coarray
#endif
#ifdef GENERATOR_FILE
extern int flag_code_hoisting;
#else
  int x_flag_code_hoisting;
#define flag_code_hoisting global_options.x_flag_code_hoisting
#endif
#ifdef GENERATOR_FILE
extern int flag_combine_stack_adjustments;
#else
  int x_flag_combine_stack_adjustments;
#define flag_combine_stack_adjustments global_options.x_flag_combine_stack_adjustments
#endif
#ifdef GENERATOR_FILE
extern int flag_no_common;
#else
  int x_flag_no_common;
#define flag_no_common global_options.x_flag_no_common
#endif
#ifdef GENERATOR_FILE
extern int flag_compare_debug;
#else
  int x_flag_compare_debug;
#define flag_compare_debug global_options.x_flag_compare_debug
#endif
#ifdef GENERATOR_FILE
extern const char *flag_compare_debug_opt;
#else
  const char *x_flag_compare_debug_opt;
#define flag_compare_debug_opt global_options.x_flag_compare_debug_opt
#endif
#ifdef GENERATOR_FILE
extern int flag_compare_elim_after_reload;
#else
  int x_flag_compare_elim_after_reload;
#define flag_compare_elim_after_reload global_options.x_flag_compare_elim_after_reload
#endif
#ifdef GENERATOR_FILE
extern int flag_concepts;
#else
  int x_flag_concepts;
#define flag_concepts global_options.x_flag_concepts
#endif
#ifdef GENERATOR_FILE
extern int concepts_diagnostics_max_depth;
#else
  int x_concepts_diagnostics_max_depth;
#define concepts_diagnostics_max_depth global_options.x_concepts_diagnostics_max_depth
#endif
#ifdef GENERATOR_FILE
extern int flag_concepts_ts;
#else
  int x_flag_concepts_ts;
#define flag_concepts_ts global_options.x_flag_concepts_ts
#endif
#ifdef GENERATOR_FILE
extern int condition_coverage_flag;
#else
  int x_condition_coverage_flag;
#define condition_coverage_flag global_options.x_condition_coverage_flag
#endif
#ifdef GENERATOR_FILE
extern int flag_conserve_stack;
#else
  int x_flag_conserve_stack;
#define flag_conserve_stack global_options.x_flag_conserve_stack
#endif
#ifdef GENERATOR_FILE
extern int constexpr_cache_depth;
#else
  int x_constexpr_cache_depth;
#define constexpr_cache_depth global_options.x_constexpr_cache_depth
#endif
#ifdef GENERATOR_FILE
extern int max_constexpr_depth;
#else
  int x_max_constexpr_depth;
#define max_constexpr_depth global_options.x_max_constexpr_depth
#endif
#ifdef GENERATOR_FILE
extern int flag_constexpr_fp_except;
#else
  int x_flag_constexpr_fp_except;
#define flag_constexpr_fp_except global_options.x_flag_constexpr_fp_except
#endif
#ifdef GENERATOR_FILE
extern int constexpr_loop_limit;
#else
  int x_constexpr_loop_limit;
#define constexpr_loop_limit global_options.x_constexpr_loop_limit
#endif
#ifdef GENERATOR_FILE
extern HOST_WIDE_INT constexpr_ops_limit;
#else
  HOST_WIDE_INT x_constexpr_ops_limit;
#define constexpr_ops_limit global_options.x_constexpr_ops_limit
#endif
#ifdef GENERATOR_FILE
extern int flag_contract_mode;
#else
  int x_flag_contract_mode;
#define flag_contract_mode global_options.x_flag_contract_mode
#endif
#ifdef GENERATOR_FILE
extern int flag_contract_strict_declarations;
#else
  int x_flag_contract_strict_declarations;
#define flag_contract_strict_declarations global_options.x_flag_contract_strict_declarations
#endif
#ifdef GENERATOR_FILE
extern int flag_contracts;
#else
  int x_flag_contracts;
#define flag_contracts global_options.x_flag_contracts
#endif
#ifdef GENERATOR_FILE
extern enum gfc_convert flag_convert;
#else
  enum gfc_convert x_flag_convert;
#define flag_convert global_options.x_flag_convert
#endif
#ifdef GENERATOR_FILE
extern int flag_coroutines;
#else
  int x_flag_coroutines;
#define flag_coroutines global_options.x_flag_coroutines
#endif
#ifdef GENERATOR_FILE
extern int flag_cprop_registers;
#else
  int x_flag_cprop_registers;
#define flag_cprop_registers global_options.x_flag_cprop_registers
#endif
#ifdef GENERATOR_FILE
extern int flag_cray_pointer;
#else
  int x_flag_cray_pointer;
#define flag_cray_pointer global_options.x_flag_cray_pointer
#endif
#ifdef GENERATOR_FILE
extern int flag_crossjumping;
#else
  int x_flag_crossjumping;
#define flag_crossjumping global_options.x_flag_crossjumping
#endif
#ifdef GENERATOR_FILE
extern int flag_cse_follow_jumps;
#else
  int x_flag_cse_follow_jumps;
#define flag_cse_follow_jumps global_options.x_flag_cse_follow_jumps
#endif
#ifdef GENERATOR_FILE
extern int flag_cx_fortran_rules;
#else
  int x_flag_cx_fortran_rules;
#define flag_cx_fortran_rules global_options.x_flag_cx_fortran_rules
#endif
#ifdef GENERATOR_FILE
extern int flag_cx_limited_range;
#else
  int x_flag_cx_limited_range;
#define flag_cx_limited_range global_options.x_flag_cx_limited_range
#endif
#ifdef GENERATOR_FILE
extern int flag_data_sections;
#else
  int x_flag_data_sections;
#define flag_data_sections global_options.x_flag_data_sections
#endif
#ifdef GENERATOR_FILE
extern int flag_dbg_cnt_list;
#else
  int x_flag_dbg_cnt_list;
#define flag_dbg_cnt_list global_options.x_flag_dbg_cnt_list
#endif
#ifdef GENERATOR_FILE
extern int flag_dce;
#else
  int x_flag_dce;
#define flag_dce global_options.x_flag_dce
#endif
#ifdef GENERATOR_FILE
extern int flag_debug_aux_vars;
#else
  int x_flag_debug_aux_vars;
#define flag_debug_aux_vars global_options.x_flag_debug_aux_vars
#endif
#ifdef GENERATOR_FILE
extern int flag_debug_types_section;
#else
  int x_flag_debug_types_section;
#define flag_debug_types_section global_options.x_flag_debug_types_section
#endif
#ifdef GENERATOR_FILE
extern int flag_dec;
#else
  int x_flag_dec;
#define flag_dec global_options.x_flag_dec
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_blank_format_item;
#else
  int x_flag_dec_blank_format_item;
#define flag_dec_blank_format_item global_options.x_flag_dec_blank_format_item
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_char_conversions;
#else
  int x_flag_dec_char_conversions;
#define flag_dec_char_conversions global_options.x_flag_dec_char_conversions
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_format_defaults;
#else
  int x_flag_dec_format_defaults;
#define flag_dec_format_defaults global_options.x_flag_dec_format_defaults
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_include;
#else
  int x_flag_dec_include;
#define flag_dec_include global_options.x_flag_dec_include
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_intrinsic_ints;
#else
  int x_flag_dec_intrinsic_ints;
#define flag_dec_intrinsic_ints global_options.x_flag_dec_intrinsic_ints
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_math;
#else
  int x_flag_dec_math;
#define flag_dec_math global_options.x_flag_dec_math
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_static;
#else
  int x_flag_dec_static;
#define flag_dec_static global_options.x_flag_dec_static
#endif
#ifdef GENERATOR_FILE
extern int flag_dec_structure;
#else
  int x_flag_dec_structure;
#define flag_dec_structure global_options.x_flag_dec_structure
#endif
#ifdef GENERATOR_FILE
extern int flag_declone_ctor_dtor;
#else
  int x_flag_declone_ctor_dtor;
#define flag_declone_ctor_dtor global_options.x_flag_declone_ctor_dtor
#endif
#ifdef GENERATOR_FILE
extern int flag_default_double;
#else
  int x_flag_default_double;
#define flag_default_double global_options.x_flag_default_double
#endif
#ifdef GENERATOR_FILE
extern int flag_default_integer;
#else
  int x_flag_default_integer;
#define flag_default_integer global_options.x_flag_default_integer
#endif
#ifdef GENERATOR_FILE
extern int flag_default_real_10;
#else
  int x_flag_default_real_10;
#define flag_default_real_10 global_options.x_flag_default_real_10
#endif
#ifdef GENERATOR_FILE
extern int flag_default_real_16;
#else
  int x_flag_default_real_16;
#define flag_default_real_16 global_options.x_flag_default_real_16
#endif
#ifdef GENERATOR_FILE
extern int flag_default_real_8;
#else
  int x_flag_default_real_8;
#define flag_default_real_8 global_options.x_flag_default_real_8
#endif
#ifdef GENERATOR_FILE
extern int flag_defer_pop;
#else
  int x_flag_defer_pop;
#define flag_defer_pop global_options.x_flag_defer_pop
#endif
#ifdef GENERATOR_FILE
extern int flag_delayed_branch;
#else
  int x_flag_delayed_branch;
#define flag_delayed_branch global_options.x_flag_delayed_branch
#endif
#ifdef GENERATOR_FILE
extern int flag_delete_dead_exceptions;
#else
  int x_flag_delete_dead_exceptions;
#define flag_delete_dead_exceptions global_options.x_flag_delete_dead_exceptions
#endif
#ifdef GENERATOR_FILE
extern int flag_delete_null_pointer_checks;
#else
  int x_flag_delete_null_pointer_checks;
#define flag_delete_null_pointer_checks global_options.x_flag_delete_null_pointer_checks
#endif
#ifdef GENERATOR_FILE
extern int flag_devirtualize;
#else
  int x_flag_devirtualize;
#define flag_devirtualize global_options.x_flag_devirtualize
#endif
#ifdef GENERATOR_FILE
extern int flag_ltrans_devirtualize;
#else
  int x_flag_ltrans_devirtualize;
#define flag_ltrans_devirtualize global_options.x_flag_ltrans_devirtualize
#endif
#ifdef GENERATOR_FILE
extern int flag_devirtualize_speculatively;
#else
  int x_flag_devirtualize_speculatively;
#define flag_devirtualize_speculatively global_options.x_flag_devirtualize_speculatively
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_all_candidates;
#else
  int x_flag_diagnostics_all_candidates;
#define flag_diagnostics_all_candidates global_options.x_flag_diagnostics_all_candidates
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_color;
#else
  int x_flag_diagnostics_show_color;
#define flag_diagnostics_show_color global_options.x_flag_diagnostics_show_color
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_generate_patch;
#else
  int x_flag_diagnostics_generate_patch;
#define flag_diagnostics_generate_patch global_options.x_flag_diagnostics_generate_patch
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_json_formatting;
#else
  int x_flag_diagnostics_json_formatting;
#define flag_diagnostics_json_formatting global_options.x_flag_diagnostics_json_formatting
#endif
#ifdef GENERATOR_FILE
extern int diagnostics_minimum_margin_width;
#else
  int x_diagnostics_minimum_margin_width;
#define diagnostics_minimum_margin_width global_options.x_diagnostics_minimum_margin_width
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_parseable_fixits;
#else
  int x_flag_diagnostics_parseable_fixits;
#define flag_diagnostics_parseable_fixits global_options.x_flag_diagnostics_parseable_fixits
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_path_format;
#else
  int x_flag_diagnostics_path_format;
#define flag_diagnostics_path_format global_options.x_flag_diagnostics_path_format
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_caret;
#else
  int x_flag_diagnostics_show_caret;
#define flag_diagnostics_show_caret global_options.x_flag_diagnostics_show_caret
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_cwe;
#else
  int x_flag_diagnostics_show_cwe;
#define flag_diagnostics_show_cwe global_options.x_flag_diagnostics_show_cwe
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_labels;
#else
  int x_flag_diagnostics_show_labels;
#define flag_diagnostics_show_labels global_options.x_flag_diagnostics_show_labels
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_line_numbers;
#else
  int x_flag_diagnostics_show_line_numbers;
#define flag_diagnostics_show_line_numbers global_options.x_flag_diagnostics_show_line_numbers
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_option;
#else
  int x_flag_diagnostics_show_option;
#define flag_diagnostics_show_option global_options.x_flag_diagnostics_show_option
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_path_depths;
#else
  int x_flag_diagnostics_show_path_depths;
#define flag_diagnostics_show_path_depths global_options.x_flag_diagnostics_show_path_depths
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_rules;
#else
  int x_flag_diagnostics_show_rules;
#define flag_diagnostics_show_rules global_options.x_flag_diagnostics_show_rules
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_template_tree;
#else
  int x_flag_diagnostics_show_template_tree;
#define flag_diagnostics_show_template_tree global_options.x_flag_diagnostics_show_template_tree
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_text_art_charset;
#else
  int x_flag_diagnostics_text_art_charset;
#define flag_diagnostics_text_art_charset global_options.x_flag_diagnostics_text_art_charset
#endif
#ifdef GENERATOR_FILE
extern int flag_diagnostics_show_urls;
#else
  int x_flag_diagnostics_show_urls;
#define flag_diagnostics_show_urls global_options.x_flag_diagnostics_show_urls
#endif
#ifdef GENERATOR_FILE
extern int flag_dollar_ok;
#else
  int x_flag_dollar_ok;
#define flag_dollar_ok global_options.x_flag_dollar_ok
#endif
#ifdef GENERATOR_FILE
extern int flag_dse;
#else
  int x_flag_dse;
#define flag_dse global_options.x_flag_dse
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_ada_spec;
#else
  int x_flag_dump_ada_spec;
#define flag_dump_ada_spec global_options.x_flag_dump_ada_spec
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_ada_spec_slim;
#else
  int x_flag_dump_ada_spec_slim;
#define flag_dump_ada_spec_slim global_options.x_flag_dump_ada_spec_slim
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer;
#else
  int x_flag_dump_analyzer;
#define flag_dump_analyzer global_options.x_flag_dump_analyzer
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_callgraph;
#else
  int x_flag_dump_analyzer_callgraph;
#define flag_dump_analyzer_callgraph global_options.x_flag_dump_analyzer_callgraph
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_exploded_graph;
#else
  int x_flag_dump_analyzer_exploded_graph;
#define flag_dump_analyzer_exploded_graph global_options.x_flag_dump_analyzer_exploded_graph
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_exploded_nodes;
#else
  int x_flag_dump_analyzer_exploded_nodes;
#define flag_dump_analyzer_exploded_nodes global_options.x_flag_dump_analyzer_exploded_nodes
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_exploded_nodes_2;
#else
  int x_flag_dump_analyzer_exploded_nodes_2;
#define flag_dump_analyzer_exploded_nodes_2 global_options.x_flag_dump_analyzer_exploded_nodes_2
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_exploded_nodes_3;
#else
  int x_flag_dump_analyzer_exploded_nodes_3;
#define flag_dump_analyzer_exploded_nodes_3 global_options.x_flag_dump_analyzer_exploded_nodes_3
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_exploded_paths;
#else
  int x_flag_dump_analyzer_exploded_paths;
#define flag_dump_analyzer_exploded_paths global_options.x_flag_dump_analyzer_exploded_paths
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_feasibility;
#else
  int x_flag_dump_analyzer_feasibility;
#define flag_dump_analyzer_feasibility global_options.x_flag_dump_analyzer_feasibility
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_infinite_loop;
#else
  int x_flag_dump_analyzer_infinite_loop;
#define flag_dump_analyzer_infinite_loop global_options.x_flag_dump_analyzer_infinite_loop
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_json;
#else
  int x_flag_dump_analyzer_json;
#define flag_dump_analyzer_json global_options.x_flag_dump_analyzer_json
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_state_purge;
#else
  int x_flag_dump_analyzer_state_purge;
#define flag_dump_analyzer_state_purge global_options.x_flag_dump_analyzer_state_purge
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_stderr;
#else
  int x_flag_dump_analyzer_stderr;
#define flag_dump_analyzer_stderr global_options.x_flag_dump_analyzer_stderr
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_supergraph;
#else
  int x_flag_dump_analyzer_supergraph;
#define flag_dump_analyzer_supergraph global_options.x_flag_dump_analyzer_supergraph
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_analyzer_untracked;
#else
  int x_flag_dump_analyzer_untracked;
#define flag_dump_analyzer_untracked global_options.x_flag_dump_analyzer_untracked
#endif
#ifdef GENERATOR_FILE
extern const char *flag_dump_final_insns;
#else
  const char *x_flag_dump_final_insns;
#define flag_dump_final_insns global_options.x_flag_dump_final_insns
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_fortran_global;
#else
  int x_flag_dump_fortran_global;
#define flag_dump_fortran_global global_options.x_flag_dump_fortran_global
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_fortran_optimized;
#else
  int x_flag_dump_fortran_optimized;
#define flag_dump_fortran_optimized global_options.x_flag_dump_fortran_optimized
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_fortran_original;
#else
  int x_flag_dump_fortran_original;
#define flag_dump_fortran_original global_options.x_flag_dump_fortran_original
#endif
#ifdef GENERATOR_FILE
extern const char *flag_dump_go_spec;
#else
  const char *x_flag_dump_go_spec;
#define flag_dump_go_spec global_options.x_flag_dump_go_spec
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_locations;
#else
  int x_flag_dump_locations;
#define flag_dump_locations global_options.x_flag_dump_locations
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_noaddr;
#else
  int x_flag_dump_noaddr;
#define flag_dump_noaddr global_options.x_flag_dump_noaddr
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_passes;
#else
  int x_flag_dump_passes;
#define flag_dump_passes global_options.x_flag_dump_passes
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_scos;
#else
  int x_flag_dump_scos;
#define flag_dump_scos global_options.x_flag_dump_scos
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_unnumbered;
#else
  int x_flag_dump_unnumbered;
#define flag_dump_unnumbered global_options.x_flag_dump_unnumbered
#endif
#ifdef GENERATOR_FILE
extern int flag_dump_unnumbered_links;
#else
  int x_flag_dump_unnumbered_links;
#define flag_dump_unnumbered_links global_options.x_flag_dump_unnumbered_links
#endif
#ifdef GENERATOR_FILE
extern int flag_dwarf2_cfi_asm;
#else
  int x_flag_dwarf2_cfi_asm;
#define flag_dwarf2_cfi_asm global_options.x_flag_dwarf2_cfi_asm
#endif
#ifdef GENERATOR_FILE
extern int flag_early_inlining;
#else
  int x_flag_early_inlining;
#define flag_early_inlining global_options.x_flag_early_inlining
#endif
#ifdef GENERATOR_FILE
extern int flag_elide_constructors;
#else
  int x_flag_elide_constructors;
#define flag_elide_constructors global_options.x_flag_elide_constructors
#endif
#ifdef GENERATOR_FILE
extern int flag_elide_type;
#else
  int x_flag_elide_type;
#define flag_elide_type global_options.x_flag_elide_type
#endif
#ifdef GENERATOR_FILE
extern int flag_debug_only_used_symbols;
#else
  int x_flag_debug_only_used_symbols;
#define flag_debug_only_used_symbols global_options.x_flag_debug_only_used_symbols
#endif
#ifdef GENERATOR_FILE
extern int flag_eliminate_unused_debug_types;
#else
  int x_flag_eliminate_unused_debug_types;
#define flag_eliminate_unused_debug_types global_options.x_flag_eliminate_unused_debug_types
#endif
#ifdef GENERATOR_FILE
extern int flag_emit_class_debug_always;
#else
  int x_flag_emit_class_debug_always;
#define flag_emit_class_debug_always global_options.x_flag_emit_class_debug_always
#endif
#ifdef GENERATOR_FILE
extern int flag_enforce_eh_specs;
#else
  int x_flag_enforce_eh_specs;
#define flag_enforce_eh_specs global_options.x_flag_enforce_eh_specs
#endif
#ifdef GENERATOR_FILE
extern int flag_exceptions;
#else
  int x_flag_exceptions;
#define flag_exceptions global_options.x_flag_exceptions
#endif
#ifdef GENERATOR_FILE
extern enum excess_precision flag_excess_precision;
#else
  enum excess_precision x_flag_excess_precision;
#define flag_excess_precision global_options.x_flag_excess_precision
#endif
#ifdef GENERATOR_FILE
extern int flag_expensive_optimizations;
#else
  int x_flag_expensive_optimizations;
#define flag_expensive_optimizations global_options.x_flag_expensive_optimizations
#endif
#ifdef GENERATOR_FILE
extern int flag_ext_numeric_literals;
#else
  int x_flag_ext_numeric_literals;
#define flag_ext_numeric_literals global_options.x_flag_ext_numeric_literals
#endif
#ifdef GENERATOR_FILE
extern int flag_extern_stdcpp;
#else
  int x_flag_extern_stdcpp;
#define flag_extern_stdcpp global_options.x_flag_extern_stdcpp
#endif
#ifdef GENERATOR_FILE
extern int flag_extern_tls_init;
#else
  int x_flag_extern_tls_init;
#define flag_extern_tls_init global_options.x_flag_extern_tls_init
#endif
#ifdef GENERATOR_FILE
extern int flag_external_blas;
#else
  int x_flag_external_blas;
#define flag_external_blas global_options.x_flag_external_blas
#endif
#ifdef GENERATOR_FILE
extern int flag_f2c;
#else
  int x_flag_f2c;
#define flag_f2c global_options.x_flag_f2c
#endif
#ifdef GENERATOR_FILE
extern int flag_fat_lto_objects;
#else
  int x_flag_fat_lto_objects;
#define flag_fat_lto_objects global_options.x_flag_fat_lto_objects
#endif
#ifdef GENERATOR_FILE
extern int flag_finite_loops;
#else
  int x_flag_finite_loops;
#define flag_finite_loops global_options.x_flag_finite_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_finite_math_only;
#else
  int x_flag_finite_math_only;
#define flag_finite_math_only global_options.x_flag_finite_math_only
#endif
#ifdef GENERATOR_FILE
extern int flag_fixed_line_length;
#else
  int x_flag_fixed_line_length;
#define flag_fixed_line_length global_options.x_flag_fixed_line_length
#endif
#ifdef GENERATOR_FILE
extern int flag_float_store;
#else
  int x_flag_float_store;
#define flag_float_store global_options.x_flag_float_store
#endif
#ifdef GENERATOR_FILE
extern int flag_fold_mem_offsets;
#else
  int x_flag_fold_mem_offsets;
#define flag_fold_mem_offsets global_options.x_flag_fold_mem_offsets
#endif
#ifdef GENERATOR_FILE
extern int flag_fold_simple_inlines;
#else
  int x_flag_fold_simple_inlines;
#define flag_fold_simple_inlines global_options.x_flag_fold_simple_inlines
#endif
#ifdef GENERATOR_FILE
extern int flag_forward_propagate;
#else
  int x_flag_forward_propagate;
#define flag_forward_propagate global_options.x_flag_forward_propagate
#endif
#ifdef GENERATOR_FILE
extern enum fp_contract_mode flag_fp_contract_mode;
#else
  enum fp_contract_mode x_flag_fp_contract_mode;
#define flag_fp_contract_mode global_options.x_flag_fp_contract_mode
#endif
#ifdef GENERATOR_FILE
extern int flag_fp_int_builtin_inexact;
#else
  int x_flag_fp_int_builtin_inexact;
#define flag_fp_int_builtin_inexact global_options.x_flag_fp_int_builtin_inexact
#endif
#ifdef GENERATOR_FILE
extern int flag_free_line_length;
#else
  int x_flag_free_line_length;
#define flag_free_line_length global_options.x_flag_free_line_length
#endif
#ifdef GENERATOR_FILE
extern int flag_frontend_loop_interchange;
#else
  int x_flag_frontend_loop_interchange;
#define flag_frontend_loop_interchange global_options.x_flag_frontend_loop_interchange
#endif
#ifdef GENERATOR_FILE
extern int flag_frontend_optimize;
#else
  int x_flag_frontend_optimize;
#define flag_frontend_optimize global_options.x_flag_frontend_optimize
#endif
#ifdef GENERATOR_FILE
extern int flag_no_function_cse;
#else
  int x_flag_no_function_cse;
#define flag_no_function_cse global_options.x_flag_no_function_cse
#endif
#ifdef GENERATOR_FILE
extern int flag_function_sections;
#else
  int x_flag_function_sections;
#define flag_function_sections global_options.x_flag_function_sections
#endif
#ifdef GENERATOR_FILE
extern int flag_gcse;
#else
  int x_flag_gcse;
#define flag_gcse global_options.x_flag_gcse
#endif
#ifdef GENERATOR_FILE
extern int flag_gcse_after_reload;
#else
  int x_flag_gcse_after_reload;
#define flag_gcse_after_reload global_options.x_flag_gcse_after_reload
#endif
#ifdef GENERATOR_FILE
extern int flag_gcse_las;
#else
  int x_flag_gcse_las;
#define flag_gcse_las global_options.x_flag_gcse_las
#endif
#ifdef GENERATOR_FILE
extern int flag_gcse_lm;
#else
  int x_flag_gcse_lm;
#define flag_gcse_lm global_options.x_flag_gcse_lm
#endif
#ifdef GENERATOR_FILE
extern int flag_gcse_sm;
#else
  int x_flag_gcse_sm;
#define flag_gcse_sm global_options.x_flag_gcse_sm
#endif
#ifdef GENERATOR_FILE
extern int flag_gimple;
#else
  int x_flag_gimple;
#define flag_gimple global_options.x_flag_gimple
#endif
#ifdef GENERATOR_FILE
extern int flag_no_gnu_keywords;
#else
  int x_flag_no_gnu_keywords;
#define flag_no_gnu_keywords global_options.x_flag_no_gnu_keywords
#endif
#ifdef GENERATOR_FILE
extern int flag_next_runtime;
#else
  int x_flag_next_runtime;
#define flag_next_runtime global_options.x_flag_next_runtime
#endif
#ifdef GENERATOR_FILE
extern int flag_tm;
#else
  int x_flag_tm;
#define flag_tm global_options.x_flag_tm
#endif
#ifdef GENERATOR_FILE
extern int flag_gnu_unique;
#else
  int x_flag_gnu_unique;
#define flag_gnu_unique global_options.x_flag_gnu_unique
#endif
#ifdef GENERATOR_FILE
extern int flag_gnu89_inline;
#else
  int x_flag_gnu89_inline;
#define flag_gnu89_inline global_options.x_flag_gnu89_inline
#endif
#ifdef GENERATOR_FILE
extern int go_check_divide_overflow;
#else
  int x_go_check_divide_overflow;
#define go_check_divide_overflow global_options.x_go_check_divide_overflow
#endif
#ifdef GENERATOR_FILE
extern int go_check_divide_zero;
#else
  int x_go_check_divide_zero;
#define go_check_divide_zero global_options.x_go_check_divide_zero
#endif
#ifdef GENERATOR_FILE
extern int go_compiling_runtime;
#else
  int x_go_compiling_runtime;
#define go_compiling_runtime global_options.x_go_compiling_runtime
#endif
#ifdef GENERATOR_FILE
extern int go_debug_escape_level;
#else
  int x_go_debug_escape_level;
#define go_debug_escape_level global_options.x_go_debug_escape_level
#endif
#ifdef GENERATOR_FILE
extern const char *go_debug_escape_hash;
#else
  const char *x_go_debug_escape_hash;
#define go_debug_escape_hash global_options.x_go_debug_escape_hash
#endif
#ifdef GENERATOR_FILE
extern int go_debug_optimization;
#else
  int x_go_debug_optimization;
#define go_debug_optimization global_options.x_go_debug_optimization
#endif
#ifdef GENERATOR_FILE
extern int flag_graphite;
#else
  int x_flag_graphite;
#define flag_graphite global_options.x_flag_graphite
#endif
#ifdef GENERATOR_FILE
extern int flag_graphite_identity;
#else
  int x_flag_graphite_identity;
#define flag_graphite_identity global_options.x_flag_graphite_identity
#endif
#ifdef GENERATOR_FILE
extern int flag_guess_branch_prob;
#else
  int x_flag_guess_branch_prob;
#define flag_guess_branch_prob global_options.x_flag_guess_branch_prob
#endif
#ifdef GENERATOR_FILE
extern int flag_harden_control_flow_redundancy_check_exceptions;
#else
  int x_flag_harden_control_flow_redundancy_check_exceptions;
#define flag_harden_control_flow_redundancy_check_exceptions global_options.x_flag_harden_control_flow_redundancy_check_exceptions
#endif
#ifdef GENERATOR_FILE
extern enum hardcfr_noret flag_harden_control_flow_redundancy_check_noreturn;
#else
  enum hardcfr_noret x_flag_harden_control_flow_redundancy_check_noreturn;
#define flag_harden_control_flow_redundancy_check_noreturn global_options.x_flag_harden_control_flow_redundancy_check_noreturn
#endif
#ifdef GENERATOR_FILE
extern int flag_harden_control_flow_redundancy_check_returning_calls;
#else
  int x_flag_harden_control_flow_redundancy_check_returning_calls;
#define flag_harden_control_flow_redundancy_check_returning_calls global_options.x_flag_harden_control_flow_redundancy_check_returning_calls
#endif
#ifdef GENERATOR_FILE
extern int flag_harden_control_flow_redundancy_skip_leaf;
#else
  int x_flag_harden_control_flow_redundancy_skip_leaf;
#define flag_harden_control_flow_redundancy_skip_leaf global_options.x_flag_harden_control_flow_redundancy_skip_leaf
#endif
#ifdef GENERATOR_FILE
extern int flag_harden_compares;
#else
  int x_flag_harden_compares;
#define flag_harden_compares global_options.x_flag_harden_compares
#endif
#ifdef GENERATOR_FILE
extern int flag_harden_conditional_branches;
#else
  int x_flag_harden_conditional_branches;
#define flag_harden_conditional_branches global_options.x_flag_harden_conditional_branches
#endif
#ifdef GENERATOR_FILE
extern int flag_harden_control_flow_redundancy;
#else
  int x_flag_harden_control_flow_redundancy;
#define flag_harden_control_flow_redundancy global_options.x_flag_harden_control_flow_redundancy
#endif
#ifdef GENERATOR_FILE
extern int flag_hardened;
#else
  int x_flag_hardened;
#define flag_hardened global_options.x_flag_hardened
#endif
#ifdef GENERATOR_FILE
extern int flag_hoist_adjacent_loads;
#else
  int x_flag_hoist_adjacent_loads;
#define flag_hoist_adjacent_loads global_options.x_flag_hoist_adjacent_loads
#endif
#ifdef GENERATOR_FILE
extern int flag_no_ident;
#else
  int x_flag_no_ident;
#define flag_no_ident global_options.x_flag_no_ident
#endif
#ifdef GENERATOR_FILE
extern int flag_if_conversion;
#else
  int x_flag_if_conversion;
#define flag_if_conversion global_options.x_flag_if_conversion
#endif
#ifdef GENERATOR_FILE
extern int flag_if_conversion2;
#else
  int x_flag_if_conversion2;
#define flag_if_conversion2 global_options.x_flag_if_conversion2
#endif
#ifdef GENERATOR_FILE
extern int flag_immediate_escalation;
#else
  int x_flag_immediate_escalation;
#define flag_immediate_escalation global_options.x_flag_immediate_escalation
#endif
#ifdef GENERATOR_FILE
extern int flag_implement_inlines;
#else
  int x_flag_implement_inlines;
#define flag_implement_inlines global_options.x_flag_implement_inlines
#endif
#ifdef GENERATOR_FILE
extern int flag_implicit_constexpr;
#else
  int x_flag_implicit_constexpr;
#define flag_implicit_constexpr global_options.x_flag_implicit_constexpr
#endif
#ifdef GENERATOR_FILE
extern int flag_implicit_inline_templates;
#else
  int x_flag_implicit_inline_templates;
#define flag_implicit_inline_templates global_options.x_flag_implicit_inline_templates
#endif
#ifdef GENERATOR_FILE
extern int flag_implicit_none;
#else
  int x_flag_implicit_none;
#define flag_implicit_none global_options.x_flag_implicit_none
#endif
#ifdef GENERATOR_FILE
extern int flag_implicit_templates;
#else
  int x_flag_implicit_templates;
#define flag_implicit_templates global_options.x_flag_implicit_templates
#endif
#ifdef GENERATOR_FILE
extern int flag_indirect_inlining;
#else
  int x_flag_indirect_inlining;
#define flag_indirect_inlining global_options.x_flag_indirect_inlining
#endif
#ifdef GENERATOR_FILE
extern int flag_inhibit_size_directive;
#else
  int x_flag_inhibit_size_directive;
#define flag_inhibit_size_directive global_options.x_flag_inhibit_size_directive
#endif
#ifdef GENERATOR_FILE
extern int flag_init_derived;
#else
  int x_flag_init_derived;
#define flag_init_derived global_options.x_flag_init_derived
#endif
#ifdef GENERATOR_FILE
extern enum gfc_init_local_real flag_init_real;
#else
  enum gfc_init_local_real x_flag_init_real;
#define flag_init_real global_options.x_flag_init_real
#endif
#ifdef GENERATOR_FILE
extern int flag_no_inline;
#else
  int x_flag_no_inline;
#define flag_no_inline global_options.x_flag_no_inline
#endif
#ifdef GENERATOR_FILE
extern int flag_inline_arg_packing;
#else
  int x_flag_inline_arg_packing;
#define flag_inline_arg_packing global_options.x_flag_inline_arg_packing
#endif
#ifdef GENERATOR_FILE
extern int flag_inline_atomics;
#else
  int x_flag_inline_atomics;
#define flag_inline_atomics global_options.x_flag_inline_atomics
#endif
#ifdef GENERATOR_FILE
extern int flag_inline_functions;
#else
  int x_flag_inline_functions;
#define flag_inline_functions global_options.x_flag_inline_functions
#endif
#ifdef GENERATOR_FILE
extern int flag_inline_functions_called_once;
#else
  int x_flag_inline_functions_called_once;
#define flag_inline_functions_called_once global_options.x_flag_inline_functions_called_once
#endif
#ifdef GENERATOR_FILE
extern int flag_inline_matmul_limit;
#else
  int x_flag_inline_matmul_limit;
#define flag_inline_matmul_limit global_options.x_flag_inline_matmul_limit
#endif
#ifdef GENERATOR_FILE
extern int flag_inline_small_functions;
#else
  int x_flag_inline_small_functions;
#define flag_inline_small_functions global_options.x_flag_inline_small_functions
#endif
#ifdef GENERATOR_FILE
extern enum ilsop_fn flag_inline_stringops;
#else
  enum ilsop_fn x_flag_inline_stringops;
#define flag_inline_stringops global_options.x_flag_inline_stringops
#endif
#ifdef GENERATOR_FILE
extern int flag_instrument_function_entry_exit;
#else
  int x_flag_instrument_function_entry_exit;
#define flag_instrument_function_entry_exit global_options.x_flag_instrument_function_entry_exit
#endif
#ifdef GENERATOR_FILE
extern int flag_integer4_kind;
#else
  int x_flag_integer4_kind;
#define flag_integer4_kind global_options.x_flag_integer4_kind
#endif
#ifdef GENERATOR_FILE
extern int flag_invariants;
#else
  int x_flag_invariants;
#define flag_invariants global_options.x_flag_invariants
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_bit_cp;
#else
  int x_flag_ipa_bit_cp;
#define flag_ipa_bit_cp global_options.x_flag_ipa_bit_cp
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_cp;
#else
  int x_flag_ipa_cp;
#define flag_ipa_cp global_options.x_flag_ipa_cp
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_cp_clone;
#else
  int x_flag_ipa_cp_clone;
#define flag_ipa_cp_clone global_options.x_flag_ipa_cp_clone
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_icf;
#else
  int x_flag_ipa_icf;
#define flag_ipa_icf global_options.x_flag_ipa_icf
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_icf_functions;
#else
  int x_flag_ipa_icf_functions;
#define flag_ipa_icf_functions global_options.x_flag_ipa_icf_functions
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_icf_variables;
#else
  int x_flag_ipa_icf_variables;
#define flag_ipa_icf_variables global_options.x_flag_ipa_icf_variables
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_modref;
#else
  int x_flag_ipa_modref;
#define flag_ipa_modref global_options.x_flag_ipa_modref
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_profile;
#else
  int x_flag_ipa_profile;
#define flag_ipa_profile global_options.x_flag_ipa_profile
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_pta;
#else
  int x_flag_ipa_pta;
#define flag_ipa_pta global_options.x_flag_ipa_pta
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_pure_const;
#else
  int x_flag_ipa_pure_const;
#define flag_ipa_pure_const global_options.x_flag_ipa_pure_const
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_ra;
#else
  int x_flag_ipa_ra;
#define flag_ipa_ra global_options.x_flag_ipa_ra
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_reference;
#else
  int x_flag_ipa_reference;
#define flag_ipa_reference global_options.x_flag_ipa_reference
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_reference_addressable;
#else
  int x_flag_ipa_reference_addressable;
#define flag_ipa_reference_addressable global_options.x_flag_ipa_reference_addressable
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_sra;
#else
  int x_flag_ipa_sra;
#define flag_ipa_sra global_options.x_flag_ipa_sra
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_stack_alignment;
#else
  int x_flag_ipa_stack_alignment;
#define flag_ipa_stack_alignment global_options.x_flag_ipa_stack_alignment
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_strict_aliasing;
#else
  int x_flag_ipa_strict_aliasing;
#define flag_ipa_strict_aliasing global_options.x_flag_ipa_strict_aliasing
#endif
#ifdef GENERATOR_FILE
extern int flag_ipa_vrp;
#else
  int x_flag_ipa_vrp;
#define flag_ipa_vrp global_options.x_flag_ipa_vrp
#endif
#ifdef GENERATOR_FILE
extern enum ira_algorithm flag_ira_algorithm;
#else
  enum ira_algorithm x_flag_ira_algorithm;
#define flag_ira_algorithm global_options.x_flag_ira_algorithm
#endif
#ifdef GENERATOR_FILE
extern int flag_ira_hoist_pressure;
#else
  int x_flag_ira_hoist_pressure;
#define flag_ira_hoist_pressure global_options.x_flag_ira_hoist_pressure
#endif
#ifdef GENERATOR_FILE
extern int flag_ira_loop_pressure;
#else
  int x_flag_ira_loop_pressure;
#define flag_ira_loop_pressure global_options.x_flag_ira_loop_pressure
#endif
#ifdef GENERATOR_FILE
extern enum ira_region flag_ira_region;
#else
  enum ira_region x_flag_ira_region;
#define flag_ira_region global_options.x_flag_ira_region
#endif
#ifdef GENERATOR_FILE
extern int flag_ira_share_save_slots;
#else
  int x_flag_ira_share_save_slots;
#define flag_ira_share_save_slots global_options.x_flag_ira_share_save_slots
#endif
#ifdef GENERATOR_FILE
extern int flag_ira_share_spill_slots;
#else
  int x_flag_ira_share_spill_slots;
#define flag_ira_share_spill_slots global_options.x_flag_ira_share_spill_slots
#endif
#ifdef GENERATOR_FILE
extern int flag_ira_verbose;
#else
  int x_flag_ira_verbose;
#define flag_ira_verbose global_options.x_flag_ira_verbose
#endif
#ifdef GENERATOR_FILE
extern int flag_isolate_erroneous_paths_attribute;
#else
  int x_flag_isolate_erroneous_paths_attribute;
#define flag_isolate_erroneous_paths_attribute global_options.x_flag_isolate_erroneous_paths_attribute
#endif
#ifdef GENERATOR_FILE
extern int flag_isolate_erroneous_paths_dereference;
#else
  int x_flag_isolate_erroneous_paths_dereference;
#define flag_isolate_erroneous_paths_dereference global_options.x_flag_isolate_erroneous_paths_dereference
#endif
#ifdef GENERATOR_FILE
extern enum ivar_visibility default_ivar_visibility;
#else
  enum ivar_visibility x_default_ivar_visibility;
#define default_ivar_visibility global_options.x_default_ivar_visibility
#endif
#ifdef GENERATOR_FILE
extern int flag_ivopts;
#else
  int x_flag_ivopts;
#define flag_ivopts global_options.x_flag_ivopts
#endif
#ifdef GENERATOR_FILE
extern int flag_jump_tables;
#else
  int x_flag_jump_tables;
#define flag_jump_tables global_options.x_flag_jump_tables
#endif
#ifdef GENERATOR_FILE
extern int flag_keep_gc_roots_live;
#else
  int x_flag_keep_gc_roots_live;
#define flag_keep_gc_roots_live global_options.x_flag_keep_gc_roots_live
#endif
#ifdef GENERATOR_FILE
extern int flag_keep_inline_dllexport;
#else
  int x_flag_keep_inline_dllexport;
#define flag_keep_inline_dllexport global_options.x_flag_keep_inline_dllexport
#endif
#ifdef GENERATOR_FILE
extern int flag_keep_inline_functions;
#else
  int x_flag_keep_inline_functions;
#define flag_keep_inline_functions global_options.x_flag_keep_inline_functions
#endif
#ifdef GENERATOR_FILE
extern int flag_keep_static_consts;
#else
  int x_flag_keep_static_consts;
#define flag_keep_static_consts global_options.x_flag_keep_static_consts
#endif
#ifdef GENERATOR_FILE
extern int flag_keep_static_functions;
#else
  int x_flag_keep_static_functions;
#define flag_keep_static_functions global_options.x_flag_keep_static_functions
#endif
#ifdef GENERATOR_FILE
extern int note_include_translate_yes;
#else
  int x_note_include_translate_yes;
#define note_include_translate_yes global_options.x_note_include_translate_yes
#endif
#ifdef GENERATOR_FILE
extern int note_include_translate_no;
#else
  int x_note_include_translate_no;
#define note_include_translate_no global_options.x_note_include_translate_no
#endif
#ifdef GENERATOR_FILE
extern int note_module_cmi_yes;
#else
  int x_note_module_cmi_yes;
#define note_module_cmi_yes global_options.x_note_module_cmi_yes
#endif
#ifdef GENERATOR_FILE
extern int flag_large_source_files;
#else
  int x_flag_large_source_files;
#define flag_large_source_files global_options.x_flag_large_source_files
#endif
#ifdef GENERATOR_FILE
extern int flag_lax_vector_conversions;
#else
  int x_flag_lax_vector_conversions;
#define flag_lax_vector_conversions global_options.x_flag_lax_vector_conversions
#endif
#ifdef GENERATOR_FILE
extern int flag_leading_underscore;
#else
  int x_flag_leading_underscore;
#define flag_leading_underscore global_options.x_flag_leading_underscore
#endif
#ifdef GENERATOR_FILE
extern int flag_lifetime_dse;
#else
  int x_flag_lifetime_dse;
#define flag_lifetime_dse global_options.x_flag_lifetime_dse
#endif
#ifdef GENERATOR_FILE
extern int flag_limit_function_alignment;
#else
  int x_flag_limit_function_alignment;
#define flag_limit_function_alignment global_options.x_flag_limit_function_alignment
#endif
#ifdef GENERATOR_FILE
extern enum lto_linker_output flag_lto_linker_output;
#else
  enum lto_linker_output x_flag_lto_linker_output;
#define flag_lto_linker_output global_options.x_flag_lto_linker_output
#endif
#ifdef GENERATOR_FILE
extern enum live_patching_level flag_live_patching;
#else
  enum live_patching_level x_flag_live_patching;
#define flag_live_patching global_options.x_flag_live_patching
#endif
#ifdef GENERATOR_FILE
extern int flag_live_range_shrinkage;
#else
  int x_flag_live_range_shrinkage;
#define flag_live_range_shrinkage global_options.x_flag_live_range_shrinkage
#endif
#ifdef GENERATOR_FILE
extern int flag_local_ivars;
#else
  int x_flag_local_ivars;
#define flag_local_ivars global_options.x_flag_local_ivars
#endif
#ifdef GENERATOR_FILE
extern int flag_loop_interchange;
#else
  int x_flag_loop_interchange;
#define flag_loop_interchange global_options.x_flag_loop_interchange
#endif
#ifdef GENERATOR_FILE
extern int flag_loop_nest_optimize;
#else
  int x_flag_loop_nest_optimize;
#define flag_loop_nest_optimize global_options.x_flag_loop_nest_optimize
#endif
#ifdef GENERATOR_FILE
extern int flag_loop_parallelize_all;
#else
  int x_flag_loop_parallelize_all;
#define flag_loop_parallelize_all global_options.x_flag_loop_parallelize_all
#endif
#ifdef GENERATOR_FILE
extern int flag_unroll_jam;
#else
  int x_flag_unroll_jam;
#define flag_unroll_jam global_options.x_flag_unroll_jam
#endif
#ifdef GENERATOR_FILE
extern int flag_lra_remat;
#else
  int x_flag_lra_remat;
#define flag_lra_remat global_options.x_flag_lra_remat
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_compression_level;
#else
  int x_flag_lto_compression_level;
#define flag_lto_compression_level global_options.x_flag_lto_compression_level
#endif
#ifdef GENERATOR_FILE
extern enum lto_partition_model flag_lto_partition;
#else
  enum lto_partition_model x_flag_lto_partition;
#define flag_lto_partition global_options.x_flag_lto_partition
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_report;
#else
  int x_flag_lto_report;
#define flag_lto_report global_options.x_flag_lto_report
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_report_wpa;
#else
  int x_flag_lto_report_wpa;
#define flag_lto_report_wpa global_options.x_flag_lto_report_wpa
#endif
#ifdef GENERATOR_FILE
extern const char *flag_lto;
#else
  const char *x_flag_lto;
#define flag_lto global_options.x_flag_lto
#endif
#ifdef GENERATOR_FILE
extern int flag_ltrans;
#else
  int x_flag_ltrans;
#define flag_ltrans global_options.x_flag_ltrans
#endif
#ifdef GENERATOR_FILE
extern const char *ltrans_output_list;
#else
  const char *x_ltrans_output_list;
#define ltrans_output_list global_options.x_ltrans_output_list
#endif
#ifdef GENERATOR_FILE
extern int flag_errno_math;
#else
  int x_flag_errno_math;
#define flag_errno_math global_options.x_flag_errno_math
#endif
#ifdef GENERATOR_FILE
extern int flag_max_array_constructor;
#else
  int x_flag_max_array_constructor;
#define flag_max_array_constructor global_options.x_flag_max_array_constructor
#endif
#ifdef GENERATOR_FILE
extern int flag_max_errors;
#else
  int x_flag_max_errors;
#define flag_max_errors global_options.x_flag_max_errors
#endif
#ifdef GENERATOR_FILE
extern int flag_max_stack_var_size;
#else
  int x_flag_max_stack_var_size;
#define flag_max_stack_var_size global_options.x_flag_max_stack_var_size
#endif
#ifdef GENERATOR_FILE
extern int flag_max_subrecord_length;
#else
  int x_flag_max_subrecord_length;
#define flag_max_subrecord_length global_options.x_flag_max_subrecord_length
#endif
#ifdef GENERATOR_FILE
extern int mem_report;
#else
  int x_mem_report;
#define mem_report global_options.x_mem_report
#endif
#ifdef GENERATOR_FILE
extern int mem_report_wpa;
#else
  int x_mem_report_wpa;
#define mem_report_wpa global_options.x_mem_report_wpa
#endif
#ifdef GENERATOR_FILE
extern int flag_merge_constants;
#else
  int x_flag_merge_constants;
#define flag_merge_constants global_options.x_flag_merge_constants
#endif
#ifdef GENERATOR_FILE
extern int flag_merge_debug_strings;
#else
  int x_flag_merge_debug_strings;
#define flag_merge_debug_strings global_options.x_flag_merge_debug_strings
#endif
#ifdef GENERATOR_FILE
extern int flag_min_function_alignment;
#else
  int x_flag_min_function_alignment;
#define flag_min_function_alignment global_options.x_flag_min_function_alignment
#endif
#ifdef GENERATOR_FILE
extern int flag_header_unit;
#else
  int x_flag_header_unit;
#define flag_header_unit global_options.x_flag_header_unit
#endif
#ifdef GENERATOR_FILE
extern int flag_module_implicit_inline;
#else
  int x_flag_module_implicit_inline;
#define flag_module_implicit_inline global_options.x_flag_module_implicit_inline
#endif
#ifdef GENERATOR_FILE
extern int flag_module_lazy;
#else
  int x_flag_module_lazy;
#define flag_module_lazy global_options.x_flag_module_lazy
#endif
#ifdef GENERATOR_FILE
extern int flag_module_only;
#else
  int x_flag_module_only;
#define flag_module_only global_options.x_flag_module_only
#endif
#ifdef GENERATOR_FILE
extern int flag_module_private;
#else
  int x_flag_module_private;
#define flag_module_private global_options.x_flag_module_private
#endif
#ifdef GENERATOR_FILE
extern int flag_module_version_ignore;
#else
  int x_flag_module_version_ignore;
#define flag_module_version_ignore global_options.x_flag_module_version_ignore
#endif
#ifdef GENERATOR_FILE
extern int flag_moduleinfo;
#else
  int x_flag_moduleinfo;
#define flag_moduleinfo global_options.x_flag_moduleinfo
#endif
#ifdef GENERATOR_FILE
extern int flag_modules;
#else
  int x_flag_modules;
#define flag_modules global_options.x_flag_modules
#endif
#ifdef GENERATOR_FILE
extern int flag_modulo_sched;
#else
  int x_flag_modulo_sched;
#define flag_modulo_sched global_options.x_flag_modulo_sched
#endif
#ifdef GENERATOR_FILE
extern int flag_modulo_sched_allow_regmoves;
#else
  int x_flag_modulo_sched_allow_regmoves;
#define flag_modulo_sched_allow_regmoves global_options.x_flag_modulo_sched_allow_regmoves
#endif
#ifdef GENERATOR_FILE
extern int flag_move_loop_invariants;
#else
  int x_flag_move_loop_invariants;
#define flag_move_loop_invariants global_options.x_flag_move_loop_invariants
#endif
#ifdef GENERATOR_FILE
extern int flag_move_loop_stores;
#else
  int x_flag_move_loop_stores;
#define flag_move_loop_stores global_options.x_flag_move_loop_stores
#endif
#ifdef GENERATOR_FILE
extern int flag_ms_extensions;
#else
  int x_flag_ms_extensions;
#define flag_ms_extensions global_options.x_flag_ms_extensions
#endif
#ifdef GENERATOR_FILE
extern int flag_new_inheriting_ctors;
#else
  int x_flag_new_inheriting_ctors;
#define flag_new_inheriting_ctors global_options.x_flag_new_inheriting_ctors
#endif
#ifdef GENERATOR_FILE
extern int flag_new_ttp;
#else
  int x_flag_new_ttp;
#define flag_new_ttp global_options.x_flag_new_ttp
#endif
#ifdef GENERATOR_FILE
extern int flag_nil_receivers;
#else
  int x_flag_nil_receivers;
#define flag_nil_receivers global_options.x_flag_nil_receivers
#endif
#ifdef GENERATOR_FILE
extern int flag_non_call_exceptions;
#else
  int x_flag_non_call_exceptions;
#define flag_non_call_exceptions global_options.x_flag_non_call_exceptions
#endif
#ifdef GENERATOR_FILE
extern int flag_no_nonansi_builtin;
#else
  int x_flag_no_nonansi_builtin;
#define flag_no_nonansi_builtin global_options.x_flag_no_nonansi_builtin
#endif
#ifdef GENERATOR_FILE
extern int flag_nothrow_opt;
#else
  int x_flag_nothrow_opt;
#define flag_nothrow_opt global_options.x_flag_nothrow_opt
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_abi;
#else
  int x_flag_objc_abi;
#define flag_objc_abi global_options.x_flag_objc_abi
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_call_cxx_cdtors;
#else
  int x_flag_objc_call_cxx_cdtors;
#define flag_objc_call_cxx_cdtors global_options.x_flag_objc_call_cxx_cdtors
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_direct_dispatch;
#else
  int x_flag_objc_direct_dispatch;
#define flag_objc_direct_dispatch global_options.x_flag_objc_direct_dispatch
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_exceptions;
#else
  int x_flag_objc_exceptions;
#define flag_objc_exceptions global_options.x_flag_objc_exceptions
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_gc;
#else
  int x_flag_objc_gc;
#define flag_objc_gc global_options.x_flag_objc_gc
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_nilcheck;
#else
  int x_flag_objc_nilcheck;
#define flag_objc_nilcheck global_options.x_flag_objc_nilcheck
#endif
#ifdef GENERATOR_FILE
extern int flag_objc_sjlj_exceptions;
#else
  int x_flag_objc_sjlj_exceptions;
#define flag_objc_sjlj_exceptions global_options.x_flag_objc_sjlj_exceptions
#endif
#ifdef GENERATOR_FILE
extern int flag_objc1_only;
#else
  int x_flag_objc1_only;
#define flag_objc1_only global_options.x_flag_objc1_only
#endif
#ifdef GENERATOR_FILE
extern int flag_omit_frame_pointer;
#else
  int x_flag_omit_frame_pointer;
#define flag_omit_frame_pointer global_options.x_flag_omit_frame_pointer
#endif
#ifdef GENERATOR_FILE
extern int flag_openacc;
#else
  int x_flag_openacc;
#define flag_openacc global_options.x_flag_openacc
#endif
#ifdef GENERATOR_FILE
extern const char *flag_openacc_dims;
#else
  const char *x_flag_openacc_dims;
#define flag_openacc_dims global_options.x_flag_openacc_dims
#endif
#ifdef GENERATOR_FILE
extern int flag_openmp;
#else
  int x_flag_openmp;
#define flag_openmp global_options.x_flag_openmp
#endif
#ifdef GENERATOR_FILE
extern int flag_openmp_allocators;
#else
  int x_flag_openmp_allocators;
#define flag_openmp_allocators global_options.x_flag_openmp_allocators
#endif
#ifdef GENERATOR_FILE
extern int flag_openmp_simd;
#else
  int x_flag_openmp_simd;
#define flag_openmp_simd global_options.x_flag_openmp_simd
#endif
#ifdef GENERATOR_FILE
extern int flag_openmp_target_simd_clone;
#else
  int x_flag_openmp_target_simd_clone;
#define flag_openmp_target_simd_clone global_options.x_flag_openmp_target_simd_clone
#endif
#ifdef GENERATOR_FILE
extern int flag_opt_info;
#else
  int x_flag_opt_info;
#define flag_opt_info global_options.x_flag_opt_info
#endif
#ifdef GENERATOR_FILE
extern int flag_optimize_sibling_calls;
#else
  int x_flag_optimize_sibling_calls;
#define flag_optimize_sibling_calls global_options.x_flag_optimize_sibling_calls
#endif
#ifdef GENERATOR_FILE
extern int flag_optimize_strlen;
#else
  int x_flag_optimize_strlen;
#define flag_optimize_strlen global_options.x_flag_optimize_strlen
#endif
#ifdef GENERATOR_FILE
extern int flag_pack_derived;
#else
  int x_flag_pack_derived;
#define flag_pack_derived global_options.x_flag_pack_derived
#endif
#ifdef GENERATOR_FILE
extern int flag_pack_struct;
#else
  int x_flag_pack_struct;
#define flag_pack_struct global_options.x_flag_pack_struct
#endif
#ifdef GENERATOR_FILE
extern int flag_pad_source;
#else
  int x_flag_pad_source;
#define flag_pad_source global_options.x_flag_pad_source
#endif
#ifdef GENERATOR_FILE
extern int flag_partial_inlining;
#else
  int x_flag_partial_inlining;
#define flag_partial_inlining global_options.x_flag_partial_inlining
#endif
#ifdef GENERATOR_FILE
extern const char *flag_patchable_function_entry;
#else
  const char *x_flag_patchable_function_entry;
#define flag_patchable_function_entry global_options.x_flag_patchable_function_entry
#endif
#ifdef GENERATOR_FILE
extern int flag_pcc_struct_return;
#else
  int x_flag_pcc_struct_return;
#define flag_pcc_struct_return global_options.x_flag_pcc_struct_return
#endif
#ifdef GENERATOR_FILE
extern int flag_peel_loops;
#else
  int x_flag_peel_loops;
#define flag_peel_loops global_options.x_flag_peel_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_no_peephole;
#else
  int x_flag_no_peephole;
#define flag_no_peephole global_options.x_flag_no_peephole
#endif
#ifdef GENERATOR_FILE
extern int flag_peephole2;
#else
  int x_flag_peephole2;
#define flag_peephole2 global_options.x_flag_peephole2
#endif
#ifdef GENERATOR_FILE
extern int flag_permissive;
#else
  int x_flag_permissive;
#define flag_permissive global_options.x_flag_permissive
#endif
#ifdef GENERATOR_FILE
extern enum permitted_flt_eval_methods flag_permitted_flt_eval_methods;
#else
  enum permitted_flt_eval_methods x_flag_permitted_flt_eval_methods;
#define flag_permitted_flt_eval_methods global_options.x_flag_permitted_flt_eval_methods
#endif
#ifdef GENERATOR_FILE
extern int flag_plan9_extensions;
#else
  int x_flag_plan9_extensions;
#define flag_plan9_extensions global_options.x_flag_plan9_extensions
#endif
#ifdef GENERATOR_FILE
extern int flag_plt;
#else
  int x_flag_plt;
#define flag_plt global_options.x_flag_plt
#endif
#ifdef GENERATOR_FILE
extern int post_ipa_mem_report;
#else
  int x_post_ipa_mem_report;
#define post_ipa_mem_report global_options.x_post_ipa_mem_report
#endif
#ifdef GENERATOR_FILE
extern int flag_postconditions;
#else
  int x_flag_postconditions;
#define flag_postconditions global_options.x_flag_postconditions
#endif
#ifdef GENERATOR_FILE
extern const char *flag_pre_include;
#else
  const char *x_flag_pre_include;
#define flag_pre_include global_options.x_flag_pre_include
#endif
#ifdef GENERATOR_FILE
extern int pre_ipa_mem_report;
#else
  int x_pre_ipa_mem_report;
#define pre_ipa_mem_report global_options.x_pre_ipa_mem_report
#endif
#ifdef GENERATOR_FILE
extern int flag_preconditions;
#else
  int x_flag_preconditions;
#define flag_preconditions global_options.x_flag_preconditions
#endif
#ifdef GENERATOR_FILE
extern int flag_predictive_commoning;
#else
  int x_flag_predictive_commoning;
#define flag_predictive_commoning global_options.x_flag_predictive_commoning
#endif
#ifdef GENERATOR_FILE
extern int flag_prefetch_loop_arrays;
#else
  int x_flag_prefetch_loop_arrays;
#define flag_prefetch_loop_arrays global_options.x_flag_prefetch_loop_arrays
#endif
#ifdef GENERATOR_FILE
extern int flag_pretty_templates;
#else
  int x_flag_pretty_templates;
#define flag_pretty_templates global_options.x_flag_pretty_templates
#endif
#ifdef GENERATOR_FILE
extern int flag_printf_return_value;
#else
  int x_flag_printf_return_value;
#define flag_printf_return_value global_options.x_flag_printf_return_value
#endif
#ifdef GENERATOR_FILE
extern int profile_flag;
#else
  int x_profile_flag;
#define profile_flag global_options.x_profile_flag
#endif
#ifdef GENERATOR_FILE
extern int profile_abs_path_flag;
#else
  int x_profile_abs_path_flag;
#define profile_abs_path_flag global_options.x_profile_abs_path_flag
#endif
#ifdef GENERATOR_FILE
extern int profile_arc_flag;
#else
  int x_profile_arc_flag;
#define profile_arc_flag global_options.x_profile_arc_flag
#endif
#ifdef GENERATOR_FILE
extern int flag_profile_correction;
#else
  int x_flag_profile_correction;
#define flag_profile_correction global_options.x_flag_profile_correction
#endif
#ifdef GENERATOR_FILE
extern const char *profile_data_prefix;
#else
  const char *x_profile_data_prefix;
#define profile_data_prefix global_options.x_profile_data_prefix
#endif
#ifdef GENERATOR_FILE
extern const char *flag_profile_exclude_files;
#else
  const char *x_flag_profile_exclude_files;
#define flag_profile_exclude_files global_options.x_flag_profile_exclude_files
#endif
#ifdef GENERATOR_FILE
extern const char *flag_profile_filter_files;
#else
  const char *x_flag_profile_filter_files;
#define flag_profile_filter_files global_options.x_flag_profile_filter_files
#endif
#ifdef GENERATOR_FILE
extern const char *profile_info_section;
#else
  const char *x_profile_info_section;
#define profile_info_section global_options.x_profile_info_section
#endif
#ifdef GENERATOR_FILE
extern const char *profile_note_location;
#else
  const char *x_profile_note_location;
#define profile_note_location global_options.x_profile_note_location
#endif
#ifdef GENERATOR_FILE
extern int flag_profile_partial_training;
#else
  int x_flag_profile_partial_training;
#define flag_profile_partial_training global_options.x_flag_profile_partial_training
#endif
#ifdef GENERATOR_FILE
extern const char *profile_prefix_path;
#else
  const char *x_profile_prefix_path;
#define profile_prefix_path global_options.x_profile_prefix_path
#endif
#ifdef GENERATOR_FILE
extern int flag_profile_reorder_functions;
#else
  int x_flag_profile_reorder_functions;
#define flag_profile_reorder_functions global_options.x_flag_profile_reorder_functions
#endif
#ifdef GENERATOR_FILE
extern int profile_report;
#else
  int x_profile_report;
#define profile_report global_options.x_profile_report
#endif
#ifdef GENERATOR_FILE
extern enum profile_reproducibility flag_profile_reproducible;
#else
  enum profile_reproducibility x_flag_profile_reproducible;
#define flag_profile_reproducible global_options.x_flag_profile_reproducible
#endif
#ifdef GENERATOR_FILE
extern enum profile_update flag_profile_update;
#else
  enum profile_update x_flag_profile_update;
#define flag_profile_update global_options.x_flag_profile_update
#endif
#ifdef GENERATOR_FILE
extern int flag_profile_use;
#else
  int x_flag_profile_use;
#define flag_profile_use global_options.x_flag_profile_use
#endif
#ifdef GENERATOR_FILE
extern int flag_profile_values;
#else
  int x_flag_profile_values;
#define flag_profile_values global_options.x_flag_profile_values
#endif
#ifdef GENERATOR_FILE
extern int flag_protect_parens;
#else
  int x_flag_protect_parens;
#define flag_protect_parens global_options.x_flag_protect_parens
#endif
#ifdef GENERATOR_FILE
extern int flag_range_check;
#else
  int x_flag_range_check;
#define flag_range_check global_options.x_flag_range_check
#endif
#ifdef GENERATOR_FILE
extern int flag_real4_kind;
#else
  int x_flag_real4_kind;
#define flag_real4_kind global_options.x_flag_real4_kind
#endif
#ifdef GENERATOR_FILE
extern int flag_real8_kind;
#else
  int x_flag_real8_kind;
#define flag_real8_kind global_options.x_flag_real8_kind
#endif
#ifdef GENERATOR_FILE
extern int flag_realloc_lhs;
#else
  int x_flag_realloc_lhs;
#define flag_realloc_lhs global_options.x_flag_realloc_lhs
#endif
#ifdef GENERATOR_FILE
extern int flag_reciprocal_math;
#else
  int x_flag_reciprocal_math;
#define flag_reciprocal_math global_options.x_flag_reciprocal_math
#endif
#ifdef GENERATOR_FILE
extern int flag_record_gcc_switches;
#else
  int x_flag_record_gcc_switches;
#define flag_record_gcc_switches global_options.x_flag_record_gcc_switches
#endif
#ifdef GENERATOR_FILE
extern int flag_record_marker;
#else
  int x_flag_record_marker;
#define flag_record_marker global_options.x_flag_record_marker
#endif
#ifdef GENERATOR_FILE
extern int flag_recursive;
#else
  int x_flag_recursive;
#define flag_recursive global_options.x_flag_recursive
#endif
#ifdef GENERATOR_FILE
extern int flag_ree;
#else
  int x_flag_ree;
#define flag_ree global_options.x_flag_ree
#endif
#ifdef GENERATOR_FILE
extern int flag_rename_registers;
#else
  int x_flag_rename_registers;
#define flag_rename_registers global_options.x_flag_rename_registers
#endif
#ifdef GENERATOR_FILE
extern int flag_reorder_blocks;
#else
  int x_flag_reorder_blocks;
#define flag_reorder_blocks global_options.x_flag_reorder_blocks
#endif
#ifdef GENERATOR_FILE
extern enum reorder_blocks_algorithm flag_reorder_blocks_algorithm;
#else
  enum reorder_blocks_algorithm x_flag_reorder_blocks_algorithm;
#define flag_reorder_blocks_algorithm global_options.x_flag_reorder_blocks_algorithm
#endif
#ifdef GENERATOR_FILE
extern int flag_reorder_blocks_and_partition;
#else
  int x_flag_reorder_blocks_and_partition;
#define flag_reorder_blocks_and_partition global_options.x_flag_reorder_blocks_and_partition
#endif
#ifdef GENERATOR_FILE
extern int flag_reorder_functions;
#else
  int x_flag_reorder_functions;
#define flag_reorder_functions global_options.x_flag_reorder_functions
#endif
#ifdef GENERATOR_FILE
extern int flag_repack_arrays;
#else
  int x_flag_repack_arrays;
#define flag_repack_arrays global_options.x_flag_repack_arrays
#endif
#ifdef GENERATOR_FILE
extern int flag_replace_objc_classes;
#else
  int x_flag_replace_objc_classes;
#define flag_replace_objc_classes global_options.x_flag_replace_objc_classes
#endif
#ifdef GENERATOR_FILE
extern int flag_report_bug;
#else
  int x_flag_report_bug;
#define flag_report_bug global_options.x_flag_report_bug
#endif
#ifdef GENERATOR_FILE
extern int go_require_return_statement;
#else
  int x_go_require_return_statement;
#define go_require_return_statement global_options.x_go_require_return_statement
#endif
#ifdef GENERATOR_FILE
extern int flag_rerun_cse_after_loop;
#else
  int x_flag_rerun_cse_after_loop;
#define flag_rerun_cse_after_loop global_options.x_flag_rerun_cse_after_loop
#endif
#ifdef GENERATOR_FILE
extern int flag_resched_modulo_sched;
#else
  int x_flag_resched_modulo_sched;
#define flag_resched_modulo_sched global_options.x_flag_resched_modulo_sched
#endif
#ifdef GENERATOR_FILE
extern int flag_rounding_math;
#else
  int x_flag_rounding_math;
#define flag_rounding_math global_options.x_flag_rounding_math
#endif
#ifdef GENERATOR_FILE
extern int flag_rtti;
#else
  int x_flag_rtti;
#define flag_rtti global_options.x_flag_rtti
#endif
#ifdef GENERATOR_FILE
extern int flag_borrowcheck;
#else
  int x_flag_borrowcheck;
#define flag_borrowcheck global_options.x_flag_borrowcheck
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_compile_until;
#else
  int x_flag_rust_compile_until;
#define flag_rust_compile_until global_options.x_flag_rust_compile_until
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_crate_type;
#else
  int x_flag_rust_crate_type;
#define flag_rust_crate_type global_options.x_flag_rust_crate_type
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_debug;
#else
  int x_flag_rust_debug;
#define flag_rust_debug global_options.x_flag_rust_debug
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_edition;
#else
  int x_flag_rust_edition;
#define flag_rust_edition global_options.x_flag_rust_edition
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_embed_metadata;
#else
  int x_flag_rust_embed_metadata;
#define flag_rust_embed_metadata global_options.x_flag_rust_embed_metadata
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_experimental;
#else
  int x_flag_rust_experimental;
#define flag_rust_experimental global_options.x_flag_rust_experimental
#endif
#ifdef GENERATOR_FILE
extern int flag_rust_mangling;
#else
  int x_flag_rust_mangling;
#define flag_rust_mangling global_options.x_flag_rust_mangling
#endif
#ifdef GENERATOR_FILE
extern int rust_max_recursion_depth;
#else
  int x_rust_max_recursion_depth;
#define rust_max_recursion_depth global_options.x_rust_max_recursion_depth
#endif
#ifdef GENERATOR_FILE
extern int flag_name_resolution_2_0;
#else
  int x_flag_name_resolution_2_0;
#define flag_name_resolution_2_0 global_options.x_flag_name_resolution_2_0
#endif
#ifdef GENERATOR_FILE
extern int flag_sanitize_address_use_after_scope;
#else
  int x_flag_sanitize_address_use_after_scope;
#define flag_sanitize_address_use_after_scope global_options.x_flag_sanitize_address_use_after_scope
#endif
#ifdef GENERATOR_FILE
extern int flag_sanitize_coverage;
#else
  int x_flag_sanitize_coverage;
#define flag_sanitize_coverage global_options.x_flag_sanitize_coverage
#endif
#ifdef GENERATOR_FILE
extern int flag_save_optimization_record;
#else
  int x_flag_save_optimization_record;
#define flag_save_optimization_record global_options.x_flag_save_optimization_record
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_critical_path_heuristic;
#else
  int x_flag_sched_critical_path_heuristic;
#define flag_sched_critical_path_heuristic global_options.x_flag_sched_critical_path_heuristic
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_dep_count_heuristic;
#else
  int x_flag_sched_dep_count_heuristic;
#define flag_sched_dep_count_heuristic global_options.x_flag_sched_dep_count_heuristic
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_group_heuristic;
#else
  int x_flag_sched_group_heuristic;
#define flag_sched_group_heuristic global_options.x_flag_sched_group_heuristic
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_interblock;
#else
  int x_flag_schedule_interblock;
#define flag_schedule_interblock global_options.x_flag_schedule_interblock
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_last_insn_heuristic;
#else
  int x_flag_sched_last_insn_heuristic;
#define flag_sched_last_insn_heuristic global_options.x_flag_sched_last_insn_heuristic
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_pressure;
#else
  int x_flag_sched_pressure;
#define flag_sched_pressure global_options.x_flag_sched_pressure
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_rank_heuristic;
#else
  int x_flag_sched_rank_heuristic;
#define flag_sched_rank_heuristic global_options.x_flag_sched_rank_heuristic
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_speculative;
#else
  int x_flag_schedule_speculative;
#define flag_schedule_speculative global_options.x_flag_schedule_speculative
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_spec_insn_heuristic;
#else
  int x_flag_sched_spec_insn_heuristic;
#define flag_sched_spec_insn_heuristic global_options.x_flag_sched_spec_insn_heuristic
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_speculative_load;
#else
  int x_flag_schedule_speculative_load;
#define flag_schedule_speculative_load global_options.x_flag_schedule_speculative_load
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_speculative_load_dangerous;
#else
  int x_flag_schedule_speculative_load_dangerous;
#define flag_schedule_speculative_load_dangerous global_options.x_flag_schedule_speculative_load_dangerous
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_stalled_insns;
#else
  int x_flag_sched_stalled_insns;
#define flag_sched_stalled_insns global_options.x_flag_sched_stalled_insns
#endif
#ifdef GENERATOR_FILE
extern int flag_sched_stalled_insns_dep;
#else
  int x_flag_sched_stalled_insns_dep;
#define flag_sched_stalled_insns_dep global_options.x_flag_sched_stalled_insns_dep
#endif
#ifdef GENERATOR_FILE
extern int sched_verbose_param;
#else
  int x_sched_verbose_param;
#define sched_verbose_param global_options.x_sched_verbose_param
#endif
#ifdef GENERATOR_FILE
extern int flag_sched2_use_superblocks;
#else
  int x_flag_sched2_use_superblocks;
#define flag_sched2_use_superblocks global_options.x_flag_sched2_use_superblocks
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_fusion;
#else
  int x_flag_schedule_fusion;
#define flag_schedule_fusion global_options.x_flag_schedule_fusion
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_insns;
#else
  int x_flag_schedule_insns;
#define flag_schedule_insns global_options.x_flag_schedule_insns
#endif
#ifdef GENERATOR_FILE
extern int flag_schedule_insns_after_reload;
#else
  int x_flag_schedule_insns_after_reload;
#define flag_schedule_insns_after_reload global_options.x_flag_schedule_insns_after_reload
#endif
#ifdef GENERATOR_FILE
extern int flag_second_underscore;
#else
  int x_flag_second_underscore;
#define flag_second_underscore global_options.x_flag_second_underscore
#endif
#ifdef GENERATOR_FILE
extern int flag_section_anchors;
#else
  int x_flag_section_anchors;
#define flag_section_anchors global_options.x_flag_section_anchors
#endif
#ifdef GENERATOR_FILE
extern int flag_sel_sched_pipelining;
#else
  int x_flag_sel_sched_pipelining;
#define flag_sel_sched_pipelining global_options.x_flag_sel_sched_pipelining
#endif
#ifdef GENERATOR_FILE
extern int flag_sel_sched_pipelining_outer_loops;
#else
  int x_flag_sel_sched_pipelining_outer_loops;
#define flag_sel_sched_pipelining_outer_loops global_options.x_flag_sel_sched_pipelining_outer_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_sel_sched_reschedule_pipelined;
#else
  int x_flag_sel_sched_reschedule_pipelined;
#define flag_sel_sched_reschedule_pipelined global_options.x_flag_sel_sched_reschedule_pipelined
#endif
#ifdef GENERATOR_FILE
extern int flag_selective_scheduling;
#else
  int x_flag_selective_scheduling;
#define flag_selective_scheduling global_options.x_flag_selective_scheduling
#endif
#ifdef GENERATOR_FILE
extern int flag_selective_scheduling2;
#else
  int x_flag_selective_scheduling2;
#define flag_selective_scheduling2 global_options.x_flag_selective_scheduling2
#endif
#ifdef GENERATOR_FILE
extern const char *flag_self_test;
#else
  const char *x_flag_self_test;
#define flag_self_test global_options.x_flag_self_test
#endif
#ifdef GENERATOR_FILE
extern int flag_semantic_interposition;
#else
  int x_flag_semantic_interposition;
#define flag_semantic_interposition global_options.x_flag_semantic_interposition
#endif
#ifdef GENERATOR_FILE
extern int flag_short_enums;
#else
  int x_flag_short_enums;
#define flag_short_enums global_options.x_flag_short_enums
#endif
#ifdef GENERATOR_FILE
extern int flag_short_wchar;
#else
  int x_flag_short_wchar;
#define flag_short_wchar global_options.x_flag_short_wchar
#endif
#ifdef GENERATOR_FILE
extern int flag_show_column;
#else
  int x_flag_show_column;
#define flag_show_column global_options.x_flag_show_column
#endif
#ifdef GENERATOR_FILE
extern int flag_shrink_wrap;
#else
  int x_flag_shrink_wrap;
#define flag_shrink_wrap global_options.x_flag_shrink_wrap
#endif
#ifdef GENERATOR_FILE
extern int flag_shrink_wrap_separate;
#else
  int x_flag_shrink_wrap_separate;
#define flag_shrink_wrap_separate global_options.x_flag_shrink_wrap_separate
#endif
#ifdef GENERATOR_FILE
extern int flag_sign_zero;
#else
  int x_flag_sign_zero;
#define flag_sign_zero global_options.x_flag_sign_zero
#endif
#ifdef GENERATOR_FILE
extern int flag_signaling_nans;
#else
  int x_flag_signaling_nans;
#define flag_signaling_nans global_options.x_flag_signaling_nans
#endif
#ifdef GENERATOR_FILE
extern int flag_signed_bitfields;
#else
  int x_flag_signed_bitfields;
#define flag_signed_bitfields global_options.x_flag_signed_bitfields
#endif
#ifdef GENERATOR_FILE
extern int flag_signed_char;
#else
  int x_flag_signed_char;
#define flag_signed_char global_options.x_flag_signed_char
#endif
#ifdef GENERATOR_FILE
extern int flag_signed_zeros;
#else
  int x_flag_signed_zeros;
#define flag_signed_zeros global_options.x_flag_signed_zeros
#endif
#ifdef GENERATOR_FILE
extern enum vect_cost_model flag_simd_cost_model;
#else
  enum vect_cost_model x_flag_simd_cost_model;
#define flag_simd_cost_model global_options.x_flag_simd_cost_model
#endif
#ifdef GENERATOR_FILE
extern int flag_single_precision_constant;
#else
  int x_flag_single_precision_constant;
#define flag_single_precision_constant global_options.x_flag_single_precision_constant
#endif
#ifdef GENERATOR_FILE
extern int flag_sized_deallocation;
#else
  int x_flag_sized_deallocation;
#define flag_sized_deallocation global_options.x_flag_sized_deallocation
#endif
#ifdef GENERATOR_FILE
extern int flag_split_ivs_in_unroller;
#else
  int x_flag_split_ivs_in_unroller;
#define flag_split_ivs_in_unroller global_options.x_flag_split_ivs_in_unroller
#endif
#ifdef GENERATOR_FILE
extern int flag_split_loops;
#else
  int x_flag_split_loops;
#define flag_split_loops global_options.x_flag_split_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_split_paths;
#else
  int x_flag_split_paths;
#define flag_split_paths global_options.x_flag_split_paths
#endif
#ifdef GENERATOR_FILE
extern int flag_split_stack;
#else
  int x_flag_split_stack;
#define flag_split_stack global_options.x_flag_split_stack
#endif
#ifdef GENERATOR_FILE
extern int flag_split_wide_types;
#else
  int x_flag_split_wide_types;
#define flag_split_wide_types global_options.x_flag_split_wide_types
#endif
#ifdef GENERATOR_FILE
extern int flag_split_wide_types_early;
#else
  int x_flag_split_wide_types_early;
#define flag_split_wide_types_early global_options.x_flag_split_wide_types_early
#endif
#ifdef GENERATOR_FILE
extern int flag_ssa_backprop;
#else
  int x_flag_ssa_backprop;
#define flag_ssa_backprop global_options.x_flag_ssa_backprop
#endif
#ifdef GENERATOR_FILE
extern int flag_ssa_phiopt;
#else
  int x_flag_ssa_phiopt;
#define flag_ssa_phiopt global_options.x_flag_ssa_phiopt
#endif
#ifdef GENERATOR_FILE
extern enum scalar_storage_order_kind default_sso;
#else
  enum scalar_storage_order_kind x_default_sso;
#define default_sso global_options.x_default_sso
#endif
#ifdef GENERATOR_FILE
extern int flag_stack_arrays;
#else
  int x_flag_stack_arrays;
#define flag_stack_arrays global_options.x_flag_stack_arrays
#endif
#ifdef GENERATOR_FILE
extern int flag_stack_clash_protection;
#else
  int x_flag_stack_clash_protection;
#define flag_stack_clash_protection global_options.x_flag_stack_clash_protection
#endif
#ifdef GENERATOR_FILE
extern int flag_stack_protect;
#else
  int x_flag_stack_protect;
#define flag_stack_protect global_options.x_flag_stack_protect
#endif
#ifdef GENERATOR_FILE
extern enum stack_reuse_level flag_stack_reuse;
#else
  enum stack_reuse_level x_flag_stack_reuse;
#define flag_stack_reuse global_options.x_flag_stack_reuse
#endif
#ifdef GENERATOR_FILE
extern int flag_stack_usage;
#else
  int x_flag_stack_usage;
#define flag_stack_usage global_options.x_flag_stack_usage
#endif
#ifdef GENERATOR_FILE
extern int flag_detailed_statistics;
#else
  int x_flag_detailed_statistics;
#define flag_detailed_statistics global_options.x_flag_detailed_statistics
#endif
#ifdef GENERATOR_FILE
extern int flag_stdarg_opt;
#else
  int x_flag_stdarg_opt;
#define flag_stdarg_opt global_options.x_flag_stdarg_opt
#endif
#ifdef GENERATOR_FILE
extern int flag_store_merging;
#else
  int x_flag_store_merging;
#define flag_store_merging global_options.x_flag_store_merging
#endif
#ifdef GENERATOR_FILE
extern int flag_strict_aliasing;
#else
  int x_flag_strict_aliasing;
#define flag_strict_aliasing global_options.x_flag_strict_aliasing
#endif
#ifdef GENERATOR_FILE
extern int flag_strict_enums;
#else
  int x_flag_strict_enums;
#define flag_strict_enums global_options.x_flag_strict_enums
#endif
#ifdef GENERATOR_FILE
extern int flag_strict_flex_arrays;
#else
  int x_flag_strict_flex_arrays;
#define flag_strict_flex_arrays global_options.x_flag_strict_flex_arrays
#endif
#ifdef GENERATOR_FILE
extern int flag_strict_volatile_bitfields;
#else
  int x_flag_strict_volatile_bitfields;
#define flag_strict_volatile_bitfields global_options.x_flag_strict_volatile_bitfields
#endif
#ifdef GENERATOR_FILE
extern int flag_strong_eval_order;
#else
  int x_flag_strong_eval_order;
#define flag_strong_eval_order global_options.x_flag_strong_eval_order
#endif
#ifdef GENERATOR_FILE
extern int flag_strub;
#else
  int x_flag_strub;
#define flag_strub global_options.x_flag_strub
#endif
#ifdef GENERATOR_FILE
extern int flag_switch_errors;
#else
  int x_flag_switch_errors;
#define flag_switch_errors global_options.x_flag_switch_errors
#endif
#ifdef GENERATOR_FILE
extern int flag_sync_libcalls;
#else
  int x_flag_sync_libcalls;
#define flag_sync_libcalls global_options.x_flag_sync_libcalls
#endif
#ifdef GENERATOR_FILE
extern int flag_syntax_only;
#else
  int x_flag_syntax_only;
#define flag_syntax_only global_options.x_flag_syntax_only
#endif
#ifdef GENERATOR_FILE
extern int flag_tail_call_workaround;
#else
  int x_flag_tail_call_workaround;
#define flag_tail_call_workaround global_options.x_flag_tail_call_workaround
#endif
#ifdef GENERATOR_FILE
extern int template_backtrace_limit;
#else
  int x_template_backtrace_limit;
#define template_backtrace_limit global_options.x_template_backtrace_limit
#endif
#ifdef GENERATOR_FILE
extern int flag_test_coverage;
#else
  int x_flag_test_coverage;
#define flag_test_coverage global_options.x_flag_test_coverage
#endif
#ifdef GENERATOR_FILE
extern int flag_test_forall_temp;
#else
  int x_flag_test_forall_temp;
#define flag_test_forall_temp global_options.x_flag_test_forall_temp
#endif
#ifdef GENERATOR_FILE
extern int flag_thread_jumps;
#else
  int x_flag_thread_jumps;
#define flag_thread_jumps global_options.x_flag_thread_jumps
#endif
#ifdef GENERATOR_FILE
extern int flag_threadsafe_statics;
#else
  int x_flag_threadsafe_statics;
#define flag_threadsafe_statics global_options.x_flag_threadsafe_statics
#endif
#ifdef GENERATOR_FILE
extern int time_report;
#else
  int x_time_report;
#define time_report global_options.x_time_report
#endif
#ifdef GENERATOR_FILE
extern int time_report_details;
#else
  int x_time_report_details;
#define time_report_details global_options.x_time_report_details
#endif
#ifdef GENERATOR_FILE
extern enum tls_model flag_tls_default;
#else
  enum tls_model x_flag_tls_default;
#define flag_tls_default global_options.x_flag_tls_default
#endif
#ifdef GENERATOR_FILE
extern int flag_toplevel_reorder;
#else
  int x_flag_toplevel_reorder;
#define flag_toplevel_reorder global_options.x_flag_toplevel_reorder
#endif
#ifdef GENERATOR_FILE
extern int flag_tracer;
#else
  int x_flag_tracer;
#define flag_tracer global_options.x_flag_tracer
#endif
#ifdef GENERATOR_FILE
extern enum trampoline_impl flag_trampoline_impl;
#else
  enum trampoline_impl x_flag_trampoline_impl;
#define flag_trampoline_impl global_options.x_flag_trampoline_impl
#endif
#ifdef GENERATOR_FILE
extern int flag_trampolines;
#else
  int x_flag_trampolines;
#define flag_trampolines global_options.x_flag_trampolines
#endif
#ifdef GENERATOR_FILE
extern int flag_trapping_math;
#else
  int x_flag_trapping_math;
#define flag_trapping_math global_options.x_flag_trapping_math
#endif
#ifdef GENERATOR_FILE
extern int flag_trapv;
#else
  int x_flag_trapv;
#define flag_trapv global_options.x_flag_trapv
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_bit_ccp;
#else
  int x_flag_tree_bit_ccp;
#define flag_tree_bit_ccp global_options.x_flag_tree_bit_ccp
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_builtin_call_dce;
#else
  int x_flag_tree_builtin_call_dce;
#define flag_tree_builtin_call_dce global_options.x_flag_tree_builtin_call_dce
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_ccp;
#else
  int x_flag_tree_ccp;
#define flag_tree_ccp global_options.x_flag_tree_ccp
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_ch;
#else
  int x_flag_tree_ch;
#define flag_tree_ch global_options.x_flag_tree_ch
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_coalesce_vars;
#else
  int x_flag_tree_coalesce_vars;
#define flag_tree_coalesce_vars global_options.x_flag_tree_coalesce_vars
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_copy_prop;
#else
  int x_flag_tree_copy_prop;
#define flag_tree_copy_prop global_options.x_flag_tree_copy_prop
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_cselim;
#else
  int x_flag_tree_cselim;
#define flag_tree_cselim global_options.x_flag_tree_cselim
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_dce;
#else
  int x_flag_tree_dce;
#define flag_tree_dce global_options.x_flag_tree_dce
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_dom;
#else
  int x_flag_tree_dom;
#define flag_tree_dom global_options.x_flag_tree_dom
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_dse;
#else
  int x_flag_tree_dse;
#define flag_tree_dse global_options.x_flag_tree_dse
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_forwprop;
#else
  int x_flag_tree_forwprop;
#define flag_tree_forwprop global_options.x_flag_tree_forwprop
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_fre;
#else
  int x_flag_tree_fre;
#define flag_tree_fre global_options.x_flag_tree_fre
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_distribute_patterns;
#else
  int x_flag_tree_loop_distribute_patterns;
#define flag_tree_loop_distribute_patterns global_options.x_flag_tree_loop_distribute_patterns
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_distribution;
#else
  int x_flag_tree_loop_distribution;
#define flag_tree_loop_distribution global_options.x_flag_tree_loop_distribution
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_if_convert;
#else
  int x_flag_tree_loop_if_convert;
#define flag_tree_loop_if_convert global_options.x_flag_tree_loop_if_convert
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_im;
#else
  int x_flag_tree_loop_im;
#define flag_tree_loop_im global_options.x_flag_tree_loop_im
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_ivcanon;
#else
  int x_flag_tree_loop_ivcanon;
#define flag_tree_loop_ivcanon global_options.x_flag_tree_loop_ivcanon
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_optimize;
#else
  int x_flag_tree_loop_optimize;
#define flag_tree_loop_optimize global_options.x_flag_tree_loop_optimize
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_loop_vectorize;
#else
  int x_flag_tree_loop_vectorize;
#define flag_tree_loop_vectorize global_options.x_flag_tree_loop_vectorize
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_live_range_split;
#else
  int x_flag_tree_live_range_split;
#define flag_tree_live_range_split global_options.x_flag_tree_live_range_split
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_parallelize_loops;
#else
  int x_flag_tree_parallelize_loops;
#define flag_tree_parallelize_loops global_options.x_flag_tree_parallelize_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_partial_pre;
#else
  int x_flag_tree_partial_pre;
#define flag_tree_partial_pre global_options.x_flag_tree_partial_pre
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_phiprop;
#else
  int x_flag_tree_phiprop;
#define flag_tree_phiprop global_options.x_flag_tree_phiprop
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_pre;
#else
  int x_flag_tree_pre;
#define flag_tree_pre global_options.x_flag_tree_pre
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_pta;
#else
  int x_flag_tree_pta;
#define flag_tree_pta global_options.x_flag_tree_pta
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_reassoc;
#else
  int x_flag_tree_reassoc;
#define flag_tree_reassoc global_options.x_flag_tree_reassoc
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_scev_cprop;
#else
  int x_flag_tree_scev_cprop;
#define flag_tree_scev_cprop global_options.x_flag_tree_scev_cprop
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_sink;
#else
  int x_flag_tree_sink;
#define flag_tree_sink global_options.x_flag_tree_sink
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_slp_vectorize;
#else
  int x_flag_tree_slp_vectorize;
#define flag_tree_slp_vectorize global_options.x_flag_tree_slp_vectorize
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_slsr;
#else
  int x_flag_tree_slsr;
#define flag_tree_slsr global_options.x_flag_tree_slsr
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_sra;
#else
  int x_flag_tree_sra;
#define flag_tree_sra global_options.x_flag_tree_sra
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_switch_conversion;
#else
  int x_flag_tree_switch_conversion;
#define flag_tree_switch_conversion global_options.x_flag_tree_switch_conversion
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_tail_merge;
#else
  int x_flag_tree_tail_merge;
#define flag_tree_tail_merge global_options.x_flag_tree_tail_merge
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_ter;
#else
  int x_flag_tree_ter;
#define flag_tree_ter global_options.x_flag_tree_ter
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_vectorize;
#else
  int x_flag_tree_vectorize;
#define flag_tree_vectorize global_options.x_flag_tree_vectorize
#endif
#ifdef GENERATOR_FILE
extern int flag_tree_vrp;
#else
  int x_flag_tree_vrp;
#define flag_tree_vrp global_options.x_flag_tree_vrp
#endif
#ifdef GENERATOR_FILE
extern enum auto_init_type flag_auto_var_init;
#else
  enum auto_init_type x_flag_auto_var_init;
#define flag_auto_var_init global_options.x_flag_auto_var_init
#endif
#ifdef GENERATOR_FILE
extern int flag_unconstrained_commons;
#else
  int x_flag_unconstrained_commons;
#define flag_unconstrained_commons global_options.x_flag_unconstrained_commons
#endif
#ifdef GENERATOR_FILE
extern int flag_underscoring;
#else
  int x_flag_underscoring;
#define flag_underscoring global_options.x_flag_underscoring
#endif
#ifdef GENERATOR_FILE
extern int flag_unit_at_a_time;
#else
  int x_flag_unit_at_a_time;
#define flag_unit_at_a_time global_options.x_flag_unit_at_a_time
#endif
#ifdef GENERATOR_FILE
extern int flag_unreachable_traps;
#else
  int x_flag_unreachable_traps;
#define flag_unreachable_traps global_options.x_flag_unreachable_traps
#endif
#ifdef GENERATOR_FILE
extern int flag_unroll_all_loops;
#else
  int x_flag_unroll_all_loops;
#define flag_unroll_all_loops global_options.x_flag_unroll_all_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_cunroll_grow_size;
#else
  int x_flag_cunroll_grow_size;
#define flag_cunroll_grow_size global_options.x_flag_cunroll_grow_size
#endif
#ifdef GENERATOR_FILE
extern int flag_unroll_loops;
#else
  int x_flag_unroll_loops;
#define flag_unroll_loops global_options.x_flag_unroll_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_unsafe_math_optimizations;
#else
  int x_flag_unsafe_math_optimizations;
#define flag_unsafe_math_optimizations global_options.x_flag_unsafe_math_optimizations
#endif
#ifdef GENERATOR_FILE
extern int flag_unswitch_loops;
#else
  int x_flag_unswitch_loops;
#define flag_unswitch_loops global_options.x_flag_unswitch_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_unwind_tables;
#else
  int x_flag_unwind_tables;
#define flag_unwind_tables global_options.x_flag_unwind_tables
#endif
#ifdef GENERATOR_FILE
extern int flag_use_cxa_atexit;
#else
  int x_flag_use_cxa_atexit;
#define flag_use_cxa_atexit global_options.x_flag_use_cxa_atexit
#endif
#ifdef GENERATOR_FILE
extern int flag_use_cxa_get_exception_ptr;
#else
  int x_flag_use_cxa_get_exception_ptr;
#define flag_use_cxa_get_exception_ptr global_options.x_flag_use_cxa_get_exception_ptr
#endif
#ifdef GENERATOR_FILE
extern int flag_use_linker_plugin;
#else
  int x_flag_use_linker_plugin;
#define flag_use_linker_plugin global_options.x_flag_use_linker_plugin
#endif
#ifdef GENERATOR_FILE
extern int flag_var_tracking;
#else
  int x_flag_var_tracking;
#define flag_var_tracking global_options.x_flag_var_tracking
#endif
#ifdef GENERATOR_FILE
extern int flag_var_tracking_assignments;
#else
  int x_flag_var_tracking_assignments;
#define flag_var_tracking_assignments global_options.x_flag_var_tracking_assignments
#endif
#ifdef GENERATOR_FILE
extern int flag_var_tracking_assignments_toggle;
#else
  int x_flag_var_tracking_assignments_toggle;
#define flag_var_tracking_assignments_toggle global_options.x_flag_var_tracking_assignments_toggle
#endif
#ifdef GENERATOR_FILE
extern int flag_var_tracking_uninit;
#else
  int x_flag_var_tracking_uninit;
#define flag_var_tracking_uninit global_options.x_flag_var_tracking_uninit
#endif
#ifdef GENERATOR_FILE
extern int flag_variable_expansion_in_unroller;
#else
  int x_flag_variable_expansion_in_unroller;
#define flag_variable_expansion_in_unroller global_options.x_flag_variable_expansion_in_unroller
#endif
#ifdef GENERATOR_FILE
extern enum vect_cost_model flag_vect_cost_model;
#else
  enum vect_cost_model x_flag_vect_cost_model;
#define flag_vect_cost_model global_options.x_flag_vect_cost_model
#endif
#ifdef GENERATOR_FILE
extern int flag_verbose_asm;
#else
  int x_flag_verbose_asm;
#define flag_verbose_asm global_options.x_flag_verbose_asm
#endif
#ifdef GENERATOR_FILE
extern int flag_version_loops_for_strides;
#else
  int x_flag_version_loops_for_strides;
#define flag_version_loops_for_strides global_options.x_flag_version_loops_for_strides
#endif
#ifdef GENERATOR_FILE
extern int flag_visibility_ms_compat;
#else
  int x_flag_visibility_ms_compat;
#define flag_visibility_ms_compat global_options.x_flag_visibility_ms_compat
#endif
#ifdef GENERATOR_FILE
extern enum symbol_visibility default_visibility;
#else
  enum symbol_visibility x_default_visibility;
#define default_visibility global_options.x_default_visibility
#endif
#ifdef GENERATOR_FILE
extern int flag_value_profile_transformations;
#else
  int x_flag_value_profile_transformations;
#define flag_value_profile_transformations global_options.x_flag_value_profile_transformations
#endif
#ifdef GENERATOR_FILE
extern enum vtv_priority flag_vtable_verify;
#else
  enum vtv_priority x_flag_vtable_verify;
#define flag_vtable_verify global_options.x_flag_vtable_verify
#endif
#ifdef GENERATOR_FILE
extern int flag_vtv_counts;
#else
  int x_flag_vtv_counts;
#define flag_vtv_counts global_options.x_flag_vtv_counts
#endif
#ifdef GENERATOR_FILE
extern int flag_vtv_debug;
#else
  int x_flag_vtv_debug;
#define flag_vtv_debug global_options.x_flag_vtv_debug
#endif
#ifdef GENERATOR_FILE
extern int flag_weak;
#else
  int x_flag_weak;
#define flag_weak global_options.x_flag_weak
#endif
#ifdef GENERATOR_FILE
extern int flag_weak_templates;
#else
  int x_flag_weak_templates;
#define flag_weak_templates global_options.x_flag_weak_templates
#endif
#ifdef GENERATOR_FILE
extern int flag_web;
#else
  int x_flag_web;
#define flag_web global_options.x_flag_web
#endif
#ifdef GENERATOR_FILE
extern int flag_whole_program;
#else
  int x_flag_whole_program;
#define flag_whole_program global_options.x_flag_whole_program
#endif
#ifdef GENERATOR_FILE
extern int flag_working_directory;
#else
  int x_flag_working_directory;
#define flag_working_directory global_options.x_flag_working_directory
#endif
#ifdef GENERATOR_FILE
extern const char *flag_wpa;
#else
  const char *x_flag_wpa;
#define flag_wpa global_options.x_flag_wpa
#endif
#ifdef GENERATOR_FILE
extern int flag_wrapv;
#else
  int x_flag_wrapv;
#define flag_wrapv global_options.x_flag_wrapv
#endif
#ifdef GENERATOR_FILE
extern int flag_wrapv_pointer;
#else
  int x_flag_wrapv_pointer;
#define flag_wrapv_pointer global_options.x_flag_wrapv_pointer
#endif
#ifdef GENERATOR_FILE
extern int flag_zero_initialized_in_bss;
#else
  int x_flag_zero_initialized_in_bss;
#define flag_zero_initialized_in_bss global_options.x_flag_zero_initialized_in_bss
#endif
#ifdef GENERATOR_FILE
extern int flag_zero_link;
#else
  int x_flag_zero_link;
#define flag_zero_link global_options.x_flag_zero_link
#endif
#ifdef GENERATOR_FILE
extern int dwarf2out_as_loc_support;
#else
  int x_dwarf2out_as_loc_support;
#define dwarf2out_as_loc_support global_options.x_dwarf2out_as_loc_support
#endif
#ifdef GENERATOR_FILE
extern int dwarf2out_as_locview_support;
#else
  int x_dwarf2out_as_locview_support;
#define dwarf2out_as_locview_support global_options.x_dwarf2out_as_locview_support
#endif
#ifdef GENERATOR_FILE
extern int debug_column_info;
#else
  int x_debug_column_info;
#define debug_column_info global_options.x_debug_column_info
#endif
#ifdef GENERATOR_FILE
extern int flag_describe_dies;
#else
  int x_flag_describe_dies;
#define flag_describe_dies global_options.x_flag_describe_dies
#endif
#ifdef GENERATOR_FILE
extern int dwarf_version;
#else
  int x_dwarf_version;
#define dwarf_version global_options.x_dwarf_version
#endif
#ifdef GENERATOR_FILE
extern int dwarf_offset_size;
#else
  int x_dwarf_offset_size;
#define dwarf_offset_size global_options.x_dwarf_offset_size
#endif
#ifdef GENERATOR_FILE
extern int flag_gen_declaration;
#else
  int x_flag_gen_declaration;
#define flag_gen_declaration global_options.x_flag_gen_declaration
#endif
#ifdef GENERATOR_FILE
extern int debug_generate_pub_sections;
#else
  int x_debug_generate_pub_sections;
#define debug_generate_pub_sections global_options.x_debug_generate_pub_sections
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_gimple_stats;
#else
  int x_flag_lto_gimple_stats;
#define flag_lto_gimple_stats global_options.x_flag_lto_gimple_stats
#endif
#ifdef GENERATOR_FILE
extern int debug_inline_points;
#else
  int x_debug_inline_points;
#define debug_inline_points global_options.x_debug_inline_points
#endif
#ifdef GENERATOR_FILE
extern int debug_internal_reset_location_views;
#else
  int x_debug_internal_reset_location_views;
#define debug_internal_reset_location_views global_options.x_debug_internal_reset_location_views
#endif
#ifdef GENERATOR_FILE
extern int dwarf_record_gcc_switches;
#else
  int x_dwarf_record_gcc_switches;
#define dwarf_record_gcc_switches global_options.x_dwarf_record_gcc_switches
#endif
#ifdef GENERATOR_FILE
extern int dwarf_split_debug_info;
#else
  int x_dwarf_split_debug_info;
#define dwarf_split_debug_info global_options.x_dwarf_split_debug_info
#endif
#ifdef GENERATOR_FILE
extern int debug_nonbind_markers_p;
#else
  int x_debug_nonbind_markers_p;
#define debug_nonbind_markers_p global_options.x_debug_nonbind_markers_p
#endif
#ifdef GENERATOR_FILE
extern int dwarf_strict;
#else
  int x_dwarf_strict;
#define dwarf_strict global_options.x_dwarf_strict
#endif
#ifdef GENERATOR_FILE
extern int flag_gtoggle;
#else
  int x_flag_gtoggle;
#define flag_gtoggle global_options.x_flag_gtoggle
#endif
#ifdef GENERATOR_FILE
extern int debug_variable_location_views;
#else
  int x_debug_variable_location_views;
#define debug_variable_location_views global_options.x_debug_variable_location_views
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_dump_tool_help;
#else
  int x_flag_lto_dump_tool_help;
#define flag_lto_dump_tool_help global_options.x_flag_lto_dump_tool_help
#endif
#ifdef GENERATOR_FILE
extern const char *imultiarch;
#else
  const char *x_imultiarch;
#define imultiarch global_options.x_imultiarch
#endif
#ifdef GENERATOR_FILE
extern const char *plugindir_string;
#else
  const char *x_plugindir_string;
#define plugindir_string global_options.x_plugindir_string
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_dump_list;
#else
  int x_flag_lto_dump_list;
#define flag_lto_dump_list global_options.x_flag_lto_dump_list
#endif
#ifdef GENERATOR_FILE
extern enum ix86_align_data ix86_align_data_type;
#else
  enum ix86_align_data x_ix86_align_data_type;
#define ix86_align_data_type global_options.x_ix86_align_data_type
#endif
#ifdef GENERATOR_FILE
extern int ix86_apx_features;
#else
  int x_ix86_apx_features;
#define ix86_apx_features global_options.x_ix86_apx_features
#endif
#ifdef GENERATOR_FILE
extern int ix86_apx_inline_asm_use_gpr32;
#else
  int x_ix86_apx_inline_asm_use_gpr32;
#define ix86_apx_inline_asm_use_gpr32 global_options.x_ix86_apx_inline_asm_use_gpr32
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_arch_string;
#else
  const char *x_ix86_arch_string;
#define ix86_arch_string global_options.x_ix86_arch_string
#endif
#ifdef GENERATOR_FILE
extern enum asm_dialect ix86_asm_dialect;
#else
  enum asm_dialect x_ix86_asm_dialect;
#define ix86_asm_dialect global_options.x_ix86_asm_dialect
#endif
#ifdef GENERATOR_FILE
extern int ix86_branch_cost;
#else
  int x_ix86_branch_cost;
#define ix86_branch_cost global_options.x_ix86_branch_cost
#endif
#ifdef GENERATOR_FILE
extern int flag_cet_switch;
#else
  int x_flag_cet_switch;
#define flag_cet_switch global_options.x_flag_cet_switch
#endif
#ifdef GENERATOR_FILE
extern int ix86_direct_extern_access;
#else
  int x_ix86_direct_extern_access;
#define ix86_direct_extern_access global_options.x_ix86_direct_extern_access
#endif
#ifdef GENERATOR_FILE
extern int flag_dispatch_scheduler;
#else
  int x_flag_dispatch_scheduler;
#define flag_dispatch_scheduler global_options.x_flag_dispatch_scheduler
#endif
#ifdef GENERATOR_FILE
extern int ix86_dump_tunes;
#else
  int x_ix86_dump_tunes;
#define ix86_dump_tunes global_options.x_ix86_dump_tunes
#endif
#ifdef GENERATOR_FILE
extern int flag_fentry;
#else
  int x_flag_fentry;
#define flag_fentry global_options.x_flag_fentry
#endif
#ifdef GENERATOR_FILE
extern const char *fentry_name;
#else
  const char *x_fentry_name;
#define fentry_name global_options.x_fentry_name
#endif
#ifdef GENERATOR_FILE
extern const char *fentry_section;
#else
  const char *x_fentry_section;
#define fentry_section global_options.x_fentry_section
#endif
#ifdef GENERATOR_FILE
extern int ix86_force_drap;
#else
  int x_ix86_force_drap;
#define ix86_force_drap global_options.x_ix86_force_drap
#endif
#ifdef GENERATOR_FILE
extern int flag_force_indirect_call;
#else
  int x_flag_force_indirect_call;
#define flag_force_indirect_call global_options.x_flag_force_indirect_call
#endif
#ifdef GENERATOR_FILE
extern enum fpmath_unit ix86_fpmath;
#else
  enum fpmath_unit x_ix86_fpmath;
#define ix86_fpmath global_options.x_ix86_fpmath
#endif
#ifdef GENERATOR_FILE
extern enum indirect_branch ix86_function_return;
#else
  enum indirect_branch x_ix86_function_return;
#define ix86_function_return global_options.x_ix86_function_return
#endif
#ifdef GENERATOR_FILE
extern enum harden_sls ix86_harden_sls;
#else
  enum harden_sls x_ix86_harden_sls;
#define ix86_harden_sls global_options.x_ix86_harden_sls
#endif
#ifdef GENERATOR_FILE
extern int ix86_indirect_branch_cs_prefix;
#else
  int x_ix86_indirect_branch_cs_prefix;
#define ix86_indirect_branch_cs_prefix global_options.x_ix86_indirect_branch_cs_prefix
#endif
#ifdef GENERATOR_FILE
extern int ix86_indirect_branch_register;
#else
  int x_ix86_indirect_branch_register;
#define ix86_indirect_branch_register global_options.x_ix86_indirect_branch_register
#endif
#ifdef GENERATOR_FILE
extern enum indirect_branch ix86_indirect_branch;
#else
  enum indirect_branch x_ix86_indirect_branch;
#define ix86_indirect_branch global_options.x_ix86_indirect_branch
#endif
#ifdef GENERATOR_FILE
extern enum instrument_return ix86_instrument_return;
#else
  enum instrument_return x_ix86_instrument_return;
#define ix86_instrument_return global_options.x_ix86_instrument_return
#endif
#ifdef GENERATOR_FILE
extern enum lam_type ix86_lam_type;
#else
  enum lam_type x_ix86_lam_type;
#define ix86_lam_type global_options.x_ix86_lam_type
#endif
#ifdef GENERATOR_FILE
extern int ix86_section_threshold;
#else
  int x_ix86_section_threshold;
#define ix86_section_threshold global_options.x_ix86_section_threshold
#endif
#ifdef GENERATOR_FILE
extern int flag_manual_endbr;
#else
  int x_flag_manual_endbr;
#define flag_manual_endbr global_options.x_flag_manual_endbr
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_tune_memcpy_strategy;
#else
  const char *x_ix86_tune_memcpy_strategy;
#define ix86_tune_memcpy_strategy global_options.x_ix86_tune_memcpy_strategy
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_tune_memset_strategy;
#else
  const char *x_ix86_tune_memset_strategy;
#define ix86_tune_memset_strategy global_options.x_ix86_tune_memset_strategy
#endif
#ifdef GENERATOR_FILE
extern enum prefer_vector_width ix86_move_max;
#else
  enum prefer_vector_width x_ix86_move_max;
#define ix86_move_max global_options.x_ix86_move_max
#endif
#ifdef GENERATOR_FILE
extern int ix86_needed;
#else
  int x_ix86_needed;
#define ix86_needed global_options.x_ix86_needed
#endif
#ifdef GENERATOR_FILE
extern int ix86_tune_no_default;
#else
  int x_ix86_tune_no_default;
#define ix86_tune_no_default global_options.x_ix86_tune_no_default
#endif
#ifdef GENERATOR_FILE
extern int flag_nop_mcount;
#else
  int x_flag_nop_mcount;
#define flag_nop_mcount global_options.x_flag_nop_mcount
#endif
#ifdef GENERATOR_FILE
extern int ix86_noreturn_no_callee_saved_registers;
#else
  int x_ix86_noreturn_no_callee_saved_registers;
#define ix86_noreturn_no_callee_saved_registers global_options.x_ix86_noreturn_no_callee_saved_registers
#endif
#ifdef GENERATOR_FILE
extern int ix86_partial_vec_fp_math;
#else
  int x_ix86_partial_vec_fp_math;
#define ix86_partial_vec_fp_math global_options.x_ix86_partial_vec_fp_math
#endif
#ifdef GENERATOR_FILE
extern enum prefer_vector_width prefer_vector_width_type;
#else
  enum prefer_vector_width x_prefer_vector_width_type;
#define prefer_vector_width_type global_options.x_prefer_vector_width_type
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_recip_name;
#else
  const char *x_ix86_recip_name;
#define ix86_recip_name global_options.x_ix86_recip_name
#endif
#ifdef GENERATOR_FILE
extern int flag_record_mcount;
#else
  int x_flag_record_mcount;
#define flag_record_mcount global_options.x_flag_record_mcount
#endif
#ifdef GENERATOR_FILE
extern int ix86_flag_record_return;
#else
  int x_ix86_flag_record_return;
#define ix86_flag_record_return global_options.x_ix86_flag_record_return
#endif
#ifdef GENERATOR_FILE
extern int flag_skip_rax_setup;
#else
  int x_flag_skip_rax_setup;
#define flag_skip_rax_setup global_options.x_flag_skip_rax_setup
#endif
#ifdef GENERATOR_FILE
extern int ix86_sse2avx;
#else
  int x_ix86_sse2avx;
#define ix86_sse2avx global_options.x_ix86_sse2avx
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_stack_protector_guard_offset_str;
#else
  const char *x_ix86_stack_protector_guard_offset_str;
#define ix86_stack_protector_guard_offset_str global_options.x_ix86_stack_protector_guard_offset_str
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_stack_protector_guard_reg_str;
#else
  const char *x_ix86_stack_protector_guard_reg_str;
#define ix86_stack_protector_guard_reg_str global_options.x_ix86_stack_protector_guard_reg_str
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_stack_protector_guard_symbol_str;
#else
  const char *x_ix86_stack_protector_guard_symbol_str;
#define ix86_stack_protector_guard_symbol_str global_options.x_ix86_stack_protector_guard_symbol_str
#endif
#ifdef GENERATOR_FILE
extern enum stack_protector_guard ix86_stack_protector_guard;
#else
  enum stack_protector_guard x_ix86_stack_protector_guard;
#define ix86_stack_protector_guard global_options.x_ix86_stack_protector_guard
#endif
#ifdef GENERATOR_FILE
extern int ix86_force_align_arg_pointer;
#else
  int x_ix86_force_align_arg_pointer;
#define ix86_force_align_arg_pointer global_options.x_ix86_force_align_arg_pointer
#endif
#ifdef GENERATOR_FILE
extern enum prefer_vector_width ix86_store_max;
#else
  enum prefer_vector_width x_ix86_store_max;
#define ix86_store_max global_options.x_ix86_store_max
#endif
#ifdef GENERATOR_FILE
extern enum stringop_alg ix86_stringop_alg;
#else
  enum stringop_alg x_ix86_stringop_alg;
#define ix86_stringop_alg global_options.x_ix86_stringop_alg
#endif
#ifdef GENERATOR_FILE
extern enum tls_dialect ix86_tls_dialect;
#else
  enum tls_dialect x_ix86_tls_dialect;
#define ix86_tls_dialect global_options.x_ix86_tls_dialect
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_tune_ctrl_string;
#else
  const char *x_ix86_tune_ctrl_string;
#define ix86_tune_ctrl_string global_options.x_ix86_tune_ctrl_string
#endif
#ifdef GENERATOR_FILE
extern const char *ix86_tune_string;
#else
  const char *x_ix86_tune_string;
#define ix86_tune_string global_options.x_ix86_tune_string
#endif
#ifdef GENERATOR_FILE
extern int ix86_unroll_only_small_loops;
#else
  int x_ix86_unroll_only_small_loops;
#define ix86_unroll_only_small_loops global_options.x_ix86_unroll_only_small_loops
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_name_sort;
#else
  int x_flag_lto_name_sort;
#define flag_lto_name_sort global_options.x_flag_lto_name_sort
#endif
#ifdef GENERATOR_FILE
extern const char *asm_file_name;
#else
  const char *x_asm_file_name;
#define asm_file_name global_options.x_asm_file_name
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_dump_objects;
#else
  int x_flag_lto_dump_objects;
#define flag_lto_dump_objects global_options.x_flag_lto_dump_objects
#endif
#ifdef GENERATOR_FILE
extern int pass_exit_codes;
#else
  int x_pass_exit_codes;
#define pass_exit_codes global_options.x_pass_exit_codes
#endif
#ifdef GENERATOR_FILE
extern int flag_pedantic_errors;
#else
  int x_flag_pedantic_errors;
#define flag_pedantic_errors global_options.x_flag_pedantic_errors
#endif
#ifdef GENERATOR_FILE
extern int use_pipes;
#else
  int x_use_pipes;
#define use_pipes global_options.x_use_pipes
#endif
#ifdef GENERATOR_FILE
extern const char *print_file_name;
#else
  const char *x_print_file_name;
#define print_file_name global_options.x_print_file_name
#endif
#ifdef GENERATOR_FILE
extern int print_multi_directory;
#else
  int x_print_multi_directory;
#define print_multi_directory global_options.x_print_multi_directory
#endif
#ifdef GENERATOR_FILE
extern int print_multi_lib;
#else
  int x_print_multi_lib;
#define print_multi_lib global_options.x_print_multi_lib
#endif
#ifdef GENERATOR_FILE
extern int print_multi_os_directory;
#else
  int x_print_multi_os_directory;
#define print_multi_os_directory global_options.x_print_multi_os_directory
#endif
#ifdef GENERATOR_FILE
extern int print_multiarch;
#else
  int x_print_multiarch;
#define print_multiarch global_options.x_print_multiarch
#endif
#ifdef GENERATOR_FILE
extern const char *print_prog_name;
#else
  const char *x_print_prog_name;
#define print_prog_name global_options.x_print_prog_name
#endif
#ifdef GENERATOR_FILE
extern int print_search_dirs;
#else
  int x_print_search_dirs;
#define print_search_dirs global_options.x_print_search_dirs
#endif
#ifdef GENERATOR_FILE
extern int print_sysroot;
#else
  int x_print_sysroot;
#define print_sysroot global_options.x_print_sysroot
#endif
#ifdef GENERATOR_FILE
extern int print_sysroot_headers_suffix;
#else
  int x_print_sysroot_headers_suffix;
#define print_sysroot_headers_suffix global_options.x_print_sysroot_headers_suffix
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_print_value;
#else
  int x_flag_lto_print_value;
#define flag_lto_print_value global_options.x_flag_lto_print_value
#endif
#ifdef GENERATOR_FILE
extern int quiet_flag;
#else
  int x_quiet_flag;
#define quiet_flag global_options.x_quiet_flag
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_reverse_sort;
#else
  int x_flag_lto_reverse_sort;
#define flag_lto_reverse_sort global_options.x_flag_lto_reverse_sort
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_size_sort;
#else
  int x_flag_lto_size_sort;
#define flag_lto_size_sort global_options.x_flag_lto_size_sort
#endif
#ifdef GENERATOR_FILE
extern int flag_stdlib_kind;
#else
  int x_flag_stdlib_kind;
#define flag_stdlib_kind global_options.x_flag_stdlib_kind
#endif
#ifdef GENERATOR_FILE
extern const char *flag_lto_dump_symbol;
#else
  const char *x_flag_lto_dump_symbol;
#define flag_lto_dump_symbol global_options.x_flag_lto_dump_symbol
#endif
#ifdef GENERATOR_FILE
extern int report_times;
#else
  int x_report_times;
#define report_times global_options.x_report_times
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_tree_stats;
#else
  int x_flag_lto_tree_stats;
#define flag_lto_tree_stats global_options.x_flag_lto_tree_stats
#endif
#ifdef GENERATOR_FILE
extern int flag_lto_dump_type_stats;
#else
  int x_flag_lto_dump_type_stats;
#define flag_lto_dump_type_stats global_options.x_flag_lto_dump_type_stats
#endif
#ifdef GENERATOR_FILE
extern int flag_undef;
#else
  int x_flag_undef;
#define flag_undef global_options.x_flag_undef
#endif
#ifdef GENERATOR_FILE
extern int verbose_flag;
#else
  int x_verbose_flag;
#define verbose_flag global_options.x_verbose_flag
#endif
#ifdef GENERATOR_FILE
extern int version_flag;
#else
  int x_version_flag;
#define version_flag global_options.x_version_flag
#endif
#ifdef GENERATOR_FILE
extern int inhibit_warnings;
#else
  int x_inhibit_warnings;
#define inhibit_warnings global_options.x_inhibit_warnings
#endif
#ifdef GENERATOR_FILE
extern const char *wrapper_string;
#else
  const char *x_wrapper_string;
#define wrapper_string global_options.x_wrapper_string
#endif
#ifndef GENERATOR_FILE
  int x_VAR_malign_functions_;
#define x_VAR_malign_functions_ do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_malign_jumps_;
#define x_VAR_malign_jumps_ do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_malign_loops_;
#define x_VAR_malign_loops_ do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mdaz_ftz;
#define x_VAR_mdaz_ftz do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mmitigate_rop;
#define x_VAR_mmitigate_rop do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mmpx;
#define x_VAR_mmpx do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mpc32;
#define x_VAR_mpc32 do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mpc64;
#define x_VAR_mpc64 do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mpc80;
#define x_VAR_mpc80 do_not_use
#endif
#ifndef GENERATOR_FILE
  int x_VAR_mpcommit;
#define x_VAR_mpcommit do_not_use
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_associative_math;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_cx_limited_range;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_excess_precision;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_finite_math_only;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_errno_math;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_reciprocal_math;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_rounding_math;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_signaling_nans;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_signed_zeros;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_trapping_math;
#endif
#ifndef GENERATOR_FILE
  bool frontend_set_flag_unsafe_math_optimizations;
#endif
#ifndef GENERATOR_FILE
};
extern struct gcc_options global_options;
extern const struct gcc_options global_options_init;
extern struct gcc_options global_options_set;
#define target_flags_explicit global_options_set.x_target_flags
#endif
#endif

#if !defined(IN_LIBGCC2) && !defined(IN_TARGET_LIBS) && !defined(IN_RTS)

/* Structure to save/restore optimization and target specific options.  */
struct GTY(()) cl_optimization
{
  const char *x_str_align_functions;
  const char *x_str_align_jumps;
  const char *x_str_align_labels;
  const char *x_str_align_loops;
  const char *x_flag_patchable_function_entry;
  int x_param_align_loop_iterations;
  int x_param_align_threshold;
  int x_param_asan_protect_allocas;
  int x_param_asan_instrument_reads;
  int x_param_asan_instrument_writes;
  int x_param_asan_instrumentation_with_call_threshold;
  int x_param_asan_kernel_mem_intrinsic_prefix;
  int x_param_asan_memintrin;
  int x_param_asan_stack;
  int x_param_asan_use_after_return;
  int x_param_avg_loop_niter;
  int x_param_avoid_fma_max_bits;
  int x_param_builtin_expect_probability;
  int x_param_builtin_string_cmp_inline_length;
  int x_param_case_values_threshold;
  int x_param_comdat_sharing_probability;
  int x_param_construct_interfere_size;
  int x_param_destruct_interfere_size;
  int x_param_dse_max_alias_queries_per_store;
  int x_param_dse_max_object_size;
  int x_param_early_inlining_insns;
  int x_param_fsm_scale_path_stmts;
  int x_param_fully_pipelined_fma;
  int x_param_gcse_after_reload_critical_fraction;
  int x_param_gcse_after_reload_partial_fraction;
  int x_param_gcse_cost_distance_ratio;
  int x_param_gcse_unrestricted_cost;
  int x_param_graphite_max_arrays_per_scop;
  int x_param_graphite_max_nb_scop_params;
  int x_param_hardcfr_max_blocks;
  int x_param_hardcfr_max_inline_blocks;
  int x_param_hwasan_instrument_allocas;
  int x_param_hwasan_instrument_mem_intrinsics;
  int x_param_hwasan_instrument_reads;
  int x_param_hwasan_instrument_stack;
  int x_param_hwasan_instrument_writes;
  int x_param_hwasan_random_frame_tag;
  int x_param_inline_heuristics_hint_percent;
  int x_param_inline_min_speedup;
  int x_param_inline_unit_growth;
  int x_param_ipa_cp_eval_threshold;
  int x_param_ipa_cp_large_unit_insns;
  int x_param_ipa_cp_loop_hint_bonus;
  int x_param_ipa_cp_max_recursive_depth;
  int x_param_ipa_cp_min_recursive_probability;
  int x_param_ipa_cp_profile_count_base;
  int x_param_ipa_cp_recursion_penalty;
  int x_param_ipa_cp_recursive_freq_factor;
  int x_param_ipa_cp_single_call_penalty;
  int x_param_ipa_cp_unit_growth;
  int x_param_ipa_cp_value_list_size;
  int x_param_ipa_jump_function_lookups;
  int x_param_ipa_max_aa_steps;
  int x_param_ipa_max_agg_items;
  int x_param_ipa_max_loop_predicates;
  int x_param_ipa_max_param_expr_ops;
  int x_param_ipa_max_switch_predicate_bounds;
  int x_param_ipa_sra_deref_prob_threshold;
  int x_param_ipa_sra_max_replacements;
  int x_param_ipa_sra_ptr_growth_factor;
  int x_param_ipa_sra_ptrwrap_growth_factor;
  int x_param_ira_consider_dup_in_all_alts;
  int x_param_ira_loop_reserved_regs;
  int x_param_ira_max_conflict_table_size;
  int x_param_ira_max_loops_num;
  int x_param_ira_simple_lra_insn_threshold;
  int x_param_iv_always_prune_cand_set_bound;
  int x_param_iv_consider_all_candidates_bound;
  int x_param_iv_max_considered_uses;
  int x_param_jump_table_max_growth_ratio_for_size;
  int x_param_jump_table_max_growth_ratio_for_speed;
  int x_param_l1_cache_line_size;
  int x_param_l1_cache_size;
  int x_param_l2_cache_size;
  int x_param_large_function_growth;
  int x_param_large_function_insns;
  int x_param_stack_frame_growth;
  int x_param_large_stack_frame;
  int x_param_large_unit_insns;
  int x_param_lim_expensive;
  int x_param_loop_block_tile_size;
  int x_param_loop_interchange_max_num_stmts;
  int x_param_loop_interchange_stride_ratio;
  int x_param_loop_invariant_max_bbs_in_loop;
  int x_param_loop_max_datarefs_for_datadeps;
  int x_param_loop_versioning_max_inner_insns;
  int x_param_loop_versioning_max_outer_insns;
  int x_param_lra_inheritance_ebb_probability_cutoff;
  int x_param_lra_max_considered_reload_pseudos;
  int x_param_max_average_unrolled_insns;
  int x_param_max_combine_insns;
  int x_param_max_unroll_iterations;
  int x_param_max_completely_peel_times;
  int x_param_max_completely_peeled_insns;
  int x_param_max_crossjump_edges;
  int x_param_max_cse_insns;
  int x_param_max_cse_path_length;
  int x_param_max_cselib_memory_locations;
  int x_param_max_debug_marker_count;
  int x_param_max_delay_slot_insn_search;
  int x_param_max_delay_slot_live_search;
  int x_param_max_dse_active_local_stores;
  int x_param_early_inliner_max_iterations;
  int x_param_max_find_base_term_values;
  int x_param_max_fsm_thread_path_insns;
  int x_param_max_gcse_insertion_ratio;
  int x_param_max_gcse_memory;
  int x_param_max_goto_duplication_insns;
  int x_param_max_grow_copy_bb_insns;
  int x_param_max_hoist_depth;
  int x_param_inline_functions_called_once_insns;
  int x_param_inline_functions_called_once_loop_depth;
  int x_param_max_inline_insns_auto;
  int x_param_max_inline_insns_recursive_auto;
  int x_param_max_inline_insns_recursive;
  int x_param_max_inline_insns_single;
  int x_param_max_inline_insns_size;
  int x_param_max_inline_insns_small;
  int x_param_max_inline_recursive_depth_auto;
  int x_param_max_inline_recursive_depth;
  int x_param_max_isl_operations;
  int x_param_max_iterations_computation_cost;
  int x_param_max_iterations_to_track;
  int x_param_max_jump_thread_duplication_stmts;
  int x_param_max_jump_thread_paths;
  int x_param_max_last_value_rtl;
  int x_param_max_loop_header_insns;
  int x_param_max_modulo_backtrack_attempts;
  int x_param_max_partial_antic_length;
  int x_param_max_peel_branches;
  int x_param_max_peel_times;
  int x_param_max_peeled_insns;
  int x_param_max_pending_list_length;
  int x_param_max_pipeline_region_blocks;
  int x_param_max_pipeline_region_insns;
  int x_param_max_pow_sqrt_depth;
  int x_param_max_predicted_iterations;
  int x_param_max_reload_search_insns;
  int x_param_max_rtl_if_conversion_insns;
  int x_param_max_rtl_if_conversion_predictable_cost;
  int x_param_max_rtl_if_conversion_unpredictable_cost;
  int x_param_max_sched_extend_regions_iters;
  int x_param_max_sched_insn_conflict_delay;
  int x_param_max_sched_ready_insns;
  int x_param_max_sched_region_blocks;
  int x_param_max_sched_region_insns;
  int x_param_max_slsr_candidate_scan;
  int x_param_max_speculative_devirt_maydefs;
  int x_param_max_stores_to_merge;
  int x_param_max_stores_to_sink;
  int x_param_max_tail_merge_comparisons;
  int x_param_max_tail_merge_iterations;
  int x_param_max_tracked_strlens;
  int x_param_max_tree_if_conversion_phi_args;
  int x_param_max_unroll_times;
  int x_param_max_unrolled_insns;
  int x_param_max_unswitch_depth;
  int x_param_max_unswitch_insns;
  int x_param_max_variable_expansions;
  int x_param_max_vartrack_expr_depth;
  int x_param_max_vartrack_reverse_op_size;
  int x_param_max_vartrack_size;
  int x_param_min_crossjump_insns;
  int x_param_min_inline_recursive_probability;
  int x_param_min_insn_to_prefetch_ratio;
  int x_param_min_loop_cond_split_prob;
  int x_param_min_pagesize;
  int x_param_min_size_for_stack_sharing;
  int x_param_min_spec_prob;
  int x_param_min_vect_loop_bound;
  int x_param_modref_max_accesses;
  int x_param_modref_max_adjustments;
  int x_param_modref_max_bases;
  int x_param_modref_max_depth;
  int x_param_modref_max_escape_points;
  int x_param_modref_max_refs;
  int x_param_modref_max_tests;
  int x_param_parloops_chunk_size;
  int x_param_parloops_min_per_thread;
  int x_param_parloops_schedule;
  int x_param_partial_inlining_entry_probability;
  int x_param_predictable_branch_outcome;
  int x_param_prefetch_dynamic_strides;
  int x_param_prefetch_latency;
  int x_param_prefetch_min_insn_to_mem_ratio;
  int x_param_prefetch_minimum_stride;
  int x_param_ranger_logical_depth;
  int x_param_ranger_recompute_depth;
  int x_param_relation_block_limit;
  int x_param_rpo_vn_max_loop_depth;
  int x_param_sccvn_max_alias_queries_per_access;
  int x_param_scev_max_expr_complexity;
  int x_param_scev_max_expr_size;
  int x_param_sched_autopref_queue_depth;
  int x_param_sched_mem_true_dep_cost;
  int x_param_sched_pressure_algorithm;
  int x_param_sched_spec_prob_cutoff;
  int x_param_sched_state_edge_prob_cutoff;
  int x_param_selsched_insns_to_rename;
  int x_param_selsched_max_lookahead;
  int x_param_selsched_max_sched_times;
  int x_param_simultaneous_prefetches;
  int x_param_sink_frequency_threshold;
  int x_param_sms_dfa_history;
  int x_param_sms_loop_average_count_threshold;
  int x_param_sms_max_ii_factor;
  int x_param_sms_min_sc;
  int x_param_sra_max_propagations;
  int x_param_sra_max_scalarization_size_size;
  int x_param_sra_max_scalarization_size_speed;
  int x_param_ssa_name_def_chain_limit;
  int x_param_ssp_buffer_size;
  int x_param_stack_clash_protection_guard_size;
  int x_param_stack_clash_protection_probe_interval;
  int x_param_store_merging_allow_unaligned;
  int x_param_store_merging_max_size;
  int x_param_switch_conversion_branch_ratio;
  int x_param_tm_max_aggregate_size;
  int x_param_tracer_dynamic_coverage_feedback;
  int x_param_tracer_dynamic_coverage;
  int x_param_tracer_max_code_growth;
  int x_param_tracer_min_branch_probability_feedback;
  int x_param_tracer_min_branch_probability;
  int x_param_tracer_min_branch_ratio;
  int x_param_tree_reassoc_width;
  int x_param_uninit_control_dep_attempts;
  int x_param_uninit_max_chain_len;
  int x_param_uninit_max_num_chains;
  int x_param_uninlined_function_insns;
  int x_param_uninlined_function_time;
  int x_param_uninlined_function_thunk_insns;
  int x_param_uninlined_function_thunk_time;
  int x_param_unlikely_bb_count_fraction;
  int x_param_unroll_jam_max_unroll;
  int x_param_unroll_jam_min_percent;
  int x_param_use_after_scope_direct_emission_threshold;
  int x_param_vect_epilogues_nomask;
  int x_param_vect_induction_float;
  int x_param_vect_inner_loop_cost_factor;
  int x_param_vect_max_layout_candidates;
  int x_param_vect_max_peeling_for_alignment;
  int x_param_vect_max_version_for_alias_checks;
  int x_param_vect_max_version_for_alignment_checks;
  int x_param_vect_partial_vector_usage;
  int x_param_vrp_sparse_threshold;
  int x_param_vrp_switch_limit;
  int x_param_vrp_vector_threshold;
  int x_flag_min_function_alignment;
  int x_flag_openmp_target_simd_clone;
  int x_flag_sched_stalled_insns;
  int x_flag_sched_stalled_insns_dep;
  int x_flag_tree_parallelize_loops;
  enum ranger_debug x_param_ranger_debug;
  enum threader_debug x_param_threader_debug;
  enum excess_precision x_flag_excess_precision;
  enum fp_contract_mode x_flag_fp_contract_mode;
  enum hardcfr_noret x_flag_harden_control_flow_redundancy_check_noreturn;
  enum ilsop_fn x_flag_inline_stringops;
  enum ira_algorithm x_flag_ira_algorithm;
  enum ira_region x_flag_ira_region;
  enum live_patching_level x_flag_live_patching;
  enum reorder_blocks_algorithm x_flag_reorder_blocks_algorithm;
  enum vect_cost_model x_flag_simd_cost_model;
  enum stack_reuse_level x_flag_stack_reuse;
  enum auto_init_type x_flag_auto_var_init;
  enum vect_cost_model x_flag_vect_cost_model;
  unsigned char x_optimize;
  unsigned char x_optimize_size;
  unsigned char x_optimize_debug;
  unsigned char x_optimize_fast;
  signed char x_warn_inline;
  signed char x_flag_aggressive_loop_optimizations;
  signed char x_flag_align_functions;
  signed char x_flag_align_jumps;
  signed char x_flag_align_labels;
  signed char x_flag_align_loops;
  signed char x_flag_allocation_dce;
  signed char x_flag_store_data_races;
  signed char x_flag_associative_math;
  signed char x_flag_asynchronous_unwind_tables;
  signed char x_flag_auto_inc_dec;
  signed char x_flag_bit_tests;
  signed char x_flag_branch_on_count_reg;
  signed char x_flag_branch_probabilities;
  signed char x_flag_caller_saves;
  signed char x_flag_code_hoisting;
  signed char x_flag_combine_stack_adjustments;
  signed char x_flag_compare_elim_after_reload;
  signed char x_flag_conserve_stack;
  signed char x_flag_cprop_registers;
  signed char x_flag_crossjumping;
  signed char x_flag_cse_follow_jumps;
  signed char x_flag_cx_fortran_rules;
  signed char x_flag_cx_limited_range;
  signed char x_flag_dce;
  signed char x_flag_defer_pop;
  signed char x_flag_delayed_branch;
  signed char x_flag_delete_dead_exceptions;
  signed char x_flag_delete_null_pointer_checks;
  signed char x_flag_devirtualize;
  signed char x_flag_devirtualize_speculatively;
  signed char x_flag_dse;
  signed char x_flag_early_inlining;
  signed char x_flag_exceptions;
  signed char x_flag_expensive_optimizations;
  signed char x_flag_finite_loops;
  signed char x_flag_finite_math_only;
  signed char x_flag_float_store;
  signed char x_flag_fold_mem_offsets;
  signed char x_flag_fold_simple_inlines;
  signed char x_flag_forward_propagate;
  signed char x_flag_fp_int_builtin_inexact;
  signed char x_flag_no_function_cse;
  signed char x_flag_gcse;
  signed char x_flag_gcse_after_reload;
  signed char x_flag_gcse_las;
  signed char x_flag_gcse_lm;
  signed char x_flag_gcse_sm;
  signed char x_flag_graphite;
  signed char x_flag_graphite_identity;
  signed char x_flag_guess_branch_prob;
  signed char x_flag_harden_control_flow_redundancy_check_exceptions;
  signed char x_flag_harden_control_flow_redundancy_check_returning_calls;
  signed char x_flag_harden_control_flow_redundancy_skip_leaf;
  signed char x_flag_harden_compares;
  signed char x_flag_harden_conditional_branches;
  signed char x_flag_harden_control_flow_redundancy;
  signed char x_flag_hoist_adjacent_loads;
  signed char x_flag_if_conversion;
  signed char x_flag_if_conversion2;
  signed char x_flag_indirect_inlining;
  signed char x_flag_no_inline;
  signed char x_flag_inline_atomics;
  signed char x_flag_inline_functions;
  signed char x_flag_inline_functions_called_once;
  signed char x_flag_inline_small_functions;
  signed char x_flag_ipa_bit_cp;
  signed char x_flag_ipa_cp;
  signed char x_flag_ipa_cp_clone;
  signed char x_flag_ipa_icf;
  signed char x_flag_ipa_icf_functions;
  signed char x_flag_ipa_icf_variables;
  signed char x_flag_ipa_modref;
  signed char x_flag_ipa_profile;
  signed char x_flag_ipa_pta;
  signed char x_flag_ipa_pure_const;
  signed char x_flag_ipa_ra;
  signed char x_flag_ipa_reference;
  signed char x_flag_ipa_reference_addressable;
  signed char x_flag_ipa_sra;
  signed char x_flag_ipa_stack_alignment;
  signed char x_flag_ipa_strict_aliasing;
  signed char x_flag_ipa_vrp;
  signed char x_flag_ira_hoist_pressure;
  signed char x_flag_ira_loop_pressure;
  signed char x_flag_ira_share_save_slots;
  signed char x_flag_ira_share_spill_slots;
  signed char x_flag_isolate_erroneous_paths_attribute;
  signed char x_flag_isolate_erroneous_paths_dereference;
  signed char x_flag_ivopts;
  signed char x_flag_jump_tables;
  signed char x_flag_keep_gc_roots_live;
  signed char x_flag_lifetime_dse;
  signed char x_flag_limit_function_alignment;
  signed char x_flag_live_range_shrinkage;
  signed char x_flag_loop_interchange;
  signed char x_flag_loop_nest_optimize;
  signed char x_flag_loop_parallelize_all;
  signed char x_flag_unroll_jam;
  signed char x_flag_lra_remat;
  signed char x_flag_errno_math;
  signed char x_flag_modulo_sched;
  signed char x_flag_modulo_sched_allow_regmoves;
  signed char x_flag_move_loop_invariants;
  signed char x_flag_move_loop_stores;
  signed char x_flag_non_call_exceptions;
  signed char x_flag_nothrow_opt;
  signed char x_flag_omit_frame_pointer;
  signed char x_flag_opt_info;
  signed char x_flag_optimize_sibling_calls;
  signed char x_flag_optimize_strlen;
  signed char x_flag_pack_struct;
  signed char x_flag_partial_inlining;
  signed char x_flag_peel_loops;
  signed char x_flag_no_peephole;
  signed char x_flag_peephole2;
  signed char x_flag_plt;
  signed char x_flag_predictive_commoning;
  signed char x_flag_prefetch_loop_arrays;
  signed char x_flag_printf_return_value;
  signed char x_flag_profile_partial_training;
  signed char x_flag_profile_reorder_functions;
  signed char x_flag_reciprocal_math;
  signed char x_flag_ree;
  signed char x_flag_pcc_struct_return;
  signed char x_flag_rename_registers;
  signed char x_flag_reorder_blocks;
  signed char x_flag_reorder_blocks_and_partition;
  signed char x_flag_reorder_functions;
  signed char x_flag_rerun_cse_after_loop;
  signed char x_flag_resched_modulo_sched;
  signed char x_flag_rounding_math;
  signed char x_flag_rtti;
  signed char x_flag_save_optimization_record;
  signed char x_flag_sched_critical_path_heuristic;
  signed char x_flag_sched_dep_count_heuristic;
  signed char x_flag_sched_group_heuristic;
  signed char x_flag_schedule_interblock;
  signed char x_flag_sched_last_insn_heuristic;
  signed char x_flag_sched_pressure;
  signed char x_flag_sched_rank_heuristic;
  signed char x_flag_schedule_speculative;
  signed char x_flag_sched_spec_insn_heuristic;
  signed char x_flag_schedule_speculative_load;
  signed char x_flag_schedule_speculative_load_dangerous;
  signed char x_flag_sched2_use_superblocks;
  signed char x_flag_schedule_fusion;
  signed char x_flag_schedule_insns;
  signed char x_flag_schedule_insns_after_reload;
  signed char x_flag_section_anchors;
  signed char x_flag_sel_sched_pipelining;
  signed char x_flag_sel_sched_pipelining_outer_loops;
  signed char x_flag_sel_sched_reschedule_pipelined;
  signed char x_flag_selective_scheduling;
  signed char x_flag_selective_scheduling2;
  signed char x_flag_semantic_interposition;
  signed char x_flag_short_enums;
  signed char x_flag_short_wchar;
  signed char x_flag_shrink_wrap;
  signed char x_flag_shrink_wrap_separate;
  signed char x_flag_signaling_nans;
  signed char x_flag_signed_zeros;
  signed char x_flag_single_precision_constant;
  signed char x_flag_split_ivs_in_unroller;
  signed char x_flag_split_loops;
  signed char x_flag_split_paths;
  signed char x_flag_split_wide_types;
  signed char x_flag_split_wide_types_early;
  signed char x_flag_ssa_backprop;
  signed char x_flag_ssa_phiopt;
  signed char x_flag_stack_clash_protection;
  signed char x_flag_stack_protect;
  signed char x_flag_stdarg_opt;
  signed char x_flag_store_merging;
  signed char x_flag_strict_aliasing;
  signed char x_flag_strict_enums;
  signed char x_flag_strict_volatile_bitfields;
  signed char x_flag_thread_jumps;
  signed char x_flag_threadsafe_statics;
  signed char x_flag_toplevel_reorder;
  signed char x_flag_tracer;
  signed char x_flag_trapping_math;
  signed char x_flag_trapv;
  signed char x_flag_tree_bit_ccp;
  signed char x_flag_tree_builtin_call_dce;
  signed char x_flag_tree_ccp;
  signed char x_flag_tree_ch;
  signed char x_flag_tree_coalesce_vars;
  signed char x_flag_tree_copy_prop;
  signed char x_flag_tree_cselim;
  signed char x_flag_tree_dce;
  signed char x_flag_tree_dom;
  signed char x_flag_tree_dse;
  signed char x_flag_tree_forwprop;
  signed char x_flag_tree_fre;
  signed char x_flag_tree_loop_distribute_patterns;
  signed char x_flag_tree_loop_distribution;
  signed char x_flag_tree_loop_if_convert;
  signed char x_flag_tree_loop_im;
  signed char x_flag_tree_loop_ivcanon;
  signed char x_flag_tree_loop_optimize;
  signed char x_flag_tree_loop_vectorize;
  signed char x_flag_tree_live_range_split;
  signed char x_flag_tree_partial_pre;
  signed char x_flag_tree_phiprop;
  signed char x_flag_tree_pre;
  signed char x_flag_tree_pta;
  signed char x_flag_tree_reassoc;
  signed char x_flag_tree_scev_cprop;
  signed char x_flag_tree_sink;
  signed char x_flag_tree_slp_vectorize;
  signed char x_flag_tree_slsr;
  signed char x_flag_tree_sra;
  signed char x_flag_tree_switch_conversion;
  signed char x_flag_tree_tail_merge;
  signed char x_flag_tree_ter;
  signed char x_flag_tree_vectorize;
  signed char x_flag_tree_vrp;
  signed char x_flag_unconstrained_commons;
  signed char x_flag_unreachable_traps;
  signed char x_flag_unroll_all_loops;
  signed char x_flag_cunroll_grow_size;
  signed char x_flag_unroll_loops;
  signed char x_flag_unsafe_math_optimizations;
  signed char x_flag_unswitch_loops;
  signed char x_flag_unwind_tables;
  signed char x_flag_var_tracking;
  signed char x_flag_var_tracking_assignments;
  signed char x_flag_var_tracking_assignments_toggle;
  signed char x_flag_var_tracking_uninit;
  signed char x_flag_variable_expansion_in_unroller;
  signed char x_flag_version_loops_for_strides;
  signed char x_flag_value_profile_transformations;
  signed char x_flag_web;
  signed char x_flag_wrapv;
  signed char x_flag_wrapv_pointer;
  signed char x_debug_nonbind_markers_p;
  /* 512 members */
  unsigned HOST_WIDE_INT explicit_mask[8];
};

/* Structure to save/restore selected target specific options.  */
struct GTY(()) cl_target_option
{
  HOST_WIDE_INT x_ix86_isa_flags2_explicit;
  HOST_WIDE_INT x_ix86_isa_flags_explicit;
  HOST_WIDE_INT x_ix86_no_avx10_1_explicit;
  HOST_WIDE_INT x_ix86_no_avx512_explicit;
  const char *x_ix86_arch_string;
  const char *x_ix86_recip_name;
  const char *x_ix86_tune_ctrl_string;
  const char *x_ix86_tune_memcpy_strategy;
  const char *x_ix86_tune_memset_strategy;
  const char *x_ix86_tune_string;
  HOST_WIDE_INT x_ix86_stack_protector_guard_offset;
  addr_space_t x_ix86_stack_protector_guard_reg;
  bool x_ix86_unsafe_math_optimizations;
  /* - */ int x_target_flags;
  HOST_WIDE_INT x_ix86_isa_flags;
  HOST_WIDE_INT x_ix86_isa_flags2;
  enum fpmath_unit x_ix86_fpmath;
  /* - */ int x_ix86_target_flags;
  enum prefer_vector_width x_ix86_move_max;
  enum prefer_vector_width x_prefer_vector_width_type;
  const char *x_ix86_stack_protector_guard_offset_str;
  const char *x_ix86_stack_protector_guard_reg_str;
  const char *x_ix86_stack_protector_guard_symbol_str;
  enum prefer_vector_width x_ix86_store_max;
  enum asm_dialect x_ix86_asm_dialect;
  enum stack_protector_guard x_ix86_stack_protector_guard;
  enum stringop_alg x_ix86_stringop_alg;
  enum tls_dialect x_ix86_tls_dialect;
  enum calling_abi x_ix86_abi;
  enum cmodel x_ix86_cmodel;
  enum excess_precision x_ix86_excess_precision;
  enum ix86_veclibabi x_ix86_veclibabi_type;
  enum pmode x_ix86_pmode;
  int x_ix86_branch_cost;
  int x_ix86_dump_tunes;
  int x_ix86_force_align_arg_pointer;
  int x_ix86_force_drap;
  int x_ix86_section_threshold;
  int x_ix86_sse2avx;
  int x_ix86_tune_no_default;
  int x_recip_mask_explicit;
  int x_ix86_incoming_stack_boundary_arg;
  int x_ix86_preferred_stack_boundary_arg;
  int x_ix86_regparm;
  int x_recip_mask;
  int x_ix86_apx_features;
  unsigned char arch;
  unsigned char arch_specified;
  unsigned char branch_cost;
  unsigned char prefetch_sse;
  unsigned char schedule;
  unsigned char tune;
  unsigned char tune_defaulted;
  signed char x_flag_fentry;
  signed char x_ix86_needed;
  signed char x_ix86_unroll_only_small_loops;
  /* 23 members */
  unsigned HOST_WIDE_INT explicit_mask[1];
  /* - */ int explicit_mask_target_flags;
  HOST_WIDE_INT explicit_mask_ix86_isa_flags;
  HOST_WIDE_INT explicit_mask_ix86_isa_flags2;
  /* - */ int explicit_mask_ix86_target_flags;
};


/* Save optimization variables into a structure.  */
extern void cl_optimization_save (struct cl_optimization *, struct gcc_options *, struct gcc_options *);

/* Restore optimization variables from a structure.  */
extern void cl_optimization_restore (struct gcc_options *, struct gcc_options *, struct cl_optimization *);

/* Print optimization variables from a structure.  */
extern void cl_optimization_print (FILE *, int, struct cl_optimization *);

/* Print different optimization variables from structures provided as arguments.  */
extern void cl_optimization_print_diff (FILE *, int, cl_optimization *ptr1, cl_optimization *ptr2);

/* Save selected option variables into a structure.  */
extern void cl_target_option_save (struct cl_target_option *, struct gcc_options *, struct gcc_options *);

/* Restore selected option variables from a structure.  */
extern void cl_target_option_restore (struct gcc_options *, struct gcc_options *, struct cl_target_option *);

/* Print target option variables from a structure.  */
extern void cl_target_option_print (FILE *, int, struct cl_target_option *);

/* Print different target option variables from structures provided as arguments.  */
extern void cl_target_option_print_diff (FILE *, int, cl_target_option *ptr1, cl_target_option *ptr2);

/* Compare two target option variables from a structure.  */
extern bool cl_target_option_eq (const struct cl_target_option *, const struct cl_target_option *);

/* Free heap memory used by target option variables.  */
extern void cl_target_option_free (struct cl_target_option *);

/* Hash option variables from a structure.  */
extern hashval_t cl_target_option_hash (const struct cl_target_option *);

/* Hash optimization from a structure.  */
extern hashval_t cl_optimization_hash (const struct cl_optimization *);

/* Compare two optimization options.  */
extern bool cl_optimization_option_eq (cl_optimization const *ptr1, cl_optimization const *ptr2);

/* Free heap memory used by optimization options.  */
extern void cl_optimization_option_free (cl_optimization *ptr1);

/* Compare and report difference for a part of cl_optimization options.  */
extern void cl_optimization_compare (gcc_options *ptr1, gcc_options *ptr2);

/* Generator files may not have access to location_t, and don't need these.  */
#if defined(UNKNOWN_LOCATION)
bool                                                                  
common_handle_option_auto (struct gcc_options *opts,                  
                           struct gcc_options *opts_set,              
                           const struct cl_decoded_option *decoded,   
                           unsigned int lang_mask, int kind,          
                           location_t loc,                            
                           const struct cl_option_handlers *handlers, 
                           diagnostic_context *dc);                   
bool
Ada_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
AdaSCIL_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
AdaWhy_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
C_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
CXX_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
D_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
Fortran_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
Go_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
LTO_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
LTODump_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
ModulaX2_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
ObjC_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
ObjCXX_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
bool
Rust_handle_option_auto (struct gcc_options *opts,
                           struct gcc_options *opts_set,
                           size_t scode, const char *arg,
                           HOST_WIDE_INT value,
                           unsigned int lang_mask, int kind,
                           location_t loc,
                           const struct cl_option_handlers *handlers,
                           diagnostic_context *dc);
void cpp_handle_option_auto (const struct gcc_options * opts, size_t scode,
                             struct cpp_options * cpp_opts);
void init_global_opts_from_cpp(struct gcc_options * opts,      
                               const struct cpp_options * cpp_opts);
#endif
#endif

#define MASK_128BIT_LONG_DOUBLE (1U << 0)
#define OPTION_MASK_CODE16 (HOST_WIDE_INT_1U << 0)
#define OPTION_MASK_ISA_64BIT (HOST_WIDE_INT_1U << 1)
#define OPTION_MASK_ISA_3DNOW (HOST_WIDE_INT_1U << 2)
#define OPTION_MASK_ISA_3DNOW_A (HOST_WIDE_INT_1U << 3)
#define OPTION_MASK_ABI_64 (HOST_WIDE_INT_1U << 4)
#define MASK_80387 (1U << 1)
#define MASK_USE_8BIT_IDIV (1U << 2)
#define OPTION_MASK_ISA_ABM (HOST_WIDE_INT_1U << 5)
#define MASK_ACCUMULATE_OUTGOING_ARGS (1U << 3)
#define OPTION_MASK_ISA_ADX (HOST_WIDE_INT_1U << 6)
#define OPTION_MASK_ISA_AES (HOST_WIDE_INT_1U << 7)
#define MASK_ALIGN_DOUBLE (1U << 4)
#define MASK_NO_ALIGN_STRINGOPS (1U << 5)
#define OPTION_MASK_ISA2_AMX_BF16 (HOST_WIDE_INT_1U << 0)
#define OPTION_MASK_ISA2_AMX_COMPLEX (HOST_WIDE_INT_1U << 1)
#define OPTION_MASK_ISA2_AMX_FP16 (HOST_WIDE_INT_1U << 2)
#define OPTION_MASK_ISA2_AMX_INT8 (HOST_WIDE_INT_1U << 3)
#define OPTION_MASK_ISA2_AMX_TILE (HOST_WIDE_INT_1U << 4)
#define OPTION_MASK_ISA2_APX_F (HOST_WIDE_INT_1U << 5)
#define OPTION_MASK_ISA_AVX (HOST_WIDE_INT_1U << 8)
#define OPTION_MASK_ISA2_AVX10_1_256 (HOST_WIDE_INT_1U << 6)
#define OPTION_MASK_ISA2_AVX10_1_512 (HOST_WIDE_INT_1U << 7)
#define OPTION_MASK_ISA_AVX2 (HOST_WIDE_INT_1U << 9)
#define MASK_AVX256_SPLIT_UNALIGNED_LOAD (1U << 6)
#define MASK_AVX256_SPLIT_UNALIGNED_STORE (1U << 7)
#define OPTION_MASK_ISA2_AVX5124FMAPS (HOST_WIDE_INT_1U << 8)
#define OPTION_MASK_ISA2_AVX5124VNNIW (HOST_WIDE_INT_1U << 9)
#define OPTION_MASK_ISA2_AVX512BF16 (HOST_WIDE_INT_1U << 10)
#define OPTION_MASK_ISA_AVX512BITALG (HOST_WIDE_INT_1U << 10)
#define OPTION_MASK_ISA_AVX512BW (HOST_WIDE_INT_1U << 11)
#define OPTION_MASK_ISA_AVX512CD (HOST_WIDE_INT_1U << 12)
#define OPTION_MASK_ISA_AVX512DQ (HOST_WIDE_INT_1U << 13)
#define OPTION_MASK_ISA_AVX512ER (HOST_WIDE_INT_1U << 14)
#define OPTION_MASK_ISA_AVX512F (HOST_WIDE_INT_1U << 15)
#define OPTION_MASK_ISA2_AVX512FP16 (HOST_WIDE_INT_1U << 11)
#define OPTION_MASK_ISA_AVX512IFMA (HOST_WIDE_INT_1U << 16)
#define OPTION_MASK_ISA_AVX512PF (HOST_WIDE_INT_1U << 17)
#define OPTION_MASK_ISA_AVX512VBMI (HOST_WIDE_INT_1U << 18)
#define OPTION_MASK_ISA_AVX512VBMI2 (HOST_WIDE_INT_1U << 19)
#define OPTION_MASK_ISA_AVX512VL (HOST_WIDE_INT_1U << 20)
#define OPTION_MASK_ISA_AVX512VNNI (HOST_WIDE_INT_1U << 21)
#define OPTION_MASK_ISA2_AVX512VP2INTERSECT (HOST_WIDE_INT_1U << 12)
#define OPTION_MASK_ISA_AVX512VPOPCNTDQ (HOST_WIDE_INT_1U << 22)
#define OPTION_MASK_ISA2_AVXIFMA (HOST_WIDE_INT_1U << 13)
#define OPTION_MASK_ISA2_AVXNECONVERT (HOST_WIDE_INT_1U << 14)
#define OPTION_MASK_ISA2_AVXVNNI (HOST_WIDE_INT_1U << 15)
#define OPTION_MASK_ISA2_AVXVNNIINT16 (HOST_WIDE_INT_1U << 16)
#define OPTION_MASK_ISA2_AVXVNNIINT8 (HOST_WIDE_INT_1U << 17)
#define OPTION_MASK_ISA_BMI (HOST_WIDE_INT_1U << 23)
#define OPTION_MASK_ISA_BMI2 (HOST_WIDE_INT_1U << 24)
#define MASK_CALL_MS2SYSV_XLOGUES (1U << 8)
#define MASK_CLD (1U << 9)
#define OPTION_MASK_ISA2_CLDEMOTE (HOST_WIDE_INT_1U << 18)
#define OPTION_MASK_ISA_CLFLUSHOPT (HOST_WIDE_INT_1U << 25)
#define OPTION_MASK_ISA_CLWB (HOST_WIDE_INT_1U << 26)
#define OPTION_MASK_ISA2_CLZERO (HOST_WIDE_INT_1U << 19)
#define OPTION_MASK_ISA2_CMPCCXADD (HOST_WIDE_INT_1U << 20)
#define OPTION_MASK_ISA_CRC32 (HOST_WIDE_INT_1U << 27)
#define OPTION_MASK_ISA2_CX16 (HOST_WIDE_INT_1U << 21)
#define OPTION_MASK_ISA2_ENQCMD (HOST_WIDE_INT_1U << 22)
#define OPTION_MASK_ISA2_EVEX512 (HOST_WIDE_INT_1U << 23)
#define OPTION_MASK_ISA_F16C (HOST_WIDE_INT_1U << 28)
#define MASK_NO_FANCY_MATH_387 (1U << 10)
#define OPTION_MASK_ISA_FMA (HOST_WIDE_INT_1U << 29)
#define OPTION_MASK_ISA_FMA4 (HOST_WIDE_INT_1U << 30)
#define MASK_FLOAT_RETURNS (1U << 11)
#define OPTION_MASK_ISA_FSGSBASE (HOST_WIDE_INT_1U << 31)
#define OPTION_MASK_ISA_FXSR (HOST_WIDE_INT_1U << 32)
#define OPTION_MASK_GENERAL_REGS_ONLY (1U << 0)
#define OPTION_MASK_ISA_GFNI (HOST_WIDE_INT_1U << 33)
#define OPTION_MASK_ISA2_HLE (HOST_WIDE_INT_1U << 24)
#define OPTION_MASK_ISA2_HRESET (HOST_WIDE_INT_1U << 25)
#define MASK_IAMCU (1U << 12)
#define MASK_IEEE_FP (1U << 13)
#define MASK_INLINE_ALL_STRINGOPS (1U << 14)
#define MASK_INLINE_STRINGOPS_DYNAMICALLY (1U << 15)
#define OPTION_MASK_ISA2_KL (HOST_WIDE_INT_1U << 26)
#define MASK_LONG_DOUBLE_128 (1U << 16)
#define MASK_LONG_DOUBLE_64 (1U << 17)
#define OPTION_MASK_ISA_LWP (HOST_WIDE_INT_1U << 34)
#define OPTION_MASK_ISA_LZCNT (HOST_WIDE_INT_1U << 35)
#define OPTION_MASK_ISA_MMX (HOST_WIDE_INT_1U << 36)
#define OPTION_MASK_ISA2_MOVBE (HOST_WIDE_INT_1U << 27)
#define OPTION_MASK_ISA2_MOVDIR64B (HOST_WIDE_INT_1U << 28)
#define OPTION_MASK_ISA_MOVDIRI (HOST_WIDE_INT_1U << 37)
#define MASK_MS_BITFIELD_LAYOUT (1U << 18)
#define OPTION_MASK_ISA2_MWAIT (HOST_WIDE_INT_1U << 29)
#define OPTION_MASK_ISA2_MWAITX (HOST_WIDE_INT_1U << 30)
#define MASK_NO_PUSH_ARGS (1U << 19)
#define MASK_NO_RED_ZONE (1U << 20)
#define OPTION_MASK_ISA_SSE4_1 (HOST_WIDE_INT_1U << 38)
#define MASK_OMIT_LEAF_FRAME_POINTER (1U << 21)
#define OPTION_MASK_ISA_PCLMUL (HOST_WIDE_INT_1U << 39)
#define OPTION_MASK_ISA2_PCONFIG (HOST_WIDE_INT_1U << 31)
#define OPTION_MASK_ISA_PKU (HOST_WIDE_INT_1U << 40)
#define OPTION_MASK_ISA_POPCNT (HOST_WIDE_INT_1U << 41)
#define OPTION_MASK_ISA2_PREFETCHI (HOST_WIDE_INT_1U << 32)
#define OPTION_MASK_ISA_PREFETCHWT1 (HOST_WIDE_INT_1U << 42)
#define OPTION_MASK_ISA_PRFCHW (HOST_WIDE_INT_1U << 43)
#define OPTION_MASK_ISA2_PTWRITE (HOST_WIDE_INT_1U << 33)
#define OPTION_MASK_ISA2_RAOINT (HOST_WIDE_INT_1U << 34)
#define OPTION_MASK_ISA2_RDPID (HOST_WIDE_INT_1U << 35)
#define OPTION_MASK_ISA_RDRND (HOST_WIDE_INT_1U << 44)
#define OPTION_MASK_ISA_RDSEED (HOST_WIDE_INT_1U << 45)
#define MASK_RECIP (1U << 22)
#define MASK_RELAX_CMPXCHG_LOOP (1U << 23)
#define MASK_RTD (1U << 24)
#define OPTION_MASK_ISA_RTM (HOST_WIDE_INT_1U << 46)
#define OPTION_MASK_ISA_SAHF (HOST_WIDE_INT_1U << 47)
#define OPTION_MASK_ISA2_SERIALIZE (HOST_WIDE_INT_1U << 36)
#define OPTION_MASK_ISA2_SGX (HOST_WIDE_INT_1U << 37)
#define OPTION_MASK_ISA_SHA (HOST_WIDE_INT_1U << 48)
#define OPTION_MASK_ISA2_SHA512 (HOST_WIDE_INT_1U << 38)
#define OPTION_MASK_ISA_SHSTK (HOST_WIDE_INT_1U << 49)
#define OPTION_MASK_ISA2_SM3 (HOST_WIDE_INT_1U << 39)
#define OPTION_MASK_ISA2_SM4 (HOST_WIDE_INT_1U << 40)
#define OPTION_MASK_ISA_SSE (HOST_WIDE_INT_1U << 50)
#define OPTION_MASK_ISA_SSE2 (HOST_WIDE_INT_1U << 51)
#define OPTION_MASK_ISA_SSE3 (HOST_WIDE_INT_1U << 52)
#define OPTION_MASK_ISA_SSE4_2 (HOST_WIDE_INT_1U << 53)
#define OPTION_MASK_ISA_SSE4A (HOST_WIDE_INT_1U << 54)
#define MASK_SSEREGPARM (1U << 25)
#define OPTION_MASK_ISA_SSSE3 (HOST_WIDE_INT_1U << 55)
#define MASK_STACK_PROBE (1U << 26)
#define MASK_STV (1U << 27)
#define OPTION_MASK_ISA_TBM (HOST_WIDE_INT_1U << 56)
#define MASK_TLS_DIRECT_SEG_REFS (1U << 28)
#define OPTION_MASK_ISA2_TSXLDTRK (HOST_WIDE_INT_1U << 41)
#define OPTION_MASK_ISA2_UINTR (HOST_WIDE_INT_1U << 42)
#define OPTION_MASK_ISA2_USER_MSR (HOST_WIDE_INT_1U << 43)
#define OPTION_MASK_ISA2_VAES (HOST_WIDE_INT_1U << 44)
#define MASK_VECT8_RETURNS (1U << 29)
#define OPTION_MASK_ISA_VPCLMULQDQ (HOST_WIDE_INT_1U << 57)
#define MASK_VZEROUPPER (1U << 30)
#define OPTION_MASK_ISA2_WAITPKG (HOST_WIDE_INT_1U << 45)
#define OPTION_MASK_ISA2_WBNOINVD (HOST_WIDE_INT_1U << 46)
#define OPTION_MASK_ISA2_WIDEKL (HOST_WIDE_INT_1U << 47)
#define OPTION_MASK_ABI_X32 (HOST_WIDE_INT_1U << 58)
#define OPTION_MASK_ISA_XOP (HOST_WIDE_INT_1U << 59)
#define OPTION_MASK_ISA_XSAVE (HOST_WIDE_INT_1U << 60)
#define OPTION_MASK_ISA_XSAVEC (HOST_WIDE_INT_1U << 61)
#define OPTION_MASK_ISA_XSAVEOPT (HOST_WIDE_INT_1U << 62)
#define OPTION_MASK_ISA_XSAVES (HOST_WIDE_INT_1U << 63)
#if defined(HOST_BITS_PER_WIDE_INT) && 64 > HOST_BITS_PER_WIDE_INT
#error too many masks for ix86_isa_flags
#endif
#if defined(HOST_BITS_PER_WIDE_INT) && 48 > HOST_BITS_PER_WIDE_INT
#error too many masks for ix86_isa_flags2
#endif

/* 128BIT_LONG_DOUBLE mask */
#define TARGET_128BIT_LONG_DOUBLE ((target_flags & MASK_128BIT_LONG_DOUBLE) != 0)
#define TARGET_128BIT_LONG_DOUBLE_P(target_flags) (((target_flags) & MASK_128BIT_LONG_DOUBLE) != 0)
#define TARGET_128BIT_LONG_DOUBLE_OPTS_P(opts) (((opts->x_target_flags) & MASK_128BIT_LONG_DOUBLE) != 0)
#define TARGET_EXPLICIT_128BIT_LONG_DOUBLE_P(opts) ((opts->x_target_flags_explicit & MASK_128BIT_LONG_DOUBLE) != 0)
#define SET_TARGET_128BIT_LONG_DOUBLE(opts) opts->x_target_flags |= MASK_128BIT_LONG_DOUBLE
/* CODE16 mask */
#define TARGET_CODE16 ((ix86_isa_flags & OPTION_MASK_CODE16) != 0)
#define TARGET_CODE16_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_CODE16) != 0)
#define TARGET_CODE16_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_CODE16) != 0)
#define TARGET_EXPLICIT_CODE16_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_CODE16) != 0)
#define SET_TARGET_CODE16(opts) opts->x_ix86_isa_flags |= OPTION_MASK_CODE16
/* ISA_64BIT mask */
#define TARGET_64BIT ((ix86_isa_flags & OPTION_MASK_ISA_64BIT) != 0)
#define TARGET_64BIT_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_64BIT) != 0)
#define TARGET_64BIT_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_64BIT) != 0)
#define TARGET_EXPLICIT_64BIT_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_64BIT) != 0)
#define SET_TARGET_64BIT(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_64BIT
/* ISA_3DNOW mask */
#define TARGET_3DNOW ((ix86_isa_flags & OPTION_MASK_ISA_3DNOW) != 0)
#define TARGET_3DNOW_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_3DNOW) != 0)
#define TARGET_3DNOW_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_3DNOW) != 0)
#define TARGET_EXPLICIT_3DNOW_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_3DNOW) != 0)
#define SET_TARGET_3DNOW(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_3DNOW
/* ISA_3DNOW_A mask */
#define TARGET_3DNOW_A ((ix86_isa_flags & OPTION_MASK_ISA_3DNOW_A) != 0)
#define TARGET_3DNOW_A_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_3DNOW_A) != 0)
#define TARGET_3DNOW_A_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_3DNOW_A) != 0)
#define TARGET_EXPLICIT_3DNOW_A_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_3DNOW_A) != 0)
#define SET_TARGET_3DNOW_A(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_3DNOW_A
/* ABI_64 mask */
#define TARGET_ABI_64 ((ix86_isa_flags & OPTION_MASK_ABI_64) != 0)
#define TARGET_ABI_64_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ABI_64) != 0)
#define TARGET_ABI_64_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ABI_64) != 0)
#define TARGET_EXPLICIT_ABI_64_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ABI_64) != 0)
#define SET_TARGET_ABI_64(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ABI_64
/* 80387 mask */
#define TARGET_80387 ((target_flags & MASK_80387) != 0)
#define TARGET_80387_P(target_flags) (((target_flags) & MASK_80387) != 0)
#define TARGET_80387_OPTS_P(opts) (((opts->x_target_flags) & MASK_80387) != 0)
#define TARGET_EXPLICIT_80387_P(opts) ((opts->x_target_flags_explicit & MASK_80387) != 0)
#define SET_TARGET_80387(opts) opts->x_target_flags |= MASK_80387
/* USE_8BIT_IDIV mask */
#define TARGET_USE_8BIT_IDIV ((target_flags & MASK_USE_8BIT_IDIV) != 0)
#define TARGET_USE_8BIT_IDIV_P(target_flags) (((target_flags) & MASK_USE_8BIT_IDIV) != 0)
#define TARGET_USE_8BIT_IDIV_OPTS_P(opts) (((opts->x_target_flags) & MASK_USE_8BIT_IDIV) != 0)
#define TARGET_EXPLICIT_USE_8BIT_IDIV_P(opts) ((opts->x_target_flags_explicit & MASK_USE_8BIT_IDIV) != 0)
#define SET_TARGET_USE_8BIT_IDIV(opts) opts->x_target_flags |= MASK_USE_8BIT_IDIV
/* ISA_ABM mask */
#define TARGET_ABM ((ix86_isa_flags & OPTION_MASK_ISA_ABM) != 0)
#define TARGET_ABM_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_ABM) != 0)
#define TARGET_ABM_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_ABM) != 0)
#define TARGET_EXPLICIT_ABM_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_ABM) != 0)
#define SET_TARGET_ABM(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_ABM
/* ACCUMULATE_OUTGOING_ARGS mask */
#define TARGET_ACCUMULATE_OUTGOING_ARGS ((target_flags & MASK_ACCUMULATE_OUTGOING_ARGS) != 0)
#define TARGET_ACCUMULATE_OUTGOING_ARGS_P(target_flags) (((target_flags) & MASK_ACCUMULATE_OUTGOING_ARGS) != 0)
#define TARGET_ACCUMULATE_OUTGOING_ARGS_OPTS_P(opts) (((opts->x_target_flags) & MASK_ACCUMULATE_OUTGOING_ARGS) != 0)
#define TARGET_EXPLICIT_ACCUMULATE_OUTGOING_ARGS_P(opts) ((opts->x_target_flags_explicit & MASK_ACCUMULATE_OUTGOING_ARGS) != 0)
#define SET_TARGET_ACCUMULATE_OUTGOING_ARGS(opts) opts->x_target_flags |= MASK_ACCUMULATE_OUTGOING_ARGS
/* ISA_ADX mask */
#define TARGET_ADX ((ix86_isa_flags & OPTION_MASK_ISA_ADX) != 0)
#define TARGET_ADX_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_ADX) != 0)
#define TARGET_ADX_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_ADX) != 0)
#define TARGET_EXPLICIT_ADX_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_ADX) != 0)
#define SET_TARGET_ADX(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_ADX
/* ISA_AES mask */
#define TARGET_AES ((ix86_isa_flags & OPTION_MASK_ISA_AES) != 0)
#define TARGET_AES_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AES) != 0)
#define TARGET_AES_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AES) != 0)
#define TARGET_EXPLICIT_AES_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AES) != 0)
#define SET_TARGET_AES(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AES
/* ALIGN_DOUBLE mask */
#define TARGET_ALIGN_DOUBLE ((target_flags & MASK_ALIGN_DOUBLE) != 0)
#define TARGET_ALIGN_DOUBLE_P(target_flags) (((target_flags) & MASK_ALIGN_DOUBLE) != 0)
#define TARGET_ALIGN_DOUBLE_OPTS_P(opts) (((opts->x_target_flags) & MASK_ALIGN_DOUBLE) != 0)
#define TARGET_EXPLICIT_ALIGN_DOUBLE_P(opts) ((opts->x_target_flags_explicit & MASK_ALIGN_DOUBLE) != 0)
#define SET_TARGET_ALIGN_DOUBLE(opts) opts->x_target_flags |= MASK_ALIGN_DOUBLE
/* NO_ALIGN_STRINGOPS mask */
#define TARGET_NO_ALIGN_STRINGOPS ((target_flags & MASK_NO_ALIGN_STRINGOPS) != 0)
#define TARGET_NO_ALIGN_STRINGOPS_P(target_flags) (((target_flags) & MASK_NO_ALIGN_STRINGOPS) != 0)
#define TARGET_NO_ALIGN_STRINGOPS_OPTS_P(opts) (((opts->x_target_flags) & MASK_NO_ALIGN_STRINGOPS) != 0)
#define TARGET_EXPLICIT_NO_ALIGN_STRINGOPS_P(opts) ((opts->x_target_flags_explicit & MASK_NO_ALIGN_STRINGOPS) != 0)
#define SET_TARGET_NO_ALIGN_STRINGOPS(opts) opts->x_target_flags |= MASK_NO_ALIGN_STRINGOPS
/* ISA2_AMX_BF16 mask */
#define TARGET_AMX_BF16 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AMX_BF16) != 0)
#define TARGET_AMX_BF16_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_BF16) != 0)
#define TARGET_AMX_BF16_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_BF16) != 0)
#define TARGET_EXPLICIT_AMX_BF16_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AMX_BF16) != 0)
#define SET_TARGET_AMX_BF16(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AMX_BF16
/* ISA2_AMX_COMPLEX mask */
#define TARGET_AMX_COMPLEX ((ix86_isa_flags2 & OPTION_MASK_ISA2_AMX_COMPLEX) != 0)
#define TARGET_AMX_COMPLEX_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_COMPLEX) != 0)
#define TARGET_AMX_COMPLEX_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_COMPLEX) != 0)
#define TARGET_EXPLICIT_AMX_COMPLEX_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AMX_COMPLEX) != 0)
#define SET_TARGET_AMX_COMPLEX(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AMX_COMPLEX
/* ISA2_AMX_FP16 mask */
#define TARGET_AMX_FP16 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AMX_FP16) != 0)
#define TARGET_AMX_FP16_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_FP16) != 0)
#define TARGET_AMX_FP16_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_FP16) != 0)
#define TARGET_EXPLICIT_AMX_FP16_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AMX_FP16) != 0)
#define SET_TARGET_AMX_FP16(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AMX_FP16
/* ISA2_AMX_INT8 mask */
#define TARGET_AMX_INT8 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AMX_INT8) != 0)
#define TARGET_AMX_INT8_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_INT8) != 0)
#define TARGET_AMX_INT8_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_INT8) != 0)
#define TARGET_EXPLICIT_AMX_INT8_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AMX_INT8) != 0)
#define SET_TARGET_AMX_INT8(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AMX_INT8
/* ISA2_AMX_TILE mask */
#define TARGET_AMX_TILE ((ix86_isa_flags2 & OPTION_MASK_ISA2_AMX_TILE) != 0)
#define TARGET_AMX_TILE_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_TILE) != 0)
#define TARGET_AMX_TILE_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AMX_TILE) != 0)
#define TARGET_EXPLICIT_AMX_TILE_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AMX_TILE) != 0)
#define SET_TARGET_AMX_TILE(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AMX_TILE
/* ISA2_APX_F mask */
#define TARGET_APX_F ((ix86_isa_flags2 & OPTION_MASK_ISA2_APX_F) != 0)
#define TARGET_APX_F_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_APX_F) != 0)
#define TARGET_APX_F_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_APX_F) != 0)
#define TARGET_EXPLICIT_APX_F_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_APX_F) != 0)
#define SET_TARGET_APX_F(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_APX_F
/* ISA_AVX mask */
#define TARGET_AVX ((ix86_isa_flags & OPTION_MASK_ISA_AVX) != 0)
#define TARGET_AVX_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX) != 0)
#define TARGET_AVX_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX) != 0)
#define TARGET_EXPLICIT_AVX_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX) != 0)
#define SET_TARGET_AVX(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX
/* ISA2_AVX10_1_256 mask */
#define TARGET_AVX10_1_256 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX10_1_256) != 0)
#define TARGET_AVX10_1_256_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX10_1_256) != 0)
#define TARGET_AVX10_1_256_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX10_1_256) != 0)
#define TARGET_EXPLICIT_AVX10_1_256_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX10_1_256) != 0)
#define SET_TARGET_AVX10_1_256(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX10_1_256
/* ISA2_AVX10_1_512 mask */
#define TARGET_AVX10_1_512 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX10_1_512) != 0)
#define TARGET_AVX10_1_512_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX10_1_512) != 0)
#define TARGET_AVX10_1_512_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX10_1_512) != 0)
#define TARGET_EXPLICIT_AVX10_1_512_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX10_1_512) != 0)
#define SET_TARGET_AVX10_1_512(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX10_1_512
/* ISA_AVX2 mask */
#define TARGET_AVX2 ((ix86_isa_flags & OPTION_MASK_ISA_AVX2) != 0)
#define TARGET_AVX2_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX2) != 0)
#define TARGET_AVX2_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX2) != 0)
#define TARGET_EXPLICIT_AVX2_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX2) != 0)
#define SET_TARGET_AVX2(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX2
/* AVX256_SPLIT_UNALIGNED_LOAD mask */
#define TARGET_AVX256_SPLIT_UNALIGNED_LOAD ((target_flags & MASK_AVX256_SPLIT_UNALIGNED_LOAD) != 0)
#define TARGET_AVX256_SPLIT_UNALIGNED_LOAD_P(target_flags) (((target_flags) & MASK_AVX256_SPLIT_UNALIGNED_LOAD) != 0)
#define TARGET_AVX256_SPLIT_UNALIGNED_LOAD_OPTS_P(opts) (((opts->x_target_flags) & MASK_AVX256_SPLIT_UNALIGNED_LOAD) != 0)
#define TARGET_EXPLICIT_AVX256_SPLIT_UNALIGNED_LOAD_P(opts) ((opts->x_target_flags_explicit & MASK_AVX256_SPLIT_UNALIGNED_LOAD) != 0)
#define SET_TARGET_AVX256_SPLIT_UNALIGNED_LOAD(opts) opts->x_target_flags |= MASK_AVX256_SPLIT_UNALIGNED_LOAD
/* AVX256_SPLIT_UNALIGNED_STORE mask */
#define TARGET_AVX256_SPLIT_UNALIGNED_STORE ((target_flags & MASK_AVX256_SPLIT_UNALIGNED_STORE) != 0)
#define TARGET_AVX256_SPLIT_UNALIGNED_STORE_P(target_flags) (((target_flags) & MASK_AVX256_SPLIT_UNALIGNED_STORE) != 0)
#define TARGET_AVX256_SPLIT_UNALIGNED_STORE_OPTS_P(opts) (((opts->x_target_flags) & MASK_AVX256_SPLIT_UNALIGNED_STORE) != 0)
#define TARGET_EXPLICIT_AVX256_SPLIT_UNALIGNED_STORE_P(opts) ((opts->x_target_flags_explicit & MASK_AVX256_SPLIT_UNALIGNED_STORE) != 0)
#define SET_TARGET_AVX256_SPLIT_UNALIGNED_STORE(opts) opts->x_target_flags |= MASK_AVX256_SPLIT_UNALIGNED_STORE
/* ISA2_AVX5124FMAPS mask */
#define TARGET_AVX5124FMAPS ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX5124FMAPS) != 0)
#define TARGET_AVX5124FMAPS_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX5124FMAPS) != 0)
#define TARGET_AVX5124FMAPS_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX5124FMAPS) != 0)
#define TARGET_EXPLICIT_AVX5124FMAPS_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX5124FMAPS) != 0)
#define SET_TARGET_AVX5124FMAPS(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX5124FMAPS
/* ISA2_AVX5124VNNIW mask */
#define TARGET_AVX5124VNNIW ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX5124VNNIW) != 0)
#define TARGET_AVX5124VNNIW_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX5124VNNIW) != 0)
#define TARGET_AVX5124VNNIW_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX5124VNNIW) != 0)
#define TARGET_EXPLICIT_AVX5124VNNIW_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX5124VNNIW) != 0)
#define SET_TARGET_AVX5124VNNIW(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX5124VNNIW
/* ISA2_AVX512BF16 mask */
#define TARGET_AVX512BF16 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX512BF16) != 0)
#define TARGET_AVX512BF16_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX512BF16) != 0)
#define TARGET_AVX512BF16_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX512BF16) != 0)
#define TARGET_EXPLICIT_AVX512BF16_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX512BF16) != 0)
#define SET_TARGET_AVX512BF16(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX512BF16
/* ISA_AVX512BITALG mask */
#define TARGET_AVX512BITALG ((ix86_isa_flags & OPTION_MASK_ISA_AVX512BITALG) != 0)
#define TARGET_AVX512BITALG_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512BITALG) != 0)
#define TARGET_AVX512BITALG_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512BITALG) != 0)
#define TARGET_EXPLICIT_AVX512BITALG_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512BITALG) != 0)
#define SET_TARGET_AVX512BITALG(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512BITALG
/* ISA_AVX512BW mask */
#define TARGET_AVX512BW ((ix86_isa_flags & OPTION_MASK_ISA_AVX512BW) != 0)
#define TARGET_AVX512BW_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512BW) != 0)
#define TARGET_AVX512BW_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512BW) != 0)
#define TARGET_EXPLICIT_AVX512BW_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512BW) != 0)
#define SET_TARGET_AVX512BW(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512BW
/* ISA_AVX512CD mask */
#define TARGET_AVX512CD ((ix86_isa_flags & OPTION_MASK_ISA_AVX512CD) != 0)
#define TARGET_AVX512CD_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512CD) != 0)
#define TARGET_AVX512CD_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512CD) != 0)
#define TARGET_EXPLICIT_AVX512CD_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512CD) != 0)
#define SET_TARGET_AVX512CD(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512CD
/* ISA_AVX512DQ mask */
#define TARGET_AVX512DQ ((ix86_isa_flags & OPTION_MASK_ISA_AVX512DQ) != 0)
#define TARGET_AVX512DQ_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512DQ) != 0)
#define TARGET_AVX512DQ_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512DQ) != 0)
#define TARGET_EXPLICIT_AVX512DQ_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512DQ) != 0)
#define SET_TARGET_AVX512DQ(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512DQ
/* ISA_AVX512ER mask */
#define TARGET_AVX512ER ((ix86_isa_flags & OPTION_MASK_ISA_AVX512ER) != 0)
#define TARGET_AVX512ER_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512ER) != 0)
#define TARGET_AVX512ER_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512ER) != 0)
#define TARGET_EXPLICIT_AVX512ER_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512ER) != 0)
#define SET_TARGET_AVX512ER(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512ER
/* ISA_AVX512F mask */
#define TARGET_AVX512F ((ix86_isa_flags & OPTION_MASK_ISA_AVX512F) != 0)
#define TARGET_AVX512F_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512F) != 0)
#define TARGET_AVX512F_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512F) != 0)
#define TARGET_EXPLICIT_AVX512F_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512F) != 0)
#define SET_TARGET_AVX512F(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512F
/* ISA2_AVX512FP16 mask */
#define TARGET_AVX512FP16 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX512FP16) != 0)
#define TARGET_AVX512FP16_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX512FP16) != 0)
#define TARGET_AVX512FP16_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX512FP16) != 0)
#define TARGET_EXPLICIT_AVX512FP16_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX512FP16) != 0)
#define SET_TARGET_AVX512FP16(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX512FP16
/* ISA_AVX512IFMA mask */
#define TARGET_AVX512IFMA ((ix86_isa_flags & OPTION_MASK_ISA_AVX512IFMA) != 0)
#define TARGET_AVX512IFMA_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512IFMA) != 0)
#define TARGET_AVX512IFMA_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512IFMA) != 0)
#define TARGET_EXPLICIT_AVX512IFMA_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512IFMA) != 0)
#define SET_TARGET_AVX512IFMA(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512IFMA
/* ISA_AVX512PF mask */
#define TARGET_AVX512PF ((ix86_isa_flags & OPTION_MASK_ISA_AVX512PF) != 0)
#define TARGET_AVX512PF_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512PF) != 0)
#define TARGET_AVX512PF_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512PF) != 0)
#define TARGET_EXPLICIT_AVX512PF_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512PF) != 0)
#define SET_TARGET_AVX512PF(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512PF
/* ISA_AVX512VBMI mask */
#define TARGET_AVX512VBMI ((ix86_isa_flags & OPTION_MASK_ISA_AVX512VBMI) != 0)
#define TARGET_AVX512VBMI_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512VBMI) != 0)
#define TARGET_AVX512VBMI_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512VBMI) != 0)
#define TARGET_EXPLICIT_AVX512VBMI_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512VBMI) != 0)
#define SET_TARGET_AVX512VBMI(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512VBMI
/* ISA_AVX512VBMI2 mask */
#define TARGET_AVX512VBMI2 ((ix86_isa_flags & OPTION_MASK_ISA_AVX512VBMI2) != 0)
#define TARGET_AVX512VBMI2_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512VBMI2) != 0)
#define TARGET_AVX512VBMI2_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512VBMI2) != 0)
#define TARGET_EXPLICIT_AVX512VBMI2_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512VBMI2) != 0)
#define SET_TARGET_AVX512VBMI2(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512VBMI2
/* ISA_AVX512VL mask */
#define TARGET_AVX512VL ((ix86_isa_flags & OPTION_MASK_ISA_AVX512VL) != 0)
#define TARGET_AVX512VL_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512VL) != 0)
#define TARGET_AVX512VL_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512VL) != 0)
#define TARGET_EXPLICIT_AVX512VL_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512VL) != 0)
#define SET_TARGET_AVX512VL(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512VL
/* ISA_AVX512VNNI mask */
#define TARGET_AVX512VNNI ((ix86_isa_flags & OPTION_MASK_ISA_AVX512VNNI) != 0)
#define TARGET_AVX512VNNI_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512VNNI) != 0)
#define TARGET_AVX512VNNI_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512VNNI) != 0)
#define TARGET_EXPLICIT_AVX512VNNI_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512VNNI) != 0)
#define SET_TARGET_AVX512VNNI(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512VNNI
/* ISA2_AVX512VP2INTERSECT mask */
#define TARGET_AVX512VP2INTERSECT ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVX512VP2INTERSECT) != 0)
#define TARGET_AVX512VP2INTERSECT_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVX512VP2INTERSECT) != 0)
#define TARGET_AVX512VP2INTERSECT_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVX512VP2INTERSECT) != 0)
#define TARGET_EXPLICIT_AVX512VP2INTERSECT_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVX512VP2INTERSECT) != 0)
#define SET_TARGET_AVX512VP2INTERSECT(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVX512VP2INTERSECT
/* ISA_AVX512VPOPCNTDQ mask */
#define TARGET_AVX512VPOPCNTDQ ((ix86_isa_flags & OPTION_MASK_ISA_AVX512VPOPCNTDQ) != 0)
#define TARGET_AVX512VPOPCNTDQ_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_AVX512VPOPCNTDQ) != 0)
#define TARGET_AVX512VPOPCNTDQ_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_AVX512VPOPCNTDQ) != 0)
#define TARGET_EXPLICIT_AVX512VPOPCNTDQ_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_AVX512VPOPCNTDQ) != 0)
#define SET_TARGET_AVX512VPOPCNTDQ(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_AVX512VPOPCNTDQ
/* ISA2_AVXIFMA mask */
#define TARGET_AVXIFMA ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVXIFMA) != 0)
#define TARGET_AVXIFMA_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVXIFMA) != 0)
#define TARGET_AVXIFMA_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVXIFMA) != 0)
#define TARGET_EXPLICIT_AVXIFMA_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVXIFMA) != 0)
#define SET_TARGET_AVXIFMA(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVXIFMA
/* ISA2_AVXNECONVERT mask */
#define TARGET_AVXNECONVERT ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVXNECONVERT) != 0)
#define TARGET_AVXNECONVERT_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVXNECONVERT) != 0)
#define TARGET_AVXNECONVERT_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVXNECONVERT) != 0)
#define TARGET_EXPLICIT_AVXNECONVERT_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVXNECONVERT) != 0)
#define SET_TARGET_AVXNECONVERT(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVXNECONVERT
/* ISA2_AVXVNNI mask */
#define TARGET_AVXVNNI ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVXVNNI) != 0)
#define TARGET_AVXVNNI_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVXVNNI) != 0)
#define TARGET_AVXVNNI_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVXVNNI) != 0)
#define TARGET_EXPLICIT_AVXVNNI_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVXVNNI) != 0)
#define SET_TARGET_AVXVNNI(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVXVNNI
/* ISA2_AVXVNNIINT16 mask */
#define TARGET_AVXVNNIINT16 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVXVNNIINT16) != 0)
#define TARGET_AVXVNNIINT16_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVXVNNIINT16) != 0)
#define TARGET_AVXVNNIINT16_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVXVNNIINT16) != 0)
#define TARGET_EXPLICIT_AVXVNNIINT16_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVXVNNIINT16) != 0)
#define SET_TARGET_AVXVNNIINT16(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVXVNNIINT16
/* ISA2_AVXVNNIINT8 mask */
#define TARGET_AVXVNNIINT8 ((ix86_isa_flags2 & OPTION_MASK_ISA2_AVXVNNIINT8) != 0)
#define TARGET_AVXVNNIINT8_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_AVXVNNIINT8) != 0)
#define TARGET_AVXVNNIINT8_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_AVXVNNIINT8) != 0)
#define TARGET_EXPLICIT_AVXVNNIINT8_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_AVXVNNIINT8) != 0)
#define SET_TARGET_AVXVNNIINT8(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_AVXVNNIINT8
/* ISA_BMI mask */
#define TARGET_BMI ((ix86_isa_flags & OPTION_MASK_ISA_BMI) != 0)
#define TARGET_BMI_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_BMI) != 0)
#define TARGET_BMI_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_BMI) != 0)
#define TARGET_EXPLICIT_BMI_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_BMI) != 0)
#define SET_TARGET_BMI(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_BMI
/* ISA_BMI2 mask */
#define TARGET_BMI2 ((ix86_isa_flags & OPTION_MASK_ISA_BMI2) != 0)
#define TARGET_BMI2_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_BMI2) != 0)
#define TARGET_BMI2_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_BMI2) != 0)
#define TARGET_EXPLICIT_BMI2_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_BMI2) != 0)
#define SET_TARGET_BMI2(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_BMI2
/* CALL_MS2SYSV_XLOGUES mask */
#define TARGET_CALL_MS2SYSV_XLOGUES ((target_flags & MASK_CALL_MS2SYSV_XLOGUES) != 0)
#define TARGET_CALL_MS2SYSV_XLOGUES_P(target_flags) (((target_flags) & MASK_CALL_MS2SYSV_XLOGUES) != 0)
#define TARGET_CALL_MS2SYSV_XLOGUES_OPTS_P(opts) (((opts->x_target_flags) & MASK_CALL_MS2SYSV_XLOGUES) != 0)
#define TARGET_EXPLICIT_CALL_MS2SYSV_XLOGUES_P(opts) ((opts->x_target_flags_explicit & MASK_CALL_MS2SYSV_XLOGUES) != 0)
#define SET_TARGET_CALL_MS2SYSV_XLOGUES(opts) opts->x_target_flags |= MASK_CALL_MS2SYSV_XLOGUES
/* CLD mask */
#define TARGET_CLD ((target_flags & MASK_CLD) != 0)
#define TARGET_CLD_P(target_flags) (((target_flags) & MASK_CLD) != 0)
#define TARGET_CLD_OPTS_P(opts) (((opts->x_target_flags) & MASK_CLD) != 0)
#define TARGET_EXPLICIT_CLD_P(opts) ((opts->x_target_flags_explicit & MASK_CLD) != 0)
#define SET_TARGET_CLD(opts) opts->x_target_flags |= MASK_CLD
/* ISA2_CLDEMOTE mask */
#define TARGET_CLDEMOTE ((ix86_isa_flags2 & OPTION_MASK_ISA2_CLDEMOTE) != 0)
#define TARGET_CLDEMOTE_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_CLDEMOTE) != 0)
#define TARGET_CLDEMOTE_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_CLDEMOTE) != 0)
#define TARGET_EXPLICIT_CLDEMOTE_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_CLDEMOTE) != 0)
#define SET_TARGET_CLDEMOTE(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_CLDEMOTE
/* ISA_CLFLUSHOPT mask */
#define TARGET_CLFLUSHOPT ((ix86_isa_flags & OPTION_MASK_ISA_CLFLUSHOPT) != 0)
#define TARGET_CLFLUSHOPT_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_CLFLUSHOPT) != 0)
#define TARGET_CLFLUSHOPT_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_CLFLUSHOPT) != 0)
#define TARGET_EXPLICIT_CLFLUSHOPT_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_CLFLUSHOPT) != 0)
#define SET_TARGET_CLFLUSHOPT(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_CLFLUSHOPT
/* ISA_CLWB mask */
#define TARGET_CLWB ((ix86_isa_flags & OPTION_MASK_ISA_CLWB) != 0)
#define TARGET_CLWB_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_CLWB) != 0)
#define TARGET_CLWB_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_CLWB) != 0)
#define TARGET_EXPLICIT_CLWB_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_CLWB) != 0)
#define SET_TARGET_CLWB(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_CLWB
/* ISA2_CLZERO mask */
#define TARGET_CLZERO ((ix86_isa_flags2 & OPTION_MASK_ISA2_CLZERO) != 0)
#define TARGET_CLZERO_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_CLZERO) != 0)
#define TARGET_CLZERO_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_CLZERO) != 0)
#define TARGET_EXPLICIT_CLZERO_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_CLZERO) != 0)
#define SET_TARGET_CLZERO(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_CLZERO
/* ISA2_CMPCCXADD mask */
#define TARGET_CMPCCXADD ((ix86_isa_flags2 & OPTION_MASK_ISA2_CMPCCXADD) != 0)
#define TARGET_CMPCCXADD_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_CMPCCXADD) != 0)
#define TARGET_CMPCCXADD_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_CMPCCXADD) != 0)
#define TARGET_EXPLICIT_CMPCCXADD_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_CMPCCXADD) != 0)
#define SET_TARGET_CMPCCXADD(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_CMPCCXADD
/* ISA_CRC32 mask */
#define TARGET_CRC32 ((ix86_isa_flags & OPTION_MASK_ISA_CRC32) != 0)
#define TARGET_CRC32_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_CRC32) != 0)
#define TARGET_CRC32_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_CRC32) != 0)
#define TARGET_EXPLICIT_CRC32_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_CRC32) != 0)
#define SET_TARGET_CRC32(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_CRC32
/* ISA2_CX16 mask */
#define TARGET_CX16 ((ix86_isa_flags2 & OPTION_MASK_ISA2_CX16) != 0)
#define TARGET_CX16_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_CX16) != 0)
#define TARGET_CX16_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_CX16) != 0)
#define TARGET_EXPLICIT_CX16_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_CX16) != 0)
#define SET_TARGET_CX16(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_CX16
/* ISA2_ENQCMD mask */
#define TARGET_ENQCMD ((ix86_isa_flags2 & OPTION_MASK_ISA2_ENQCMD) != 0)
#define TARGET_ENQCMD_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_ENQCMD) != 0)
#define TARGET_ENQCMD_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_ENQCMD) != 0)
#define TARGET_EXPLICIT_ENQCMD_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_ENQCMD) != 0)
#define SET_TARGET_ENQCMD(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_ENQCMD
/* ISA2_EVEX512 mask */
#define TARGET_EVEX512 ((ix86_isa_flags2 & OPTION_MASK_ISA2_EVEX512) != 0)
#define TARGET_EVEX512_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_EVEX512) != 0)
#define TARGET_EVEX512_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_EVEX512) != 0)
#define TARGET_EXPLICIT_EVEX512_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_EVEX512) != 0)
#define SET_TARGET_EVEX512(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_EVEX512
/* ISA_F16C mask */
#define TARGET_F16C ((ix86_isa_flags & OPTION_MASK_ISA_F16C) != 0)
#define TARGET_F16C_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_F16C) != 0)
#define TARGET_F16C_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_F16C) != 0)
#define TARGET_EXPLICIT_F16C_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_F16C) != 0)
#define SET_TARGET_F16C(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_F16C
/* NO_FANCY_MATH_387 mask */
#define TARGET_NO_FANCY_MATH_387 ((target_flags & MASK_NO_FANCY_MATH_387) != 0)
#define TARGET_NO_FANCY_MATH_387_P(target_flags) (((target_flags) & MASK_NO_FANCY_MATH_387) != 0)
#define TARGET_NO_FANCY_MATH_387_OPTS_P(opts) (((opts->x_target_flags) & MASK_NO_FANCY_MATH_387) != 0)
#define TARGET_EXPLICIT_NO_FANCY_MATH_387_P(opts) ((opts->x_target_flags_explicit & MASK_NO_FANCY_MATH_387) != 0)
#define SET_TARGET_NO_FANCY_MATH_387(opts) opts->x_target_flags |= MASK_NO_FANCY_MATH_387
/* ISA_FMA mask */
#define TARGET_FMA ((ix86_isa_flags & OPTION_MASK_ISA_FMA) != 0)
#define TARGET_FMA_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_FMA) != 0)
#define TARGET_FMA_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_FMA) != 0)
#define TARGET_EXPLICIT_FMA_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_FMA) != 0)
#define SET_TARGET_FMA(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_FMA
/* ISA_FMA4 mask */
#define TARGET_FMA4 ((ix86_isa_flags & OPTION_MASK_ISA_FMA4) != 0)
#define TARGET_FMA4_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_FMA4) != 0)
#define TARGET_FMA4_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_FMA4) != 0)
#define TARGET_EXPLICIT_FMA4_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_FMA4) != 0)
#define SET_TARGET_FMA4(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_FMA4
/* FLOAT_RETURNS mask */
#define TARGET_FLOAT_RETURNS ((target_flags & MASK_FLOAT_RETURNS) != 0)
#define TARGET_FLOAT_RETURNS_P(target_flags) (((target_flags) & MASK_FLOAT_RETURNS) != 0)
#define TARGET_FLOAT_RETURNS_OPTS_P(opts) (((opts->x_target_flags) & MASK_FLOAT_RETURNS) != 0)
#define TARGET_EXPLICIT_FLOAT_RETURNS_P(opts) ((opts->x_target_flags_explicit & MASK_FLOAT_RETURNS) != 0)
#define SET_TARGET_FLOAT_RETURNS(opts) opts->x_target_flags |= MASK_FLOAT_RETURNS
/* ISA_FSGSBASE mask */
#define TARGET_FSGSBASE ((ix86_isa_flags & OPTION_MASK_ISA_FSGSBASE) != 0)
#define TARGET_FSGSBASE_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_FSGSBASE) != 0)
#define TARGET_FSGSBASE_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_FSGSBASE) != 0)
#define TARGET_EXPLICIT_FSGSBASE_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_FSGSBASE) != 0)
#define SET_TARGET_FSGSBASE(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_FSGSBASE
/* ISA_FXSR mask */
#define TARGET_FXSR ((ix86_isa_flags & OPTION_MASK_ISA_FXSR) != 0)
#define TARGET_FXSR_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_FXSR) != 0)
#define TARGET_FXSR_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_FXSR) != 0)
#define TARGET_EXPLICIT_FXSR_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_FXSR) != 0)
#define SET_TARGET_FXSR(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_FXSR
/* GENERAL_REGS_ONLY mask */
#define TARGET_GENERAL_REGS_ONLY ((ix86_target_flags & OPTION_MASK_GENERAL_REGS_ONLY) != 0)
#define TARGET_GENERAL_REGS_ONLY_P(ix86_target_flags) (((ix86_target_flags) & OPTION_MASK_GENERAL_REGS_ONLY) != 0)
#define TARGET_GENERAL_REGS_ONLY_OPTS_P(opts) (((opts->x_ix86_target_flags) & OPTION_MASK_GENERAL_REGS_ONLY) != 0)
#define TARGET_EXPLICIT_GENERAL_REGS_ONLY_P(opts) ((opts->x_ix86_target_flags_explicit & OPTION_MASK_GENERAL_REGS_ONLY) != 0)
#define SET_TARGET_GENERAL_REGS_ONLY(opts) opts->x_ix86_target_flags |= OPTION_MASK_GENERAL_REGS_ONLY
/* ISA_GFNI mask */
#define TARGET_GFNI ((ix86_isa_flags & OPTION_MASK_ISA_GFNI) != 0)
#define TARGET_GFNI_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_GFNI) != 0)
#define TARGET_GFNI_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_GFNI) != 0)
#define TARGET_EXPLICIT_GFNI_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_GFNI) != 0)
#define SET_TARGET_GFNI(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_GFNI
/* ISA2_HLE mask */
#define TARGET_HLE ((ix86_isa_flags2 & OPTION_MASK_ISA2_HLE) != 0)
#define TARGET_HLE_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_HLE) != 0)
#define TARGET_HLE_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_HLE) != 0)
#define TARGET_EXPLICIT_HLE_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_HLE) != 0)
#define SET_TARGET_HLE(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_HLE
/* ISA2_HRESET mask */
#define TARGET_HRESET ((ix86_isa_flags2 & OPTION_MASK_ISA2_HRESET) != 0)
#define TARGET_HRESET_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_HRESET) != 0)
#define TARGET_HRESET_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_HRESET) != 0)
#define TARGET_EXPLICIT_HRESET_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_HRESET) != 0)
#define SET_TARGET_HRESET(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_HRESET
/* IAMCU mask */
#define TARGET_IAMCU ((target_flags & MASK_IAMCU) != 0)
#define TARGET_IAMCU_P(target_flags) (((target_flags) & MASK_IAMCU) != 0)
#define TARGET_IAMCU_OPTS_P(opts) (((opts->x_target_flags) & MASK_IAMCU) != 0)
#define TARGET_EXPLICIT_IAMCU_P(opts) ((opts->x_target_flags_explicit & MASK_IAMCU) != 0)
#define SET_TARGET_IAMCU(opts) opts->x_target_flags |= MASK_IAMCU
/* IEEE_FP mask */
#define TARGET_IEEE_FP ((target_flags & MASK_IEEE_FP) != 0)
#define TARGET_IEEE_FP_P(target_flags) (((target_flags) & MASK_IEEE_FP) != 0)
#define TARGET_IEEE_FP_OPTS_P(opts) (((opts->x_target_flags) & MASK_IEEE_FP) != 0)
#define TARGET_EXPLICIT_IEEE_FP_P(opts) ((opts->x_target_flags_explicit & MASK_IEEE_FP) != 0)
#define SET_TARGET_IEEE_FP(opts) opts->x_target_flags |= MASK_IEEE_FP
/* INLINE_ALL_STRINGOPS mask */
#define TARGET_INLINE_ALL_STRINGOPS ((target_flags & MASK_INLINE_ALL_STRINGOPS) != 0)
#define TARGET_INLINE_ALL_STRINGOPS_P(target_flags) (((target_flags) & MASK_INLINE_ALL_STRINGOPS) != 0)
#define TARGET_INLINE_ALL_STRINGOPS_OPTS_P(opts) (((opts->x_target_flags) & MASK_INLINE_ALL_STRINGOPS) != 0)
#define TARGET_EXPLICIT_INLINE_ALL_STRINGOPS_P(opts) ((opts->x_target_flags_explicit & MASK_INLINE_ALL_STRINGOPS) != 0)
#define SET_TARGET_INLINE_ALL_STRINGOPS(opts) opts->x_target_flags |= MASK_INLINE_ALL_STRINGOPS
/* INLINE_STRINGOPS_DYNAMICALLY mask */
#define TARGET_INLINE_STRINGOPS_DYNAMICALLY ((target_flags & MASK_INLINE_STRINGOPS_DYNAMICALLY) != 0)
#define TARGET_INLINE_STRINGOPS_DYNAMICALLY_P(target_flags) (((target_flags) & MASK_INLINE_STRINGOPS_DYNAMICALLY) != 0)
#define TARGET_INLINE_STRINGOPS_DYNAMICALLY_OPTS_P(opts) (((opts->x_target_flags) & MASK_INLINE_STRINGOPS_DYNAMICALLY) != 0)
#define TARGET_EXPLICIT_INLINE_STRINGOPS_DYNAMICALLY_P(opts) ((opts->x_target_flags_explicit & MASK_INLINE_STRINGOPS_DYNAMICALLY) != 0)
#define SET_TARGET_INLINE_STRINGOPS_DYNAMICALLY(opts) opts->x_target_flags |= MASK_INLINE_STRINGOPS_DYNAMICALLY
/* ISA2_KL mask */
#define TARGET_KL ((ix86_isa_flags2 & OPTION_MASK_ISA2_KL) != 0)
#define TARGET_KL_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_KL) != 0)
#define TARGET_KL_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_KL) != 0)
#define TARGET_EXPLICIT_KL_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_KL) != 0)
#define SET_TARGET_KL(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_KL
/* LONG_DOUBLE_128 mask */
#define TARGET_LONG_DOUBLE_128 ((target_flags & MASK_LONG_DOUBLE_128) != 0)
#define TARGET_LONG_DOUBLE_128_P(target_flags) (((target_flags) & MASK_LONG_DOUBLE_128) != 0)
#define TARGET_LONG_DOUBLE_128_OPTS_P(opts) (((opts->x_target_flags) & MASK_LONG_DOUBLE_128) != 0)
#define TARGET_EXPLICIT_LONG_DOUBLE_128_P(opts) ((opts->x_target_flags_explicit & MASK_LONG_DOUBLE_128) != 0)
#define SET_TARGET_LONG_DOUBLE_128(opts) opts->x_target_flags |= MASK_LONG_DOUBLE_128
/* LONG_DOUBLE_64 mask */
#define TARGET_LONG_DOUBLE_64 ((target_flags & MASK_LONG_DOUBLE_64) != 0)
#define TARGET_LONG_DOUBLE_64_P(target_flags) (((target_flags) & MASK_LONG_DOUBLE_64) != 0)
#define TARGET_LONG_DOUBLE_64_OPTS_P(opts) (((opts->x_target_flags) & MASK_LONG_DOUBLE_64) != 0)
#define TARGET_EXPLICIT_LONG_DOUBLE_64_P(opts) ((opts->x_target_flags_explicit & MASK_LONG_DOUBLE_64) != 0)
#define SET_TARGET_LONG_DOUBLE_64(opts) opts->x_target_flags |= MASK_LONG_DOUBLE_64
/* ISA_LWP mask */
#define TARGET_LWP ((ix86_isa_flags & OPTION_MASK_ISA_LWP) != 0)
#define TARGET_LWP_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_LWP) != 0)
#define TARGET_LWP_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_LWP) != 0)
#define TARGET_EXPLICIT_LWP_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_LWP) != 0)
#define SET_TARGET_LWP(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_LWP
/* ISA_LZCNT mask */
#define TARGET_LZCNT ((ix86_isa_flags & OPTION_MASK_ISA_LZCNT) != 0)
#define TARGET_LZCNT_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_LZCNT) != 0)
#define TARGET_LZCNT_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_LZCNT) != 0)
#define TARGET_EXPLICIT_LZCNT_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_LZCNT) != 0)
#define SET_TARGET_LZCNT(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_LZCNT
/* ISA_MMX mask */
#define TARGET_MMX ((ix86_isa_flags & OPTION_MASK_ISA_MMX) != 0)
#define TARGET_MMX_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_MMX) != 0)
#define TARGET_MMX_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_MMX) != 0)
#define TARGET_EXPLICIT_MMX_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_MMX) != 0)
#define SET_TARGET_MMX(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_MMX
/* ISA2_MOVBE mask */
#define TARGET_MOVBE ((ix86_isa_flags2 & OPTION_MASK_ISA2_MOVBE) != 0)
#define TARGET_MOVBE_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_MOVBE) != 0)
#define TARGET_MOVBE_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_MOVBE) != 0)
#define TARGET_EXPLICIT_MOVBE_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_MOVBE) != 0)
#define SET_TARGET_MOVBE(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_MOVBE
/* ISA2_MOVDIR64B mask */
#define TARGET_MOVDIR64B ((ix86_isa_flags2 & OPTION_MASK_ISA2_MOVDIR64B) != 0)
#define TARGET_MOVDIR64B_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_MOVDIR64B) != 0)
#define TARGET_MOVDIR64B_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_MOVDIR64B) != 0)
#define TARGET_EXPLICIT_MOVDIR64B_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_MOVDIR64B) != 0)
#define SET_TARGET_MOVDIR64B(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_MOVDIR64B
/* ISA_MOVDIRI mask */
#define TARGET_MOVDIRI ((ix86_isa_flags & OPTION_MASK_ISA_MOVDIRI) != 0)
#define TARGET_MOVDIRI_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_MOVDIRI) != 0)
#define TARGET_MOVDIRI_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_MOVDIRI) != 0)
#define TARGET_EXPLICIT_MOVDIRI_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_MOVDIRI) != 0)
#define SET_TARGET_MOVDIRI(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_MOVDIRI
/* MS_BITFIELD_LAYOUT mask */
#define TARGET_MS_BITFIELD_LAYOUT ((target_flags & MASK_MS_BITFIELD_LAYOUT) != 0)
#define TARGET_MS_BITFIELD_LAYOUT_P(target_flags) (((target_flags) & MASK_MS_BITFIELD_LAYOUT) != 0)
#define TARGET_MS_BITFIELD_LAYOUT_OPTS_P(opts) (((opts->x_target_flags) & MASK_MS_BITFIELD_LAYOUT) != 0)
#define TARGET_EXPLICIT_MS_BITFIELD_LAYOUT_P(opts) ((opts->x_target_flags_explicit & MASK_MS_BITFIELD_LAYOUT) != 0)
#define SET_TARGET_MS_BITFIELD_LAYOUT(opts) opts->x_target_flags |= MASK_MS_BITFIELD_LAYOUT
/* ISA2_MWAIT mask */
#define TARGET_MWAIT ((ix86_isa_flags2 & OPTION_MASK_ISA2_MWAIT) != 0)
#define TARGET_MWAIT_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_MWAIT) != 0)
#define TARGET_MWAIT_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_MWAIT) != 0)
#define TARGET_EXPLICIT_MWAIT_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_MWAIT) != 0)
#define SET_TARGET_MWAIT(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_MWAIT
/* ISA2_MWAITX mask */
#define TARGET_MWAITX ((ix86_isa_flags2 & OPTION_MASK_ISA2_MWAITX) != 0)
#define TARGET_MWAITX_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_MWAITX) != 0)
#define TARGET_MWAITX_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_MWAITX) != 0)
#define TARGET_EXPLICIT_MWAITX_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_MWAITX) != 0)
#define SET_TARGET_MWAITX(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_MWAITX
/* NO_PUSH_ARGS mask */
#define TARGET_NO_PUSH_ARGS ((target_flags & MASK_NO_PUSH_ARGS) != 0)
#define TARGET_NO_PUSH_ARGS_P(target_flags) (((target_flags) & MASK_NO_PUSH_ARGS) != 0)
#define TARGET_NO_PUSH_ARGS_OPTS_P(opts) (((opts->x_target_flags) & MASK_NO_PUSH_ARGS) != 0)
#define TARGET_EXPLICIT_NO_PUSH_ARGS_P(opts) ((opts->x_target_flags_explicit & MASK_NO_PUSH_ARGS) != 0)
#define SET_TARGET_NO_PUSH_ARGS(opts) opts->x_target_flags |= MASK_NO_PUSH_ARGS
/* NO_RED_ZONE mask */
#define TARGET_NO_RED_ZONE ((target_flags & MASK_NO_RED_ZONE) != 0)
#define TARGET_NO_RED_ZONE_P(target_flags) (((target_flags) & MASK_NO_RED_ZONE) != 0)
#define TARGET_NO_RED_ZONE_OPTS_P(opts) (((opts->x_target_flags) & MASK_NO_RED_ZONE) != 0)
#define TARGET_EXPLICIT_NO_RED_ZONE_P(opts) ((opts->x_target_flags_explicit & MASK_NO_RED_ZONE) != 0)
#define SET_TARGET_NO_RED_ZONE(opts) opts->x_target_flags |= MASK_NO_RED_ZONE
/* ISA_SSE4_1 mask */
#define TARGET_SSE4_1 ((ix86_isa_flags & OPTION_MASK_ISA_SSE4_1) != 0)
#define TARGET_SSE4_1_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSE4_1) != 0)
#define TARGET_SSE4_1_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSE4_1) != 0)
#define TARGET_EXPLICIT_SSE4_1_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSE4_1) != 0)
#define SET_TARGET_SSE4_1(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSE4_1
/* OMIT_LEAF_FRAME_POINTER mask */
#define TARGET_OMIT_LEAF_FRAME_POINTER ((target_flags & MASK_OMIT_LEAF_FRAME_POINTER) != 0)
#define TARGET_OMIT_LEAF_FRAME_POINTER_P(target_flags) (((target_flags) & MASK_OMIT_LEAF_FRAME_POINTER) != 0)
#define TARGET_OMIT_LEAF_FRAME_POINTER_OPTS_P(opts) (((opts->x_target_flags) & MASK_OMIT_LEAF_FRAME_POINTER) != 0)
#define TARGET_EXPLICIT_OMIT_LEAF_FRAME_POINTER_P(opts) ((opts->x_target_flags_explicit & MASK_OMIT_LEAF_FRAME_POINTER) != 0)
#define SET_TARGET_OMIT_LEAF_FRAME_POINTER(opts) opts->x_target_flags |= MASK_OMIT_LEAF_FRAME_POINTER
/* ISA_PCLMUL mask */
#define TARGET_PCLMUL ((ix86_isa_flags & OPTION_MASK_ISA_PCLMUL) != 0)
#define TARGET_PCLMUL_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_PCLMUL) != 0)
#define TARGET_PCLMUL_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_PCLMUL) != 0)
#define TARGET_EXPLICIT_PCLMUL_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_PCLMUL) != 0)
#define SET_TARGET_PCLMUL(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_PCLMUL
/* ISA2_PCONFIG mask */
#define TARGET_PCONFIG ((ix86_isa_flags2 & OPTION_MASK_ISA2_PCONFIG) != 0)
#define TARGET_PCONFIG_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_PCONFIG) != 0)
#define TARGET_PCONFIG_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_PCONFIG) != 0)
#define TARGET_EXPLICIT_PCONFIG_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_PCONFIG) != 0)
#define SET_TARGET_PCONFIG(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_PCONFIG
/* ISA_PKU mask */
#define TARGET_PKU ((ix86_isa_flags & OPTION_MASK_ISA_PKU) != 0)
#define TARGET_PKU_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_PKU) != 0)
#define TARGET_PKU_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_PKU) != 0)
#define TARGET_EXPLICIT_PKU_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_PKU) != 0)
#define SET_TARGET_PKU(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_PKU
/* ISA_POPCNT mask */
#define TARGET_POPCNT ((ix86_isa_flags & OPTION_MASK_ISA_POPCNT) != 0)
#define TARGET_POPCNT_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_POPCNT) != 0)
#define TARGET_POPCNT_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_POPCNT) != 0)
#define TARGET_EXPLICIT_POPCNT_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_POPCNT) != 0)
#define SET_TARGET_POPCNT(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_POPCNT
/* ISA2_PREFETCHI mask */
#define TARGET_PREFETCHI ((ix86_isa_flags2 & OPTION_MASK_ISA2_PREFETCHI) != 0)
#define TARGET_PREFETCHI_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_PREFETCHI) != 0)
#define TARGET_PREFETCHI_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_PREFETCHI) != 0)
#define TARGET_EXPLICIT_PREFETCHI_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_PREFETCHI) != 0)
#define SET_TARGET_PREFETCHI(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_PREFETCHI
/* ISA_PREFETCHWT1 mask */
#define TARGET_PREFETCHWT1 ((ix86_isa_flags & OPTION_MASK_ISA_PREFETCHWT1) != 0)
#define TARGET_PREFETCHWT1_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_PREFETCHWT1) != 0)
#define TARGET_PREFETCHWT1_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_PREFETCHWT1) != 0)
#define TARGET_EXPLICIT_PREFETCHWT1_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_PREFETCHWT1) != 0)
#define SET_TARGET_PREFETCHWT1(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_PREFETCHWT1
/* ISA_PRFCHW mask */
#define TARGET_PRFCHW ((ix86_isa_flags & OPTION_MASK_ISA_PRFCHW) != 0)
#define TARGET_PRFCHW_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_PRFCHW) != 0)
#define TARGET_PRFCHW_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_PRFCHW) != 0)
#define TARGET_EXPLICIT_PRFCHW_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_PRFCHW) != 0)
#define SET_TARGET_PRFCHW(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_PRFCHW
/* ISA2_PTWRITE mask */
#define TARGET_PTWRITE ((ix86_isa_flags2 & OPTION_MASK_ISA2_PTWRITE) != 0)
#define TARGET_PTWRITE_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_PTWRITE) != 0)
#define TARGET_PTWRITE_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_PTWRITE) != 0)
#define TARGET_EXPLICIT_PTWRITE_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_PTWRITE) != 0)
#define SET_TARGET_PTWRITE(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_PTWRITE
/* ISA2_RAOINT mask */
#define TARGET_RAOINT ((ix86_isa_flags2 & OPTION_MASK_ISA2_RAOINT) != 0)
#define TARGET_RAOINT_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_RAOINT) != 0)
#define TARGET_RAOINT_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_RAOINT) != 0)
#define TARGET_EXPLICIT_RAOINT_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_RAOINT) != 0)
#define SET_TARGET_RAOINT(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_RAOINT
/* ISA2_RDPID mask */
#define TARGET_RDPID ((ix86_isa_flags2 & OPTION_MASK_ISA2_RDPID) != 0)
#define TARGET_RDPID_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_RDPID) != 0)
#define TARGET_RDPID_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_RDPID) != 0)
#define TARGET_EXPLICIT_RDPID_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_RDPID) != 0)
#define SET_TARGET_RDPID(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_RDPID
/* ISA_RDRND mask */
#define TARGET_RDRND ((ix86_isa_flags & OPTION_MASK_ISA_RDRND) != 0)
#define TARGET_RDRND_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_RDRND) != 0)
#define TARGET_RDRND_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_RDRND) != 0)
#define TARGET_EXPLICIT_RDRND_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_RDRND) != 0)
#define SET_TARGET_RDRND(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_RDRND
/* ISA_RDSEED mask */
#define TARGET_RDSEED ((ix86_isa_flags & OPTION_MASK_ISA_RDSEED) != 0)
#define TARGET_RDSEED_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_RDSEED) != 0)
#define TARGET_RDSEED_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_RDSEED) != 0)
#define TARGET_EXPLICIT_RDSEED_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_RDSEED) != 0)
#define SET_TARGET_RDSEED(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_RDSEED
/* RECIP mask */
#define TARGET_RECIP ((target_flags & MASK_RECIP) != 0)
#define TARGET_RECIP_P(target_flags) (((target_flags) & MASK_RECIP) != 0)
#define TARGET_RECIP_OPTS_P(opts) (((opts->x_target_flags) & MASK_RECIP) != 0)
#define TARGET_EXPLICIT_RECIP_P(opts) ((opts->x_target_flags_explicit & MASK_RECIP) != 0)
#define SET_TARGET_RECIP(opts) opts->x_target_flags |= MASK_RECIP
/* RELAX_CMPXCHG_LOOP mask */
#define TARGET_RELAX_CMPXCHG_LOOP ((target_flags & MASK_RELAX_CMPXCHG_LOOP) != 0)
#define TARGET_RELAX_CMPXCHG_LOOP_P(target_flags) (((target_flags) & MASK_RELAX_CMPXCHG_LOOP) != 0)
#define TARGET_RELAX_CMPXCHG_LOOP_OPTS_P(opts) (((opts->x_target_flags) & MASK_RELAX_CMPXCHG_LOOP) != 0)
#define TARGET_EXPLICIT_RELAX_CMPXCHG_LOOP_P(opts) ((opts->x_target_flags_explicit & MASK_RELAX_CMPXCHG_LOOP) != 0)
#define SET_TARGET_RELAX_CMPXCHG_LOOP(opts) opts->x_target_flags |= MASK_RELAX_CMPXCHG_LOOP
/* RTD mask */
#define TARGET_RTD ((target_flags & MASK_RTD) != 0)
#define TARGET_RTD_P(target_flags) (((target_flags) & MASK_RTD) != 0)
#define TARGET_RTD_OPTS_P(opts) (((opts->x_target_flags) & MASK_RTD) != 0)
#define TARGET_EXPLICIT_RTD_P(opts) ((opts->x_target_flags_explicit & MASK_RTD) != 0)
#define SET_TARGET_RTD(opts) opts->x_target_flags |= MASK_RTD
/* ISA_RTM mask */
#define TARGET_RTM ((ix86_isa_flags & OPTION_MASK_ISA_RTM) != 0)
#define TARGET_RTM_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_RTM) != 0)
#define TARGET_RTM_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_RTM) != 0)
#define TARGET_EXPLICIT_RTM_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_RTM) != 0)
#define SET_TARGET_RTM(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_RTM
/* ISA_SAHF mask */
#define TARGET_SAHF ((ix86_isa_flags & OPTION_MASK_ISA_SAHF) != 0)
#define TARGET_SAHF_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SAHF) != 0)
#define TARGET_SAHF_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SAHF) != 0)
#define TARGET_EXPLICIT_SAHF_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SAHF) != 0)
#define SET_TARGET_SAHF(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SAHF
/* ISA2_SERIALIZE mask */
#define TARGET_SERIALIZE ((ix86_isa_flags2 & OPTION_MASK_ISA2_SERIALIZE) != 0)
#define TARGET_SERIALIZE_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_SERIALIZE) != 0)
#define TARGET_SERIALIZE_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_SERIALIZE) != 0)
#define TARGET_EXPLICIT_SERIALIZE_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_SERIALIZE) != 0)
#define SET_TARGET_SERIALIZE(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_SERIALIZE
/* ISA2_SGX mask */
#define TARGET_SGX ((ix86_isa_flags2 & OPTION_MASK_ISA2_SGX) != 0)
#define TARGET_SGX_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_SGX) != 0)
#define TARGET_SGX_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_SGX) != 0)
#define TARGET_EXPLICIT_SGX_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_SGX) != 0)
#define SET_TARGET_SGX(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_SGX
/* ISA_SHA mask */
#define TARGET_SHA ((ix86_isa_flags & OPTION_MASK_ISA_SHA) != 0)
#define TARGET_SHA_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SHA) != 0)
#define TARGET_SHA_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SHA) != 0)
#define TARGET_EXPLICIT_SHA_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SHA) != 0)
#define SET_TARGET_SHA(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SHA
/* ISA2_SHA512 mask */
#define TARGET_SHA512 ((ix86_isa_flags2 & OPTION_MASK_ISA2_SHA512) != 0)
#define TARGET_SHA512_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_SHA512) != 0)
#define TARGET_SHA512_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_SHA512) != 0)
#define TARGET_EXPLICIT_SHA512_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_SHA512) != 0)
#define SET_TARGET_SHA512(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_SHA512
/* ISA_SHSTK mask */
#define TARGET_SHSTK ((ix86_isa_flags & OPTION_MASK_ISA_SHSTK) != 0)
#define TARGET_SHSTK_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SHSTK) != 0)
#define TARGET_SHSTK_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SHSTK) != 0)
#define TARGET_EXPLICIT_SHSTK_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SHSTK) != 0)
#define SET_TARGET_SHSTK(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SHSTK
/* ISA2_SM3 mask */
#define TARGET_SM3 ((ix86_isa_flags2 & OPTION_MASK_ISA2_SM3) != 0)
#define TARGET_SM3_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_SM3) != 0)
#define TARGET_SM3_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_SM3) != 0)
#define TARGET_EXPLICIT_SM3_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_SM3) != 0)
#define SET_TARGET_SM3(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_SM3
/* ISA2_SM4 mask */
#define TARGET_SM4 ((ix86_isa_flags2 & OPTION_MASK_ISA2_SM4) != 0)
#define TARGET_SM4_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_SM4) != 0)
#define TARGET_SM4_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_SM4) != 0)
#define TARGET_EXPLICIT_SM4_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_SM4) != 0)
#define SET_TARGET_SM4(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_SM4
/* ISA_SSE mask */
#define TARGET_SSE ((ix86_isa_flags & OPTION_MASK_ISA_SSE) != 0)
#define TARGET_SSE_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSE) != 0)
#define TARGET_SSE_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSE) != 0)
#define TARGET_EXPLICIT_SSE_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSE) != 0)
#define SET_TARGET_SSE(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSE
/* ISA_SSE2 mask */
#define TARGET_SSE2 ((ix86_isa_flags & OPTION_MASK_ISA_SSE2) != 0)
#define TARGET_SSE2_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSE2) != 0)
#define TARGET_SSE2_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSE2) != 0)
#define TARGET_EXPLICIT_SSE2_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSE2) != 0)
#define SET_TARGET_SSE2(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSE2
/* ISA_SSE3 mask */
#define TARGET_SSE3 ((ix86_isa_flags & OPTION_MASK_ISA_SSE3) != 0)
#define TARGET_SSE3_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSE3) != 0)
#define TARGET_SSE3_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSE3) != 0)
#define TARGET_EXPLICIT_SSE3_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSE3) != 0)
#define SET_TARGET_SSE3(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSE3
/* ISA_SSE4_2 mask */
#define TARGET_SSE4_2 ((ix86_isa_flags & OPTION_MASK_ISA_SSE4_2) != 0)
#define TARGET_SSE4_2_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSE4_2) != 0)
#define TARGET_SSE4_2_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSE4_2) != 0)
#define TARGET_EXPLICIT_SSE4_2_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSE4_2) != 0)
#define SET_TARGET_SSE4_2(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSE4_2
/* ISA_SSE4A mask */
#define TARGET_SSE4A ((ix86_isa_flags & OPTION_MASK_ISA_SSE4A) != 0)
#define TARGET_SSE4A_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSE4A) != 0)
#define TARGET_SSE4A_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSE4A) != 0)
#define TARGET_EXPLICIT_SSE4A_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSE4A) != 0)
#define SET_TARGET_SSE4A(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSE4A
/* SSEREGPARM mask */
#define TARGET_SSEREGPARM ((target_flags & MASK_SSEREGPARM) != 0)
#define TARGET_SSEREGPARM_P(target_flags) (((target_flags) & MASK_SSEREGPARM) != 0)
#define TARGET_SSEREGPARM_OPTS_P(opts) (((opts->x_target_flags) & MASK_SSEREGPARM) != 0)
#define TARGET_EXPLICIT_SSEREGPARM_P(opts) ((opts->x_target_flags_explicit & MASK_SSEREGPARM) != 0)
#define SET_TARGET_SSEREGPARM(opts) opts->x_target_flags |= MASK_SSEREGPARM
/* ISA_SSSE3 mask */
#define TARGET_SSSE3 ((ix86_isa_flags & OPTION_MASK_ISA_SSSE3) != 0)
#define TARGET_SSSE3_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_SSSE3) != 0)
#define TARGET_SSSE3_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_SSSE3) != 0)
#define TARGET_EXPLICIT_SSSE3_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_SSSE3) != 0)
#define SET_TARGET_SSSE3(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_SSSE3
/* STACK_PROBE mask */
#define TARGET_STACK_PROBE ((target_flags & MASK_STACK_PROBE) != 0)
#define TARGET_STACK_PROBE_P(target_flags) (((target_flags) & MASK_STACK_PROBE) != 0)
#define TARGET_STACK_PROBE_OPTS_P(opts) (((opts->x_target_flags) & MASK_STACK_PROBE) != 0)
#define TARGET_EXPLICIT_STACK_PROBE_P(opts) ((opts->x_target_flags_explicit & MASK_STACK_PROBE) != 0)
#define SET_TARGET_STACK_PROBE(opts) opts->x_target_flags |= MASK_STACK_PROBE
/* STV mask */
#define TARGET_STV ((target_flags & MASK_STV) != 0)
#define TARGET_STV_P(target_flags) (((target_flags) & MASK_STV) != 0)
#define TARGET_STV_OPTS_P(opts) (((opts->x_target_flags) & MASK_STV) != 0)
#define TARGET_EXPLICIT_STV_P(opts) ((opts->x_target_flags_explicit & MASK_STV) != 0)
#define SET_TARGET_STV(opts) opts->x_target_flags |= MASK_STV
/* ISA_TBM mask */
#define TARGET_TBM ((ix86_isa_flags & OPTION_MASK_ISA_TBM) != 0)
#define TARGET_TBM_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_TBM) != 0)
#define TARGET_TBM_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_TBM) != 0)
#define TARGET_EXPLICIT_TBM_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_TBM) != 0)
#define SET_TARGET_TBM(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_TBM
/* TLS_DIRECT_SEG_REFS mask */
#define TARGET_TLS_DIRECT_SEG_REFS ((target_flags & MASK_TLS_DIRECT_SEG_REFS) != 0)
#define TARGET_TLS_DIRECT_SEG_REFS_P(target_flags) (((target_flags) & MASK_TLS_DIRECT_SEG_REFS) != 0)
#define TARGET_TLS_DIRECT_SEG_REFS_OPTS_P(opts) (((opts->x_target_flags) & MASK_TLS_DIRECT_SEG_REFS) != 0)
#define TARGET_EXPLICIT_TLS_DIRECT_SEG_REFS_P(opts) ((opts->x_target_flags_explicit & MASK_TLS_DIRECT_SEG_REFS) != 0)
#define SET_TARGET_TLS_DIRECT_SEG_REFS(opts) opts->x_target_flags |= MASK_TLS_DIRECT_SEG_REFS
/* ISA2_TSXLDTRK mask */
#define TARGET_TSXLDTRK ((ix86_isa_flags2 & OPTION_MASK_ISA2_TSXLDTRK) != 0)
#define TARGET_TSXLDTRK_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_TSXLDTRK) != 0)
#define TARGET_TSXLDTRK_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_TSXLDTRK) != 0)
#define TARGET_EXPLICIT_TSXLDTRK_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_TSXLDTRK) != 0)
#define SET_TARGET_TSXLDTRK(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_TSXLDTRK
/* ISA2_UINTR mask */
#define TARGET_UINTR ((ix86_isa_flags2 & OPTION_MASK_ISA2_UINTR) != 0)
#define TARGET_UINTR_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_UINTR) != 0)
#define TARGET_UINTR_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_UINTR) != 0)
#define TARGET_EXPLICIT_UINTR_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_UINTR) != 0)
#define SET_TARGET_UINTR(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_UINTR
/* ISA2_USER_MSR mask */
#define TARGET_USER_MSR ((ix86_isa_flags2 & OPTION_MASK_ISA2_USER_MSR) != 0)
#define TARGET_USER_MSR_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_USER_MSR) != 0)
#define TARGET_USER_MSR_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_USER_MSR) != 0)
#define TARGET_EXPLICIT_USER_MSR_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_USER_MSR) != 0)
#define SET_TARGET_USER_MSR(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_USER_MSR
/* ISA2_VAES mask */
#define TARGET_VAES ((ix86_isa_flags2 & OPTION_MASK_ISA2_VAES) != 0)
#define TARGET_VAES_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_VAES) != 0)
#define TARGET_VAES_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_VAES) != 0)
#define TARGET_EXPLICIT_VAES_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_VAES) != 0)
#define SET_TARGET_VAES(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_VAES
/* VECT8_RETURNS mask */
#define TARGET_VECT8_RETURNS ((target_flags & MASK_VECT8_RETURNS) != 0)
#define TARGET_VECT8_RETURNS_P(target_flags) (((target_flags) & MASK_VECT8_RETURNS) != 0)
#define TARGET_VECT8_RETURNS_OPTS_P(opts) (((opts->x_target_flags) & MASK_VECT8_RETURNS) != 0)
#define TARGET_EXPLICIT_VECT8_RETURNS_P(opts) ((opts->x_target_flags_explicit & MASK_VECT8_RETURNS) != 0)
#define SET_TARGET_VECT8_RETURNS(opts) opts->x_target_flags |= MASK_VECT8_RETURNS
/* ISA_VPCLMULQDQ mask */
#define TARGET_VPCLMULQDQ ((ix86_isa_flags & OPTION_MASK_ISA_VPCLMULQDQ) != 0)
#define TARGET_VPCLMULQDQ_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_VPCLMULQDQ) != 0)
#define TARGET_VPCLMULQDQ_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_VPCLMULQDQ) != 0)
#define TARGET_EXPLICIT_VPCLMULQDQ_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_VPCLMULQDQ) != 0)
#define SET_TARGET_VPCLMULQDQ(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_VPCLMULQDQ
/* VZEROUPPER mask */
#define TARGET_VZEROUPPER ((target_flags & MASK_VZEROUPPER) != 0)
#define TARGET_VZEROUPPER_P(target_flags) (((target_flags) & MASK_VZEROUPPER) != 0)
#define TARGET_VZEROUPPER_OPTS_P(opts) (((opts->x_target_flags) & MASK_VZEROUPPER) != 0)
#define TARGET_EXPLICIT_VZEROUPPER_P(opts) ((opts->x_target_flags_explicit & MASK_VZEROUPPER) != 0)
#define SET_TARGET_VZEROUPPER(opts) opts->x_target_flags |= MASK_VZEROUPPER
/* ISA2_WAITPKG mask */
#define TARGET_WAITPKG ((ix86_isa_flags2 & OPTION_MASK_ISA2_WAITPKG) != 0)
#define TARGET_WAITPKG_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_WAITPKG) != 0)
#define TARGET_WAITPKG_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_WAITPKG) != 0)
#define TARGET_EXPLICIT_WAITPKG_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_WAITPKG) != 0)
#define SET_TARGET_WAITPKG(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_WAITPKG
/* ISA2_WBNOINVD mask */
#define TARGET_WBNOINVD ((ix86_isa_flags2 & OPTION_MASK_ISA2_WBNOINVD) != 0)
#define TARGET_WBNOINVD_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_WBNOINVD) != 0)
#define TARGET_WBNOINVD_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_WBNOINVD) != 0)
#define TARGET_EXPLICIT_WBNOINVD_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_WBNOINVD) != 0)
#define SET_TARGET_WBNOINVD(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_WBNOINVD
/* ISA2_WIDEKL mask */
#define TARGET_WIDEKL ((ix86_isa_flags2 & OPTION_MASK_ISA2_WIDEKL) != 0)
#define TARGET_WIDEKL_P(ix86_isa_flags2) (((ix86_isa_flags2) & OPTION_MASK_ISA2_WIDEKL) != 0)
#define TARGET_WIDEKL_OPTS_P(opts) (((opts->x_ix86_isa_flags2) & OPTION_MASK_ISA2_WIDEKL) != 0)
#define TARGET_EXPLICIT_WIDEKL_P(opts) ((opts->x_ix86_isa_flags2_explicit & OPTION_MASK_ISA2_WIDEKL) != 0)
#define SET_TARGET_WIDEKL(opts) opts->x_ix86_isa_flags2 |= OPTION_MASK_ISA2_WIDEKL
/* ABI_X32 mask */
#define TARGET_ABI_X32 ((ix86_isa_flags & OPTION_MASK_ABI_X32) != 0)
#define TARGET_ABI_X32_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ABI_X32) != 0)
#define TARGET_ABI_X32_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ABI_X32) != 0)
#define TARGET_EXPLICIT_ABI_X32_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ABI_X32) != 0)
#define SET_TARGET_ABI_X32(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ABI_X32
/* ISA_XOP mask */
#define TARGET_XOP ((ix86_isa_flags & OPTION_MASK_ISA_XOP) != 0)
#define TARGET_XOP_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_XOP) != 0)
#define TARGET_XOP_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_XOP) != 0)
#define TARGET_EXPLICIT_XOP_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_XOP) != 0)
#define SET_TARGET_XOP(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_XOP
/* ISA_XSAVE mask */
#define TARGET_XSAVE ((ix86_isa_flags & OPTION_MASK_ISA_XSAVE) != 0)
#define TARGET_XSAVE_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_XSAVE) != 0)
#define TARGET_XSAVE_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_XSAVE) != 0)
#define TARGET_EXPLICIT_XSAVE_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_XSAVE) != 0)
#define SET_TARGET_XSAVE(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_XSAVE
/* ISA_XSAVEC mask */
#define TARGET_XSAVEC ((ix86_isa_flags & OPTION_MASK_ISA_XSAVEC) != 0)
#define TARGET_XSAVEC_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_XSAVEC) != 0)
#define TARGET_XSAVEC_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_XSAVEC) != 0)
#define TARGET_EXPLICIT_XSAVEC_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_XSAVEC) != 0)
#define SET_TARGET_XSAVEC(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_XSAVEC
/* ISA_XSAVEOPT mask */
#define TARGET_XSAVEOPT ((ix86_isa_flags & OPTION_MASK_ISA_XSAVEOPT) != 0)
#define TARGET_XSAVEOPT_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_XSAVEOPT) != 0)
#define TARGET_XSAVEOPT_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_XSAVEOPT) != 0)
#define TARGET_EXPLICIT_XSAVEOPT_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_XSAVEOPT) != 0)
#define SET_TARGET_XSAVEOPT(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_XSAVEOPT
/* ISA_XSAVES mask */
#define TARGET_XSAVES ((ix86_isa_flags & OPTION_MASK_ISA_XSAVES) != 0)
#define TARGET_XSAVES_P(ix86_isa_flags) (((ix86_isa_flags) & OPTION_MASK_ISA_XSAVES) != 0)
#define TARGET_XSAVES_OPTS_P(opts) (((opts->x_ix86_isa_flags) & OPTION_MASK_ISA_XSAVES) != 0)
#define TARGET_EXPLICIT_XSAVES_P(opts) ((opts->x_ix86_isa_flags_explicit & OPTION_MASK_ISA_XSAVES) != 0)
#define SET_TARGET_XSAVES(opts) opts->x_ix86_isa_flags |= OPTION_MASK_ISA_XSAVES

#define TARGET_ALIGN_STRINGOPS ((target_flags & MASK_NO_ALIGN_STRINGOPS) == 0)
#define TARGET_USE_FANCY_MATH_387 ((target_flags & MASK_NO_FANCY_MATH_387) == 0)
#define TARGET_PUSH_ARGS ((target_flags & MASK_NO_PUSH_ARGS) == 0)
#define TARGET_RED_ZONE ((target_flags & MASK_NO_RED_ZONE) == 0)

#define CL_Ada        (1U << 0)
#define CL_AdaSCIL    (1U << 1)
#define CL_AdaWhy     (1U << 2)
#define CL_C          (1U << 3)
#define CL_CXX        (1U << 4)
#define CL_D          (1U << 5)
#define CL_Fortran    (1U << 6)
#define CL_Go         (1U << 7)
#define CL_LTO        (1U << 8)
#define CL_LTODump    (1U << 9)
#define CL_ModulaX2   (1U << 10)
#define CL_ObjC       (1U << 11)
#define CL_ObjCXX     (1U << 12)
#define CL_Rust       (1U << 13)
#define CL_LANG_ALL   ((1U << 14) - 1)

enum opt_code
{
  OPT____ = 0,                               /* -### */
  /* OPT__all_warnings = 1, */               /* --all-warnings */
  /* OPT__ansi = 2, */                       /* --ansi */
  /* OPT__assemble = 3, */                   /* --assemble */
  /* OPT__assert = 4, */                     /* --assert */
  /* OPT__assert_ = 5, */                    /* --assert= */
  /* OPT__comments = 6, */                   /* --comments */
  /* OPT__comments_in_macros = 7, */         /* --comments-in-macros */
  /* OPT__compile = 8, */                    /* --compile */
  OPT__completion_ = 9,                      /* --completion= */
  /* OPT__coverage = 10, */                  /* --coverage */
  /* OPT__debug = 11, */                     /* --debug */
  /* OPT__define_macro = 12, */              /* --define-macro */
  /* OPT__define_macro_ = 13, */             /* --define-macro= */
  /* OPT__dependencies = 14, */              /* --dependencies */
  /* OPT__dump = 15, */                      /* --dump */
  /* OPT__dump_ = 16, */                     /* --dump= */
  /* OPT__dumpbase = 17, */                  /* --dumpbase */
  /* OPT__dumpbase_ext = 18, */              /* --dumpbase-ext */
  /* OPT__dumpdir = 19, */                   /* --dumpdir */
  /* OPT__entry = 20, */                     /* --entry */
  /* OPT__entry_ = 21, */                    /* --entry= */
  /* OPT__extra_warnings = 22, */            /* --extra-warnings */
  /* OPT__for_assembler = 23, */             /* --for-assembler */
  /* OPT__for_assembler_ = 24, */            /* --for-assembler= */
  /* OPT__for_linker = 25, */                /* --for-linker */
  /* OPT__for_linker_ = 26, */               /* --for-linker= */
  /* OPT__force_link = 27, */                /* --force-link */
  /* OPT__force_link_ = 28, */               /* --force-link= */
  OPT__help = 29,                            /* --help */
  OPT__help_ = 30,                           /* --help= */
  /* OPT__imacros = 31, */                   /* --imacros */
  /* OPT__imacros_ = 32, */                  /* --imacros= */
  /* OPT__include = 33, */                   /* --include */
  /* OPT__include_barrier = 34, */           /* --include-barrier */
  /* OPT__include_directory = 35, */         /* --include-directory */
  /* OPT__include_directory_after = 36, */   /* --include-directory-after */
  /* OPT__include_directory_after_ = 37, */  /* --include-directory-after= */
  /* OPT__include_directory_ = 38, */        /* --include-directory= */
  /* OPT__include_prefix = 39, */            /* --include-prefix */
  /* OPT__include_prefix_ = 40, */           /* --include-prefix= */
  /* OPT__include_with_prefix = 41, */       /* --include-with-prefix */
  /* OPT__include_with_prefix_after = 42, */ /* --include-with-prefix-after */
  /* OPT__include_with_prefix_after_ = 43, *//* --include-with-prefix-after= */
  /* OPT__include_with_prefix_before = 44, *//* --include-with-prefix-before */
  /* OPT__include_with_prefix_before_ = 45, *//* --include-with-prefix-before= */
  /* OPT__include_with_prefix_ = 46, */      /* --include-with-prefix= */
  /* OPT__include_ = 47, */                  /* --include= */
  /* OPT__language = 48, */                  /* --language */
  /* OPT__language_ = 49, */                 /* --language= */
  /* OPT__library_directory = 50, */         /* --library-directory */
  /* OPT__library_directory_ = 51, */        /* --library-directory= */
  /* OPT__no_canonical_prefixes = 52, */     /* --no-canonical-prefixes */
  /* OPT__no_integrated_cpp = 53, */         /* --no-integrated-cpp */
  /* OPT__no_line_commands = 54, */          /* --no-line-commands */
  /* OPT__no_standard_includes = 55, */      /* --no-standard-includes */
  /* OPT__no_standard_libraries = 56, */     /* --no-standard-libraries */
  OPT__no_sysroot_suffix = 57,               /* --no-sysroot-suffix */
  /* OPT__no_warnings = 58, */               /* --no-warnings */
  /* OPT__optimize = 59, */                  /* --optimize */
  /* OPT__output = 60, */                    /* --output */
  OPT__output_pch = 61,                      /* --output-pch */
  /* OPT__output_ = 62, */                   /* --output= */
  OPT__param_align_loop_iterations_ = 63,    /* --param=align-loop-iterations= */
  OPT__param_align_threshold_ = 64,          /* --param=align-threshold= */
  OPT__param_analyzer_bb_explosion_factor_ = 65,/* --param=analyzer-bb-explosion-factor= */
  OPT__param_analyzer_max_constraints_ = 66, /* --param=analyzer-max-constraints= */
  OPT__param_analyzer_max_enodes_for_full_dump_ = 67,/* --param=analyzer-max-enodes-for-full-dump= */
  OPT__param_analyzer_max_enodes_per_program_point_ = 68,/* --param=analyzer-max-enodes-per-program-point= */
  OPT__param_analyzer_max_infeasible_edges_ = 69,/* --param=analyzer-max-infeasible-edges= */
  OPT__param_analyzer_max_recursion_depth_ = 70,/* --param=analyzer-max-recursion-depth= */
  OPT__param_analyzer_max_svalue_depth_ = 71,/* --param=analyzer-max-svalue-depth= */
  OPT__param_analyzer_min_snodes_for_call_summary_ = 72,/* --param=analyzer-min-snodes-for-call-summary= */
  OPT__param_analyzer_text_art_ideal_canvas_width_ = 73,/* --param=analyzer-text-art-ideal-canvas-width= */
  OPT__param_analyzer_text_art_string_ellipsis_head_len_ = 74,/* --param=analyzer-text-art-string-ellipsis-head-len= */
  OPT__param_analyzer_text_art_string_ellipsis_tail_len_ = 75,/* --param=analyzer-text-art-string-ellipsis-tail-len= */
  OPT__param_analyzer_text_art_string_ellipsis_threshold_ = 76,/* --param=analyzer-text-art-string-ellipsis-threshold= */
  OPT__param_asan_globals_ = 77,             /* --param=asan-globals= */
  OPT__param_asan_instrument_allocas_ = 78,  /* --param=asan-instrument-allocas= */
  OPT__param_asan_instrument_reads_ = 79,    /* --param=asan-instrument-reads= */
  OPT__param_asan_instrument_writes_ = 80,   /* --param=asan-instrument-writes= */
  OPT__param_asan_instrumentation_with_call_threshold_ = 81,/* --param=asan-instrumentation-with-call-threshold= */
  OPT__param_asan_kernel_mem_intrinsic_prefix_ = 82,/* --param=asan-kernel-mem-intrinsic-prefix= */
  OPT__param_asan_memintrin_ = 83,           /* --param=asan-memintrin= */
  OPT__param_asan_stack_ = 84,               /* --param=asan-stack= */
  OPT__param_asan_use_after_return_ = 85,    /* --param=asan-use-after-return= */
  OPT__param_avg_loop_niter_ = 86,           /* --param=avg-loop-niter= */
  OPT__param_avoid_fma_max_bits_ = 87,       /* --param=avoid-fma-max-bits= */
  OPT__param_builtin_expect_probability_ = 88,/* --param=builtin-expect-probability= */
  OPT__param_builtin_string_cmp_inline_length_ = 89,/* --param=builtin-string-cmp-inline-length= */
  OPT__param_case_values_threshold_ = 90,    /* --param=case-values-threshold= */
  OPT__param_comdat_sharing_probability_ = 91,/* --param=comdat-sharing-probability= */
  OPT__param_constructive_interference_size_ = 92,/* --param=constructive-interference-size= */
  OPT__param_cxx_max_namespaces_for_diagnostic_help_ = 93,/* --param=cxx-max-namespaces-for-diagnostic-help= */
  OPT__param_destructive_interference_size_ = 94,/* --param=destructive-interference-size= */
  OPT__param_dse_max_alias_queries_per_store_ = 95,/* --param=dse-max-alias-queries-per-store= */
  OPT__param_dse_max_object_size_ = 96,      /* --param=dse-max-object-size= */
  OPT__param_early_inlining_insns_ = 97,     /* --param=early-inlining-insns= */
  OPT__param_fsm_scale_path_stmts_ = 98,     /* --param=fsm-scale-path-stmts= */
  OPT__param_fully_pipelined_fma_ = 99,      /* --param=fully-pipelined-fma= */
  OPT__param_gcse_after_reload_critical_fraction_ = 100,/* --param=gcse-after-reload-critical-fraction= */
  OPT__param_gcse_after_reload_partial_fraction_ = 101,/* --param=gcse-after-reload-partial-fraction= */
  OPT__param_gcse_cost_distance_ratio_ = 102,/* --param=gcse-cost-distance-ratio= */
  OPT__param_gcse_unrestricted_cost_ = 103,  /* --param=gcse-unrestricted-cost= */
  OPT__param_ggc_min_expand_ = 104,          /* --param=ggc-min-expand= */
  OPT__param_ggc_min_heapsize_ = 105,        /* --param=ggc-min-heapsize= */
  OPT__param_gimple_fe_computed_hot_bb_threshold_ = 106,/* --param=gimple-fe-computed-hot-bb-threshold= */
  OPT__param_graphite_allow_codegen_errors_ = 107,/* --param=graphite-allow-codegen-errors= */
  OPT__param_graphite_max_arrays_per_scop_ = 108,/* --param=graphite-max-arrays-per-scop= */
  OPT__param_graphite_max_nb_scop_params_ = 109,/* --param=graphite-max-nb-scop-params= */
  OPT__param_hardcfr_max_blocks_ = 110,      /* --param=hardcfr-max-blocks= */
  OPT__param_hardcfr_max_inline_blocks_ = 111,/* --param=hardcfr-max-inline-blocks= */
  OPT__param_hash_table_verification_limit_ = 112,/* --param=hash-table-verification-limit= */
  OPT__param_hot_bb_count_fraction_ = 113,   /* --param=hot-bb-count-fraction= */
  OPT__param_hot_bb_count_ws_permille_ = 114,/* --param=hot-bb-count-ws-permille= */
  OPT__param_hot_bb_frequency_fraction_ = 115,/* --param=hot-bb-frequency-fraction= */
  OPT__param_hwasan_instrument_allocas_ = 116,/* --param=hwasan-instrument-allocas= */
  OPT__param_hwasan_instrument_mem_intrinsics_ = 117,/* --param=hwasan-instrument-mem-intrinsics= */
  OPT__param_hwasan_instrument_reads_ = 118, /* --param=hwasan-instrument-reads= */
  OPT__param_hwasan_instrument_stack_ = 119, /* --param=hwasan-instrument-stack= */
  OPT__param_hwasan_instrument_writes_ = 120,/* --param=hwasan-instrument-writes= */
  OPT__param_hwasan_random_frame_tag_ = 121, /* --param=hwasan-random-frame-tag= */
  OPT__param_inline_heuristics_hint_percent_ = 122,/* --param=inline-heuristics-hint-percent= */
  OPT__param_inline_min_speedup_ = 123,      /* --param=inline-min-speedup= */
  OPT__param_inline_unit_growth_ = 124,      /* --param=inline-unit-growth= */
  OPT__param_integer_share_limit_ = 125,     /* --param=integer-share-limit= */
  OPT__param_ipa_cp_eval_threshold_ = 126,   /* --param=ipa-cp-eval-threshold= */
  OPT__param_ipa_cp_large_unit_insns_ = 127, /* --param=ipa-cp-large-unit-insns= */
  OPT__param_ipa_cp_loop_hint_bonus_ = 128,  /* --param=ipa-cp-loop-hint-bonus= */
  OPT__param_ipa_cp_max_recursive_depth_ = 129,/* --param=ipa-cp-max-recursive-depth= */
  OPT__param_ipa_cp_min_recursive_probability_ = 130,/* --param=ipa-cp-min-recursive-probability= */
  OPT__param_ipa_cp_profile_count_base_ = 131,/* --param=ipa-cp-profile-count-base= */
  OPT__param_ipa_cp_recursion_penalty_ = 132,/* --param=ipa-cp-recursion-penalty= */
  OPT__param_ipa_cp_recursive_freq_factor_ = 133,/* --param=ipa-cp-recursive-freq-factor= */
  OPT__param_ipa_cp_single_call_penalty_ = 134,/* --param=ipa-cp-single-call-penalty= */
  OPT__param_ipa_cp_unit_growth_ = 135,      /* --param=ipa-cp-unit-growth= */
  OPT__param_ipa_cp_value_list_size_ = 136,  /* --param=ipa-cp-value-list-size= */
  OPT__param_ipa_jump_function_lookups_ = 137,/* --param=ipa-jump-function-lookups= */
  OPT__param_ipa_max_aa_steps_ = 138,        /* --param=ipa-max-aa-steps= */
  OPT__param_ipa_max_agg_items_ = 139,       /* --param=ipa-max-agg-items= */
  OPT__param_ipa_max_loop_predicates_ = 140, /* --param=ipa-max-loop-predicates= */
  OPT__param_ipa_max_param_expr_ops_ = 141,  /* --param=ipa-max-param-expr-ops= */
  OPT__param_ipa_max_switch_predicate_bounds_ = 142,/* --param=ipa-max-switch-predicate-bounds= */
  OPT__param_ipa_sra_deref_prob_threshold_ = 143,/* --param=ipa-sra-deref-prob-threshold= */
  OPT__param_ipa_sra_max_replacements_ = 144,/* --param=ipa-sra-max-replacements= */
  OPT__param_ipa_sra_ptr_growth_factor_ = 145,/* --param=ipa-sra-ptr-growth-factor= */
  OPT__param_ipa_sra_ptrwrap_growth_factor_ = 146,/* --param=ipa-sra-ptrwrap-growth-factor= */
  OPT__param_ira_consider_dup_in_all_alts_ = 147,/* --param=ira-consider-dup-in-all-alts= */
  OPT__param_ira_loop_reserved_regs_ = 148,  /* --param=ira-loop-reserved-regs= */
  OPT__param_ira_max_conflict_table_size_ = 149,/* --param=ira-max-conflict-table-size= */
  OPT__param_ira_max_loops_num_ = 150,       /* --param=ira-max-loops-num= */
  OPT__param_ira_simple_lra_insn_threshold_ = 151,/* --param=ira-simple-lra-insn-threshold= */
  OPT__param_iv_always_prune_cand_set_bound_ = 152,/* --param=iv-always-prune-cand-set-bound= */
  OPT__param_iv_consider_all_candidates_bound_ = 153,/* --param=iv-consider-all-candidates-bound= */
  OPT__param_iv_max_considered_uses_ = 154,  /* --param=iv-max-considered-uses= */
  OPT__param_jump_table_max_growth_ratio_for_size_ = 155,/* --param=jump-table-max-growth-ratio-for-size= */
  OPT__param_jump_table_max_growth_ratio_for_speed_ = 156,/* --param=jump-table-max-growth-ratio-for-speed= */
  OPT__param_l1_cache_line_size_ = 157,      /* --param=l1-cache-line-size= */
  OPT__param_l1_cache_size_ = 158,           /* --param=l1-cache-size= */
  OPT__param_l2_cache_size_ = 159,           /* --param=l2-cache-size= */
  OPT__param_large_function_growth_ = 160,   /* --param=large-function-growth= */
  OPT__param_large_function_insns_ = 161,    /* --param=large-function-insns= */
  OPT__param_large_stack_frame_growth_ = 162,/* --param=large-stack-frame-growth= */
  OPT__param_large_stack_frame_ = 163,       /* --param=large-stack-frame= */
  OPT__param_large_unit_insns_ = 164,        /* --param=large-unit-insns= */
  OPT__param_lazy_modules_ = 165,            /* --param=lazy-modules= */
  OPT__param_lim_expensive_ = 166,           /* --param=lim-expensive= */
  OPT__param_logical_op_non_short_circuit_ = 167,/* --param=logical-op-non-short-circuit= */
  OPT__param_loop_block_tile_size_ = 168,    /* --param=loop-block-tile-size= */
  OPT__param_loop_interchange_max_num_stmts_ = 169,/* --param=loop-interchange-max-num-stmts= */
  OPT__param_loop_interchange_stride_ratio_ = 170,/* --param=loop-interchange-stride-ratio= */
  OPT__param_loop_invariant_max_bbs_in_loop_ = 171,/* --param=loop-invariant-max-bbs-in-loop= */
  OPT__param_loop_max_datarefs_for_datadeps_ = 172,/* --param=loop-max-datarefs-for-datadeps= */
  OPT__param_loop_versioning_max_inner_insns_ = 173,/* --param=loop-versioning-max-inner-insns= */
  OPT__param_loop_versioning_max_outer_insns_ = 174,/* --param=loop-versioning-max-outer-insns= */
  OPT__param_lra_inheritance_ebb_probability_cutoff_ = 175,/* --param=lra-inheritance-ebb-probability-cutoff= */
  OPT__param_lra_max_considered_reload_pseudos_ = 176,/* --param=lra-max-considered-reload-pseudos= */
  OPT__param_lto_max_partition_ = 177,       /* --param=lto-max-partition= */
  OPT__param_lto_max_streaming_parallelism_ = 178,/* --param=lto-max-streaming-parallelism= */
  OPT__param_lto_min_partition_ = 179,       /* --param=lto-min-partition= */
  OPT__param_lto_partitions_ = 180,          /* --param=lto-partitions= */
  OPT__param_max_average_unrolled_insns_ = 181,/* --param=max-average-unrolled-insns= */
  OPT__param_max_combine_insns_ = 182,       /* --param=max-combine-insns= */
  OPT__param_max_completely_peel_loop_nest_depth_ = 183,/* --param=max-completely-peel-loop-nest-depth= */
  OPT__param_max_completely_peel_times_ = 184,/* --param=max-completely-peel-times= */
  OPT__param_max_completely_peeled_insns_ = 185,/* --param=max-completely-peeled-insns= */
  OPT__param_max_crossjump_edges_ = 186,     /* --param=max-crossjump-edges= */
  OPT__param_max_cse_insns_ = 187,           /* --param=max-cse-insns= */
  OPT__param_max_cse_path_length_ = 188,     /* --param=max-cse-path-length= */
  OPT__param_max_cselib_memory_locations_ = 189,/* --param=max-cselib-memory-locations= */
  OPT__param_max_debug_marker_count_ = 190,  /* --param=max-debug-marker-count= */
  OPT__param_max_delay_slot_insn_search_ = 191,/* --param=max-delay-slot-insn-search= */
  OPT__param_max_delay_slot_live_search_ = 192,/* --param=max-delay-slot-live-search= */
  OPT__param_max_dse_active_local_stores_ = 193,/* --param=max-dse-active-local-stores= */
  OPT__param_max_early_inliner_iterations_ = 194,/* --param=max-early-inliner-iterations= */
  OPT__param_max_fields_for_field_sensitive_ = 195,/* --param=max-fields-for-field-sensitive= */
  OPT__param_max_find_base_term_values_ = 196,/* --param=max-find-base-term-values= */
  OPT__param_max_fsm_thread_path_insns_ = 197,/* --param=max-fsm-thread-path-insns= */
  OPT__param_max_gcse_insertion_ratio_ = 198,/* --param=max-gcse-insertion-ratio= */
  OPT__param_max_gcse_memory_ = 199,         /* --param=max-gcse-memory= */
  OPT__param_max_goto_duplication_insns_ = 200,/* --param=max-goto-duplication-insns= */
  OPT__param_max_grow_copy_bb_insns_ = 201,  /* --param=max-grow-copy-bb-insns= */
  OPT__param_max_hoist_depth_ = 202,         /* --param=max-hoist-depth= */
  OPT__param_max_inline_functions_called_once_insns_ = 203,/* --param=max-inline-functions-called-once-insns= */
  OPT__param_max_inline_functions_called_once_loop_depth_ = 204,/* --param=max-inline-functions-called-once-loop-depth= */
  OPT__param_max_inline_insns_auto_ = 205,   /* --param=max-inline-insns-auto= */
  OPT__param_max_inline_insns_recursive_auto_ = 206,/* --param=max-inline-insns-recursive-auto= */
  OPT__param_max_inline_insns_recursive_ = 207,/* --param=max-inline-insns-recursive= */
  OPT__param_max_inline_insns_single_ = 208, /* --param=max-inline-insns-single= */
  OPT__param_max_inline_insns_size_ = 209,   /* --param=max-inline-insns-size= */
  OPT__param_max_inline_insns_small_ = 210,  /* --param=max-inline-insns-small= */
  OPT__param_max_inline_recursive_depth_auto_ = 211,/* --param=max-inline-recursive-depth-auto= */
  OPT__param_max_inline_recursive_depth_ = 212,/* --param=max-inline-recursive-depth= */
  OPT__param_max_isl_operations_ = 213,      /* --param=max-isl-operations= */
  OPT__param_max_iterations_computation_cost_ = 214,/* --param=max-iterations-computation-cost= */
  OPT__param_max_iterations_to_track_ = 215, /* --param=max-iterations-to-track= */
  OPT__param_max_jump_thread_duplication_stmts_ = 216,/* --param=max-jump-thread-duplication-stmts= */
  OPT__param_max_jump_thread_paths_ = 217,   /* --param=max-jump-thread-paths= */
  OPT__param_max_last_value_rtl_ = 218,      /* --param=max-last-value-rtl= */
  OPT__param_max_loop_header_insns_ = 219,   /* --param=max-loop-header-insns= */
  OPT__param_max_modulo_backtrack_attempts_ = 220,/* --param=max-modulo-backtrack-attempts= */
  OPT__param_max_partial_antic_length_ = 221,/* --param=max-partial-antic-length= */
  OPT__param_max_peel_branches_ = 222,       /* --param=max-peel-branches= */
  OPT__param_max_peel_times_ = 223,          /* --param=max-peel-times= */
  OPT__param_max_peeled_insns_ = 224,        /* --param=max-peeled-insns= */
  OPT__param_max_pending_list_length_ = 225, /* --param=max-pending-list-length= */
  OPT__param_max_pipeline_region_blocks_ = 226,/* --param=max-pipeline-region-blocks= */
  OPT__param_max_pipeline_region_insns_ = 227,/* --param=max-pipeline-region-insns= */
  OPT__param_max_pow_sqrt_depth_ = 228,      /* --param=max-pow-sqrt-depth= */
  OPT__param_max_predicted_iterations_ = 229,/* --param=max-predicted-iterations= */
  OPT__param_max_reload_search_insns_ = 230, /* --param=max-reload-search-insns= */
  OPT__param_max_rtl_if_conversion_insns_ = 231,/* --param=max-rtl-if-conversion-insns= */
  OPT__param_max_rtl_if_conversion_predictable_cost_ = 232,/* --param=max-rtl-if-conversion-predictable-cost= */
  OPT__param_max_rtl_if_conversion_unpredictable_cost_ = 233,/* --param=max-rtl-if-conversion-unpredictable-cost= */
  OPT__param_max_sched_extend_regions_iters_ = 234,/* --param=max-sched-extend-regions-iters= */
  OPT__param_max_sched_insn_conflict_delay_ = 235,/* --param=max-sched-insn-conflict-delay= */
  OPT__param_max_sched_ready_insns_ = 236,   /* --param=max-sched-ready-insns= */
  OPT__param_max_sched_region_blocks_ = 237, /* --param=max-sched-region-blocks= */
  OPT__param_max_sched_region_insns_ = 238,  /* --param=max-sched-region-insns= */
  OPT__param_max_slsr_cand_scan_ = 239,      /* --param=max-slsr-cand-scan= */
  OPT__param_max_speculative_devirt_maydefs_ = 240,/* --param=max-speculative-devirt-maydefs= */
  OPT__param_max_ssa_name_query_depth_ = 241,/* --param=max-ssa-name-query-depth= */
  OPT__param_max_store_chains_to_track_ = 242,/* --param=max-store-chains-to-track= */
  OPT__param_max_stores_to_merge_ = 243,     /* --param=max-stores-to-merge= */
  OPT__param_max_stores_to_sink_ = 244,      /* --param=max-stores-to-sink= */
  OPT__param_max_stores_to_track_ = 245,     /* --param=max-stores-to-track= */
  OPT__param_max_tail_merge_comparisons_ = 246,/* --param=max-tail-merge-comparisons= */
  OPT__param_max_tail_merge_iterations_ = 247,/* --param=max-tail-merge-iterations= */
  OPT__param_max_tracked_strlens_ = 248,     /* --param=max-tracked-strlens= */
  OPT__param_max_tree_if_conversion_phi_args_ = 249,/* --param=max-tree-if-conversion-phi-args= */
  OPT__param_max_unroll_times_ = 250,        /* --param=max-unroll-times= */
  OPT__param_max_unrolled_insns_ = 251,      /* --param=max-unrolled-insns= */
  OPT__param_max_unswitch_depth_ = 252,      /* --param=max-unswitch-depth= */
  OPT__param_max_unswitch_insns_ = 253,      /* --param=max-unswitch-insns= */
  OPT__param_max_variable_expansions_in_unroller_ = 254,/* --param=max-variable-expansions-in-unroller= */
  OPT__param_max_vartrack_expr_depth_ = 255, /* --param=max-vartrack-expr-depth= */
  OPT__param_max_vartrack_reverse_op_size_ = 256,/* --param=max-vartrack-reverse-op-size= */
  OPT__param_max_vartrack_size_ = 257,       /* --param=max-vartrack-size= */
  OPT__param_min_crossjump_insns_ = 258,     /* --param=min-crossjump-insns= */
  OPT__param_min_inline_recursive_probability_ = 259,/* --param=min-inline-recursive-probability= */
  OPT__param_min_insn_to_prefetch_ratio_ = 260,/* --param=min-insn-to-prefetch-ratio= */
  OPT__param_min_loop_cond_split_prob_ = 261,/* --param=min-loop-cond-split-prob= */
  OPT__param_min_nondebug_insn_uid_ = 262,   /* --param=min-nondebug-insn-uid= */
  OPT__param_min_pagesize_ = 263,            /* --param=min-pagesize= */
  OPT__param_min_size_for_stack_sharing_ = 264,/* --param=min-size-for-stack-sharing= */
  OPT__param_min_spec_prob_ = 265,           /* --param=min-spec-prob= */
  OPT__param_min_vect_loop_bound_ = 266,     /* --param=min-vect-loop-bound= */
  OPT__param_modref_max_accesses_ = 267,     /* --param=modref-max-accesses= */
  OPT__param_modref_max_adjustments_ = 268,  /* --param=modref-max-adjustments= */
  OPT__param_modref_max_bases_ = 269,        /* --param=modref-max-bases= */
  OPT__param_modref_max_depth_ = 270,        /* --param=modref-max-depth= */
  OPT__param_modref_max_escape_points_ = 271,/* --param=modref-max-escape-points= */
  OPT__param_modref_max_refs_ = 272,         /* --param=modref-max-refs= */
  OPT__param_modref_max_tests_ = 273,        /* --param=modref-max-tests= */
  OPT__param_openacc_kernels_ = 274,         /* --param=openacc-kernels= */
  OPT__param_openacc_privatization_ = 275,   /* --param=openacc-privatization= */
  OPT__param_parloops_chunk_size_ = 276,     /* --param=parloops-chunk-size= */
  OPT__param_parloops_min_per_thread_ = 277, /* --param=parloops-min-per-thread= */
  OPT__param_parloops_schedule_ = 278,       /* --param=parloops-schedule= */
  OPT__param_partial_inlining_entry_probability_ = 279,/* --param=partial-inlining-entry-probability= */
  OPT__param_predictable_branch_outcome_ = 280,/* --param=predictable-branch-outcome= */
  OPT__param_prefetch_dynamic_strides_ = 281,/* --param=prefetch-dynamic-strides= */
  OPT__param_prefetch_latency_ = 282,        /* --param=prefetch-latency= */
  OPT__param_prefetch_min_insn_to_mem_ratio_ = 283,/* --param=prefetch-min-insn-to-mem-ratio= */
  OPT__param_prefetch_minimum_stride_ = 284, /* --param=prefetch-minimum-stride= */
  OPT__param_profile_func_internal_id_ = 285,/* --param=profile-func-internal-id= */
  OPT__param_ranger_debug_ = 286,            /* --param=ranger-debug= */
  OPT__param_ranger_logical_depth_ = 287,    /* --param=ranger-logical-depth= */
  OPT__param_ranger_recompute_depth_ = 288,  /* --param=ranger-recompute-depth= */
  OPT__param_relation_block_limit_ = 289,    /* --param=relation-block-limit= */
  OPT__param_rpo_vn_max_loop_depth_ = 290,   /* --param=rpo-vn-max-loop-depth= */
  OPT__param_sccvn_max_alias_queries_per_access_ = 291,/* --param=sccvn-max-alias-queries-per-access= */
  OPT__param_scev_max_expr_complexity_ = 292,/* --param=scev-max-expr-complexity= */
  OPT__param_scev_max_expr_size_ = 293,      /* --param=scev-max-expr-size= */
  OPT__param_sched_autopref_queue_depth_ = 294,/* --param=sched-autopref-queue-depth= */
  OPT__param_sched_mem_true_dep_cost_ = 295, /* --param=sched-mem-true-dep-cost= */
  OPT__param_sched_pressure_algorithm_ = 296,/* --param=sched-pressure-algorithm= */
  OPT__param_sched_spec_prob_cutoff_ = 297,  /* --param=sched-spec-prob-cutoff= */
  OPT__param_sched_state_edge_prob_cutoff_ = 298,/* --param=sched-state-edge-prob-cutoff= */
  OPT__param_selsched_insns_to_rename_ = 299,/* --param=selsched-insns-to-rename= */
  OPT__param_selsched_max_lookahead_ = 300,  /* --param=selsched-max-lookahead= */
  OPT__param_selsched_max_sched_times_ = 301,/* --param=selsched-max-sched-times= */
  OPT__param_simultaneous_prefetches_ = 302, /* --param=simultaneous-prefetches= */
  OPT__param_sink_frequency_threshold_ = 303,/* --param=sink-frequency-threshold= */
  OPT__param_sms_dfa_history_ = 304,         /* --param=sms-dfa-history= */
  OPT__param_sms_loop_average_count_threshold_ = 305,/* --param=sms-loop-average-count-threshold= */
  OPT__param_sms_max_ii_factor_ = 306,       /* --param=sms-max-ii-factor= */
  OPT__param_sms_min_sc_ = 307,              /* --param=sms-min-sc= */
  OPT__param_sra_max_propagations_ = 308,    /* --param=sra-max-propagations= */
  OPT__param_sra_max_scalarization_size_Osize_ = 309,/* --param=sra-max-scalarization-size-Osize= */
  OPT__param_sra_max_scalarization_size_Ospeed_ = 310,/* --param=sra-max-scalarization-size-Ospeed= */
  OPT__param_ssa_name_def_chain_limit_ = 311,/* --param=ssa-name-def-chain-limit= */
  OPT__param_ssp_buffer_size_ = 312,         /* --param=ssp-buffer-size= */
  OPT__param_stack_clash_protection_guard_size_ = 313,/* --param=stack-clash-protection-guard-size= */
  OPT__param_stack_clash_protection_probe_interval_ = 314,/* --param=stack-clash-protection-probe-interval= */
  OPT__param_store_merging_allow_unaligned_ = 315,/* --param=store-merging-allow-unaligned= */
  OPT__param_store_merging_max_size_ = 316,  /* --param=store-merging-max-size= */
  OPT__param_switch_conversion_max_branch_ratio_ = 317,/* --param=switch-conversion-max-branch-ratio= */
  OPT__param_threader_debug_ = 318,          /* --param=threader-debug= */
  OPT__param_tm_max_aggregate_size_ = 319,   /* --param=tm-max-aggregate-size= */
  OPT__param_tracer_dynamic_coverage_feedback_ = 320,/* --param=tracer-dynamic-coverage-feedback= */
  OPT__param_tracer_dynamic_coverage_ = 321, /* --param=tracer-dynamic-coverage= */
  OPT__param_tracer_max_code_growth_ = 322,  /* --param=tracer-max-code-growth= */
  OPT__param_tracer_min_branch_probability_feedback_ = 323,/* --param=tracer-min-branch-probability-feedback= */
  OPT__param_tracer_min_branch_probability_ = 324,/* --param=tracer-min-branch-probability= */
  OPT__param_tracer_min_branch_ratio_ = 325, /* --param=tracer-min-branch-ratio= */
  OPT__param_tree_reassoc_width_ = 326,      /* --param=tree-reassoc-width= */
  OPT__param_tsan_distinguish_volatile_ = 327,/* --param=tsan-distinguish-volatile= */
  OPT__param_tsan_instrument_func_entry_exit_ = 328,/* --param=tsan-instrument-func-entry-exit= */
  OPT__param_uninit_control_dep_attempts_ = 329,/* --param=uninit-control-dep-attempts= */
  OPT__param_uninit_max_chain_len_ = 330,    /* --param=uninit-max-chain-len= */
  OPT__param_uninit_max_num_chains_ = 331,   /* --param=uninit-max-num-chains= */
  OPT__param_uninlined_function_insns_ = 332,/* --param=uninlined-function-insns= */
  OPT__param_uninlined_function_time_ = 333, /* --param=uninlined-function-time= */
  OPT__param_uninlined_thunk_insns_ = 334,   /* --param=uninlined-thunk-insns= */
  OPT__param_uninlined_thunk_time_ = 335,    /* --param=uninlined-thunk-time= */
  OPT__param_unlikely_bb_count_fraction_ = 336,/* --param=unlikely-bb-count-fraction= */
  OPT__param_unroll_jam_max_unroll_ = 337,   /* --param=unroll-jam-max-unroll= */
  OPT__param_unroll_jam_min_percent_ = 338,  /* --param=unroll-jam-min-percent= */
  OPT__param_use_after_scope_direct_emission_threshold_ = 339,/* --param=use-after-scope-direct-emission-threshold= */
  OPT__param_use_canonical_types_ = 340,     /* --param=use-canonical-types= */
  OPT__param_vect_epilogues_nomask_ = 341,   /* --param=vect-epilogues-nomask= */
  OPT__param_vect_induction_float_ = 342,    /* --param=vect-induction-float= */
  OPT__param_vect_inner_loop_cost_factor_ = 343,/* --param=vect-inner-loop-cost-factor= */
  OPT__param_vect_max_layout_candidates_ = 344,/* --param=vect-max-layout-candidates= */
  OPT__param_vect_max_peeling_for_alignment_ = 345,/* --param=vect-max-peeling-for-alignment= */
  OPT__param_vect_max_version_for_alias_checks_ = 346,/* --param=vect-max-version-for-alias-checks= */
  OPT__param_vect_max_version_for_alignment_checks_ = 347,/* --param=vect-max-version-for-alignment-checks= */
  OPT__param_vect_partial_vector_usage_ = 348,/* --param=vect-partial-vector-usage= */
  OPT__param_vrp_sparse_threshold_ = 349,    /* --param=vrp-sparse-threshold= */
  OPT__param_vrp_switch_limit_ = 350,        /* --param=vrp-switch-limit= */
  OPT__param_vrp_vector_threshold_ = 351,    /* --param=vrp-vector-threshold= */
  OPT__param_x86_stlf_window_ninsns_ = 352,  /* --param=x86-stlf-window-ninsns= */
  OPT__param_x86_stv_max_visits_ = 353,      /* --param=x86-stv-max-visits= */
  /* OPT__pass_exit_codes = 354, */          /* --pass-exit-codes */
  /* OPT__pedantic = 355, */                 /* --pedantic */
  /* OPT__pedantic_errors = 356, */          /* --pedantic-errors */
  /* OPT__pie = 357, */                      /* --pie */
  /* OPT__pipe = 358, */                     /* --pipe */
  /* OPT__prefix = 359, */                   /* --prefix */
  /* OPT__prefix_ = 360, */                  /* --prefix= */
  /* OPT__preprocess = 361, */               /* --preprocess */
  /* OPT__print_file_name = 362, */          /* --print-file-name */
  /* OPT__print_file_name_ = 363, */         /* --print-file-name= */
  /* OPT__print_libgcc_file_name = 364, */   /* --print-libgcc-file-name */
  /* OPT__print_missing_file_dependencies = 365, *//* --print-missing-file-dependencies */
  /* OPT__print_multi_directory = 366, */    /* --print-multi-directory */
  /* OPT__print_multi_lib = 367, */          /* --print-multi-lib */
  /* OPT__print_multi_os_directory = 368, */ /* --print-multi-os-directory */
  /* OPT__print_multiarch = 369, */          /* --print-multiarch */
  /* OPT__print_prog_name = 370, */          /* --print-prog-name */
  /* OPT__print_prog_name_ = 371, */         /* --print-prog-name= */
  /* OPT__print_search_dirs = 372, */        /* --print-search-dirs */
  /* OPT__print_sysroot = 373, */            /* --print-sysroot */
  /* OPT__print_sysroot_headers_suffix = 374, *//* --print-sysroot-headers-suffix */
  /* OPT__profile = 375, */                  /* --profile */
  /* OPT__save_temps = 376, */               /* --save-temps */
  /* OPT__shared = 377, */                   /* --shared */
  /* OPT__specs = 378, */                    /* --specs */
  /* OPT__specs_ = 379, */                   /* --specs= */
  /* OPT__static = 380, */                   /* --static */
  /* OPT__static_pie = 381, */               /* --static-pie */
  /* OPT__symbolic = 382, */                 /* --symbolic */
  /* OPT__sysroot = 383, */                  /* --sysroot */
  OPT__sysroot_ = 384,                       /* --sysroot= */
  OPT__target_help = 385,                    /* --target-help */
  /* OPT__time = 386, */                     /* --time */
  /* OPT__trace_includes = 387, */           /* --trace-includes */
  /* OPT__traditional = 388, */              /* --traditional */
  /* OPT__traditional_cpp = 389, */          /* --traditional-cpp */
  /* OPT__trigraphs = 390, */                /* --trigraphs */
  /* OPT__undefine_macro = 391, */           /* --undefine-macro */
  /* OPT__undefine_macro_ = 392, */          /* --undefine-macro= */
  /* OPT__user_dependencies = 393, */        /* --user-dependencies */
  /* OPT__verbose = 394, */                  /* --verbose */
  OPT__version = 395,                        /* --version */
  /* OPT__write_dependencies = 396, */       /* --write-dependencies */
  /* OPT__write_user_dependencies = 397, */  /* --write-user-dependencies */
  OPT_A = 398,                               /* -A */
  OPT_B = 399,                               /* -B */
  OPT_C = 400,                               /* -C */
  OPT_CC = 401,                              /* -CC */
  OPT_D = 402,                               /* -D */
  OPT_E = 403,                               /* -E */
  OPT_F = 404,                               /* -F */
  OPT_H = 405,                               /* -H */
  OPT_Hd = 406,                              /* -Hd */
  OPT_Hf = 407,                              /* -Hf */
  OPT_I = 408,                               /* -I */
  OPT_J = 409,                               /* -J */
  OPT_L = 410,                               /* -L */
  OPT_M = 411,                               /* -M */
  OPT_MD = 412,                              /* -MD */
  OPT_MF = 413,                              /* -MF */
  OPT_MG = 414,                              /* -MG */
  OPT_MM = 415,                              /* -MM */
  OPT_MMD = 416,                             /* -MMD */
  OPT_MP = 417,                              /* -MP */
  OPT_MQ = 418,                              /* -MQ */
  OPT_MT = 419,                              /* -MT */
  OPT_Mmodules = 420,                        /* -Mmodules */
  OPT_Mno_modules = 421,                     /* -Mno-modules */
  OPT_N = 422,                               /* -N */
  OPT_O = 423,                               /* -O */
  OPT_Ofast = 424,                           /* -Ofast */
  OPT_Og = 425,                              /* -Og */
  OPT_Os = 426,                              /* -Os */
  OPT_Oz = 427,                              /* -Oz */
  OPT_P = 428,                               /* -P */
  OPT_Q = 429,                               /* -Q */
  OPT_Qn = 430,                              /* -Qn */
  OPT_Qy = 431,                              /* -Qy */
  OPT_R = 432,                               /* -R */
  OPT_S = 433,                               /* -S */
  OPT_T = 434,                               /* -T */
  OPT_Tbss = 435,                            /* -Tbss */
  OPT_Tbss_ = 436,                           /* -Tbss= */
  OPT_Tdata = 437,                           /* -Tdata */
  OPT_Tdata_ = 438,                          /* -Tdata= */
  OPT_Ttext = 439,                           /* -Ttext */
  OPT_Ttext_ = 440,                          /* -Ttext= */
  OPT_U = 441,                               /* -U */
  /* OPT_W = 442, */                         /* -W */
  OPT_WNSObject_attribute = 443,             /* -WNSObject-attribute */
  OPT_Wa_ = 444,                             /* -Wa, */
  OPT_Wabi = 445,                            /* -Wabi */
  OPT_Wabi_tag = 446,                        /* -Wabi-tag */
  OPT_Wabi_ = 447,                           /* -Wabi= */
  OPT_Wabsolute_value = 448,                 /* -Wabsolute-value */
  OPT_Waddress = 449,                        /* -Waddress */
  OPT_Waddress_of_packed_member = 450,       /* -Waddress-of-packed-member */
  OPT_Waggregate_return = 451,               /* -Waggregate-return */
  OPT_Waggressive_loop_optimizations = 452,  /* -Waggressive-loop-optimizations */
  OPT_Waliasing = 453,                       /* -Waliasing */
  OPT_Walign_commons = 454,                  /* -Walign-commons */
  /* OPT_Waligned_new = 455, */              /* -Waligned-new */
  OPT_Waligned_new_ = 456,                   /* -Waligned-new= */
  OPT_Wall = 457,                            /* -Wall */
  OPT_Walloc_size = 458,                     /* -Walloc-size */
  OPT_Walloc_size_larger_than_ = 459,        /* -Walloc-size-larger-than= */
  OPT_Walloc_zero = 460,                     /* -Walloc-zero */
  OPT_Walloca = 461,                         /* -Walloca */
  OPT_Walloca_larger_than_ = 462,            /* -Walloca-larger-than= */
  OPT_Wampersand = 463,                      /* -Wampersand */
  OPT_Wanalyzer_allocation_size = 464,       /* -Wanalyzer-allocation-size */
  OPT_Wanalyzer_deref_before_check = 465,    /* -Wanalyzer-deref-before-check */
  OPT_Wanalyzer_double_fclose = 466,         /* -Wanalyzer-double-fclose */
  OPT_Wanalyzer_double_free = 467,           /* -Wanalyzer-double-free */
  OPT_Wanalyzer_exposure_through_output_file = 468,/* -Wanalyzer-exposure-through-output-file */
  OPT_Wanalyzer_exposure_through_uninit_copy = 469,/* -Wanalyzer-exposure-through-uninit-copy */
  OPT_Wanalyzer_fd_access_mode_mismatch = 470,/* -Wanalyzer-fd-access-mode-mismatch */
  OPT_Wanalyzer_fd_double_close = 471,       /* -Wanalyzer-fd-double-close */
  OPT_Wanalyzer_fd_leak = 472,               /* -Wanalyzer-fd-leak */
  OPT_Wanalyzer_fd_phase_mismatch = 473,     /* -Wanalyzer-fd-phase-mismatch */
  OPT_Wanalyzer_fd_type_mismatch = 474,      /* -Wanalyzer-fd-type-mismatch */
  OPT_Wanalyzer_fd_use_after_close = 475,    /* -Wanalyzer-fd-use-after-close */
  OPT_Wanalyzer_fd_use_without_check = 476,  /* -Wanalyzer-fd-use-without-check */
  OPT_Wanalyzer_file_leak = 477,             /* -Wanalyzer-file-leak */
  OPT_Wanalyzer_free_of_non_heap = 478,      /* -Wanalyzer-free-of-non-heap */
  OPT_Wanalyzer_imprecise_fp_arithmetic = 479,/* -Wanalyzer-imprecise-fp-arithmetic */
  OPT_Wanalyzer_infinite_loop = 480,         /* -Wanalyzer-infinite-loop */
  OPT_Wanalyzer_infinite_recursion = 481,    /* -Wanalyzer-infinite-recursion */
  OPT_Wanalyzer_jump_through_null = 482,     /* -Wanalyzer-jump-through-null */
  OPT_Wanalyzer_malloc_leak = 483,           /* -Wanalyzer-malloc-leak */
  OPT_Wanalyzer_mismatching_deallocation = 484,/* -Wanalyzer-mismatching-deallocation */
  OPT_Wanalyzer_null_argument = 485,         /* -Wanalyzer-null-argument */
  OPT_Wanalyzer_null_dereference = 486,      /* -Wanalyzer-null-dereference */
  OPT_Wanalyzer_out_of_bounds = 487,         /* -Wanalyzer-out-of-bounds */
  OPT_Wanalyzer_overlapping_buffers = 488,   /* -Wanalyzer-overlapping-buffers */
  OPT_Wanalyzer_possible_null_argument = 489,/* -Wanalyzer-possible-null-argument */
  OPT_Wanalyzer_possible_null_dereference = 490,/* -Wanalyzer-possible-null-dereference */
  OPT_Wanalyzer_putenv_of_auto_var = 491,    /* -Wanalyzer-putenv-of-auto-var */
  OPT_Wanalyzer_shift_count_negative = 492,  /* -Wanalyzer-shift-count-negative */
  OPT_Wanalyzer_shift_count_overflow = 493,  /* -Wanalyzer-shift-count-overflow */
  OPT_Wanalyzer_stale_setjmp_buffer = 494,   /* -Wanalyzer-stale-setjmp-buffer */
  OPT_Wanalyzer_symbol_too_complex = 495,    /* -Wanalyzer-symbol-too-complex */
  OPT_Wanalyzer_tainted_allocation_size = 496,/* -Wanalyzer-tainted-allocation-size */
  OPT_Wanalyzer_tainted_array_index = 497,   /* -Wanalyzer-tainted-array-index */
  OPT_Wanalyzer_tainted_assertion = 498,     /* -Wanalyzer-tainted-assertion */
  OPT_Wanalyzer_tainted_divisor = 499,       /* -Wanalyzer-tainted-divisor */
  OPT_Wanalyzer_tainted_offset = 500,        /* -Wanalyzer-tainted-offset */
  OPT_Wanalyzer_tainted_size = 501,          /* -Wanalyzer-tainted-size */
  OPT_Wanalyzer_too_complex = 502,           /* -Wanalyzer-too-complex */
  OPT_Wanalyzer_undefined_behavior_strtok = 503,/* -Wanalyzer-undefined-behavior-strtok */
  OPT_Wanalyzer_unsafe_call_within_signal_handler = 504,/* -Wanalyzer-unsafe-call-within-signal-handler */
  OPT_Wanalyzer_use_after_free = 505,        /* -Wanalyzer-use-after-free */
  OPT_Wanalyzer_use_of_pointer_in_stale_stack_frame = 506,/* -Wanalyzer-use-of-pointer-in-stale-stack-frame */
  OPT_Wanalyzer_use_of_uninitialized_value = 507,/* -Wanalyzer-use-of-uninitialized-value */
  OPT_Wanalyzer_va_arg_type_mismatch = 508,  /* -Wanalyzer-va-arg-type-mismatch */
  OPT_Wanalyzer_va_list_exhausted = 509,     /* -Wanalyzer-va-list-exhausted */
  OPT_Wanalyzer_va_list_leak = 510,          /* -Wanalyzer-va-list-leak */
  OPT_Wanalyzer_va_list_use_after_va_end = 511,/* -Wanalyzer-va-list-use-after-va-end */
  OPT_Wanalyzer_write_to_const = 512,        /* -Wanalyzer-write-to-const */
  OPT_Wanalyzer_write_to_string_literal = 513,/* -Wanalyzer-write-to-string-literal */
  OPT_Wargument_mismatch = 514,              /* -Wargument-mismatch */
  OPT_Warith_conversion = 515,               /* -Warith-conversion */
  /* OPT_Warray_bounds = 516, */             /* -Warray-bounds */
  OPT_Warray_bounds_ = 517,                  /* -Warray-bounds= */
  OPT_Warray_compare = 518,                  /* -Warray-compare */
  /* OPT_Warray_parameter = 519, */          /* -Warray-parameter */
  OPT_Warray_parameter_ = 520,               /* -Warray-parameter= */
  OPT_Warray_temporaries = 521,              /* -Warray-temporaries */
  OPT_Wassign_intercept = 522,               /* -Wassign-intercept */
  /* OPT_Wattribute_alias = 523, */          /* -Wattribute-alias */
  OPT_Wattribute_alias_ = 524,               /* -Wattribute-alias= */
  OPT_Wattribute_warning = 525,              /* -Wattribute-warning */
  OPT_Wattributes = 526,                     /* -Wattributes */
  OPT_Wattributes_ = 527,                    /* -Wattributes= */
  OPT_Wbad_function_cast = 528,              /* -Wbad-function-cast */
  /* OPT_Wbidi_chars = 529, */               /* -Wbidi-chars */
  OPT_Wbidi_chars_ = 530,                    /* -Wbidi-chars= */
  OPT_Wbool_compare = 531,                   /* -Wbool-compare */
  OPT_Wbool_operation = 532,                 /* -Wbool-operation */
  OPT_Wbuiltin_declaration_mismatch = 533,   /* -Wbuiltin-declaration-mismatch */
  OPT_Wbuiltin_macro_redefined = 534,        /* -Wbuiltin-macro-redefined */
  OPT_Wc___compat = 535,                     /* -Wc++-compat */
  /* OPT_Wc__0x_compat = 536, */             /* -Wc++0x-compat */
  OPT_Wc__11_compat = 537,                   /* -Wc++11-compat */
  OPT_Wc__11_extensions = 538,               /* -Wc++11-extensions */
  OPT_Wc__14_compat = 539,                   /* -Wc++14-compat */
  OPT_Wc__14_extensions = 540,               /* -Wc++14-extensions */
  OPT_Wc__17_compat = 541,                   /* -Wc++17-compat */
  OPT_Wc__17_extensions = 542,               /* -Wc++17-extensions */
  /* OPT_Wc__1z_compat = 543, */             /* -Wc++1z-compat */
  OPT_Wc__20_compat = 544,                   /* -Wc++20-compat */
  OPT_Wc__20_extensions = 545,               /* -Wc++20-extensions */
  OPT_Wc__23_extensions = 546,               /* -Wc++23-extensions */
  OPT_Wc__26_extensions = 547,               /* -Wc++26-extensions */
  /* OPT_Wc__2a_compat = 548, */             /* -Wc++2a-compat */
  OPT_Wc_binding_type = 549,                 /* -Wc-binding-type */
  OPT_Wc11_c23_compat = 550,                 /* -Wc11-c23-compat */
  /* OPT_Wc11_c2x_compat = 551, */           /* -Wc11-c2x-compat */
  OPT_Wc90_c99_compat = 552,                 /* -Wc90-c99-compat */
  OPT_Wc99_c11_compat = 553,                 /* -Wc99-c11-compat */
  OPT_Wcalloc_transposed_args = 554,         /* -Wcalloc-transposed-args */
  OPT_Wcannot_profile = 555,                 /* -Wcannot-profile */
  OPT_Wcase_enum = 556,                      /* -Wcase-enum */
  OPT_Wcast_align = 557,                     /* -Wcast-align */
  OPT_Wcast_align_strict = 558,              /* -Wcast-align=strict */
  OPT_Wcast_function_type = 559,             /* -Wcast-function-type */
  OPT_Wcast_qual = 560,                      /* -Wcast-qual */
  OPT_Wcast_result = 561,                    /* -Wcast-result */
  OPT_Wcast_user_defined = 562,              /* -Wcast-user-defined */
  /* OPT_Wcatch_value = 563, */              /* -Wcatch-value */
  OPT_Wcatch_value_ = 564,                   /* -Wcatch-value= */
  OPT_Wchanges_meaning = 565,                /* -Wchanges-meaning */
  OPT_Wchar_subscripts = 566,                /* -Wchar-subscripts */
  OPT_Wcharacter_truncation = 567,           /* -Wcharacter-truncation */
  OPT_Wchkp = 568,                           /* -Wchkp */
  OPT_Wclass_conversion = 569,               /* -Wclass-conversion */
  OPT_Wclass_memaccess = 570,                /* -Wclass-memaccess */
  OPT_Wclobbered = 571,                      /* -Wclobbered */
  OPT_Wcomma_subscript = 572,                /* -Wcomma-subscript */
  OPT_Wcomment = 573,                        /* -Wcomment */
  /* OPT_Wcomments = 574, */                 /* -Wcomments */
  OPT_Wcompare_distinct_pointer_types = 575, /* -Wcompare-distinct-pointer-types */
  OPT_Wcompare_reals = 576,                  /* -Wcompare-reals */
  OPT_Wcomplain_wrong_lang = 577,            /* -Wcomplain-wrong-lang */
  OPT_Wconditionally_supported = 578,        /* -Wconditionally-supported */
  OPT_Wconversion = 579,                     /* -Wconversion */
  OPT_Wconversion_extra = 580,               /* -Wconversion-extra */
  OPT_Wconversion_null = 581,                /* -Wconversion-null */
  OPT_Wcoverage_invalid_line_number = 582,   /* -Wcoverage-invalid-line-number */
  OPT_Wcoverage_mismatch = 583,              /* -Wcoverage-mismatch */
  OPT_Wcoverage_too_many_conditions = 584,   /* -Wcoverage-too-many-conditions */
  OPT_Wcpp = 585,                            /* -Wcpp */
  OPT_Wctad_maybe_unsupported = 586,         /* -Wctad-maybe-unsupported */
  OPT_Wctor_dtor_privacy = 587,              /* -Wctor-dtor-privacy */
  OPT_Wdangling_else = 588,                  /* -Wdangling-else */
  /* OPT_Wdangling_pointer = 589, */         /* -Wdangling-pointer */
  OPT_Wdangling_pointer_ = 590,              /* -Wdangling-pointer= */
  OPT_Wdangling_reference = 591,             /* -Wdangling-reference */
  OPT_Wdate_time = 592,                      /* -Wdate-time */
  OPT_Wdeclaration_after_statement = 593,    /* -Wdeclaration-after-statement */
  OPT_Wdeclaration_missing_parameter_type = 594,/* -Wdeclaration-missing-parameter-type */
  OPT_Wdelete_incomplete = 595,              /* -Wdelete-incomplete */
  OPT_Wdelete_non_virtual_dtor = 596,        /* -Wdelete-non-virtual-dtor */
  OPT_Wdeprecated = 597,                     /* -Wdeprecated */
  OPT_Wdeprecated_copy = 598,                /* -Wdeprecated-copy */
  OPT_Wdeprecated_copy_dtor = 599,           /* -Wdeprecated-copy-dtor */
  OPT_Wdeprecated_declarations = 600,        /* -Wdeprecated-declarations */
  OPT_Wdeprecated_enum_enum_conversion = 601,/* -Wdeprecated-enum-enum-conversion */
  OPT_Wdeprecated_enum_float_conversion = 602,/* -Wdeprecated-enum-float-conversion */
  OPT_Wdesignated_init = 603,                /* -Wdesignated-init */
  OPT_Wdisabled_optimization = 604,          /* -Wdisabled-optimization */
  OPT_Wdiscarded_array_qualifiers = 605,     /* -Wdiscarded-array-qualifiers */
  OPT_Wdiscarded_qualifiers = 606,           /* -Wdiscarded-qualifiers */
  OPT_Wdiv_by_zero = 607,                    /* -Wdiv-by-zero */
  OPT_Wdo_subscript = 608,                   /* -Wdo-subscript */
  OPT_Wdouble_promotion = 609,               /* -Wdouble-promotion */
  OPT_Wduplicate_decl_specifier = 610,       /* -Wduplicate-decl-specifier */
  OPT_Wduplicated_branches = 611,            /* -Wduplicated-branches */
  OPT_Wduplicated_cond = 612,                /* -Wduplicated-cond */
  OPT_Weffc__ = 613,                         /* -Weffc++ */
  OPT_Welaborated_enum_base = 614,           /* -Welaborated-enum-base */
  OPT_Wempty_body = 615,                     /* -Wempty-body */
  OPT_Wendif_labels = 616,                   /* -Wendif-labels */
  OPT_Wenum_compare = 617,                   /* -Wenum-compare */
  OPT_Wenum_conversion = 618,                /* -Wenum-conversion */
  OPT_Wenum_int_mismatch = 619,              /* -Wenum-int-mismatch */
  OPT_Werror = 620,                          /* -Werror */
  /* OPT_Werror_implicit_function_declaration = 621, *//* -Werror-implicit-function-declaration */
  OPT_Werror_ = 622,                         /* -Werror= */
  OPT_Wexceptions = 623,                     /* -Wexceptions */
  OPT_Wexpansion_to_defined = 624,           /* -Wexpansion-to-defined */
  OPT_Wextra = 625,                          /* -Wextra */
  OPT_Wextra_semi = 626,                     /* -Wextra-semi */
  OPT_Wfatal_errors = 627,                   /* -Wfatal-errors */
  OPT_Wflex_array_member_not_at_end = 628,   /* -Wflex-array-member-not-at-end */
  OPT_Wfloat_conversion = 629,               /* -Wfloat-conversion */
  OPT_Wfloat_equal = 630,                    /* -Wfloat-equal */
  /* OPT_Wformat = 631, */                   /* -Wformat */
  OPT_Wformat_contains_nul = 632,            /* -Wformat-contains-nul */
  OPT_Wformat_diag = 633,                    /* -Wformat-diag */
  OPT_Wformat_extra_args = 634,              /* -Wformat-extra-args */
  OPT_Wformat_nonliteral = 635,              /* -Wformat-nonliteral */
  /* OPT_Wformat_overflow = 636, */          /* -Wformat-overflow */
  OPT_Wformat_overflow_ = 637,               /* -Wformat-overflow= */
  OPT_Wformat_security = 638,                /* -Wformat-security */
  OPT_Wformat_signedness = 639,              /* -Wformat-signedness */
  /* OPT_Wformat_truncation = 640, */        /* -Wformat-truncation */
  OPT_Wformat_truncation_ = 641,             /* -Wformat-truncation= */
  OPT_Wformat_y2k = 642,                     /* -Wformat-y2k */
  OPT_Wformat_zero_length = 643,             /* -Wformat-zero-length */
  OPT_Wformat_ = 644,                        /* -Wformat= */
  OPT_Wframe_address = 645,                  /* -Wframe-address */
  OPT_Wframe_larger_than_ = 646,             /* -Wframe-larger-than= */
  OPT_Wfree_nonheap_object = 647,            /* -Wfree-nonheap-object */
  OPT_Wfrontend_loop_interchange = 648,      /* -Wfrontend-loop-interchange */
  OPT_Wfunction_elimination = 649,           /* -Wfunction-elimination */
  OPT_Wglobal_module = 650,                  /* -Wglobal-module */
  OPT_Whardened = 651,                       /* -Whardened */
  /* OPT_Whsa = 652, */                      /* -Whsa */
  OPT_Wif_not_aligned = 653,                 /* -Wif-not-aligned */
  OPT_Wignored_attributes = 654,             /* -Wignored-attributes */
  OPT_Wignored_qualifiers = 655,             /* -Wignored-qualifiers */
  OPT_Wimplicit = 656,                       /* -Wimplicit */
  /* OPT_Wimplicit_fallthrough = 657, */     /* -Wimplicit-fallthrough */
  OPT_Wimplicit_fallthrough_ = 658,          /* -Wimplicit-fallthrough= */
  OPT_Wimplicit_function_declaration = 659,  /* -Wimplicit-function-declaration */
  OPT_Wimplicit_int = 660,                   /* -Wimplicit-int */
  OPT_Wimplicit_interface = 661,             /* -Wimplicit-interface */
  OPT_Wimplicit_procedure = 662,             /* -Wimplicit-procedure */
  /* OPT_Wimport = 663, */                   /* -Wimport */
  OPT_Winaccessible_base = 664,              /* -Winaccessible-base */
  OPT_Wincompatible_pointer_types = 665,     /* -Wincompatible-pointer-types */
  OPT_Winfinite_recursion = 666,             /* -Winfinite-recursion */
  OPT_Winherited_variadic_ctor = 667,        /* -Winherited-variadic-ctor */
  OPT_Winit_list_lifetime = 668,             /* -Winit-list-lifetime */
  OPT_Winit_self = 669,                      /* -Winit-self */
  OPT_Winline = 670,                         /* -Winline */
  OPT_Wint_conversion = 671,                 /* -Wint-conversion */
  OPT_Wint_in_bool_context = 672,            /* -Wint-in-bool-context */
  OPT_Wint_to_pointer_cast = 673,            /* -Wint-to-pointer-cast */
  OPT_Winteger_division = 674,               /* -Winteger-division */
  OPT_Winterference_size = 675,              /* -Winterference-size */
  OPT_Wintrinsic_shadow = 676,               /* -Wintrinsic-shadow */
  OPT_Wintrinsics_std = 677,                 /* -Wintrinsics-std */
  OPT_Winvalid_constexpr = 678,              /* -Winvalid-constexpr */
  OPT_Winvalid_imported_macros = 679,        /* -Winvalid-imported-macros */
  OPT_Winvalid_memory_model = 680,           /* -Winvalid-memory-model */
  OPT_Winvalid_offsetof = 681,               /* -Winvalid-offsetof */
  OPT_Winvalid_pch = 682,                    /* -Winvalid-pch */
  OPT_Winvalid_utf8 = 683,                   /* -Winvalid-utf8 */
  OPT_Wjump_misses_init = 684,               /* -Wjump-misses-init */
  OPT_Wl_ = 685,                             /* -Wl, */
  /* OPT_Wlarger_than_ = 686, */             /* -Wlarger-than- */
  OPT_Wlarger_than_ = 687,                   /* -Wlarger-than= */
  OPT_Wline_truncation = 688,                /* -Wline-truncation */
  OPT_Wliteral_suffix = 689,                 /* -Wliteral-suffix */
  OPT_Wlogical_not_parentheses = 690,        /* -Wlogical-not-parentheses */
  OPT_Wlogical_op = 691,                     /* -Wlogical-op */
  OPT_Wlong_long = 692,                      /* -Wlong-long */
  OPT_Wlto_type_mismatch = 693,              /* -Wlto-type-mismatch */
  OPT_Wmain = 694,                           /* -Wmain */
  OPT_Wmaybe_uninitialized = 695,            /* -Wmaybe-uninitialized */
  OPT_Wmemset_elt_size = 696,                /* -Wmemset-elt-size */
  OPT_Wmemset_transposed_args = 697,         /* -Wmemset-transposed-args */
  OPT_Wmisleading_indentation = 698,         /* -Wmisleading-indentation */
  OPT_Wmismatched_dealloc = 699,             /* -Wmismatched-dealloc */
  OPT_Wmismatched_new_delete = 700,          /* -Wmismatched-new-delete */
  OPT_Wmismatched_special_enum = 701,        /* -Wmismatched-special-enum */
  OPT_Wmismatched_tags = 702,                /* -Wmismatched-tags */
  OPT_Wmissing_attributes = 703,             /* -Wmissing-attributes */
  OPT_Wmissing_braces = 704,                 /* -Wmissing-braces */
  OPT_Wmissing_declarations = 705,           /* -Wmissing-declarations */
  OPT_Wmissing_field_initializers = 706,     /* -Wmissing-field-initializers */
  /* OPT_Wmissing_format_attribute = 707, */ /* -Wmissing-format-attribute */
  OPT_Wmissing_include_dirs = 708,           /* -Wmissing-include-dirs */
  /* OPT_Wmissing_noreturn = 709, */         /* -Wmissing-noreturn */
  OPT_Wmissing_parameter_type = 710,         /* -Wmissing-parameter-type */
  OPT_Wmissing_profile = 711,                /* -Wmissing-profile */
  OPT_Wmissing_prototypes = 712,             /* -Wmissing-prototypes */
  OPT_Wmissing_requires = 713,               /* -Wmissing-requires */
  OPT_Wmissing_template_keyword = 714,       /* -Wmissing-template-keyword */
  OPT_Wmissing_variable_declarations = 715,  /* -Wmissing-variable-declarations */
  OPT_Wmudflap = 716,                        /* -Wmudflap */
  OPT_Wmultichar = 717,                      /* -Wmultichar */
  OPT_Wmultiple_inheritance = 718,           /* -Wmultiple-inheritance */
  OPT_Wmultistatement_macros = 719,          /* -Wmultistatement-macros */
  OPT_Wnamespaces = 720,                     /* -Wnamespaces */
  OPT_Wnarrowing = 721,                      /* -Wnarrowing */
  OPT_Wnested_externs = 722,                 /* -Wnested-externs */
  /* OPT_Wno_alloc_size_larger_than = 723, *//* -Wno-alloc-size-larger-than */
  /* OPT_Wno_alloca_larger_than = 724, */    /* -Wno-alloca-larger-than */
  /* OPT_Wno_frame_larger_than = 725, */     /* -Wno-frame-larger-than */
  /* OPT_Wno_larger_than = 726, */           /* -Wno-larger-than */
  /* OPT_Wno_stack_usage = 727, */           /* -Wno-stack-usage */
  /* OPT_Wno_vla_larger_than = 728, */       /* -Wno-vla-larger-than */
  OPT_Wnoexcept = 729,                       /* -Wnoexcept */
  OPT_Wnoexcept_type = 730,                  /* -Wnoexcept-type */
  OPT_Wnon_template_friend = 731,            /* -Wnon-template-friend */
  OPT_Wnon_virtual_dtor = 732,               /* -Wnon-virtual-dtor */
  OPT_Wnonnull = 733,                        /* -Wnonnull */
  OPT_Wnonnull_compare = 734,                /* -Wnonnull-compare */
  /* OPT_Wnormalized = 735, */               /* -Wnormalized */
  OPT_Wnormalized_ = 736,                    /* -Wnormalized= */
  OPT_Wnrvo = 737,                           /* -Wnrvo */
  OPT_Wnull_dereference = 738,               /* -Wnull-dereference */
  OPT_Wobjc_root_class = 739,                /* -Wobjc-root-class */
  OPT_Wodr = 740,                            /* -Wodr */
  OPT_Wold_style_cast = 741,                 /* -Wold-style-cast */
  OPT_Wold_style_declaration = 742,          /* -Wold-style-declaration */
  OPT_Wold_style_definition = 743,           /* -Wold-style-definition */
  OPT_Wopenacc_parallelism = 744,            /* -Wopenacc-parallelism */
  OPT_Wopenmp = 745,                         /* -Wopenmp */
  OPT_Wopenmp_simd = 746,                    /* -Wopenmp-simd */
  OPT_Woverflow = 747,                       /* -Woverflow */
  OPT_Woverlength_strings = 748,             /* -Woverlength-strings */
  /* OPT_Woverloaded_virtual = 749, */       /* -Woverloaded-virtual */
  OPT_Woverloaded_virtual_ = 750,            /* -Woverloaded-virtual= */
  OPT_Woverride_init = 751,                  /* -Woverride-init */
  OPT_Woverride_init_side_effects = 752,     /* -Woverride-init-side-effects */
  OPT_Woverwrite_recursive = 753,            /* -Woverwrite-recursive */
  OPT_Wp_ = 754,                             /* -Wp, */
  OPT_Wpacked = 755,                         /* -Wpacked */
  OPT_Wpacked_bitfield_compat = 756,         /* -Wpacked-bitfield-compat */
  OPT_Wpacked_not_aligned = 757,             /* -Wpacked-not-aligned */
  OPT_Wpadded = 758,                         /* -Wpadded */
  OPT_Wparentheses = 759,                    /* -Wparentheses */
  OPT_Wpedantic = 760,                       /* -Wpedantic */
  OPT_Wpedantic_cast = 761,                  /* -Wpedantic-cast */
  OPT_Wpedantic_param_names = 762,           /* -Wpedantic-param-names */
  OPT_Wpessimizing_move = 763,               /* -Wpessimizing-move */
  /* OPT_Wplacement_new = 764, */            /* -Wplacement-new */
  OPT_Wplacement_new_ = 765,                 /* -Wplacement-new= */
  OPT_Wpmf_conversions = 766,                /* -Wpmf-conversions */
  OPT_Wpointer_arith = 767,                  /* -Wpointer-arith */
  OPT_Wpointer_compare = 768,                /* -Wpointer-compare */
  OPT_Wpointer_sign = 769,                   /* -Wpointer-sign */
  OPT_Wpointer_to_int_cast = 770,            /* -Wpointer-to-int-cast */
  OPT_Wpragmas = 771,                        /* -Wpragmas */
  OPT_Wprio_ctor_dtor = 772,                 /* -Wprio-ctor-dtor */
  OPT_Wproperty_assign_default = 773,        /* -Wproperty-assign-default */
  OPT_Wprotocol = 774,                       /* -Wprotocol */
  OPT_Wpsabi = 775,                          /* -Wpsabi */
  OPT_Wrange_loop_construct = 776,           /* -Wrange-loop-construct */
  OPT_Wreal_q_constant = 777,                /* -Wreal-q-constant */
  OPT_Wrealloc_lhs = 778,                    /* -Wrealloc-lhs */
  OPT_Wrealloc_lhs_all = 779,                /* -Wrealloc-lhs-all */
  OPT_Wredundant_decls = 780,                /* -Wredundant-decls */
  OPT_Wredundant_move = 781,                 /* -Wredundant-move */
  OPT_Wredundant_tags = 782,                 /* -Wredundant-tags */
  OPT_Wregister = 783,                       /* -Wregister */
  OPT_Wreorder = 784,                        /* -Wreorder */
  OPT_Wrestrict = 785,                       /* -Wrestrict */
  OPT_Wreturn_local_addr = 786,              /* -Wreturn-local-addr */
  OPT_Wreturn_mismatch = 787,                /* -Wreturn-mismatch */
  OPT_Wreturn_type = 788,                    /* -Wreturn-type */
  OPT_Wscalar_storage_order = 789,           /* -Wscalar-storage-order */
  OPT_Wselector = 790,                       /* -Wselector */
  OPT_Wself_move = 791,                      /* -Wself-move */
  OPT_Wsequence_point = 792,                 /* -Wsequence-point */
  OPT_Wshadow = 793,                         /* -Wshadow */
  /* OPT_Wshadow_compatible_local = 794, */  /* -Wshadow-compatible-local */
  OPT_Wshadow_ivar = 795,                    /* -Wshadow-ivar */
  /* OPT_Wshadow_local = 796, */             /* -Wshadow-local */
  OPT_Wshadow_compatible_local = 797,        /* -Wshadow=compatible-local */
  /* OPT_Wshadow_global = 798, */            /* -Wshadow=global */
  OPT_Wshadow_local = 799,                   /* -Wshadow=local */
  OPT_Wshift_count_negative = 800,           /* -Wshift-count-negative */
  OPT_Wshift_count_overflow = 801,           /* -Wshift-count-overflow */
  OPT_Wshift_negative_value = 802,           /* -Wshift-negative-value */
  /* OPT_Wshift_overflow = 803, */           /* -Wshift-overflow */
  OPT_Wshift_overflow_ = 804,                /* -Wshift-overflow= */
  OPT_Wsign_compare = 805,                   /* -Wsign-compare */
  OPT_Wsign_conversion = 806,                /* -Wsign-conversion */
  OPT_Wsign_promo = 807,                     /* -Wsign-promo */
  OPT_Wsized_deallocation = 808,             /* -Wsized-deallocation */
  OPT_Wsizeof_array_argument = 809,          /* -Wsizeof-array-argument */
  OPT_Wsizeof_array_div = 810,               /* -Wsizeof-array-div */
  OPT_Wsizeof_pointer_div = 811,             /* -Wsizeof-pointer-div */
  OPT_Wsizeof_pointer_memaccess = 812,       /* -Wsizeof-pointer-memaccess */
  OPT_Wspeculative = 813,                    /* -Wspeculative */
  OPT_Wstack_protector = 814,                /* -Wstack-protector */
  OPT_Wstack_usage_ = 815,                   /* -Wstack-usage= */
  OPT_Wstrict_aliasing = 816,                /* -Wstrict-aliasing */
  OPT_Wstrict_aliasing_ = 817,               /* -Wstrict-aliasing= */
  OPT_Wstrict_flex_arrays = 818,             /* -Wstrict-flex-arrays */
  OPT_Wstrict_null_sentinel = 819,           /* -Wstrict-null-sentinel */
  OPT_Wstrict_overflow = 820,                /* -Wstrict-overflow */
  OPT_Wstrict_overflow_ = 821,               /* -Wstrict-overflow= */
  OPT_Wstrict_prototypes = 822,              /* -Wstrict-prototypes */
  OPT_Wstrict_selector_match = 823,          /* -Wstrict-selector-match */
  OPT_Wstring_compare = 824,                 /* -Wstring-compare */
  /* OPT_Wstringop_overflow = 825, */        /* -Wstringop-overflow */
  OPT_Wstringop_overflow_ = 826,             /* -Wstringop-overflow= */
  OPT_Wstringop_overread = 827,              /* -Wstringop-overread */
  OPT_Wstringop_truncation = 828,            /* -Wstringop-truncation */
  OPT_Wstyle = 829,                          /* -Wstyle */
  OPT_Wsubobject_linkage = 830,              /* -Wsubobject-linkage */
  OPT_Wsuggest_attribute_cold = 831,         /* -Wsuggest-attribute=cold */
  OPT_Wsuggest_attribute_const = 832,        /* -Wsuggest-attribute=const */
  OPT_Wsuggest_attribute_format = 833,       /* -Wsuggest-attribute=format */
  OPT_Wsuggest_attribute_malloc = 834,       /* -Wsuggest-attribute=malloc */
  OPT_Wsuggest_attribute_noreturn = 835,     /* -Wsuggest-attribute=noreturn */
  OPT_Wsuggest_attribute_pure = 836,         /* -Wsuggest-attribute=pure */
  OPT_Wsuggest_attribute_returns_nonnull = 837,/* -Wsuggest-attribute=returns_nonnull */
  OPT_Wsuggest_final_methods = 838,          /* -Wsuggest-final-methods */
  OPT_Wsuggest_final_types = 839,            /* -Wsuggest-final-types */
  OPT_Wsuggest_override = 840,               /* -Wsuggest-override */
  OPT_Wsurprising = 841,                     /* -Wsurprising */
  OPT_Wswitch = 842,                         /* -Wswitch */
  OPT_Wswitch_bool = 843,                    /* -Wswitch-bool */
  OPT_Wswitch_default = 844,                 /* -Wswitch-default */
  OPT_Wswitch_enum = 845,                    /* -Wswitch-enum */
  OPT_Wswitch_outside_range = 846,           /* -Wswitch-outside-range */
  OPT_Wswitch_unreachable = 847,             /* -Wswitch-unreachable */
  OPT_Wsync_nand = 848,                      /* -Wsync-nand */
  OPT_Wsynth = 849,                          /* -Wsynth */
  OPT_Wsystem_headers = 850,                 /* -Wsystem-headers */
  OPT_Wtabs = 851,                           /* -Wtabs */
  OPT_Wtarget_lifetime = 852,                /* -Wtarget-lifetime */
  OPT_Wtautological_compare = 853,           /* -Wtautological-compare */
  OPT_Wtemplate_id_cdtor = 854,              /* -Wtemplate-id-cdtor */
  OPT_Wtemplates = 855,                      /* -Wtemplates */
  OPT_Wterminate = 856,                      /* -Wterminate */
  OPT_Wtraditional = 857,                    /* -Wtraditional */
  OPT_Wtraditional_conversion = 858,         /* -Wtraditional-conversion */
  OPT_Wtrampolines = 859,                    /* -Wtrampolines */
  OPT_Wtrigraphs = 860,                      /* -Wtrigraphs */
  OPT_Wtrivial_auto_var_init = 861,          /* -Wtrivial-auto-var-init */
  OPT_Wtsan = 862,                           /* -Wtsan */
  OPT_Wtype_limits = 863,                    /* -Wtype-limits */
  OPT_Wundeclared_selector = 864,            /* -Wundeclared-selector */
  OPT_Wundef = 865,                          /* -Wundef */
  OPT_Wundefined_do_loop = 866,              /* -Wundefined-do-loop */
  OPT_Wunderflow = 867,                      /* -Wunderflow */
  OPT_Wunicode = 868,                        /* -Wunicode */
  OPT_Wuninit_variable_checking = 869,       /* -Wuninit-variable-checking */
  OPT_Wuninit_variable_checking_ = 870,      /* -Wuninit-variable-checking= */
  OPT_Wuninitialized = 871,                  /* -Wuninitialized */
  OPT_Wunknown_pragmas = 872,                /* -Wunknown-pragmas */
  /* OPT_Wunreachable_code = 873, */         /* -Wunreachable-code */
  /* OPT_Wunsafe_loop_optimizations = 874, *//* -Wunsafe-loop-optimizations */
  OPT_Wunsuffixed_float_constants = 875,     /* -Wunsuffixed-float-constants */
  OPT_Wunused = 876,                         /* -Wunused */
  OPT_Wunused_but_set_parameter = 877,       /* -Wunused-but-set-parameter */
  OPT_Wunused_but_set_variable = 878,        /* -Wunused-but-set-variable */
  /* OPT_Wunused_const_variable = 879, */    /* -Wunused-const-variable */
  OPT_Wunused_const_variable_ = 880,         /* -Wunused-const-variable= */
  OPT_Wunused_dummy_argument = 881,          /* -Wunused-dummy-argument */
  OPT_Wunused_function = 882,                /* -Wunused-function */
  OPT_Wunused_label = 883,                   /* -Wunused-label */
  OPT_Wunused_local_typedefs = 884,          /* -Wunused-local-typedefs */
  OPT_Wunused_macros = 885,                  /* -Wunused-macros */
  OPT_Wunused_parameter = 886,               /* -Wunused-parameter */
  OPT_Wunused_result = 887,                  /* -Wunused-result */
  OPT_Wunused_value = 888,                   /* -Wunused-value */
  OPT_Wunused_variable = 889,                /* -Wunused-variable */
  OPT_Wuse_after_free = 890,                 /* -Wuse-after-free */
  OPT_Wuse_after_free_ = 891,                /* -Wuse-after-free= */
  OPT_Wuse_without_only = 892,               /* -Wuse-without-only */
  OPT_Wuseless_cast = 893,                   /* -Wuseless-cast */
  OPT_Wvarargs = 894,                        /* -Wvarargs */
  OPT_Wvariadic_macros = 895,                /* -Wvariadic-macros */
  OPT_Wvector_operation_performance = 896,   /* -Wvector-operation-performance */
  OPT_Wverbose_unbounded = 897,              /* -Wverbose-unbounded */
  OPT_Wvexing_parse = 898,                   /* -Wvexing-parse */
  OPT_Wvirtual_inheritance = 899,            /* -Wvirtual-inheritance */
  OPT_Wvirtual_move_assign = 900,            /* -Wvirtual-move-assign */
  OPT_Wvla = 901,                            /* -Wvla */
  OPT_Wvla_larger_than_ = 902,               /* -Wvla-larger-than= */
  OPT_Wvla_parameter = 903,                  /* -Wvla-parameter */
  OPT_Wvolatile = 904,                       /* -Wvolatile */
  OPT_Wvolatile_register_var = 905,          /* -Wvolatile-register-var */
  OPT_Wwrite_strings = 906,                  /* -Wwrite-strings */
  OPT_Wxor_used_as_pow = 907,                /* -Wxor-used-as-pow */
  OPT_Wzero_as_null_pointer_constant = 908,  /* -Wzero-as-null-pointer-constant */
  OPT_Wzero_length_bounds = 909,             /* -Wzero-length-bounds */
  OPT_Wzerotrip = 910,                       /* -Wzerotrip */
  OPT_X = 911,                               /* -X */
  OPT_Xassembler = 912,                      /* -Xassembler */
  OPT_Xf = 913,                              /* -Xf */
  OPT_Xlinker = 914,                         /* -Xlinker */
  OPT_Xpreprocessor = 915,                   /* -Xpreprocessor */
  OPT_Z = 916,                               /* -Z */
  OPT_ansi = 917,                            /* -ansi */
  OPT_aux_info = 918,                        /* -aux-info */
  /* OPT_aux_info_ = 919, */                 /* -aux-info= */
  OPT_c = 920,                               /* -c */
  OPT_callgraph = 921,                       /* -callgraph */
  OPT_coverage = 922,                        /* -coverage */
  OPT_cpp = 923,                             /* -cpp */
  OPT_cpp_ = 924,                            /* -cpp= */
  OPT_d = 925,                               /* -d */
  OPT_debuglib_ = 926,                       /* -debuglib= */
  OPT_defaultlib_ = 927,                     /* -defaultlib= */
  OPT_defined_only = 928,                    /* -defined-only */
  OPT_demangle = 929,                        /* -demangle */
  OPT_dstartfiles = 930,                     /* -dstartfiles */
  OPT_dump_body_ = 931,                      /* -dump-body= */
  OPT_dump_level_ = 932,                     /* -dump-level= */
  OPT_dumpbase = 933,                        /* -dumpbase */
  OPT_dumpbase_ext = 934,                    /* -dumpbase-ext */
  OPT_dumpdir = 935,                         /* -dumpdir */
  OPT_dumpfullversion = 936,                 /* -dumpfullversion */
  OPT_dumpmachine = 937,                     /* -dumpmachine */
  OPT_dumpspecs = 938,                       /* -dumpspecs */
  OPT_dumpversion = 939,                     /* -dumpversion */
  OPT_e = 940,                               /* -e */
  OPT_export_dynamic = 941,                  /* -export-dynamic */
  OPT_fPIC = 942,                            /* -fPIC */
  OPT_fPIE = 943,                            /* -fPIE */
  OPT_fRTS_ = 944,                           /* -fRTS= */
  OPT_fabi_compat_version_ = 945,            /* -fabi-compat-version= */
  OPT_fabi_version_ = 946,                   /* -fabi-version= */
  OPT_faccess_control = 947,                 /* -faccess-control */
  OPT_fada_spec_parent_ = 948,               /* -fada-spec-parent= */
  OPT_faggressive_function_elimination = 949,/* -faggressive-function-elimination */
  OPT_faggressive_loop_optimizations = 950,  /* -faggressive-loop-optimizations */
  OPT_falign_commons = 951,                  /* -falign-commons */
  OPT_falign_functions = 952,                /* -falign-functions */
  OPT_falign_functions_ = 953,               /* -falign-functions= */
  OPT_falign_jumps = 954,                    /* -falign-jumps */
  OPT_falign_jumps_ = 955,                   /* -falign-jumps= */
  OPT_falign_labels = 956,                   /* -falign-labels */
  OPT_falign_labels_ = 957,                  /* -falign-labels= */
  OPT_falign_loops = 958,                    /* -falign-loops */
  OPT_falign_loops_ = 959,                   /* -falign-loops= */
  /* OPT_faligned_new = 960, */              /* -faligned-new */
  OPT_faligned_new_ = 961,                   /* -faligned-new= */
  OPT_fall_instantiations = 962,             /* -fall-instantiations */
  OPT_fall_intrinsics = 963,                 /* -fall-intrinsics */
  OPT_fall_virtual = 964,                    /* -fall-virtual */
  OPT_fallocation_dce = 965,                 /* -fallocation-dce */
  OPT_fallow_argument_mismatch = 966,        /* -fallow-argument-mismatch */
  OPT_fallow_invalid_boz = 967,              /* -fallow-invalid-boz */
  OPT_fallow_leading_underscore = 968,       /* -fallow-leading-underscore */
  /* OPT_fallow_parameterless_variadic_functions = 969, *//* -fallow-parameterless-variadic-functions */
  OPT_fallow_store_data_races = 970,         /* -fallow-store-data-races */
  OPT_falt_external_templates = 971,         /* -falt-external-templates */
  OPT_fanalyzer = 972,                       /* -fanalyzer */
  OPT_fanalyzer_call_summaries = 973,        /* -fanalyzer-call-summaries */
  OPT_fanalyzer_checker_ = 974,              /* -fanalyzer-checker= */
  OPT_fanalyzer_debug_text_art = 975,        /* -fanalyzer-debug-text-art */
  OPT_fanalyzer_feasibility = 976,           /* -fanalyzer-feasibility */
  OPT_fanalyzer_fine_grained = 977,          /* -fanalyzer-fine-grained */
  OPT_fanalyzer_show_duplicate_count = 978,  /* -fanalyzer-show-duplicate-count */
  OPT_fanalyzer_show_events_in_system_headers = 979,/* -fanalyzer-show-events-in-system-headers */
  OPT_fanalyzer_state_merge = 980,           /* -fanalyzer-state-merge */
  OPT_fanalyzer_state_purge = 981,           /* -fanalyzer-state-purge */
  OPT_fanalyzer_suppress_followups = 982,    /* -fanalyzer-suppress-followups */
  OPT_fanalyzer_transitivity = 983,          /* -fanalyzer-transitivity */
  OPT_fanalyzer_undo_inlining = 984,         /* -fanalyzer-undo-inlining */
  OPT_fanalyzer_verbose_edges = 985,         /* -fanalyzer-verbose-edges */
  OPT_fanalyzer_verbose_state_changes = 986, /* -fanalyzer-verbose-state-changes */
  OPT_fanalyzer_verbosity_ = 987,            /* -fanalyzer-verbosity= */
  /* OPT_fargument_alias = 988, */           /* -fargument-alias */
  /* OPT_fargument_noalias = 989, */         /* -fargument-noalias */
  /* OPT_fargument_noalias_anything = 990, *//* -fargument-noalias-anything */
  /* OPT_fargument_noalias_global = 991, */  /* -fargument-noalias-global */
  OPT_fasan_shadow_offset_ = 992,            /* -fasan-shadow-offset= */
  OPT_fasm = 993,                            /* -fasm */
  OPT_fassert = 994,                         /* -fassert */
  OPT_fassociative_math = 995,               /* -fassociative-math */
  OPT_fasynchronous_unwind_tables = 996,     /* -fasynchronous-unwind-tables */
  OPT_fauto_inc_dec = 997,                   /* -fauto-inc-dec */
  OPT_fauto_init = 998,                      /* -fauto-init */
  OPT_fauto_profile = 999,                   /* -fauto-profile */
  OPT_fauto_profile_ = 1000,                 /* -fauto-profile= */
  OPT_fautomatic = 1001,                     /* -fautomatic */
  OPT_fbackslash = 1002,                     /* -fbackslash */
  OPT_fbacktrace = 1003,                     /* -fbacktrace */
  OPT_fbit_tests = 1004,                     /* -fbit-tests */
  OPT_fblas_matmul_limit_ = 1005,            /* -fblas-matmul-limit= */
  OPT_fbounds = 1006,                        /* -fbounds */
  OPT_fbounds_check = 1007,                  /* -fbounds-check */
  OPT_fbounds_check_ = 1008,                 /* -fbounds-check= */
  OPT_fbranch_count_reg = 1009,              /* -fbranch-count-reg */
  OPT_fbranch_probabilities = 1010,          /* -fbranch-probabilities */
  /* OPT_fbranch_target_load_optimize = 1011, *//* -fbranch-target-load-optimize */
  /* OPT_fbranch_target_load_optimize2 = 1012, *//* -fbranch-target-load-optimize2 */
  /* OPT_fbtr_bb_exclusive = 1013, */        /* -fbtr-bb-exclusive */
  OPT_fbuilding_libgcc = 1014,               /* -fbuilding-libgcc */
  OPT_fbuilding_libgfortran = 1015,          /* -fbuilding-libgfortran */
  OPT_fbuilding_libphobos_tests = 1016,      /* -fbuilding-libphobos-tests */
  OPT_fbuiltin = 1017,                       /* -fbuiltin */
  OPT_fbuiltin_ = 1018,                      /* -fbuiltin- */
  OPT_fbuiltin_printf = 1019,                /* -fbuiltin-printf */
  OPT_fc_prototypes = 1020,                  /* -fc-prototypes */
  OPT_fc_prototypes_external = 1021,         /* -fc-prototypes-external */
  OPT_fcall_saved_ = 1022,                   /* -fcall-saved- */
  OPT_fcall_used_ = 1023,                    /* -fcall-used- */
  OPT_fcaller_saves = 1024,                  /* -fcaller-saves */
  OPT_fcallgraph_info = 1025,                /* -fcallgraph-info */
  OPT_fcallgraph_info_ = 1026,               /* -fcallgraph-info= */
  OPT_fcanon_prefix_map = 1027,              /* -fcanon-prefix-map */
  OPT_fcanonical_system_headers = 1028,      /* -fcanonical-system-headers */
  OPT_fcase = 1029,                          /* -fcase */
  /* OPT_fcf_protection = 1030, */           /* -fcf-protection */
  OPT_fcf_protection_ = 1031,                /* -fcf-protection= */
  OPT_fchar8_t = 1032,                       /* -fchar8_t */
  OPT_fcheck_array_temporaries = 1033,       /* -fcheck-array-temporaries */
  /* OPT_fcheck_data_deps = 1034, */         /* -fcheck-data-deps */
  OPT_fcheck_new = 1035,                     /* -fcheck-new */
  OPT_fcheck_pointer_bounds = 1036,          /* -fcheck-pointer-bounds */
  OPT_fcheck_ = 1037,                        /* -fcheck= */
  /* OPT_fcheck_assert = 1038, */            /* -fcheck=assert */
  /* OPT_fcheck_bounds = 1039, */            /* -fcheck=bounds */
  /* OPT_fcheck_in = 1040, */                /* -fcheck=in */
  /* OPT_fcheck_invariant = 1041, */         /* -fcheck=invariant */
  /* OPT_fcheck_out = 1042, */               /* -fcheck=out */
  /* OPT_fcheck_switch = 1043, */            /* -fcheck=switch */
  OPT_fcheckaction_ = 1044,                  /* -fcheckaction= */
  OPT_fchecking = 1045,                      /* -fchecking */
  OPT_fchecking_ = 1046,                     /* -fchecking= */
  OPT_fchkp_check_incomplete_type = 1047,    /* -fchkp-check-incomplete-type */
  OPT_fchkp_check_read = 1048,               /* -fchkp-check-read */
  OPT_fchkp_check_write = 1049,              /* -fchkp-check-write */
  OPT_fchkp_first_field_has_own_bounds = 1050,/* -fchkp-first-field-has-own-bounds */
  OPT_fchkp_flexible_struct_trailing_arrays = 1051,/* -fchkp-flexible-struct-trailing-arrays */
  OPT_fchkp_instrument_calls = 1052,         /* -fchkp-instrument-calls */
  OPT_fchkp_instrument_marked_only = 1053,   /* -fchkp-instrument-marked-only */
  OPT_fchkp_narrow_bounds = 1054,            /* -fchkp-narrow-bounds */
  OPT_fchkp_narrow_to_innermost_array = 1055,/* -fchkp-narrow-to-innermost-array */
  OPT_fchkp_optimize = 1056,                 /* -fchkp-optimize */
  OPT_fchkp_store_bounds = 1057,             /* -fchkp-store-bounds */
  OPT_fchkp_treat_zero_dynamic_size_as_infinite = 1058,/* -fchkp-treat-zero-dynamic-size-as-infinite */
  OPT_fchkp_use_fast_string_functions = 1059,/* -fchkp-use-fast-string-functions */
  OPT_fchkp_use_nochk_string_functions = 1060,/* -fchkp-use-nochk-string-functions */
  OPT_fchkp_use_static_bounds = 1061,        /* -fchkp-use-static-bounds */
  OPT_fchkp_use_static_const_bounds = 1062,  /* -fchkp-use-static-const-bounds */
  OPT_fchkp_use_wrappers = 1063,             /* -fchkp-use-wrappers */
  OPT_fchkp_zero_input_bounds_for_main = 1064,/* -fchkp-zero-input-bounds-for-main */
  /* OPT_fcilkplus = 1065, */                /* -fcilkplus */
  OPT_fcoarray_ = 1066,                      /* -fcoarray= */
  OPT_fcode_hoisting = 1067,                 /* -fcode-hoisting */
  OPT_fcombine_stack_adjustments = 1068,     /* -fcombine-stack-adjustments */
  OPT_fcommon = 1069,                        /* -fcommon */
  OPT_fcompare_debug = 1070,                 /* -fcompare-debug */
  OPT_fcompare_debug_second = 1071,          /* -fcompare-debug-second */
  OPT_fcompare_debug_ = 1072,                /* -fcompare-debug= */
  OPT_fcompare_elim = 1073,                  /* -fcompare-elim */
  OPT_fconcepts = 1074,                      /* -fconcepts */
  OPT_fconcepts_diagnostics_depth_ = 1075,   /* -fconcepts-diagnostics-depth= */
  OPT_fconcepts_ts = 1076,                   /* -fconcepts-ts */
  OPT_fcond_mismatch = 1077,                 /* -fcond-mismatch */
  OPT_fcondition_coverage = 1078,            /* -fcondition-coverage */
  /* OPT_fconserve_space = 1079, */          /* -fconserve-space */
  OPT_fconserve_stack = 1080,                /* -fconserve-stack */
  OPT_fconstant_string_class_ = 1081,        /* -fconstant-string-class= */
  OPT_fconstexpr_cache_depth_ = 1082,        /* -fconstexpr-cache-depth= */
  OPT_fconstexpr_depth_ = 1083,              /* -fconstexpr-depth= */
  OPT_fconstexpr_fp_except = 1084,           /* -fconstexpr-fp-except */
  OPT_fconstexpr_loop_limit_ = 1085,         /* -fconstexpr-loop-limit= */
  OPT_fconstexpr_ops_limit_ = 1086,          /* -fconstexpr-ops-limit= */
  OPT_fcontract_assumption_mode_ = 1087,     /* -fcontract-assumption-mode= */
  OPT_fcontract_build_level_ = 1088,         /* -fcontract-build-level= */
  OPT_fcontract_continuation_mode_ = 1089,   /* -fcontract-continuation-mode= */
  OPT_fcontract_mode_ = 1090,                /* -fcontract-mode= */
  OPT_fcontract_role_ = 1091,                /* -fcontract-role= */
  OPT_fcontract_semantic_ = 1092,            /* -fcontract-semantic= */
  OPT_fcontract_strict_declarations_ = 1093, /* -fcontract-strict-declarations= */
  OPT_fcontracts = 1094,                     /* -fcontracts */
  OPT_fconvert_ = 1095,                      /* -fconvert= */
  OPT_fcoroutines = 1096,                    /* -fcoroutines */
  OPT_fcpp = 1097,                           /* -fcpp */
  OPT_fcpp_begin = 1098,                     /* -fcpp-begin */
  OPT_fcpp_end = 1099,                       /* -fcpp-end */
  OPT_fcprop_registers = 1100,               /* -fcprop-registers */
  OPT_fcray_pointer = 1101,                  /* -fcray-pointer */
  OPT_fcrossjumping = 1102,                  /* -fcrossjumping */
  OPT_fcse_follow_jumps = 1103,              /* -fcse-follow-jumps */
  /* OPT_fcse_skip_blocks = 1104, */         /* -fcse-skip-blocks */
  OPT_fcx_fortran_rules = 1105,              /* -fcx-fortran-rules */
  OPT_fcx_limited_range = 1106,              /* -fcx-limited-range */
  OPT_fd = 1107,                             /* -fd */
  OPT_fd_lines_as_code = 1108,               /* -fd-lines-as-code */
  OPT_fd_lines_as_comments = 1109,           /* -fd-lines-as-comments */
  OPT_fdata_sections = 1110,                 /* -fdata-sections */
  OPT_fdbg_cnt_list = 1111,                  /* -fdbg-cnt-list */
  OPT_fdbg_cnt_ = 1112,                      /* -fdbg-cnt= */
  OPT_fdce = 1113,                           /* -fdce */
  OPT_fdebug = 1114,                         /* -fdebug */
  OPT_fdebug_aux_vars = 1115,                /* -fdebug-aux-vars */
  OPT_fdebug_builtins = 1116,                /* -fdebug-builtins */
  OPT_fdebug_cpp = 1117,                     /* -fdebug-cpp */
  OPT_fdebug_function_line_numbers = 1118,   /* -fdebug-function-line-numbers */
  OPT_fdebug_prefix_map_ = 1119,             /* -fdebug-prefix-map= */
  OPT_fdebug_types_section = 1120,           /* -fdebug-types-section */
  OPT_fdebug_ = 1121,                        /* -fdebug= */
  OPT_fdec = 1122,                           /* -fdec */
  OPT_fdec_blank_format_item = 1123,         /* -fdec-blank-format-item */
  OPT_fdec_char_conversions = 1124,          /* -fdec-char-conversions */
  OPT_fdec_format_defaults = 1125,           /* -fdec-format-defaults */
  OPT_fdec_include = 1126,                   /* -fdec-include */
  OPT_fdec_intrinsic_ints = 1127,            /* -fdec-intrinsic-ints */
  OPT_fdec_math = 1128,                      /* -fdec-math */
  OPT_fdec_static = 1129,                    /* -fdec-static */
  OPT_fdec_structure = 1130,                 /* -fdec-structure */
  OPT_fdeclone_ctor_dtor = 1131,             /* -fdeclone-ctor-dtor */
  /* OPT_fdeduce_init_list = 1132, */        /* -fdeduce-init-list */
  OPT_fdef_ = 1133,                          /* -fdef= */
  OPT_fdefault_double_8 = 1134,              /* -fdefault-double-8 */
  /* OPT_fdefault_inline = 1135, */          /* -fdefault-inline */
  OPT_fdefault_integer_8 = 1136,             /* -fdefault-integer-8 */
  OPT_fdefault_real_10 = 1137,               /* -fdefault-real-10 */
  OPT_fdefault_real_16 = 1138,               /* -fdefault-real-16 */
  OPT_fdefault_real_8 = 1139,                /* -fdefault-real-8 */
  OPT_fdefer_pop = 1140,                     /* -fdefer-pop */
  OPT_fdelayed_branch = 1141,                /* -fdelayed-branch */
  OPT_fdelete_dead_exceptions = 1142,        /* -fdelete-dead-exceptions */
  OPT_fdelete_null_pointer_checks = 1143,    /* -fdelete-null-pointer-checks */
  OPT_fdeps_file_ = 1144,                    /* -fdeps-file= */
  OPT_fdeps_format_ = 1145,                  /* -fdeps-format= */
  OPT_fdeps_target_ = 1146,                  /* -fdeps-target= */
  OPT_fdevirtualize = 1147,                  /* -fdevirtualize */
  OPT_fdevirtualize_at_ltrans = 1148,        /* -fdevirtualize-at-ltrans */
  OPT_fdevirtualize_speculatively = 1149,    /* -fdevirtualize-speculatively */
  OPT_fdiagnostics_all_candidates = 1150,    /* -fdiagnostics-all-candidates */
  /* OPT_fdiagnostics_color = 1151, */       /* -fdiagnostics-color */
  OPT_fdiagnostics_color_ = 1152,            /* -fdiagnostics-color= */
  OPT_fdiagnostics_column_origin_ = 1153,    /* -fdiagnostics-column-origin= */
  OPT_fdiagnostics_column_unit_ = 1154,      /* -fdiagnostics-column-unit= */
  OPT_fdiagnostics_escape_format_ = 1155,    /* -fdiagnostics-escape-format= */
  OPT_fdiagnostics_format_ = 1156,           /* -fdiagnostics-format= */
  OPT_fdiagnostics_generate_patch = 1157,    /* -fdiagnostics-generate-patch */
  OPT_fdiagnostics_json_formatting = 1158,   /* -fdiagnostics-json-formatting */
  OPT_fdiagnostics_minimum_margin_width_ = 1159,/* -fdiagnostics-minimum-margin-width= */
  OPT_fdiagnostics_parseable_fixits = 1160,  /* -fdiagnostics-parseable-fixits */
  OPT_fdiagnostics_path_format_ = 1161,      /* -fdiagnostics-path-format= */
  OPT_fdiagnostics_plain_output = 1162,      /* -fdiagnostics-plain-output */
  OPT_fdiagnostics_show_caret = 1163,        /* -fdiagnostics-show-caret */
  OPT_fdiagnostics_show_cwe = 1164,          /* -fdiagnostics-show-cwe */
  OPT_fdiagnostics_show_labels = 1165,       /* -fdiagnostics-show-labels */
  OPT_fdiagnostics_show_line_numbers = 1166, /* -fdiagnostics-show-line-numbers */
  OPT_fdiagnostics_show_location_ = 1167,    /* -fdiagnostics-show-location= */
  OPT_fdiagnostics_show_option = 1168,       /* -fdiagnostics-show-option */
  OPT_fdiagnostics_show_path_depths = 1169,  /* -fdiagnostics-show-path-depths */
  OPT_fdiagnostics_show_rules = 1170,        /* -fdiagnostics-show-rules */
  OPT_fdiagnostics_show_template_tree = 1171,/* -fdiagnostics-show-template-tree */
  OPT_fdiagnostics_text_art_charset_ = 1172, /* -fdiagnostics-text-art-charset= */
  OPT_fdiagnostics_urls_ = 1173,             /* -fdiagnostics-urls= */
  OPT_fdirectives_only = 1174,               /* -fdirectives-only */
  OPT_fdisable_ = 1175,                      /* -fdisable- */
  OPT_fdoc = 1176,                           /* -fdoc */
  OPT_fdoc_dir_ = 1177,                      /* -fdoc-dir= */
  OPT_fdoc_file_ = 1178,                     /* -fdoc-file= */
  OPT_fdoc_inc_ = 1179,                      /* -fdoc-inc= */
  OPT_fdollar_ok = 1180,                     /* -fdollar-ok */
  OPT_fdollars_in_identifiers = 1181,        /* -fdollars-in-identifiers */
  OPT_fdruntime = 1182,                      /* -fdruntime */
  OPT_fdse = 1183,                           /* -fdse */
  OPT_fdump_ = 1184,                         /* -fdump- */
  OPT_fdump_ada_spec = 1185,                 /* -fdump-ada-spec */
  OPT_fdump_ada_spec_slim = 1186,            /* -fdump-ada-spec-slim */
  OPT_fdump_analyzer = 1187,                 /* -fdump-analyzer */
  OPT_fdump_analyzer_callgraph = 1188,       /* -fdump-analyzer-callgraph */
  OPT_fdump_analyzer_exploded_graph = 1189,  /* -fdump-analyzer-exploded-graph */
  OPT_fdump_analyzer_exploded_nodes = 1190,  /* -fdump-analyzer-exploded-nodes */
  OPT_fdump_analyzer_exploded_nodes_2 = 1191,/* -fdump-analyzer-exploded-nodes-2 */
  OPT_fdump_analyzer_exploded_nodes_3 = 1192,/* -fdump-analyzer-exploded-nodes-3 */
  OPT_fdump_analyzer_exploded_paths = 1193,  /* -fdump-analyzer-exploded-paths */
  OPT_fdump_analyzer_feasibility = 1194,     /* -fdump-analyzer-feasibility */
  OPT_fdump_analyzer_infinite_loop = 1195,   /* -fdump-analyzer-infinite-loop */
  OPT_fdump_analyzer_json = 1196,            /* -fdump-analyzer-json */
  OPT_fdump_analyzer_state_purge = 1197,     /* -fdump-analyzer-state-purge */
  OPT_fdump_analyzer_stderr = 1198,          /* -fdump-analyzer-stderr */
  OPT_fdump_analyzer_supergraph = 1199,      /* -fdump-analyzer-supergraph */
  OPT_fdump_analyzer_untracked = 1200,       /* -fdump-analyzer-untracked */
  OPT_fdump_c___spec_verbose = 1201,         /* -fdump-c++-spec-verbose */
  OPT_fdump_c___spec_ = 1202,                /* -fdump-c++-spec= */
  /* OPT_fdump_core = 1203, */               /* -fdump-core */
  OPT_fdump_d_original = 1204,               /* -fdump-d-original */
  OPT_fdump_final_insns = 1205,              /* -fdump-final-insns */
  OPT_fdump_final_insns_ = 1206,             /* -fdump-final-insns= */
  OPT_fdump_fortran_global = 1207,           /* -fdump-fortran-global */
  OPT_fdump_fortran_optimized = 1208,        /* -fdump-fortran-optimized */
  OPT_fdump_fortran_original = 1209,         /* -fdump-fortran-original */
  OPT_fdump_go_spec_ = 1210,                 /* -fdump-go-spec= */
  OPT_fdump_internal_locations = 1211,       /* -fdump-internal-locations */
  OPT_fdump_noaddr = 1212,                   /* -fdump-noaddr */
  /* OPT_fdump_parse_tree = 1213, */         /* -fdump-parse-tree */
  OPT_fdump_passes = 1214,                   /* -fdump-passes */
  OPT_fdump_scos = 1215,                     /* -fdump-scos */
  OPT_fdump_system_exports = 1216,           /* -fdump-system-exports */
  OPT_fdump_unnumbered = 1217,               /* -fdump-unnumbered */
  OPT_fdump_unnumbered_links = 1218,         /* -fdump-unnumbered-links */
  OPT_fdwarf2_cfi_asm = 1219,                /* -fdwarf2-cfi-asm */
  OPT_fearly_inlining = 1220,                /* -fearly-inlining */
  OPT_felide_constructors = 1221,            /* -felide-constructors */
  OPT_felide_type = 1222,                    /* -felide-type */
  /* OPT_feliminate_dwarf2_dups = 1223, */   /* -feliminate-dwarf2-dups */
  OPT_feliminate_unused_debug_symbols = 1224,/* -feliminate-unused-debug-symbols */
  OPT_feliminate_unused_debug_types = 1225,  /* -feliminate-unused-debug-types */
  OPT_femit_class_debug_always = 1226,       /* -femit-class-debug-always */
  OPT_femit_struct_debug_baseonly = 1227,    /* -femit-struct-debug-baseonly */
  OPT_femit_struct_debug_detailed_ = 1228,   /* -femit-struct-debug-detailed= */
  OPT_femit_struct_debug_reduced = 1229,     /* -femit-struct-debug-reduced */
  OPT_fenable_ = 1230,                       /* -fenable- */
  OPT_fenforce_eh_specs = 1231,              /* -fenforce-eh-specs */
  OPT_fenum_int_equiv = 1232,                /* -fenum-int-equiv */
  OPT_fexceptions = 1233,                    /* -fexceptions */
  OPT_fexcess_precision_ = 1234,             /* -fexcess-precision= */
  OPT_fexec_charset_ = 1235,                 /* -fexec-charset= */
  OPT_fexpensive_optimizations = 1236,       /* -fexpensive-optimizations */
  OPT_fext_numeric_literals = 1237,          /* -fext-numeric-literals */
  OPT_fextended_identifiers = 1238,          /* -fextended-identifiers */
  OPT_fextended_opaque = 1239,               /* -fextended-opaque */
  OPT_fextern_std_ = 1240,                   /* -fextern-std= */
  OPT_fextern_tls_init = 1241,               /* -fextern-tls-init */
  OPT_fexternal_blas = 1242,                 /* -fexternal-blas */
  OPT_fexternal_templates = 1243,            /* -fexternal-templates */
  OPT_ff2c = 1244,                           /* -ff2c */
  OPT_ffast_math = 1245,                     /* -ffast-math */
  OPT_ffat_lto_objects = 1246,               /* -ffat-lto-objects */
  OPT_ffile_prefix_map_ = 1247,              /* -ffile-prefix-map= */
  OPT_ffinite_loops = 1248,                  /* -ffinite-loops */
  OPT_ffinite_math_only = 1249,              /* -ffinite-math-only */
  OPT_ffixed_ = 1250,                        /* -ffixed- */
  OPT_ffixed_form = 1251,                    /* -ffixed-form */
  OPT_ffixed_line_length_ = 1252,            /* -ffixed-line-length- */
  OPT_ffixed_line_length_none = 1253,        /* -ffixed-line-length-none */
  OPT_ffloat_store = 1254,                   /* -ffloat-store */
  OPT_ffloatvalue = 1255,                    /* -ffloatvalue */
  OPT_ffold_mem_offsets = 1256,              /* -ffold-mem-offsets */
  OPT_ffold_simple_inlines = 1257,           /* -ffold-simple-inlines */
  OPT_ffor_scope = 1258,                     /* -ffor-scope */
  /* OPT_fforce_addr = 1259, */              /* -fforce-addr */
  OPT_fforward_propagate = 1260,             /* -fforward-propagate */
  OPT_ffp_contract_ = 1261,                  /* -ffp-contract= */
  OPT_ffp_int_builtin_inexact = 1262,        /* -ffp-int-builtin-inexact */
  OPT_ffpe_summary_ = 1263,                  /* -ffpe-summary= */
  OPT_ffpe_trap_ = 1264,                     /* -ffpe-trap= */
  OPT_ffree_form = 1265,                     /* -ffree-form */
  OPT_ffree_line_length_ = 1266,             /* -ffree-line-length- */
  OPT_ffree_line_length_none = 1267,         /* -ffree-line-length-none */
  OPT_ffreestanding = 1268,                  /* -ffreestanding */
  OPT_ffriend_injection = 1269,              /* -ffriend-injection */
  OPT_ffrontend_loop_interchange = 1270,     /* -ffrontend-loop-interchange */
  OPT_ffrontend_optimize = 1271,             /* -ffrontend-optimize */
  OPT_ffunction_cse = 1272,                  /* -ffunction-cse */
  OPT_ffunction_sections = 1273,             /* -ffunction-sections */
  OPT_fgcse = 1274,                          /* -fgcse */
  OPT_fgcse_after_reload = 1275,             /* -fgcse-after-reload */
  OPT_fgcse_las = 1276,                      /* -fgcse-las */
  OPT_fgcse_lm = 1277,                       /* -fgcse-lm */
  OPT_fgcse_sm = 1278,                       /* -fgcse-sm */
  OPT_fgen_module_list_ = 1279,              /* -fgen-module-list= */
  OPT_fgimple = 1280,                        /* -fgimple */
  OPT_fgnat_encodings_ = 1281,               /* -fgnat-encodings= */
  OPT_fgnu_keywords = 1282,                  /* -fgnu-keywords */
  OPT_fgnu_runtime = 1283,                   /* -fgnu-runtime */
  OPT_fgnu_tm = 1284,                        /* -fgnu-tm */
  OPT_fgnu_unique = 1285,                    /* -fgnu-unique */
  OPT_fgnu89_inline = 1286,                  /* -fgnu89-inline */
  OPT_fgo_c_header_ = 1287,                  /* -fgo-c-header= */
  OPT_fgo_check_divide_overflow = 1288,      /* -fgo-check-divide-overflow */
  OPT_fgo_check_divide_zero = 1289,          /* -fgo-check-divide-zero */
  OPT_fgo_compiling_runtime = 1290,          /* -fgo-compiling-runtime */
  OPT_fgo_debug_escape = 1291,               /* -fgo-debug-escape */
  OPT_fgo_debug_escape_hash_ = 1292,         /* -fgo-debug-escape-hash= */
  OPT_fgo_debug_optimization = 1293,         /* -fgo-debug-optimization */
  OPT_fgo_dump_ = 1294,                      /* -fgo-dump- */
  OPT_fgo_embedcfg_ = 1295,                  /* -fgo-embedcfg= */
  OPT_fgo_importcfg_ = 1296,                 /* -fgo-importcfg= */
  OPT_fgo_optimize_ = 1297,                  /* -fgo-optimize- */
  OPT_fgo_pkgpath_ = 1298,                   /* -fgo-pkgpath= */
  OPT_fgo_prefix_ = 1299,                    /* -fgo-prefix= */
  OPT_fgo_relative_import_path_ = 1300,      /* -fgo-relative-import-path= */
  OPT_fgraphite = 1301,                      /* -fgraphite */
  OPT_fgraphite_identity = 1302,             /* -fgraphite-identity */
  OPT_fguess_branch_probability = 1303,      /* -fguess-branch-probability */
  OPT_fguiding_decls = 1304,                 /* -fguiding-decls */
  /* OPT_fhandle_exceptions = 1305, */       /* -fhandle-exceptions */
  OPT_fhardcfr_check_exceptions = 1306,      /* -fhardcfr-check-exceptions */
  OPT_fhardcfr_check_noreturn_calls_ = 1307, /* -fhardcfr-check-noreturn-calls= */
  OPT_fhardcfr_check_returning_calls = 1308, /* -fhardcfr-check-returning-calls */
  OPT_fhardcfr_skip_leaf = 1309,             /* -fhardcfr-skip-leaf */
  OPT_fharden_compares = 1310,               /* -fharden-compares */
  OPT_fharden_conditional_branches = 1311,   /* -fharden-conditional-branches */
  OPT_fharden_control_flow_redundancy = 1312,/* -fharden-control-flow-redundancy */
  OPT_fhardened = 1313,                      /* -fhardened */
  /* OPT_fhelp = 1314, */                    /* -fhelp */
  /* OPT_fhelp_ = 1315, */                   /* -fhelp= */
  OPT_fhoist_adjacent_loads = 1316,          /* -fhoist-adjacent-loads */
  OPT_fhonor_std = 1317,                     /* -fhonor-std */
  OPT_fhosted = 1318,                        /* -fhosted */
  OPT_fhuge_objects = 1319,                  /* -fhuge-objects */
  OPT_fident = 1320,                         /* -fident */
  OPT_fif_conversion = 1321,                 /* -fif-conversion */
  OPT_fif_conversion2 = 1322,                /* -fif-conversion2 */
  OPT_fignore_unknown_pragmas = 1323,        /* -fignore-unknown-pragmas */
  OPT_fimmediate_escalation = 1324,          /* -fimmediate-escalation */
  OPT_fimplement_inlines = 1325,             /* -fimplement-inlines */
  OPT_fimplicit_constexpr = 1326,            /* -fimplicit-constexpr */
  OPT_fimplicit_inline_templates = 1327,     /* -fimplicit-inline-templates */
  OPT_fimplicit_none = 1328,                 /* -fimplicit-none */
  OPT_fimplicit_templates = 1329,            /* -fimplicit-templates */
  OPT_findex = 1330,                         /* -findex */
  OPT_findirect_inlining = 1331,             /* -findirect-inlining */
  OPT_finhibit_size_directive = 1332,        /* -finhibit-size-directive */
  OPT_finit_character_ = 1333,               /* -finit-character= */
  OPT_finit_derived = 1334,                  /* -finit-derived */
  OPT_finit_integer_ = 1335,                 /* -finit-integer= */
  OPT_finit_local_zero = 1336,               /* -finit-local-zero */
  OPT_finit_logical_ = 1337,                 /* -finit-logical= */
  OPT_finit_real_ = 1338,                    /* -finit-real= */
  OPT_finline = 1339,                        /* -finline */
  OPT_finline_arg_packing = 1340,            /* -finline-arg-packing */
  OPT_finline_atomics = 1341,                /* -finline-atomics */
  OPT_finline_functions = 1342,              /* -finline-functions */
  OPT_finline_functions_called_once = 1343,  /* -finline-functions-called-once */
  /* OPT_finline_limit_ = 1344, */           /* -finline-limit- */
  OPT_finline_limit_ = 1345,                 /* -finline-limit= */
  OPT_finline_matmul_limit_ = 1346,          /* -finline-matmul-limit= */
  OPT_finline_small_functions = 1347,        /* -finline-small-functions */
  OPT_finline_stringops = 1348,              /* -finline-stringops */
  OPT_finline_stringops_ = 1349,             /* -finline-stringops= */
  OPT_finput_charset_ = 1350,                /* -finput-charset= */
  OPT_finstrument_functions = 1351,          /* -finstrument-functions */
  OPT_finstrument_functions_exclude_file_list_ = 1352,/* -finstrument-functions-exclude-file-list= */
  OPT_finstrument_functions_exclude_function_list_ = 1353,/* -finstrument-functions-exclude-function-list= */
  OPT_finstrument_functions_once = 1354,     /* -finstrument-functions-once */
  OPT_finteger_4_integer_8 = 1355,           /* -finteger-4-integer-8 */
  OPT_fintrinsic_modules_path = 1356,        /* -fintrinsic-modules-path */
  OPT_fintrinsic_modules_path_ = 1357,       /* -fintrinsic-modules-path= */
  OPT_finvariants = 1358,                    /* -finvariants */
  OPT_fipa_bit_cp = 1359,                    /* -fipa-bit-cp */
  OPT_fipa_cp = 1360,                        /* -fipa-cp */
  /* OPT_fipa_cp_alignment = 1361, */        /* -fipa-cp-alignment */
  OPT_fipa_cp_clone = 1362,                  /* -fipa-cp-clone */
  OPT_fipa_icf = 1363,                       /* -fipa-icf */
  OPT_fipa_icf_functions = 1364,             /* -fipa-icf-functions */
  OPT_fipa_icf_variables = 1365,             /* -fipa-icf-variables */
  /* OPT_fipa_matrix_reorg = 1366, */        /* -fipa-matrix-reorg */
  OPT_fipa_modref = 1367,                    /* -fipa-modref */
  OPT_fipa_profile = 1368,                   /* -fipa-profile */
  OPT_fipa_pta = 1369,                       /* -fipa-pta */
  OPT_fipa_pure_const = 1370,                /* -fipa-pure-const */
  OPT_fipa_ra = 1371,                        /* -fipa-ra */
  OPT_fipa_reference = 1372,                 /* -fipa-reference */
  OPT_fipa_reference_addressable = 1373,     /* -fipa-reference-addressable */
  OPT_fipa_sra = 1374,                       /* -fipa-sra */
  OPT_fipa_stack_alignment = 1375,           /* -fipa-stack-alignment */
  OPT_fipa_strict_aliasing = 1376,           /* -fipa-strict-aliasing */
  /* OPT_fipa_struct_reorg = 1377, */        /* -fipa-struct-reorg */
  OPT_fipa_vrp = 1378,                       /* -fipa-vrp */
  OPT_fira_algorithm_ = 1379,                /* -fira-algorithm= */
  OPT_fira_hoist_pressure = 1380,            /* -fira-hoist-pressure */
  OPT_fira_loop_pressure = 1381,             /* -fira-loop-pressure */
  OPT_fira_region_ = 1382,                   /* -fira-region= */
  OPT_fira_share_save_slots = 1383,          /* -fira-share-save-slots */
  OPT_fira_share_spill_slots = 1384,         /* -fira-share-spill-slots */
  OPT_fira_verbose_ = 1385,                  /* -fira-verbose= */
  OPT_fiso = 1386,                           /* -fiso */
  OPT_fisolate_erroneous_paths_attribute = 1387,/* -fisolate-erroneous-paths-attribute */
  OPT_fisolate_erroneous_paths_dereference = 1388,/* -fisolate-erroneous-paths-dereference */
  OPT_fivar_visibility_ = 1389,              /* -fivar-visibility= */
  OPT_fivopts = 1390,                        /* -fivopts */
  OPT_fjump_tables = 1391,                   /* -fjump-tables */
  OPT_fkeep_gc_roots_live = 1392,            /* -fkeep-gc-roots-live */
  OPT_fkeep_inline_dllexport = 1393,         /* -fkeep-inline-dllexport */
  OPT_fkeep_inline_functions = 1394,         /* -fkeep-inline-functions */
  OPT_fkeep_static_consts = 1395,            /* -fkeep-static-consts */
  OPT_fkeep_static_functions = 1396,         /* -fkeep-static-functions */
  OPT_flabels_ok = 1397,                     /* -flabels-ok */
  OPT_flang_info_include_translate = 1398,   /* -flang-info-include-translate */
  OPT_flang_info_include_translate_not = 1399,/* -flang-info-include-translate-not */
  OPT_flang_info_include_translate_ = 1400,  /* -flang-info-include-translate= */
  OPT_flang_info_module_cmi = 1401,          /* -flang-info-module-cmi */
  OPT_flang_info_module_cmi_ = 1402,         /* -flang-info-module-cmi= */
  OPT_flarge_source_files = 1403,            /* -flarge-source-files */
  OPT_flax_vector_conversions = 1404,        /* -flax-vector-conversions */
  OPT_fleading_underscore = 1405,            /* -fleading-underscore */
  OPT_flibs_ = 1406,                         /* -flibs= */
  OPT_flifetime_dse = 1407,                  /* -flifetime-dse */
  OPT_flifetime_dse_ = 1408,                 /* -flifetime-dse= */
  OPT_flimit_function_alignment = 1409,      /* -flimit-function-alignment */
  OPT_flinker_output_ = 1410,                /* -flinker-output= */
  /* OPT_flive_patching = 1411, */           /* -flive-patching */
  OPT_flive_patching_ = 1412,                /* -flive-patching= */
  OPT_flive_range_shrinkage = 1413,          /* -flive-range-shrinkage */
  OPT_flocal_ivars = 1414,                   /* -flocal-ivars */
  OPT_flocation_ = 1415,                     /* -flocation= */
  /* OPT_floop_block = 1416, */              /* -floop-block */
  /* OPT_floop_flatten = 1417, */            /* -floop-flatten */
  OPT_floop_interchange = 1418,              /* -floop-interchange */
  OPT_floop_nest_optimize = 1419,            /* -floop-nest-optimize */
  /* OPT_floop_optimize = 1420, */           /* -floop-optimize */
  OPT_floop_parallelize_all = 1421,          /* -floop-parallelize-all */
  /* OPT_floop_strip_mine = 1422, */         /* -floop-strip-mine */
  OPT_floop_unroll_and_jam = 1423,           /* -floop-unroll-and-jam */
  OPT_flra_remat = 1424,                     /* -flra-remat */
  OPT_flto = 1425,                           /* -flto */
  OPT_flto_compression_level_ = 1426,        /* -flto-compression-level= */
  /* OPT_flto_odr_type_merging = 1427, */    /* -flto-odr-type-merging */
  OPT_flto_partition_ = 1428,                /* -flto-partition= */
  OPT_flto_report = 1429,                    /* -flto-report */
  OPT_flto_report_wpa = 1430,                /* -flto-report-wpa */
  OPT_flto_ = 1431,                          /* -flto= */
  OPT_fltrans = 1432,                        /* -fltrans */
  OPT_fltrans_output_list_ = 1433,           /* -fltrans-output-list= */
  OPT_fm2_debug_trace_ = 1434,               /* -fm2-debug-trace= */
  OPT_fm2_g = 1435,                          /* -fm2-g */
  OPT_fm2_lower_case = 1436,                 /* -fm2-lower-case */
  OPT_fm2_pathname_ = 1437,                  /* -fm2-pathname= */
  OPT_fm2_pathnameI = 1438,                  /* -fm2-pathnameI */
  OPT_fm2_plugin = 1439,                     /* -fm2-plugin */
  OPT_fm2_prefix_ = 1440,                    /* -fm2-prefix= */
  OPT_fm2_statistics = 1441,                 /* -fm2-statistics */
  OPT_fm2_strict_type = 1442,                /* -fm2-strict-type */
  OPT_fm2_whole_program = 1443,              /* -fm2-whole-program */
  OPT_fmacro_prefix_map_ = 1444,             /* -fmacro-prefix-map= */
  OPT_fmain = 1445,                          /* -fmain */
  OPT_fmath_errno = 1446,                    /* -fmath-errno */
  OPT_fmax_array_constructor_ = 1447,        /* -fmax-array-constructor= */
  OPT_fmax_errors_ = 1448,                   /* -fmax-errors= */
  OPT_fmax_identifier_length_ = 1449,        /* -fmax-identifier-length= */
  OPT_fmax_include_depth_ = 1450,            /* -fmax-include-depth= */
  OPT_fmax_stack_var_size_ = 1451,           /* -fmax-stack-var-size= */
  OPT_fmax_subrecord_length_ = 1452,         /* -fmax-subrecord-length= */
  OPT_fmem_report = 1453,                    /* -fmem-report */
  OPT_fmem_report_wpa = 1454,                /* -fmem-report-wpa */
  OPT_fmerge_all_constants = 1455,           /* -fmerge-all-constants */
  OPT_fmerge_constants = 1456,               /* -fmerge-constants */
  OPT_fmerge_debug_strings = 1457,           /* -fmerge-debug-strings */
  OPT_fmessage_length_ = 1458,               /* -fmessage-length= */
  OPT_fmin_function_alignment_ = 1459,       /* -fmin-function-alignment= */
  OPT_fmod_ = 1460,                          /* -fmod= */
  OPT_fmodule_file_ = 1461,                  /* -fmodule-file= */
  OPT_fmodule_header = 1462,                 /* -fmodule-header */
  OPT_fmodule_header_ = 1463,                /* -fmodule-header= */
  OPT_fmodule_implicit_inline = 1464,        /* -fmodule-implicit-inline */
  OPT_fmodule_lazy = 1465,                   /* -fmodule-lazy */
  OPT_fmodule_mapper_ = 1466,                /* -fmodule-mapper= */
  OPT_fmodule_only = 1467,                   /* -fmodule-only */
  OPT_fmodule_private = 1468,                /* -fmodule-private */
  OPT_fmodule_version_ignore = 1469,         /* -fmodule-version-ignore */
  OPT_fmoduleinfo = 1470,                    /* -fmoduleinfo */
  OPT_fmodules_ts = 1471,                    /* -fmodules-ts */
  OPT_fmodulo_sched = 1472,                  /* -fmodulo-sched */
  OPT_fmodulo_sched_allow_regmoves = 1473,   /* -fmodulo-sched-allow-regmoves */
  OPT_fmove_loop_invariants = 1474,          /* -fmove-loop-invariants */
  OPT_fmove_loop_stores = 1475,              /* -fmove-loop-stores */
  OPT_fms_extensions = 1476,                 /* -fms-extensions */
  OPT_fmudflap = 1477,                       /* -fmudflap */
  OPT_fmudflapir = 1478,                     /* -fmudflapir */
  OPT_fmudflapth = 1479,                     /* -fmudflapth */
  OPT_fmultiflags = 1480,                    /* -fmultiflags */
  OPT_fname_mangling_version_ = 1481,        /* -fname-mangling-version- */
  OPT_fnew_abi = 1482,                       /* -fnew-abi */
  OPT_fnew_inheriting_ctors = 1483,          /* -fnew-inheriting-ctors */
  OPT_fnew_ttp_matching = 1484,              /* -fnew-ttp-matching */
  OPT_fnext_runtime = 1485,                  /* -fnext-runtime */
  OPT_fnil = 1486,                           /* -fnil */
  OPT_fnil_receivers = 1487,                 /* -fnil-receivers */
  OPT_fno_inline_stringops = 1488,           /* -fno-inline-stringops */
  OPT_fno_modules = 1489,                    /* -fno-modules */
  OPT_fnon_call_exceptions = 1490,           /* -fnon-call-exceptions */
  OPT_fnonansi_builtins = 1491,              /* -fnonansi-builtins */
  OPT_fnonnull_objects = 1492,               /* -fnonnull-objects */
  OPT_fnothrow_opt = 1493,                   /* -fnothrow-opt */
  OPT_fobjc_abi_version_ = 1494,             /* -fobjc-abi-version= */
  OPT_fobjc_call_cxx_cdtors = 1495,          /* -fobjc-call-cxx-cdtors */
  OPT_fobjc_direct_dispatch = 1496,          /* -fobjc-direct-dispatch */
  OPT_fobjc_exceptions = 1497,               /* -fobjc-exceptions */
  OPT_fobjc_gc = 1498,                       /* -fobjc-gc */
  OPT_fobjc_nilcheck = 1499,                 /* -fobjc-nilcheck */
  OPT_fobjc_sjlj_exceptions = 1500,          /* -fobjc-sjlj-exceptions */
  OPT_fobjc_std_objc1 = 1501,                /* -fobjc-std=objc1 */
  OPT_foffload_abi_ = 1502,                  /* -foffload-abi= */
  OPT_foffload_options_ = 1503,              /* -foffload-options= */
  OPT_foffload_ = 1504,                      /* -foffload= */
  OPT_fomit_frame_pointer = 1505,            /* -fomit-frame-pointer */
  OPT_fonly_ = 1506,                         /* -fonly= */
  OPT_fopenacc = 1507,                       /* -fopenacc */
  OPT_fopenacc_dim_ = 1508,                  /* -fopenacc-dim= */
  OPT_fopenmp = 1509,                        /* -fopenmp */
  OPT_fopenmp_allocators = 1510,             /* -fopenmp-allocators */
  OPT_fopenmp_simd = 1511,                   /* -fopenmp-simd */
  /* OPT_fopenmp_target_simd_clone = 1512, *//* -fopenmp-target-simd-clone */
  OPT_fopenmp_target_simd_clone_ = 1513,     /* -fopenmp-target-simd-clone= */
  OPT_foperator_names = 1514,                /* -foperator-names */
  OPT_fopt_info = 1515,                      /* -fopt-info */
  OPT_fopt_info_ = 1516,                     /* -fopt-info- */
  /* OPT_foptimize_register_move = 1517, */  /* -foptimize-register-move */
  OPT_foptimize_sibling_calls = 1518,        /* -foptimize-sibling-calls */
  OPT_foptimize_strlen = 1519,               /* -foptimize-strlen */
  /* OPT_foptional_diags = 1520, */          /* -foptional-diags */
  OPT_fpack_derived = 1521,                  /* -fpack-derived */
  OPT_fpack_struct = 1522,                   /* -fpack-struct */
  OPT_fpack_struct_ = 1523,                  /* -fpack-struct= */
  OPT_fpad_source = 1524,                    /* -fpad-source */
  OPT_fpartial_inlining = 1525,              /* -fpartial-inlining */
  OPT_fpatchable_function_entry_ = 1526,     /* -fpatchable-function-entry= */
  OPT_fpcc_struct_return = 1527,             /* -fpcc-struct-return */
  OPT_fpch_deps = 1528,                      /* -fpch-deps */
  OPT_fpch_preprocess = 1529,                /* -fpch-preprocess */
  OPT_fpeel_loops = 1530,                    /* -fpeel-loops */
  OPT_fpeephole = 1531,                      /* -fpeephole */
  OPT_fpeephole2 = 1532,                     /* -fpeephole2 */
  OPT_fpermissive = 1533,                    /* -fpermissive */
  OPT_fpermitted_flt_eval_methods_ = 1534,   /* -fpermitted-flt-eval-methods= */
  OPT_fpic = 1535,                           /* -fpic */
  OPT_fpie = 1536,                           /* -fpie */
  OPT_fpim = 1537,                           /* -fpim */
  OPT_fpim2 = 1538,                          /* -fpim2 */
  OPT_fpim3 = 1539,                          /* -fpim3 */
  OPT_fpim4 = 1540,                          /* -fpim4 */
  OPT_fplan9_extensions = 1541,              /* -fplan9-extensions */
  OPT_fplt = 1542,                           /* -fplt */
  OPT_fplugin_arg_ = 1543,                   /* -fplugin-arg- */
  OPT_fplugin_ = 1544,                       /* -fplugin= */
  OPT_fpositive_mod_floor_div = 1545,        /* -fpositive-mod-floor-div */
  OPT_fpost_ipa_mem_report = 1546,           /* -fpost-ipa-mem-report */
  OPT_fpostconditions = 1547,                /* -fpostconditions */
  OPT_fpre_include_ = 1548,                  /* -fpre-include= */
  OPT_fpre_ipa_mem_report = 1549,            /* -fpre-ipa-mem-report */
  OPT_fpreconditions = 1550,                 /* -fpreconditions */
  OPT_fpredictive_commoning = 1551,          /* -fpredictive-commoning */
  OPT_fprefetch_loop_arrays = 1552,          /* -fprefetch-loop-arrays */
  OPT_fpreprocessed = 1553,                  /* -fpreprocessed */
  OPT_fpretty_templates = 1554,              /* -fpretty-templates */
  OPT_fpreview_all = 1555,                   /* -fpreview=all */
  OPT_fpreview_bitfields = 1556,             /* -fpreview=bitfields */
  OPT_fpreview_dip1000 = 1557,               /* -fpreview=dip1000 */
  OPT_fpreview_dip1008 = 1558,               /* -fpreview=dip1008 */
  OPT_fpreview_dip1021 = 1559,               /* -fpreview=dip1021 */
  OPT_fpreview_dtorfields = 1560,            /* -fpreview=dtorfields */
  OPT_fpreview_fieldwise = 1561,             /* -fpreview=fieldwise */
  OPT_fpreview_fixaliasthis = 1562,          /* -fpreview=fixaliasthis */
  OPT_fpreview_fiximmutableconv = 1563,      /* -fpreview=fiximmutableconv */
  OPT_fpreview_in = 1564,                    /* -fpreview=in */
  OPT_fpreview_inclusiveincontracts = 1565,  /* -fpreview=inclusiveincontracts */
  OPT_fpreview_nosharedaccess = 1566,        /* -fpreview=nosharedaccess */
  OPT_fpreview_rvaluerefparam = 1567,        /* -fpreview=rvaluerefparam */
  OPT_fpreview_systemvariables = 1568,       /* -fpreview=systemvariables */
  OPT_fprintf_return_value = 1569,           /* -fprintf-return-value */
  OPT_fprofile = 1570,                       /* -fprofile */
  OPT_fprofile_abs_path = 1571,              /* -fprofile-abs-path */
  OPT_fprofile_arcs = 1572,                  /* -fprofile-arcs */
  OPT_fprofile_correction = 1573,            /* -fprofile-correction */
  OPT_fprofile_dir_ = 1574,                  /* -fprofile-dir= */
  OPT_fprofile_exclude_files_ = 1575,        /* -fprofile-exclude-files= */
  OPT_fprofile_filter_files_ = 1576,         /* -fprofile-filter-files= */
  OPT_fprofile_generate = 1577,              /* -fprofile-generate */
  OPT_fprofile_generate_ = 1578,             /* -fprofile-generate= */
  OPT_fprofile_info_section = 1579,          /* -fprofile-info-section */
  OPT_fprofile_info_section_ = 1580,         /* -fprofile-info-section= */
  OPT_fprofile_note_ = 1581,                 /* -fprofile-note= */
  OPT_fprofile_partial_training = 1582,      /* -fprofile-partial-training */
  OPT_fprofile_prefix_map_ = 1583,           /* -fprofile-prefix-map= */
  OPT_fprofile_prefix_path_ = 1584,          /* -fprofile-prefix-path= */
  OPT_fprofile_reorder_functions = 1585,     /* -fprofile-reorder-functions */
  OPT_fprofile_report = 1586,                /* -fprofile-report */
  OPT_fprofile_reproducible_ = 1587,         /* -fprofile-reproducible= */
  OPT_fprofile_update_ = 1588,               /* -fprofile-update= */
  OPT_fprofile_use = 1589,                   /* -fprofile-use */
  OPT_fprofile_use_ = 1590,                  /* -fprofile-use= */
  OPT_fprofile_values = 1591,                /* -fprofile-values */
  OPT_fprotect_parens = 1592,                /* -fprotect-parens */
  OPT_fpthread = 1593,                       /* -fpthread */
  OPT_fq = 1594,                             /* -fq */
  OPT_frandom_seed = 1595,                   /* -frandom-seed */
  OPT_frandom_seed_ = 1596,                  /* -frandom-seed= */
  OPT_frange = 1597,                         /* -frange */
  OPT_frange_check = 1598,                   /* -frange-check */
  OPT_freal_4_real_10 = 1599,                /* -freal-4-real-10 */
  OPT_freal_4_real_16 = 1600,                /* -freal-4-real-16 */
  OPT_freal_4_real_8 = 1601,                 /* -freal-4-real-8 */
  OPT_freal_8_real_10 = 1602,                /* -freal-8-real-10 */
  OPT_freal_8_real_16 = 1603,                /* -freal-8-real-16 */
  OPT_freal_8_real_4 = 1604,                 /* -freal-8-real-4 */
  OPT_frealloc_lhs = 1605,                   /* -frealloc-lhs */
  OPT_freciprocal_math = 1606,               /* -freciprocal-math */
  OPT_frecord_gcc_switches = 1607,           /* -frecord-gcc-switches */
  OPT_frecord_marker_4 = 1608,               /* -frecord-marker=4 */
  OPT_frecord_marker_8 = 1609,               /* -frecord-marker=8 */
  OPT_frecursive = 1610,                     /* -frecursive */
  OPT_free = 1611,                           /* -free */
  OPT_freg_struct_return = 1612,             /* -freg-struct-return */
  /* OPT_fregmove = 1613, */                 /* -fregmove */
  OPT_frelease = 1614,                       /* -frelease */
  OPT_frename_registers = 1615,              /* -frename-registers */
  OPT_freorder_blocks = 1616,                /* -freorder-blocks */
  OPT_freorder_blocks_algorithm_ = 1617,     /* -freorder-blocks-algorithm= */
  OPT_freorder_blocks_and_partition = 1618,  /* -freorder-blocks-and-partition */
  OPT_freorder_functions = 1619,             /* -freorder-functions */
  OPT_frepack_arrays = 1620,                 /* -frepack-arrays */
  OPT_freplace_objc_classes = 1621,          /* -freplace-objc-classes */
  OPT_frepo = 1622,                          /* -frepo */
  OPT_freport_bug = 1623,                    /* -freport-bug */
  OPT_frequire_return_statement = 1624,      /* -frequire-return-statement */
  OPT_frerun_cse_after_loop = 1625,          /* -frerun-cse-after-loop */
  /* OPT_frerun_loop_opt = 1626, */          /* -frerun-loop-opt */
  OPT_freschedule_modulo_scheduled_loops = 1627,/* -freschedule-modulo-scheduled-loops */
  OPT_fresolution_ = 1628,                   /* -fresolution= */
  OPT_freturn = 1629,                        /* -freturn */
  OPT_frevert_all = 1630,                    /* -frevert=all */
  OPT_frevert_dip1000 = 1631,                /* -frevert=dip1000 */
  OPT_frevert_dtorfields = 1632,             /* -frevert=dtorfields */
  OPT_frevert_intpromote = 1633,             /* -frevert=intpromote */
  OPT_frounding_math = 1634,                 /* -frounding-math */
  OPT_frtti = 1635,                          /* -frtti */
  OPT_fruntime_modules_ = 1636,              /* -fruntime-modules= */
  OPT_frust_borrowcheck = 1637,              /* -frust-borrowcheck */
  OPT_frust_cfg_ = 1638,                     /* -frust-cfg= */
  OPT_frust_compile_until_ = 1639,           /* -frust-compile-until= */
  OPT_frust_crate_type_ = 1640,              /* -frust-crate-type= */
  OPT_frust_crate_ = 1641,                   /* -frust-crate= */
  OPT_frust_debug = 1642,                    /* -frust-debug */
  OPT_frust_dump_ = 1643,                    /* -frust-dump- */
  OPT_frust_edition_ = 1644,                 /* -frust-edition= */
  OPT_frust_embed_metadata = 1645,           /* -frust-embed-metadata */
  OPT_frust_extern_ = 1646,                  /* -frust-extern= */
  OPT_frust_incomplete_and_experimental_compiler_do_not_use = 1647,/* -frust-incomplete-and-experimental-compiler-do-not-use */
  OPT_frust_mangling_ = 1648,                /* -frust-mangling= */
  OPT_frust_max_recursion_depth_ = 1649,     /* -frust-max-recursion-depth= */
  OPT_frust_metadata_output_ = 1650,         /* -frust-metadata-output= */
  OPT_frust_name_resolution_2_0 = 1651,      /* -frust-name-resolution-2.0 */
  OPT_fsanitize_address_use_after_scope = 1652,/* -fsanitize-address-use-after-scope */
  OPT_fsanitize_coverage_ = 1653,            /* -fsanitize-coverage= */
  OPT_fsanitize_recover = 1654,              /* -fsanitize-recover */
  OPT_fsanitize_recover_ = 1655,             /* -fsanitize-recover= */
  OPT_fsanitize_sections_ = 1656,            /* -fsanitize-sections= */
  OPT_fsanitize_trap = 1657,                 /* -fsanitize-trap */
  OPT_fsanitize_trap_ = 1658,                /* -fsanitize-trap= */
  /* OPT_fsanitize_undefined_trap_on_error = 1659, *//* -fsanitize-undefined-trap-on-error */
  OPT_fsanitize_ = 1660,                     /* -fsanitize= */
  OPT_fsave_mixins_ = 1661,                  /* -fsave-mixins= */
  OPT_fsave_optimization_record = 1662,      /* -fsave-optimization-record */
  OPT_fscaffold_c = 1663,                    /* -fscaffold-c */
  OPT_fscaffold_c__ = 1664,                  /* -fscaffold-c++ */
  OPT_fscaffold_dynamic = 1665,              /* -fscaffold-dynamic */
  OPT_fscaffold_main = 1666,                 /* -fscaffold-main */
  OPT_fscaffold_static = 1667,               /* -fscaffold-static */
  OPT_fsched_critical_path_heuristic = 1668, /* -fsched-critical-path-heuristic */
  OPT_fsched_dep_count_heuristic = 1669,     /* -fsched-dep-count-heuristic */
  OPT_fsched_group_heuristic = 1670,         /* -fsched-group-heuristic */
  OPT_fsched_interblock = 1671,              /* -fsched-interblock */
  OPT_fsched_last_insn_heuristic = 1672,     /* -fsched-last-insn-heuristic */
  OPT_fsched_pressure = 1673,                /* -fsched-pressure */
  OPT_fsched_rank_heuristic = 1674,          /* -fsched-rank-heuristic */
  OPT_fsched_spec = 1675,                    /* -fsched-spec */
  OPT_fsched_spec_insn_heuristic = 1676,     /* -fsched-spec-insn-heuristic */
  OPT_fsched_spec_load = 1677,               /* -fsched-spec-load */
  OPT_fsched_spec_load_dangerous = 1678,     /* -fsched-spec-load-dangerous */
  OPT_fsched_stalled_insns = 1679,           /* -fsched-stalled-insns */
  OPT_fsched_stalled_insns_dep = 1680,       /* -fsched-stalled-insns-dep */
  OPT_fsched_stalled_insns_dep_ = 1681,      /* -fsched-stalled-insns-dep= */
  OPT_fsched_stalled_insns_ = 1682,          /* -fsched-stalled-insns= */
  OPT_fsched_verbose_ = 1683,                /* -fsched-verbose= */
  OPT_fsched2_use_superblocks = 1684,        /* -fsched2-use-superblocks */
  /* OPT_fsched2_use_traces = 1685, */       /* -fsched2-use-traces */
  OPT_fschedule_fusion = 1686,               /* -fschedule-fusion */
  OPT_fschedule_insns = 1687,                /* -fschedule-insns */
  OPT_fschedule_insns2 = 1688,               /* -fschedule-insns2 */
  OPT_fsecond_underscore = 1689,             /* -fsecond-underscore */
  OPT_fsection_anchors = 1690,               /* -fsection-anchors */
  /* OPT_fsee = 1691, */                     /* -fsee */
  OPT_fsel_sched_pipelining = 1692,          /* -fsel-sched-pipelining */
  OPT_fsel_sched_pipelining_outer_loops = 1693,/* -fsel-sched-pipelining-outer-loops */
  OPT_fsel_sched_reschedule_pipelined = 1694,/* -fsel-sched-reschedule-pipelined */
  OPT_fselective_scheduling = 1695,          /* -fselective-scheduling */
  OPT_fselective_scheduling2 = 1696,         /* -fselective-scheduling2 */
  OPT_fself_test_ = 1697,                    /* -fself-test= */
  OPT_fsemantic_interposition = 1698,        /* -fsemantic-interposition */
  OPT_fshared = 1699,                        /* -fshared */
  OPT_fshort_enums = 1700,                   /* -fshort-enums */
  OPT_fshort_wchar = 1701,                   /* -fshort-wchar */
  OPT_fshow_column = 1702,                   /* -fshow-column */
  OPT_fshrink_wrap = 1703,                   /* -fshrink-wrap */
  OPT_fshrink_wrap_separate = 1704,          /* -fshrink-wrap-separate */
  OPT_fsign_zero = 1705,                     /* -fsign-zero */
  OPT_fsignaling_nans = 1706,                /* -fsignaling-nans */
  OPT_fsigned_bitfields = 1707,              /* -fsigned-bitfields */
  OPT_fsigned_char = 1708,                   /* -fsigned-char */
  OPT_fsigned_zeros = 1709,                  /* -fsigned-zeros */
  OPT_fsimd_cost_model_ = 1710,              /* -fsimd-cost-model= */
  OPT_fsingle_precision_constant = 1711,     /* -fsingle-precision-constant */
  OPT_fsized_deallocation = 1712,            /* -fsized-deallocation */
  OPT_fsoft_check_all = 1713,                /* -fsoft-check-all */
  OPT_fsources = 1714,                       /* -fsources */
  OPT_fsplit_ivs_in_unroller = 1715,         /* -fsplit-ivs-in-unroller */
  OPT_fsplit_loops = 1716,                   /* -fsplit-loops */
  OPT_fsplit_paths = 1717,                   /* -fsplit-paths */
  OPT_fsplit_stack = 1718,                   /* -fsplit-stack */
  OPT_fsplit_wide_types = 1719,              /* -fsplit-wide-types */
  OPT_fsplit_wide_types_early = 1720,        /* -fsplit-wide-types-early */
  OPT_fsquangle = 1721,                      /* -fsquangle */
  OPT_fssa_backprop = 1722,                  /* -fssa-backprop */
  OPT_fssa_phiopt = 1723,                    /* -fssa-phiopt */
  OPT_fsso_struct_ = 1724,                   /* -fsso-struct= */
  OPT_fstack_arrays = 1725,                  /* -fstack-arrays */
  /* OPT_fstack_check = 1726, */             /* -fstack-check */
  OPT_fstack_check_ = 1727,                  /* -fstack-check= */
  OPT_fstack_clash_protection = 1728,        /* -fstack-clash-protection */
  OPT_fstack_limit = 1729,                   /* -fstack-limit */
  OPT_fstack_limit_register_ = 1730,         /* -fstack-limit-register= */
  OPT_fstack_limit_symbol_ = 1731,           /* -fstack-limit-symbol= */
  OPT_fstack_protector = 1732,               /* -fstack-protector */
  OPT_fstack_protector_all = 1733,           /* -fstack-protector-all */
  OPT_fstack_protector_explicit = 1734,      /* -fstack-protector-explicit */
  OPT_fstack_protector_strong = 1735,        /* -fstack-protector-strong */
  OPT_fstack_reuse_ = 1736,                  /* -fstack-reuse= */
  OPT_fstack_usage = 1737,                   /* -fstack-usage */
  OPT_fstats = 1738,                         /* -fstats */
  OPT_fstdarg_opt = 1739,                    /* -fstdarg-opt */
  OPT_fstore_merging = 1740,                 /* -fstore-merging */
  /* OPT_fstrength_reduce = 1741, */         /* -fstrength-reduce */
  OPT_fstrict_aliasing = 1742,               /* -fstrict-aliasing */
  OPT_fstrict_enums = 1743,                  /* -fstrict-enums */
  /* OPT_fstrict_flex_arrays = 1744, */      /* -fstrict-flex-arrays */
  OPT_fstrict_flex_arrays_ = 1745,           /* -fstrict-flex-arrays= */
  OPT_fstrict_overflow = 1746,               /* -fstrict-overflow */
  OPT_fstrict_prototype = 1747,              /* -fstrict-prototype */
  OPT_fstrict_volatile_bitfields = 1748,     /* -fstrict-volatile-bitfields */
  /* OPT_fstrong_eval_order = 1749, */       /* -fstrong-eval-order */
  OPT_fstrong_eval_order_ = 1750,            /* -fstrong-eval-order= */
  OPT_fstrub_all = 1751,                     /* -fstrub=all */
  OPT_fstrub_at_calls = 1752,                /* -fstrub=at-calls */
  OPT_fstrub_disable = 1753,                 /* -fstrub=disable */
  OPT_fstrub_internal = 1754,                /* -fstrub=internal */
  OPT_fstrub_relaxed = 1755,                 /* -fstrub=relaxed */
  OPT_fstrub_strict = 1756,                  /* -fstrub=strict */
  OPT_fswig = 1757,                          /* -fswig */
  OPT_fswitch_errors = 1758,                 /* -fswitch-errors */
  OPT_fsync_libcalls = 1759,                 /* -fsync-libcalls */
  OPT_fsyntax_only = 1760,                   /* -fsyntax-only */
  OPT_ftabstop_ = 1761,                      /* -ftabstop= */
  /* OPT_ftail_call_workaround = 1762, */    /* -ftail-call-workaround */
  OPT_ftail_call_workaround_ = 1763,         /* -ftail-call-workaround= */
  /* OPT_ftarget_help = 1764, */             /* -ftarget-help */
  OPT_ftemplate_backtrace_limit_ = 1765,     /* -ftemplate-backtrace-limit= */
  /* OPT_ftemplate_depth_ = 1766, */         /* -ftemplate-depth- */
  OPT_ftemplate_depth_ = 1767,               /* -ftemplate-depth= */
  OPT_ftest_coverage = 1768,                 /* -ftest-coverage */
  OPT_ftest_forall_temp = 1769,              /* -ftest-forall-temp */
  OPT_fthis_is_variable = 1770,              /* -fthis-is-variable */
  OPT_fthread_jumps = 1771,                  /* -fthread-jumps */
  OPT_fthreadsafe_statics = 1772,            /* -fthreadsafe-statics */
  OPT_ftime_report = 1773,                   /* -ftime-report */
  OPT_ftime_report_details = 1774,           /* -ftime-report-details */
  OPT_ftls_model_ = 1775,                    /* -ftls-model= */
  OPT_ftoplevel_reorder = 1776,              /* -ftoplevel-reorder */
  OPT_ftracer = 1777,                        /* -ftracer */
  OPT_ftrack_macro_expansion = 1778,         /* -ftrack-macro-expansion */
  OPT_ftrack_macro_expansion_ = 1779,        /* -ftrack-macro-expansion= */
  OPT_ftrampoline_impl_ = 1780,              /* -ftrampoline-impl= */
  OPT_ftrampolines = 1781,                   /* -ftrampolines */
  OPT_ftransition_all = 1782,                /* -ftransition=all */
  OPT_ftransition_field = 1783,              /* -ftransition=field */
  OPT_ftransition_in = 1784,                 /* -ftransition=in */
  OPT_ftransition_nogc = 1785,               /* -ftransition=nogc */
  OPT_ftransition_templates = 1786,          /* -ftransition=templates */
  OPT_ftransition_tls = 1787,                /* -ftransition=tls */
  OPT_ftrapping_math = 1788,                 /* -ftrapping-math */
  OPT_ftrapv = 1789,                         /* -ftrapv */
  OPT_ftree_bit_ccp = 1790,                  /* -ftree-bit-ccp */
  OPT_ftree_builtin_call_dce = 1791,         /* -ftree-builtin-call-dce */
  OPT_ftree_ccp = 1792,                      /* -ftree-ccp */
  OPT_ftree_ch = 1793,                       /* -ftree-ch */
  /* OPT_ftree_coalesce_inlined_vars = 1794, *//* -ftree-coalesce-inlined-vars */
  OPT_ftree_coalesce_vars = 1795,            /* -ftree-coalesce-vars */
  OPT_ftree_copy_prop = 1796,                /* -ftree-copy-prop */
  /* OPT_ftree_copyrename = 1797, */         /* -ftree-copyrename */
  OPT_ftree_cselim = 1798,                   /* -ftree-cselim */
  OPT_ftree_dce = 1799,                      /* -ftree-dce */
  OPT_ftree_dominator_opts = 1800,           /* -ftree-dominator-opts */
  OPT_ftree_dse = 1801,                      /* -ftree-dse */
  OPT_ftree_forwprop = 1802,                 /* -ftree-forwprop */
  OPT_ftree_fre = 1803,                      /* -ftree-fre */
  OPT_ftree_loop_distribute_patterns = 1804, /* -ftree-loop-distribute-patterns */
  OPT_ftree_loop_distribution = 1805,        /* -ftree-loop-distribution */
  OPT_ftree_loop_if_convert = 1806,          /* -ftree-loop-if-convert */
  /* OPT_ftree_loop_if_convert_stores = 1807, *//* -ftree-loop-if-convert-stores */
  OPT_ftree_loop_im = 1808,                  /* -ftree-loop-im */
  OPT_ftree_loop_ivcanon = 1809,             /* -ftree-loop-ivcanon */
  /* OPT_ftree_loop_linear = 1810, */        /* -ftree-loop-linear */
  OPT_ftree_loop_optimize = 1811,            /* -ftree-loop-optimize */
  OPT_ftree_loop_vectorize = 1812,           /* -ftree-loop-vectorize */
  OPT_ftree_lrs = 1813,                      /* -ftree-lrs */
  OPT_ftree_parallelize_loops_ = 1814,       /* -ftree-parallelize-loops= */
  OPT_ftree_partial_pre = 1815,              /* -ftree-partial-pre */
  OPT_ftree_phiprop = 1816,                  /* -ftree-phiprop */
  OPT_ftree_pre = 1817,                      /* -ftree-pre */
  OPT_ftree_pta = 1818,                      /* -ftree-pta */
  OPT_ftree_reassoc = 1819,                  /* -ftree-reassoc */
  /* OPT_ftree_salias = 1820, */             /* -ftree-salias */
  OPT_ftree_scev_cprop = 1821,               /* -ftree-scev-cprop */
  OPT_ftree_sink = 1822,                     /* -ftree-sink */
  OPT_ftree_slp_vectorize = 1823,            /* -ftree-slp-vectorize */
  OPT_ftree_slsr = 1824,                     /* -ftree-slsr */
  OPT_ftree_sra = 1825,                      /* -ftree-sra */
  /* OPT_ftree_store_ccp = 1826, */          /* -ftree-store-ccp */
  /* OPT_ftree_store_copy_prop = 1827, */    /* -ftree-store-copy-prop */
  OPT_ftree_switch_conversion = 1828,        /* -ftree-switch-conversion */
  OPT_ftree_tail_merge = 1829,               /* -ftree-tail-merge */
  OPT_ftree_ter = 1830,                      /* -ftree-ter */
  /* OPT_ftree_vect_loop_version = 1831, */  /* -ftree-vect-loop-version */
  OPT_ftree_vectorize = 1832,                /* -ftree-vectorize */
  /* OPT_ftree_vectorizer_verbose_ = 1833, *//* -ftree-vectorizer-verbose= */
  OPT_ftree_vrp = 1834,                      /* -ftree-vrp */
  OPT_ftrivial_auto_var_init_ = 1835,        /* -ftrivial-auto-var-init= */
  OPT_funbounded_by_reference = 1836,        /* -funbounded-by-reference */
  OPT_funconstrained_commons = 1837,         /* -funconstrained-commons */
  OPT_funderscoring = 1838,                  /* -funderscoring */
  OPT_funit_at_a_time = 1839,                /* -funit-at-a-time */
  OPT_funittest = 1840,                      /* -funittest */
  OPT_funreachable_traps = 1841,             /* -funreachable-traps */
  OPT_funroll_all_loops = 1842,              /* -funroll-all-loops */
  OPT_funroll_completely_grow_size = 1843,   /* -funroll-completely-grow-size */
  OPT_funroll_loops = 1844,                  /* -funroll-loops */
  /* OPT_funsafe_loop_optimizations = 1845, *//* -funsafe-loop-optimizations */
  OPT_funsafe_math_optimizations = 1846,     /* -funsafe-math-optimizations */
  OPT_funsigned_bitfields = 1847,            /* -funsigned-bitfields */
  OPT_funsigned_char = 1848,                 /* -funsigned-char */
  OPT_funswitch_loops = 1849,                /* -funswitch-loops */
  OPT_funwind_tables = 1850,                 /* -funwind-tables */
  OPT_fuse_cxa_atexit = 1851,                /* -fuse-cxa-atexit */
  OPT_fuse_cxa_get_exception_ptr = 1852,     /* -fuse-cxa-get-exception-ptr */
  OPT_fuse_ld_bfd = 1853,                    /* -fuse-ld=bfd */
  OPT_fuse_ld_gold = 1854,                   /* -fuse-ld=gold */
  OPT_fuse_ld_lld = 1855,                    /* -fuse-ld=lld */
  OPT_fuse_ld_mold = 1856,                   /* -fuse-ld=mold */
  OPT_fuse_linker_plugin = 1857,             /* -fuse-linker-plugin */
  OPT_fuse_list_ = 1858,                     /* -fuse-list= */
  OPT_fvar_tracking = 1859,                  /* -fvar-tracking */
  OPT_fvar_tracking_assignments = 1860,      /* -fvar-tracking-assignments */
  OPT_fvar_tracking_assignments_toggle = 1861,/* -fvar-tracking-assignments-toggle */
  OPT_fvar_tracking_uninit = 1862,           /* -fvar-tracking-uninit */
  OPT_fvariable_expansion_in_unroller = 1863,/* -fvariable-expansion-in-unroller */
  /* OPT_fvect_cost_model = 1864, */         /* -fvect-cost-model */
  OPT_fvect_cost_model_ = 1865,              /* -fvect-cost-model= */
  OPT_fverbose_asm = 1866,                   /* -fverbose-asm */
  /* OPT_fversion = 1867, */                 /* -fversion */
  OPT_fversion_loops_for_strides = 1868,     /* -fversion-loops-for-strides */
  OPT_fversion_ = 1869,                      /* -fversion= */
  OPT_fvisibility_inlines_hidden = 1870,     /* -fvisibility-inlines-hidden */
  OPT_fvisibility_ms_compat = 1871,          /* -fvisibility-ms-compat */
  OPT_fvisibility_ = 1872,                   /* -fvisibility= */
  OPT_fvpt = 1873,                           /* -fvpt */
  OPT_fvtable_gc = 1874,                     /* -fvtable-gc */
  OPT_fvtable_thunks = 1875,                 /* -fvtable-thunks */
  OPT_fvtable_verify_ = 1876,                /* -fvtable-verify= */
  OPT_fvtv_counts = 1877,                    /* -fvtv-counts */
  OPT_fvtv_debug = 1878,                     /* -fvtv-debug */
  OPT_fweak = 1879,                          /* -fweak */
  OPT_fweak_templates = 1880,                /* -fweak-templates */
  OPT_fweb = 1881,                           /* -fweb */
  /* OPT_fwhole_file = 1882, */              /* -fwhole-file */
  OPT_fwhole_program = 1883,                 /* -fwhole-program */
  OPT_fwholediv = 1884,                      /* -fwholediv */
  OPT_fwholevalue = 1885,                    /* -fwholevalue */
  OPT_fwide_exec_charset_ = 1886,            /* -fwide-exec-charset= */
  OPT_fworking_directory = 1887,             /* -fworking-directory */
  OPT_fwpa = 1888,                           /* -fwpa */
  OPT_fwpa_ = 1889,                          /* -fwpa= */
  OPT_fwrapv = 1890,                         /* -fwrapv */
  OPT_fwrapv_pointer = 1891,                 /* -fwrapv-pointer */
  OPT_fxref = 1892,                          /* -fxref */
  /* OPT_fzee = 1893, */                     /* -fzee */
  OPT_fzero_call_used_regs_ = 1894,          /* -fzero-call-used-regs= */
  OPT_fzero_initialized_in_bss = 1895,       /* -fzero-initialized-in-bss */
  OPT_fzero_link = 1896,                     /* -fzero-link */
  OPT_g = 1897,                              /* -g */
  OPT_gant = 1898,                           /* -gant */
  OPT_gas_loc_support = 1899,                /* -gas-loc-support */
  OPT_gas_locview_support = 1900,            /* -gas-locview-support */
  OPT_gbtf = 1901,                           /* -gbtf */
  OPT_gcodeview = 1902,                      /* -gcodeview */
  OPT_gcoff = 1903,                          /* -gcoff */
  OPT_gcoff1 = 1904,                         /* -gcoff1 */
  OPT_gcoff2 = 1905,                         /* -gcoff2 */
  OPT_gcoff3 = 1906,                         /* -gcoff3 */
  OPT_gcolumn_info = 1907,                   /* -gcolumn-info */
  OPT_gctf = 1908,                           /* -gctf */
  OPT_gdescribe_dies = 1909,                 /* -gdescribe-dies */
  OPT_gdwarf = 1910,                         /* -gdwarf */
  OPT_gdwarf_ = 1911,                        /* -gdwarf- */
  OPT_gdwarf32 = 1912,                       /* -gdwarf32 */
  OPT_gdwarf64 = 1913,                       /* -gdwarf64 */
  OPT_gen_decls = 1914,                      /* -gen-decls */
  OPT_ggdb = 1915,                           /* -ggdb */
  OPT_ggnu_pubnames = 1916,                  /* -ggnu-pubnames */
  OPT_gimple_stats = 1917,                   /* -gimple-stats */
  OPT_ginline_points = 1918,                 /* -ginline-points */
  OPT_ginternal_reset_location_views = 1919, /* -ginternal-reset-location-views */
  OPT_gnat = 1920,                           /* -gnat */
  OPT_gnatO = 1921,                          /* -gnatO */
  OPT_gno_ = 1922,                           /* -gno- */
  OPT_gno_pubnames = 1923,                   /* -gno-pubnames */
  OPT_gpubnames = 1924,                      /* -gpubnames */
  OPT_grecord_gcc_switches = 1925,           /* -grecord-gcc-switches */
  OPT_gsplit_dwarf = 1926,                   /* -gsplit-dwarf */
  OPT_gstabs = 1927,                         /* -gstabs */
  OPT_gstabs_ = 1928,                        /* -gstabs+ */
  OPT_gstatement_frontiers = 1929,           /* -gstatement-frontiers */
  OPT_gstrict_dwarf = 1930,                  /* -gstrict-dwarf */
  OPT_gtoggle = 1931,                        /* -gtoggle */
  OPT_gvariable_location_views = 1932,       /* -gvariable-location-views */
  OPT_gvariable_location_views_incompat5 = 1933,/* -gvariable-location-views=incompat5 */
  OPT_gvms = 1934,                           /* -gvms */
  OPT_gxcoff = 1935,                         /* -gxcoff */
  OPT_gxcoff_ = 1936,                        /* -gxcoff+ */
  OPT_gz = 1937,                             /* -gz */
  OPT_gz_ = 1938,                            /* -gz= */
  OPT_h = 1939,                              /* -h */
  OPT_help = 1940,                           /* -help */
  OPT_idirafter = 1941,                      /* -idirafter */
  OPT_imacros = 1942,                        /* -imacros */
  OPT_imultiarch = 1943,                     /* -imultiarch */
  OPT_imultilib = 1944,                      /* -imultilib */
  OPT_include = 1945,                        /* -include */
  OPT_iplugindir_ = 1946,                    /* -iplugindir= */
  OPT_iprefix = 1947,                        /* -iprefix */
  OPT_iquote = 1948,                         /* -iquote */
  OPT_isysroot = 1949,                       /* -isysroot */
  OPT_isystem = 1950,                        /* -isystem */
  OPT_iwithprefix = 1951,                    /* -iwithprefix */
  OPT_iwithprefixbefore = 1952,              /* -iwithprefixbefore */
  OPT_k8 = 1953,                             /* -k8 */
  OPT_l = 1954,                              /* -l */
  OPT_lang_asm = 1955,                       /* -lang-asm */
  OPT_list = 1956,                           /* -list */
  OPT_m128bit_long_double = 1957,            /* -m128bit-long-double */
  OPT_m16 = 1958,                            /* -m16 */
  OPT_m32 = 1959,                            /* -m32 */
  OPT_m3dnow = 1960,                         /* -m3dnow */
  OPT_m3dnowa = 1961,                        /* -m3dnowa */
  OPT_m64 = 1962,                            /* -m64 */
  OPT_m80387 = 1963,                         /* -m80387 */
  OPT_m8bit_idiv = 1964,                     /* -m8bit-idiv */
  OPT_m96bit_long_double = 1965,             /* -m96bit-long-double */
  OPT_mabi_ = 1966,                          /* -mabi= */
  OPT_mabm = 1967,                           /* -mabm */
  OPT_maccumulate_outgoing_args = 1968,      /* -maccumulate-outgoing-args */
  OPT_maddress_mode_ = 1969,                 /* -maddress-mode= */
  OPT_madx = 1970,                           /* -madx */
  OPT_maes = 1971,                           /* -maes */
  OPT_malign_data_ = 1972,                   /* -malign-data= */
  OPT_malign_double = 1973,                  /* -malign-double */
  OPT_malign_functions_ = 1974,              /* -malign-functions= */
  OPT_malign_jumps_ = 1975,                  /* -malign-jumps= */
  OPT_malign_loops_ = 1976,                  /* -malign-loops= */
  OPT_malign_stringops = 1977,               /* -malign-stringops */
  OPT_mamx_bf16 = 1978,                      /* -mamx-bf16 */
  OPT_mamx_complex = 1979,                   /* -mamx-complex */
  OPT_mamx_fp16 = 1980,                      /* -mamx-fp16 */
  OPT_mamx_int8 = 1981,                      /* -mamx-int8 */
  OPT_mamx_tile = 1982,                      /* -mamx-tile */
  OPT_mapx_features_ = 1983,                 /* -mapx-features= */
  OPT_mapx_inline_asm_use_gpr32 = 1984,      /* -mapx-inline-asm-use-gpr32 */
  OPT_mapxf = 1985,                          /* -mapxf */
  OPT_march_ = 1986,                         /* -march= */
  OPT_masm_ = 1987,                          /* -masm= */
  OPT_mavx = 1988,                           /* -mavx */
  /* OPT_mavx10_1 = 1989, */                 /* -mavx10.1 */
  OPT_mavx10_1_256 = 1990,                   /* -mavx10.1-256 */
  OPT_mavx10_1_512 = 1991,                   /* -mavx10.1-512 */
  OPT_mavx2 = 1992,                          /* -mavx2 */
  OPT_mavx256_split_unaligned_load = 1993,   /* -mavx256-split-unaligned-load */
  OPT_mavx256_split_unaligned_store = 1994,  /* -mavx256-split-unaligned-store */
  OPT_mavx5124fmaps = 1995,                  /* -mavx5124fmaps */
  OPT_mavx5124vnniw = 1996,                  /* -mavx5124vnniw */
  OPT_mavx512bf16 = 1997,                    /* -mavx512bf16 */
  OPT_mavx512bitalg = 1998,                  /* -mavx512bitalg */
  OPT_mavx512bw = 1999,                      /* -mavx512bw */
  OPT_mavx512cd = 2000,                      /* -mavx512cd */
  OPT_mavx512dq = 2001,                      /* -mavx512dq */
  OPT_mavx512er = 2002,                      /* -mavx512er */
  OPT_mavx512f = 2003,                       /* -mavx512f */
  OPT_mavx512fp16 = 2004,                    /* -mavx512fp16 */
  OPT_mavx512ifma = 2005,                    /* -mavx512ifma */
  OPT_mavx512pf = 2006,                      /* -mavx512pf */
  OPT_mavx512vbmi = 2007,                    /* -mavx512vbmi */
  OPT_mavx512vbmi2 = 2008,                   /* -mavx512vbmi2 */
  OPT_mavx512vl = 2009,                      /* -mavx512vl */
  OPT_mavx512vnni = 2010,                    /* -mavx512vnni */
  OPT_mavx512vp2intersect = 2011,            /* -mavx512vp2intersect */
  OPT_mavx512vpopcntdq = 2012,               /* -mavx512vpopcntdq */
  OPT_mavxifma = 2013,                       /* -mavxifma */
  OPT_mavxneconvert = 2014,                  /* -mavxneconvert */
  OPT_mavxvnni = 2015,                       /* -mavxvnni */
  OPT_mavxvnniint16 = 2016,                  /* -mavxvnniint16 */
  OPT_mavxvnniint8 = 2017,                   /* -mavxvnniint8 */
  OPT_mbmi = 2018,                           /* -mbmi */
  OPT_mbmi2 = 2019,                          /* -mbmi2 */
  OPT_mbranch_cost_ = 2020,                  /* -mbranch-cost= */
  OPT_mcall_ms2sysv_xlogues = 2021,          /* -mcall-ms2sysv-xlogues */
  OPT_mcet_switch = 2022,                    /* -mcet-switch */
  OPT_mcld = 2023,                           /* -mcld */
  OPT_mcldemote = 2024,                      /* -mcldemote */
  OPT_mclflushopt = 2025,                    /* -mclflushopt */
  OPT_mclwb = 2026,                          /* -mclwb */
  OPT_mclzero = 2027,                        /* -mclzero */
  OPT_mcmodel_ = 2028,                       /* -mcmodel= */
  OPT_mcmpccxadd = 2029,                     /* -mcmpccxadd */
  /* OPT_mcpu_ = 2030, */                    /* -mcpu= */
  OPT_mcrc32 = 2031,                         /* -mcrc32 */
  OPT_mcx16 = 2032,                          /* -mcx16 */
  OPT_mdaz_ftz = 2033,                       /* -mdaz-ftz */
  OPT_mdirect_extern_access = 2034,          /* -mdirect-extern-access */
  OPT_mdispatch_scheduler = 2035,            /* -mdispatch-scheduler */
  OPT_mdump_tune_features = 2036,            /* -mdump-tune-features */
  OPT_menqcmd = 2037,                        /* -menqcmd */
  OPT_mevex512 = 2038,                       /* -mevex512 */
  OPT_mf16c = 2039,                          /* -mf16c */
  OPT_mfancy_math_387 = 2040,                /* -mfancy-math-387 */
  OPT_mfentry = 2041,                        /* -mfentry */
  OPT_mfentry_name_ = 2042,                  /* -mfentry-name= */
  OPT_mfentry_section_ = 2043,               /* -mfentry-section= */
  OPT_mfma = 2044,                           /* -mfma */
  OPT_mfma4 = 2045,                          /* -mfma4 */
  OPT_mforce_drap = 2046,                    /* -mforce-drap */
  OPT_mforce_indirect_call = 2047,           /* -mforce-indirect-call */
  OPT_mfp_ret_in_387 = 2048,                 /* -mfp-ret-in-387 */
  OPT_mfpmath_ = 2049,                       /* -mfpmath= */
  OPT_mfsgsbase = 2050,                      /* -mfsgsbase */
  OPT_mfunction_return_ = 2051,              /* -mfunction-return= */
  /* OPT_mfused_madd = 2052, */              /* -mfused-madd */
  OPT_mfxsr = 2053,                          /* -mfxsr */
  /* OPT_mgather = 2054, */                  /* -mgather */
  OPT_mgeneral_regs_only = 2055,             /* -mgeneral-regs-only */
  OPT_mgfni = 2056,                          /* -mgfni */
  OPT_mhard_float = 2057,                    /* -mhard-float */
  OPT_mharden_sls_ = 2058,                   /* -mharden-sls= */
  OPT_mhle = 2059,                           /* -mhle */
  OPT_mhreset = 2060,                        /* -mhreset */
  OPT_miamcu = 2061,                         /* -miamcu */
  OPT_mieee_fp = 2062,                       /* -mieee-fp */
  OPT_mincoming_stack_boundary_ = 2063,      /* -mincoming-stack-boundary= */
  OPT_mindirect_branch_cs_prefix = 2064,     /* -mindirect-branch-cs-prefix */
  OPT_mindirect_branch_register = 2065,      /* -mindirect-branch-register */
  OPT_mindirect_branch_ = 2066,              /* -mindirect-branch= */
  OPT_minline_all_stringops = 2067,          /* -minline-all-stringops */
  OPT_minline_stringops_dynamically = 2068,  /* -minline-stringops-dynamically */
  OPT_minstrument_return_ = 2069,            /* -minstrument-return= */
  /* OPT_mintel_syntax = 2070, */            /* -mintel-syntax */
  OPT_mkl = 2071,                            /* -mkl */
  OPT_mlam_ = 2072,                          /* -mlam= */
  OPT_mlarge_data_threshold_ = 2073,         /* -mlarge-data-threshold= */
  OPT_mlong_double_128 = 2074,               /* -mlong-double-128 */
  OPT_mlong_double_64 = 2075,                /* -mlong-double-64 */
  OPT_mlong_double_80 = 2076,                /* -mlong-double-80 */
  OPT_mlwp = 2077,                           /* -mlwp */
  OPT_mlzcnt = 2078,                         /* -mlzcnt */
  OPT_mmanual_endbr = 2079,                  /* -mmanual-endbr */
  OPT_mmemcpy_strategy_ = 2080,              /* -mmemcpy-strategy= */
  OPT_mmemset_strategy_ = 2081,              /* -mmemset-strategy= */
  OPT_mmitigate_rop = 2082,                  /* -mmitigate-rop */
  OPT_mmmx = 2083,                           /* -mmmx */
  OPT_mmovbe = 2084,                         /* -mmovbe */
  OPT_mmovdir64b = 2085,                     /* -mmovdir64b */
  OPT_mmovdiri = 2086,                       /* -mmovdiri */
  OPT_mmove_max_ = 2087,                     /* -mmove-max= */
  OPT_mmpx = 2088,                           /* -mmpx */
  OPT_mms_bitfields = 2089,                  /* -mms-bitfields */
  OPT_mmwait = 2090,                         /* -mmwait */
  OPT_mmwaitx = 2091,                        /* -mmwaitx */
  OPT_mneeded = 2092,                        /* -mneeded */
  OPT_mno_align_stringops = 2093,            /* -mno-align-stringops */
  OPT_mno_default = 2094,                    /* -mno-default */
  OPT_mno_fancy_math_387 = 2095,             /* -mno-fancy-math-387 */
  OPT_mno_push_args = 2096,                  /* -mno-push-args */
  OPT_mno_red_zone = 2097,                   /* -mno-red-zone */
  OPT_mno_sse4 = 2098,                       /* -mno-sse4 */
  OPT_mnop_mcount = 2099,                    /* -mnop-mcount */
  OPT_mnoreturn_no_callee_saved_registers = 2100,/* -mnoreturn-no-callee-saved-registers */
  OPT_momit_leaf_frame_pointer = 2101,       /* -momit-leaf-frame-pointer */
  OPT_mpartial_vector_fp_math = 2102,        /* -mpartial-vector-fp-math */
  OPT_mpc32 = 2103,                          /* -mpc32 */
  OPT_mpc64 = 2104,                          /* -mpc64 */
  OPT_mpc80 = 2105,                          /* -mpc80 */
  OPT_mpclmul = 2106,                        /* -mpclmul */
  OPT_mpcommit = 2107,                       /* -mpcommit */
  OPT_mpconfig = 2108,                       /* -mpconfig */
  OPT_mpku = 2109,                           /* -mpku */
  OPT_mpopcnt = 2110,                        /* -mpopcnt */
  /* OPT_mprefer_avx128 = 2111, */           /* -mprefer-avx128 */
  OPT_mprefer_vector_width_ = 2112,          /* -mprefer-vector-width= */
  OPT_mpreferred_stack_boundary_ = 2113,     /* -mpreferred-stack-boundary= */
  OPT_mprefetchi = 2114,                     /* -mprefetchi */
  OPT_mprefetchwt1 = 2115,                   /* -mprefetchwt1 */
  OPT_mprfchw = 2116,                        /* -mprfchw */
  OPT_mptwrite = 2117,                       /* -mptwrite */
  OPT_mpush_args = 2118,                     /* -mpush-args */
  OPT_mraoint = 2119,                        /* -mraoint */
  OPT_mrdpid = 2120,                         /* -mrdpid */
  OPT_mrdrnd = 2121,                         /* -mrdrnd */
  OPT_mrdseed = 2122,                        /* -mrdseed */
  OPT_mrecip = 2123,                         /* -mrecip */
  OPT_mrecip_ = 2124,                        /* -mrecip= */
  OPT_mrecord_mcount = 2125,                 /* -mrecord-mcount */
  OPT_mrecord_return = 2126,                 /* -mrecord-return */
  OPT_mred_zone = 2127,                      /* -mred-zone */
  OPT_mregparm_ = 2128,                      /* -mregparm= */
  OPT_mrelax_cmpxchg_loop = 2129,            /* -mrelax-cmpxchg-loop */
  OPT_mrtd = 2130,                           /* -mrtd */
  OPT_mrtm = 2131,                           /* -mrtm */
  OPT_msahf = 2132,                          /* -msahf */
  /* OPT_mscatter = 2133, */                 /* -mscatter */
  OPT_mserialize = 2134,                     /* -mserialize */
  OPT_msgx = 2135,                           /* -msgx */
  OPT_msha = 2136,                           /* -msha */
  OPT_msha512 = 2137,                        /* -msha512 */
  OPT_mshstk = 2138,                         /* -mshstk */
  OPT_mskip_rax_setup = 2139,                /* -mskip-rax-setup */
  OPT_msm3 = 2140,                           /* -msm3 */
  OPT_msm4 = 2141,                           /* -msm4 */
  OPT_msoft_float = 2142,                    /* -msoft-float */
  OPT_msse = 2143,                           /* -msse */
  OPT_msse2 = 2144,                          /* -msse2 */
  OPT_msse2avx = 2145,                       /* -msse2avx */
  OPT_msse3 = 2146,                          /* -msse3 */
  OPT_msse4 = 2147,                          /* -msse4 */
  OPT_msse4_1 = 2148,                        /* -msse4.1 */
  OPT_msse4_2 = 2149,                        /* -msse4.2 */
  OPT_msse4a = 2150,                         /* -msse4a */
  /* OPT_msse5 = 2151, */                    /* -msse5 */
  OPT_msseregparm = 2152,                    /* -msseregparm */
  OPT_mssse3 = 2153,                         /* -mssse3 */
  OPT_mstack_arg_probe = 2154,               /* -mstack-arg-probe */
  OPT_mstack_protector_guard_offset_ = 2155, /* -mstack-protector-guard-offset= */
  OPT_mstack_protector_guard_reg_ = 2156,    /* -mstack-protector-guard-reg= */
  OPT_mstack_protector_guard_symbol_ = 2157, /* -mstack-protector-guard-symbol= */
  OPT_mstack_protector_guard_ = 2158,        /* -mstack-protector-guard= */
  OPT_mstackrealign = 2159,                  /* -mstackrealign */
  OPT_mstore_max_ = 2160,                    /* -mstore-max= */
  OPT_mstringop_strategy_ = 2161,            /* -mstringop-strategy= */
  OPT_mstv = 2162,                           /* -mstv */
  OPT_mtbm = 2163,                           /* -mtbm */
  OPT_mtls_dialect_ = 2164,                  /* -mtls-dialect= */
  OPT_mtls_direct_seg_refs = 2165,           /* -mtls-direct-seg-refs */
  OPT_mtsxldtrk = 2166,                      /* -mtsxldtrk */
  OPT_mtune_ctrl_ = 2167,                    /* -mtune-ctrl= */
  OPT_mtune_ = 2168,                         /* -mtune= */
  OPT_muintr = 2169,                         /* -muintr */
  OPT_munroll_only_small_loops = 2170,       /* -munroll-only-small-loops */
  OPT_musermsr = 2171,                       /* -musermsr */
  OPT_mvaes = 2172,                          /* -mvaes */
  OPT_mveclibabi_ = 2173,                    /* -mveclibabi= */
  OPT_mvect8_ret_in_mem = 2174,              /* -mvect8-ret-in-mem */
  OPT_mvpclmulqdq = 2175,                    /* -mvpclmulqdq */
  OPT_mvzeroupper = 2176,                    /* -mvzeroupper */
  OPT_mwaitpkg = 2177,                       /* -mwaitpkg */
  OPT_mwbnoinvd = 2178,                      /* -mwbnoinvd */
  OPT_mwidekl = 2179,                        /* -mwidekl */
  OPT_mx32 = 2180,                           /* -mx32 */
  OPT_mxop = 2181,                           /* -mxop */
  OPT_mxsave = 2182,                         /* -mxsave */
  OPT_mxsavec = 2183,                        /* -mxsavec */
  OPT_mxsaveopt = 2184,                      /* -mxsaveopt */
  OPT_mxsaves = 2185,                        /* -mxsaves */
  OPT_n = 2186,                              /* -n */
  OPT_name_sort = 2187,                      /* -name-sort */
  OPT_no_canonical_prefixes = 2188,          /* -no-canonical-prefixes */
  OPT_no_integrated_cpp = 2189,              /* -no-integrated-cpp */
  OPT_no_pie = 2190,                         /* -no-pie */
  OPT_nocpp = 2191,                          /* -nocpp */
  OPT_nodefaultlibs = 2192,                  /* -nodefaultlibs */
  OPT_nolibc = 2193,                         /* -nolibc */
  OPT_nophoboslib = 2194,                    /* -nophoboslib */
  OPT_nostartfiles = 2195,                   /* -nostartfiles */
  OPT_nostdinc = 2196,                       /* -nostdinc */
  OPT_nostdinc__ = 2197,                     /* -nostdinc++ */
  OPT_nostdlib = 2198,                       /* -nostdlib */
  OPT_nostdlib__ = 2199,                     /* -nostdlib++ */
  OPT_o = 2200,                              /* -o */
  OPT_objects = 2201,                        /* -objects */
  OPT_p = 2202,                              /* -p */
  OPT_pass_exit_codes = 2203,                /* -pass-exit-codes */
  /* OPT_pedantic = 2204, */                 /* -pedantic */
  OPT_pedantic_errors = 2205,                /* -pedantic-errors */
  OPT_pg = 2206,                             /* -pg */
  OPT_pie = 2207,                            /* -pie */
  OPT_pipe = 2208,                           /* -pipe */
  OPT_posix = 2209,                          /* -posix */
  OPT_print_file_name_ = 2210,               /* -print-file-name= */
  OPT_print_libgcc_file_name = 2211,         /* -print-libgcc-file-name */
  OPT_print_multi_directory = 2212,          /* -print-multi-directory */
  OPT_print_multi_lib = 2213,                /* -print-multi-lib */
  OPT_print_multi_os_directory = 2214,       /* -print-multi-os-directory */
  OPT_print_multiarch = 2215,                /* -print-multiarch */
  OPT_print_objc_runtime_info = 2216,        /* -print-objc-runtime-info */
  OPT_print_prog_name_ = 2217,               /* -print-prog-name= */
  OPT_print_search_dirs = 2218,              /* -print-search-dirs */
  OPT_print_sysroot = 2219,                  /* -print-sysroot */
  OPT_print_sysroot_headers_suffix = 2220,   /* -print-sysroot-headers-suffix */
  OPT_print_value = 2221,                    /* -print-value */
  OPT_profile = 2222,                        /* -profile */
  OPT_pthread = 2223,                        /* -pthread */
  OPT_quiet = 2224,                          /* -quiet */
  OPT_r = 2225,                              /* -r */
  OPT_rdynamic = 2226,                       /* -rdynamic */
  OPT_remap = 2227,                          /* -remap */
  OPT_reverse_sort = 2228,                   /* -reverse-sort */
  OPT_s = 2229,                              /* -s */
  OPT_save_temps = 2230,                     /* -save-temps */
  OPT_save_temps_ = 2231,                    /* -save-temps= */
  OPT_shared = 2232,                         /* -shared */
  OPT_shared_libgcc = 2233,                  /* -shared-libgcc */
  OPT_shared_libphobos = 2234,               /* -shared-libphobos */
  OPT_size_sort = 2235,                      /* -size-sort */
  /* OPT_specs = 2236, */                    /* -specs */
  OPT_specs_ = 2237,                         /* -specs= */
  OPT_static = 2238,                         /* -static */
  OPT_static_libasan = 2239,                 /* -static-libasan */
  OPT_static_libgcc = 2240,                  /* -static-libgcc */
  OPT_static_libgfortran = 2241,             /* -static-libgfortran */
  OPT_static_libgm2 = 2242,                  /* -static-libgm2 */
  OPT_static_libgo = 2243,                   /* -static-libgo */
  OPT_static_libhwasan = 2244,               /* -static-libhwasan */
  OPT_static_liblsan = 2245,                 /* -static-liblsan */
  OPT_static_libmpx = 2246,                  /* -static-libmpx */
  OPT_static_libmpxwrappers = 2247,          /* -static-libmpxwrappers */
  OPT_static_libphobos = 2248,               /* -static-libphobos */
  OPT_static_libquadmath = 2249,             /* -static-libquadmath */
  OPT_static_libstdc__ = 2250,               /* -static-libstdc++ */
  OPT_static_libtsan = 2251,                 /* -static-libtsan */
  OPT_static_libubsan = 2252,                /* -static-libubsan */
  OPT_static_pie = 2253,                     /* -static-pie */
  /* OPT_std_c__03 = 2254, */                /* -std=c++03 */
  /* OPT_std_c__0x = 2255, */                /* -std=c++0x */
  OPT_std_c__11 = 2256,                      /* -std=c++11 */
  OPT_std_c__14 = 2257,                      /* -std=c++14 */
  OPT_std_c__17 = 2258,                      /* -std=c++17 */
  /* OPT_std_c__1y = 2259, */                /* -std=c++1y */
  /* OPT_std_c__1z = 2260, */                /* -std=c++1z */
  OPT_std_c__20 = 2261,                      /* -std=c++20 */
  OPT_std_c__23 = 2262,                      /* -std=c++23 */
  OPT_std_c__26 = 2263,                      /* -std=c++26 */
  /* OPT_std_c__2a = 2264, */                /* -std=c++2a */
  /* OPT_std_c__2b = 2265, */                /* -std=c++2b */
  /* OPT_std_c__2c = 2266, */                /* -std=c++2c */
  OPT_std_c__98 = 2267,                      /* -std=c++98 */
  OPT_std_c11 = 2268,                        /* -std=c11 */
  OPT_std_c17 = 2269,                        /* -std=c17 */
  /* OPT_std_c18 = 2270, */                  /* -std=c18 */
  /* OPT_std_c1x = 2271, */                  /* -std=c1x */
  OPT_std_c23 = 2272,                        /* -std=c23 */
  /* OPT_std_c2x = 2273, */                  /* -std=c2x */
  /* OPT_std_c89 = 2274, */                  /* -std=c89 */
  OPT_std_c90 = 2275,                        /* -std=c90 */
  OPT_std_c99 = 2276,                        /* -std=c99 */
  /* OPT_std_c9x = 2277, */                  /* -std=c9x */
  OPT_std_f2003 = 2278,                      /* -std=f2003 */
  OPT_std_f2008 = 2279,                      /* -std=f2008 */
  OPT_std_f2008ts = 2280,                    /* -std=f2008ts */
  OPT_std_f2018 = 2281,                      /* -std=f2018 */
  OPT_std_f2023 = 2282,                      /* -std=f2023 */
  OPT_std_f95 = 2283,                        /* -std=f95 */
  OPT_std_gnu = 2284,                        /* -std=gnu */
  /* OPT_std_gnu__03 = 2285, */              /* -std=gnu++03 */
  /* OPT_std_gnu__0x = 2286, */              /* -std=gnu++0x */
  OPT_std_gnu__11 = 2287,                    /* -std=gnu++11 */
  OPT_std_gnu__14 = 2288,                    /* -std=gnu++14 */
  OPT_std_gnu__17 = 2289,                    /* -std=gnu++17 */
  /* OPT_std_gnu__1y = 2290, */              /* -std=gnu++1y */
  /* OPT_std_gnu__1z = 2291, */              /* -std=gnu++1z */
  OPT_std_gnu__20 = 2292,                    /* -std=gnu++20 */
  OPT_std_gnu__23 = 2293,                    /* -std=gnu++23 */
  OPT_std_gnu__26 = 2294,                    /* -std=gnu++26 */
  /* OPT_std_gnu__2a = 2295, */              /* -std=gnu++2a */
  /* OPT_std_gnu__2b = 2296, */              /* -std=gnu++2b */
  /* OPT_std_gnu__2c = 2297, */              /* -std=gnu++2c */
  OPT_std_gnu__98 = 2298,                    /* -std=gnu++98 */
  OPT_std_gnu11 = 2299,                      /* -std=gnu11 */
  OPT_std_gnu17 = 2300,                      /* -std=gnu17 */
  /* OPT_std_gnu18 = 2301, */                /* -std=gnu18 */
  /* OPT_std_gnu1x = 2302, */                /* -std=gnu1x */
  OPT_std_gnu23 = 2303,                      /* -std=gnu23 */
  /* OPT_std_gnu2x = 2304, */                /* -std=gnu2x */
  /* OPT_std_gnu89 = 2305, */                /* -std=gnu89 */
  OPT_std_gnu90 = 2306,                      /* -std=gnu90 */
  OPT_std_gnu99 = 2307,                      /* -std=gnu99 */
  /* OPT_std_gnu9x = 2308, */                /* -std=gnu9x */
  /* OPT_std_iso9899_1990 = 2309, */         /* -std=iso9899:1990 */
  OPT_std_iso9899_199409 = 2310,             /* -std=iso9899:199409 */
  /* OPT_std_iso9899_1999 = 2311, */         /* -std=iso9899:1999 */
  /* OPT_std_iso9899_199x = 2312, */         /* -std=iso9899:199x */
  /* OPT_std_iso9899_2011 = 2313, */         /* -std=iso9899:2011 */
  /* OPT_std_iso9899_2017 = 2314, */         /* -std=iso9899:2017 */
  /* OPT_std_iso9899_2018 = 2315, */         /* -std=iso9899:2018 */
  /* OPT_std_iso9899_2024 = 2316, */         /* -std=iso9899:2024 */
  OPT_std_legacy = 2317,                     /* -std=legacy */
  OPT_stdlib_ = 2318,                        /* -stdlib= */
  OPT_symbol_ = 2319,                        /* -symbol= */
  OPT_symbolic = 2320,                       /* -symbolic */
  OPT_t = 2321,                              /* -t */
  OPT_time = 2322,                           /* -time */
  OPT_time_ = 2323,                          /* -time= */
  OPT_traditional = 2324,                    /* -traditional */
  OPT_traditional_cpp = 2325,                /* -traditional-cpp */
  OPT_tree_stats = 2326,                     /* -tree-stats */
  OPT_trigraphs = 2327,                      /* -trigraphs */
  OPT_type_stats = 2328,                     /* -type-stats */
  OPT_u = 2329,                              /* -u */
  OPT_undef = 2330,                          /* -undef */
  OPT_v = 2331,                              /* -v */
  OPT_version = 2332,                        /* -version */
  OPT_w = 2333,                              /* -w */
  OPT_wrapper = 2334,                        /* -wrapper */
  OPT_x = 2335,                              /* -x */
  OPT_z = 2336,                              /* -z */
  N_OPTS,
  OPT_SPECIAL_unknown,
  OPT_SPECIAL_ignore,
  OPT_SPECIAL_warn_removed,
  OPT_SPECIAL_program_name,
  OPT_SPECIAL_input_file
};

#ifdef GCC_C_COMMON_C
/* Mapping from cpp message reasons to the options that enable them.  */
#include <cpplib.h>
struct cpp_reason_option_codes_t
{
  /* cpplib message reason.  */
  const enum cpp_warning_reason reason;
  /* gcc option that controls this message.  */
  const int option_code;
};

static const struct cpp_reason_option_codes_t cpp_reason_option_codes[] = {
  {CPP_W_BIDIRECTIONAL,                     OPT_Wbidi_chars_},
  {CPP_W_BUILTIN_MACRO_REDEFINED,           OPT_Wbuiltin_macro_redefined},
  {CPP_W_CXX_OPERATOR_NAMES,                OPT_Wc___compat},
  {CPP_W_CXX11_COMPAT,                      OPT_Wc__11_compat},
  {CPP_W_CXX20_COMPAT,                      OPT_Wc__20_compat},
  {CPP_W_C11_C23_COMPAT,                    OPT_Wc11_c23_compat},
  {CPP_W_C90_C99_COMPAT,                    OPT_Wc90_c99_compat},
  {CPP_W_COMMENTS,                          OPT_Wcomment},
  {CPP_W_WARNING_DIRECTIVE,                 OPT_Wcpp},
  {CPP_W_DATE_TIME,                         OPT_Wdate_time},
  {CPP_W_DEPRECATED,                        OPT_Wdeprecated},
  {CPP_W_ENDIF_LABELS,                      OPT_Wendif_labels},
  {CPP_W_EXPANSION_TO_DEFINED,              OPT_Wexpansion_to_defined},
  {CPP_W_INVALID_PCH,                       OPT_Winvalid_pch},
  {CPP_W_INVALID_UTF8,                      OPT_Winvalid_utf8},
  {CPP_W_LITERAL_SUFFIX,                    OPT_Wliteral_suffix},
  {CPP_W_LONG_LONG,                         OPT_Wlong_long},
  {CPP_W_MISSING_INCLUDE_DIRS,              OPT_Wmissing_include_dirs},
  {CPP_W_MULTICHAR,                         OPT_Wmultichar},
  {CPP_W_NORMALIZE,                         OPT_Wnormalized_},
  {CPP_W_PEDANTIC,                          OPT_Wpedantic},
  {CPP_W_TRADITIONAL,                       OPT_Wtraditional},
  {CPP_W_TRIGRAPHS,                         OPT_Wtrigraphs},
  {CPP_W_UNDEF,                             OPT_Wundef},
  {CPP_W_UNICODE,                           OPT_Wunicode},
  {CPP_W_UNUSED_MACROS,                     OPT_Wunused_macros},
  {CPP_W_VARIADIC_MACROS,                   OPT_Wvariadic_macros},
  {CPP_W_NONE,                              0},
};
#endif

#endif /* OPTIONS_H */
